/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.UiApplication;

import com.funambol.util.Log;

/**
 * Main Thread that realize the run of all the Blackberry client automatic test
 * application. It is responsible to display a first screen that contains the
 * editable url of the script to be run. Once the script url is defined or
 * confirmed by the user the test are started and finally a report with the
 * failed tests details is displayed to the user.
 */
public class AutoTestThread extends Thread {

    private String scriptUrl = null;
    private ScriptRunner runner = null;

    public AutoTestThread(String scriptUrl) {
        this.scriptUrl = scriptUrl;
    }

    /**
     * The main thread method that overrides the parent's definition
     */
    public void run() {
        runner = new ScriptRunner(scriptUrl);
        try {
            runner.test();
            Log.setLogLevel(Log.INFO);
            Log.info("***** TEST PASSED");
        } catch (Throwable e) {
            Log.error("***** TEST FAILED: " + e.toString());
        }
        showReport(runner.isTestSuitePassed(), runner.getDetails());
    }

    /**
     * Accessor method to retrieve the ScriptRunner object that is responsible
     * for this test suite
     * @return ScriptRunner the ScriptRunner instance used for this test suite
     */
    public ScriptRunner getScriptRunner() {
        return runner;
    }

    /**
     * Shows the test suite report with detailed result if they failed
     * @param passed set the result value true if passed, false otherwise
     * @param details the String content to be displayed in the report
     */
    private void showReport(boolean passed, String details) {

        if (details == null) {
            if (!passed) {
                details = runner.getDetails();
            } else {
                details = "";
            }
        }

        ReportScreen rs = new ReportScreen(passed, details);
        final UiApplication app = UiApplication.getUiApplication();

        class ShowScreenRequest implements Runnable {

            private MainScreen screen;

            public ShowScreenRequest(MainScreen scr) {
                screen = scr;
            }

            public void run() {
                synchronized (app.getEventLock()) {
                    if (screen.isDisplayed()) {
                        app.popScreen(screen);
                    }
                    app.pushScreen(screen);
                }
            }
        }

        ShowScreenRequest request = new ShowScreenRequest(rs);

        if (app.isEventThread()) {
            request.run();
        } else {
            app.invokeLater(request);
        }
    }
}

