/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Calendar;
import java.util.Vector;

import javax.microedition.pim.RepeatRule;
import javax.microedition.pim.EventList;
import javax.microedition.pim.PIM;

import net.rim.blackberry.api.pdap.BlackBerryEvent;

import com.funambol.blackberry.helper.VCalEventHelper;
import com.funambol.blackberry.source.CalendarAdditionalAttributes;
import com.funambol.client.test.CalendarCommandRunner;
import com.funambol.client.test.CalendarRobot;
import com.funambol.client.test.CheckSyncClient;
import com.funambol.client.test.CheckSyncSource;
import com.funambol.blackberry.helper.EventHelper;

import com.funambol.syncml.spds.SyncItem;

import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;
import com.funambol.util.DateUtil;
import com.funambol.util.Log;
/**
 * A Robot that executes commands in the test script file for events represented
 * in BlackBerryEvent format.
 */
public class BlackBerryCalendarsRobot extends CalendarRobot {
    private static final String TAG_LOG = "BlackBerryCalendarsRobot";

    public final static String RECURRENCE_TYPE    = "RecurrenceType";
    public final static String INTERVAL           = "Interval";
    public final static String DAY_OF_WEEK_MASK   = "DayOfWeekMask";
    public final static String PATTERN_END_DATE   = "PatternEndDate";
    public final static String MONTH_OF_YEAR      = "MonthOfYear";
    public final static String DAY_OF_MONTH       = "DayOfMonth";
    public final static String INSTANCE           = "Instance";
    public final static String OCCURRENCES        = "Occurrences";
    public final static String EXCLUDE_DATE       = "ExcludeDate";

    private final static String IS_RECURRING       = "IsRecurring";

    private static final String EVENT_KEY_NOT_FOUND_EXP_MSG = "Cannot find event on server";

    private BlackBerryEvent currentEvent = null;
    private Hashtable repeatRuleEventMap = null;
    
    private String summaryField = null;
    private CalendarAdditionalAttributes currentAdditional = null;

    private EventList eventList    = null;
    private Hashtable supportedValues;
    private String supportedFields[] = { 
                                        "BEGIN",
                                        "VERSION",
                                        "SUMMARY",
                                        "DESCRIPTION",
                                        "LOCATION",
                                        "DTSTART",
                                        "DTEND",
                                        "X-FUNAMBOL-ALLDAY",
                                        "END"
                                       };

    private Hashtable eventMap;

    /**
     * Default constructor
     */
    public BlackBerryCalendarsRobot() {
        try {
            eventList = (EventList) PIM.getInstance().openPIMList(PIM.EVENT_LIST,
                                                                  PIM.READ_WRITE);
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot open event list");
            // We will have a NPE later...
        }

        supportedValues = new Hashtable();
    }

    /**
     * Create an empty event base on the native EventList method
     * @throws Throwable
     */
    public void createEmptyEvent() throws Throwable {
        currentEvent = (BlackBerryEvent) eventList.createEvent();
        currentAdditional = new CalendarAdditionalAttributes();
        this.summaryField = null;
    }

    /**
     * Set a vlue to a given field of the loaded Event
     * @param field the field to be set referred to the CalendarCommandRunner class
     * @param value the value to be set in string format
     * @throws Throwable
     */
    public void setEventField(String field, String value) throws Throwable {

        // Keep track of the field value
        if(eventMap == null) { eventMap = new Hashtable(); }
        eventMap.put(field, value);
        
        if (CalendarCommandRunner.EVENT_FIELD_END.equals(field)) {
            Calendar date = DateUtil.parseDateTime(value);
            updateDateField(BlackBerryEvent.END, date.getTime().getTime());
        } else if (CalendarCommandRunner.EVENT_FIELD_LOCATION.equals(field)) {
            updateStringField(BlackBerryEvent.LOCATION, value);
        } else if (CalendarCommandRunner.EVENT_FIELD_START.equals(field)) {
            Calendar date = DateUtil.parseDateTime(value);
            updateDateField(BlackBerryEvent.START, date.getTime().getTime());
        } else if (CalendarCommandRunner.EVENT_FIELD_SUMMARY.equals(field)) {
            updateStringField(BlackBerryEvent.SUMMARY, value);
        } else if (CalendarCommandRunner.EVENT_FIELD_TIMEZONE.equals(field)) {
            currentAdditional.setTZID(value);
        } else if (CalendarCommandRunner.EVENT_FIELD_ALLDAY.equals(field)) {
            boolean b;
            if (value.equals("1")) {
                b = true;
            } else {
                b = false;
            }
            updateBooleanField(BlackBerryEvent.ALLDAY, b);
        } else if (CalendarCommandRunner.EVENT_FIELD_ATTENDEES.equals(field)) {
            throw new ClientTestException("Support not yet Implemented for " + field);
        } else if (CalendarCommandRunner.EVENT_FIELD_DESCRIPTION.equals(field)) {
            updateStringField(BlackBerryEvent.NOTE, value);
        } else if (CalendarCommandRunner.EVENT_FIELD_REMINDER.equals(field)) {
            updateIntField(BlackBerryEvent.ALARM,Integer.parseInt(value));
        } else {
            throw new ClientTestException("Unsupported field " + field);
        }
    }

    /**
     * Not yet implemented as recurrences are not implemented into the API right now
     * @param recField TBD
     * @param value TBD
     * @throws Throwable TBD
     */
    public void setEventRecurrenceField(String recField, String value) throws Throwable {
        
        if(repeatRuleEventMap == null){
            repeatRuleEventMap = new Hashtable();
            repeatRuleEventMap.put(IS_RECURRING,"0");
            repeatRuleEventMap.put(RECURRENCE_TYPE, "");
            repeatRuleEventMap.put(INTERVAL,"");
            repeatRuleEventMap.put(DAY_OF_WEEK_MASK,"");
            repeatRuleEventMap.put(INSTANCE, "");
            repeatRuleEventMap.put(DAY_OF_MONTH,"");
            repeatRuleEventMap.put(MONTH_OF_YEAR,"");
            repeatRuleEventMap.put(OCCURRENCES,"");
            repeatRuleEventMap.put(PATTERN_END_DATE,"");
        }
        if (CalendarCommandRunner.EVENT_REC_FIELD_FREQUENCY.equals(recField)) {
            int type;
            if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_NONE)){
                type = -1;
            }else if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_DAILY)){
                type = 0;
            }else if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_WEEKLY)){
                type = 1;
            }else if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_MONTHLY)){
                type = 2;
            }else if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_MONTHLY_NTH)){
                type = 3;
            }else if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_YEARLY)){
                type = 5;
            }else if(StringUtil.equalsIgnoreCase(value, CalendarCommandRunner.FREQUENCY_TYPE_YEARLY_NTH)){
                type = 6;
            }else{
                throw new ClientTestException("Unsupported Frequency " + value);
            }
            if(type>=0){
                repeatRuleEventMap.put(IS_RECURRING, "1");
                repeatRuleEventMap.put(RECURRENCE_TYPE, String.valueOf(type));
            }else{
                repeatRuleEventMap.put(IS_RECURRING,"0");
            }
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_INTERVAL.equals(recField)){
            repeatRuleEventMap.put(INTERVAL,value);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_DAY_OF_WEEK.equals(recField)) {
            // The day of week is a mask of days separated by semicolon. We must
            // translate it into a mask of integer (see EventHelper for details)
            int mask = 0;
            String days[] = StringUtil.split(value, ";");
            for(int i=0;i<days.length;++i) {
                if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_SUNDAY)) {
                    mask |= 1;
                } else if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_MONDAY)) {
                    mask |= 2;
                } else if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_TUESDAY)) {
                    mask |= 4;
                }  else if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_WEDNESDAY)) {
                    mask |= 8;
                } else if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_THURSDAY)) {
                    mask |= 16;
                }  else if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_FRIDAY)) {
                    mask |= 32;
                }  else if (StringUtil.equalsIgnoreCase(days[i], CalendarCommandRunner.DAY_OF_WEEK_SATURDAY)) {
                    mask |= 64;
                } 
            }
            repeatRuleEventMap.put(DAY_OF_WEEK_MASK, ""+mask);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_DAY_OF_MONTH.equals(recField)){
            repeatRuleEventMap.put(DAY_OF_MONTH,value);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_MONTH_OF_YEAR.equals(recField)){
            repeatRuleEventMap.put(MONTH_OF_YEAR,value);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_INSTANCE.equals(recField)){
            repeatRuleEventMap.put(INSTANCE,value);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_OCCURRENCES.equals(recField)){
            repeatRuleEventMap.put(OCCURRENCES,value);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_END_DATE.equals(recField)){
            repeatRuleEventMap.put(PATTERN_END_DATE,value);
        } else if (CalendarCommandRunner.EVENT_REC_FIELD_EXCEPTIONS.equals(recField)){
            // The exception is a list of comma separated dates
            int mask = 0;
            String excs[] = StringUtil.split(value, ";");
            Vector dates = new Vector();
            for(int i=0;i<excs.length;++i) {
                dates.addElement(excs[i]);
            }
            repeatRuleEventMap.put(EXCLUDE_DATE, dates);
        }else {
            throw new ClientTestException("Unsupported recurrence field " + recField);
        }
    }

    /**
     * Find the given summary event and load it into memory (currentEvent)
     * @param summary the Event.SUMMARY value to be found and loaded
     * @throws Throwable
     */
    public void loadEvent(String summary) throws Throwable {
        currentEvent = findEventBySummary(summary);
        if (currentEvent == null) {
            throw new ClientTestException("Cannot load event: " + summary);
        }
        summaryField = currentEvent.getString(BlackBerryEvent.SUMMARY, 0);
    }

    /**
     * Save the currentEvent on the client event list
     * @throws Throwable
     */
    public void saveEvent() throws Throwable {
        saveEvent(false);
    }

    /**
     * Save the currentEvent on the client event list
     *
     * @param useNativeApp true if the event shall be created through the
     *        BlackBerry native calendar app.
     * @throws Throwable
     */
    public void saveEvent(boolean useNativeApp) throws Throwable {
        if (summaryField != null) {
            updateStringField(BlackBerryEvent.SUMMARY, summaryField);
        }
        if (currentEvent == null) {
            throw new ClientTestException("Cannot save event");
        }
        if(repeatRuleEventMap!=null){
            RepeatRule repeatRule = new RepeatRule();
            VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);

            boolean allDay = false;
            if (currentEvent.countValues(BlackBerryEvent.ALLDAY) > 0) {
                allDay = currentEvent.getBoolean(BlackBerryEvent.ALLDAY, 0);
            }

            eventHelper.setRepeatRule(repeatRuleEventMap, repeatRule, false, allDay);
            currentEvent.setRepeat(repeatRule);
        }
        if(!useNativeApp) {
            currentEvent.commit();
        } else {
            if(scriptRunner != null) {
                String script = BlackBerryNativeCalendarRobot.getCreateEventScript(
                        eventMap, repeatRuleEventMap);
                Log.debug("Save event with native app: \r\n" + script);
                scriptRunner.runScript(script, null);
            }
        }
        currentEvent = null;
        eventMap = null;
        repeatRuleEventMap = null;
    }

    /**
     * Delete the event with the given summary on the client event list
     * @param summary the String representing the Event.SUMMARY value
     * @throws Throwable
     */
    public void deleteEvent(String summary) throws Throwable {
        BlackBerryEvent e = findEventBySummary(summary);
        if (e == null) {
            throw new ClientTestException("Cannot find event: " + summary);
        }
        eventList.removeEvent(e);
    }

    /**
     * Delete all the client events into the device's event list
     * @throws Throwable
     */
    public void deleteAllEvents() throws Throwable {
        Enumeration e = eventList.items();
        while(e.hasMoreElements()) {
            eventList.removeEvent((BlackBerryEvent) e.nextElement());
        }
    }

    /**
     * Check if a new contact is present on the device
     * @param summary the String representing the Event.SUMMARY value
     * @param client CheckSyncClient object
     * @param checkContent if true check deeper the content of the event
     * @throws Throwable
     */
    public void checkNewEvent(String summary, CheckSyncClient client, boolean checkContent) throws Throwable {
        CheckSyncSource source = client.getSyncSource(
                                          CheckSyncClient.SOURCE_NAME_CALENDAR);

        String key = findEventKeyOnServer(summary, client);
        Log.trace(TAG_LOG, "Found event key on server: " + key);

        SyncItem item = new SyncItem(key);
        item = source.getItemContent(item);
        String remoteVCal = new String(item.getContent());
 
        BlackBerryEvent localEvent = findEventBySummary(summary);
        checkEvent(localEvent, remoteVCal, checkContent);
    }

    /**
     * Check if a event was updated  on the device
     * @param summary the String representing the Event.SUMMARY value
     * @param client CheckSyncClient object
     * @param checkContent if true check deeper the content of the event
     * @throws Throwable
     */
    public void checkUpdatedEvent(String summary, CheckSyncClient client, boolean checkContent) throws Throwable {
        checkNewEvent(summary, client, checkContent);
    }

    /**
     * Check if a event was deleted on the device
     * @param summary the String representing the Event.SUMMARY value
     * @param client CheckSyncClient object
     * @throws Throwable
     */
    public void checkDeletedEvent(String summary, CheckSyncClient client) throws Throwable {
         // If the event is found, we assert
        if (findEventBySummary(summary) != null) {
            throw new ClientTestException("Event " + summary
                                          + " found. It should have been deleted from calendar.");
        }
    }


    /**
     * Check that the given event was added on server side
     * @param summary the summary related to the event
     * @param client CheckSyncClient object
     * @param checkContent if true check deeper the content of the new event
     * @throws Throwable
     */
    public void checkNewEventOnServer(String summary, CheckSyncClient client, boolean checkContent) throws Throwable {
        BlackBerryEvent localEvent = findEventBySummary(summary);
        Enumeration items = client.getSyncSource(
                CheckSyncClient.SOURCE_NAME_CALENDAR).getAddedItems();
        checkEventOnServer(summary, checkContent, localEvent, items);
    }

    /**
     * Check that the given event was updated on server side
     * @param summary the summary related to the event
     * @param client CheckSyncClient object
     * @throws Throwable
     */
    public void checkUpdatedEventOnServer(String summary, CheckSyncClient client, boolean checkContent) throws Throwable {
        BlackBerryEvent localEvent = findEventBySummary(summary);
        Enumeration items = client.getSyncSource(
                CheckSyncClient.SOURCE_NAME_CALENDAR).getUpdatedItems();

        checkEventOnServer(summary, checkContent, localEvent, items);
    }

    /**
     * Check that the given event was deleted on server side
     * @param summary the summary related to the event
     * @param client CheckSyncClient object
     * @throws Throwable
     */
    public void checkDeletedEventOnServer(String summary, CheckSyncClient client) throws Throwable {
        try {
            String key = findEventKeyOnServer(summary, client);
        } catch (ClientTestException cte) {
            if (cte.getMessage().indexOf(EVENT_KEY_NOT_FOUND_EXP_MSG) > -1) {
                return;
            } else {
                throw cte;
            }
        }
    }

    public void checkEventRecRule(String summary, String rrule) throws Throwable{
        BlackBerryEvent event = findEventBySummary(summary);
        VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);
        String eventVCal = eventHelper.itemToString(event, null);
        String resultStr = null;
        try {
            String[] field = new String[]{"RRULE"};
            resultStr   = orderVCal(eventVCal, field, supportedValues);
            assertTrue(resultStr.equals("\r\n"+rrule+"\r\n"), "RRULE mismatch");
        } catch(Exception ex) {
            Log.error(TAG_LOG, "Expected: " + rrule + " -- Found: " + resultStr);
            throw new ClientTestException("RRULE mismatch");
        }
    }

    public void checkEventExceptions(String summary, String exceptions) throws Throwable{
        BlackBerryEvent event = findEventBySummary(summary);
        VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);
        String eventVCal = eventHelper.itemToString(event, null);
        String resultStr = null;
        try {
            String[] field = new String[]{"EXDATE"};
            resultStr   = orderVCal(eventVCal, field, supportedValues);
            assertTrue(resultStr.equals("\r\n"+exceptions+"\r\n"), "EXDATE mismatch");
        } catch(Exception ex) {
            Log.error(TAG_LOG, "Expected: " + exceptions + " -- Found: " + resultStr);
            throw new ClientTestException("EXDATE mismatch");
        }
    }


    /**
     * Load an event stored on the server side
     * @param summary useful to retrieve the server side event
     * @param client CheckSyncClient object
     * @throws Throwable
     */
    public void loadEventOnServer(String summary, CheckSyncClient client) throws Throwable {
        CheckSyncSource css = client.getSyncSource(CheckSyncClient.SOURCE_NAME_CALENDAR);
        String key = findEventKeyOnServer(summary, client);
        currentEventId = Long.parseLong(key);

        BlackBerryEvent event = (BlackBerryEvent) (BlackBerryEvent) eventList.createEvent();
        SyncItem item = new SyncItem(key);
        item = css.getItemContent(item);
        VCalEventHelper calendarHelper = new VCalEventHelper(eventList, null, -1, -1);
        currentAdditional = (CalendarAdditionalAttributes) calendarHelper.fillItem(event, new String(item.getContent()), null);

        currentEvent = event;
        summary = currentEvent.getString(BlackBerryEvent.SUMMARY, 0);
    }

    /**
     * Finds the related key of an event onthe server accessing by summary
     * @param summary the summary of the event to be retrieved
     * @param client the CheckSyncClient object
     * @return the event's key String formatted
     * @throws Throwable
     */
    protected String findEventKeyOnServer(String summary, CheckSyncClient client) throws Throwable {

        Hashtable allItems = client.getSyncSource(CheckSyncClient.SOURCE_NAME_CALENDAR).getAllItems();

        Enumeration allElements = allItems.elements();
        VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);

        while(allElements.hasMoreElements()) {
            SyncItem item = (SyncItem)allElements.nextElement();
            Log.trace(TAG_LOG, "event key = " + item.getKey());
            BlackBerryEvent event = (BlackBerryEvent)eventList.createEvent();
            Log.trace(TAG_LOG, "event content = " + item.getContent());
            eventHelper.fillItem(event, new String(item.getContent()), null);
            String eSummary = event.getString(BlackBerryEvent.SUMMARY, BlackBerryEvent.ATTR_NONE);

            if(summary.equals(eSummary)) {
               return item.getKey();
            }
        }
        throw new ClientTestException("Cannot find event on server " + summary);
    }

    private BlackBerryEvent findEventBySummary(String summary) throws Throwable {
        // Scan all events looking for the one that matches the summary
        Enumeration items = eventList.items();
        BlackBerryEvent e = null;
        while(items.hasMoreElements()) {
            BlackBerryEvent event = (BlackBerryEvent)items.nextElement();
            String eSummary = event.getString(BlackBerryEvent.SUMMARY, 0);
            //Add here other needed checks on event's field
            if (summary.equals(eSummary)) {
                e = event;
                break;
            }
        }
        
        return e;
    }


    private void updateStringField(int field, String value) {
        if (currentEvent.countValues(field) > 0) {
            currentEvent.setString(field, 0, BlackBerryEvent.ATTR_NONE, value);
        } else {
            currentEvent.addString(field, BlackBerryEvent.ATTR_NONE, value);
        }
    }

    private void updateIntField(int field, int value){
        if (currentEvent.countValues(field) > 0) {
            currentEvent.setInt(field, 0, BlackBerryEvent.ATTR_NONE, value);
        } else {
            currentEvent.addInt(field, BlackBerryEvent.ATTR_NONE, value);
        }
        
    }

    private void updateDateField(int field, long value) {
        if (currentEvent.countValues(field) > 0) {
            currentEvent.setDate(field, 0, BlackBerryEvent.ATTR_NONE, value);
        } else {
            currentEvent.addDate(field, BlackBerryEvent.ATTR_NONE, value);
        }
    }

    private void updateBooleanField(int field, boolean value) {
        if (currentEvent.countValues(field) > 0) {
            currentEvent.setBoolean(field, 0, BlackBerryEvent.ATTR_NONE, value);
        } else {
            currentEvent.addBoolean(field, BlackBerryEvent.ATTR_NONE, value);
        }
    }

    protected String getCurrentEventVCal() throws Throwable
    {
        if(eventAsVcal != null){
            Log.trace(TAG_LOG, "getCurrentEventVCal: " + eventAsVcal);
            return eventAsVcal;
        }else{
            VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);
            String vCal = eventHelper.itemToString(currentEvent, null);
            Log.trace(TAG_LOG, "getCurrentEventVCal: " + vCal);
            return vCal;
        }
    }

    private void checkEvent(BlackBerryEvent localEvent, String remoteEvent, boolean checkContent) throws Throwable {
        if(checkContent) {
            assertEquals(localEvent, remoteEvent, "Events mismatch");
        }
    }

    private void checkEventOnServer(String summary, boolean checkContent,
                                    BlackBerryEvent localEvent, Enumeration items) throws Throwable {
        VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);
        while(items.hasMoreElements()) {
            SyncItem item = (SyncItem)items.nextElement();
            String remoteVCal = new String(item.getContent());
            BlackBerryEvent event = (BlackBerryEvent)eventList.createEvent();
            eventHelper.fillItem(event, new String(item.getContent()), null);
            String s = event.getString(BlackBerryEvent.SUMMARY, BlackBerryEvent.ATTR_NONE);
            if(summary.equals(s)) {
                checkEvent(localEvent, remoteVCal, checkContent);
                return;
            }
        }
        throw new ClientTestException("Cannot find event " + summary);
    }

    private void assertEquals(BlackBerryEvent e1, String vCal, String msg) throws ClientTestException {
        String expectedStr = null;
        String resultStr = null;

        try {
            VCalEventHelper eventHelper = new VCalEventHelper(eventList, null, -1, -1);
            String e1VCal = eventHelper.itemToString(e1, null);

            expectedStr = orderVCal(new String(e1VCal), supportedFields, supportedValues);
            resultStr   = orderVCal(vCal, supportedFields, supportedValues);

            assertTrue(resultStr.equals(expectedStr), msg);
        } catch(Exception ex) {
            Log.error(TAG_LOG, "Expected: " + expectedStr + " -- Found: " + resultStr);
            throw new ClientTestException(msg);
        }
    }
}
