/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import com.funambol.client.test.CommandRunner;

import com.funambol.util.StringUtil;
import com.funambol.util.Log;

public class BlackBerryCommandRunner extends CommandRunner {

    private static final String TAG_LOG = "BlackBerryCommandRunner";

    public static final String TRACK_WHEEL_COMMAND            = "TrackWheel";
    public static final String SET_FIELD_TEXT_COMMAND         = "SetFieldText";
    public static final String SET_CHOICE_FIELD_COMMAND       = "SetChoiceField";
    public static final String SET_CHECK_BOX_COMMAND          = "SetCheckBox";

    public static final String CLICK_MENU_ITEM_COMMAND  = "ClickMenuItem";
    public static final String WAIT_FOR_SCREEN_COMMAND  = "WaitForScreen";

    public static final String CHECK_VISIBLE_SOURCE_NAME_COMMAND  = "CheckVisibleSourceName";
    public static final String CHECK_VISIBLE_SOURCE_COUNT_COMMAND = "CheckVisibleSourceCount";

    public static final String RESET_CONFIG_COMMAND         = "ResetConfig";
    public static final String SAVE_ACCOUNT_COMMAND         = "SaveAccount";
    public static final String CHECK_ALERT_MESSAGE_COMMAND  = "CheckAlertMessage";
    public static final String RESET_DIR_COMMAND            = "ResetDir";
    public static final String COPY_FILE_COMMAND            = "CopyFile";
    public static final String RENAME_FILE_COMMAND          = "RenameFile";
    public static final String DELETE_FILE_COMMAND          = "DeleteFile";
    public static final String SET_TIMEZONE_COMMAND         = "SetTimeZone";
    public static final String SET_DATE_TIME_COMMAND        = "SetDateTime";
    public static final String WRITE_STRING_WITH_KP_COMMAND = "WriteStringWithKeypad";

    public static final String TRACK_WHEEL_LEFT         = "Left";
    public static final String TRACK_WHEEL_RIGHT        = "Right";
    public static final String TRACK_WHEEL_DOWN         = "Down";
    public static final String TRACK_WHEEL_UP           = "Up";
    public static final String TRACK_WHEEL_CLICK        = "Click";

    private BlackBerryRobot blackBerryRobot;

    public BlackBerryCommandRunner(BlackBerryRobot robot) {
        super(robot);
        blackBerryRobot = robot;
    }

    public boolean runCommand(String command, String args) throws Throwable {

        boolean handled = true;

        if (CLICK_MENU_ITEM_COMMAND.equals(command)) {
            clickMenuItem(command, args);
        } else if (WAIT_FOR_SCREEN_COMMAND.equals(command)) {
            waitForScreen(command, args);
        } else if (RESET_CONFIG_COMMAND.equals(command)) {
            resetConfig(command, args);
        } else if (CHECK_ALERT_MESSAGE_COMMAND.equals(command)) {
            checkAlertMessage(command, args);
        } else if (SAVE_ACCOUNT_COMMAND.equals(command)) {
            saveAccount(command, args);
        } else if (RESET_DIR_COMMAND.equals(command)) {
            resetDir(command, args);
        } else if (COPY_FILE_COMMAND.equals(command)) {
            copyFile(command, args);
        } else if (DELETE_FILE_COMMAND.equals(command)) {
            deleteFile(command, args);
        } else if (RENAME_FILE_COMMAND.equals(command)) {
            renameFile(command, args);
        } else if (TRACK_WHEEL_COMMAND.equals(command)) {
            trackWheel(command, args);
        } else if (SET_CHOICE_FIELD_COMMAND.equals(command)) {
            setChoiceField(command, args);
        } else if (SET_CHECK_BOX_COMMAND.equals(command)) {
            setCheckBox(command, args);
        } else if (CHECK_VISIBLE_SOURCE_NAME_COMMAND.equals(command)) {
            checkVisibleSourceName(command, args);
        } else if (CHECK_VISIBLE_SOURCE_COUNT_COMMAND.equals(command)) {
            checkVisibleSourceCount(command, args);
        } else if (SET_TIMEZONE_COMMAND.equals(command)) {
            setTimeZone(command, args);
        } else if (SET_DATE_TIME_COMMAND.equals(command)) {
            setDateTime(command, args);
        } else if (WRITE_STRING_WITH_KP_COMMAND.equals(command)) {
            writeStringWithKeypad(command, args);
        } else {
            handled = false;
        }
        return handled;
    }

    private void renameFile(String command, String args) throws Throwable {

        // Copy a file to a directory
        String sourceFile = getParameter(args, 0);
        String targetFile = getParameter(args, 1);

        checkArgument(sourceFile, "Cannot find source file in " + command);
        checkArgument(targetFile, "Cannot find target file in " + command);

        if (sourceFile.startsWith("/")) {
            // This is a local file
            sourceFile = "file://" + sourceFile;
        }

        if (targetFile.startsWith("/")) {
            targetFile = "file://" + targetFile;
        }

        if (!sourceFile.startsWith("file://")) {
            throw new ClientTestException("Unsupported file for rename: " + sourceFile);
        }

        if (!targetFile.startsWith("file://")) {
            throw new ClientTestException("Unsupported file for rename: " + targetFile);
        }
        blackBerryRobot.renameFile(sourceFile, targetFile);
    }

    private void deleteFile(String command, String args) throws Throwable {

        // Copy a file to a directory
        String file = getParameter(args, 0);

        checkArgument(file, "Cannot find file name in " + command);

        if (file.startsWith("/")) {
            // This is a local file
            file = "file://" + file;
        }

        if (!file.startsWith("file://")) {
            throw new ClientTestException("Unsupported file for delete: " + file);
        }

        blackBerryRobot.deleteFile(file);
    }

    private void trackWheel(String command, String args) throws Throwable {
        String operation = getParameter(args, 0);
        String repeat = getParameter(args, 1);
        checkArgument(operation, "Missing operation in " + command);
        Log.info(TAG_LOG, "Simulating track wheel operation " + operation);

        int r;
        if (repeat == null) {
            r = 1;
        } else {
            try {
                r = Integer.parseInt(repeat);
            } catch (Exception e) {
                Log.error(TAG_LOG, "Invalid repeat number: " + repeat);
                throw new ClientTestException("Invalid repeat number");
            }
        }
        blackBerryRobot.trackWheel(operation, r);
    }

    private void resetDir(String command, String args) throws Throwable {
        String dir = getParameter(args, 0);

        checkArgument(dir, "Missing dir in command " + command);
        blackBerryRobot.resetDir(dir);
    }

    private void copyFile(String command, String args) throws Throwable {

        // Copy a file to a directory
        String sourceFile = getParameter(args, 0);
        String targetFile = getParameter(args, 1);

        checkArgument(sourceFile, "Cannot find source file in " + command);
        checkArgument(targetFile, "Cannot find target file in " + command);

        if (sourceFile.startsWith("/")) {
            // This is a local file
            sourceFile = "file://" + sourceFile;
        }

        if (targetFile.startsWith("/")) {
            targetFile = "file://" + targetFile;
        }

        if (!sourceFile.startsWith("file://")) {
            throw new ClientTestException("Unsupported file for copy: " + sourceFile);
        }

        if (!targetFile.startsWith("file://")) {
            throw new ClientTestException("Unsupported file for copy: " + targetFile);
        }
        blackBerryRobot.copyFile(sourceFile, targetFile);
    }

    private void checkAlertMessage(String command, String args) throws Throwable {
        String message = getParameter(args, 0);

        checkArgument(message, "Missing message in " + command);
        blackBerryRobot.checkAlertMessage(message);
    }

    private void resetConfig(String command, String args) throws Throwable {
        blackBerryRobot.resetConfig();
    }

    private void setChoiceField(String command, String args) throws Throwable {
        String idx  = getParameter(args, 0);

        checkArgument(idx, "Missing index in " + command);
        blackBerryRobot.setChoiceField(Integer.parseInt(idx));
    }

    private void setCheckBox(String command, String args) throws Throwable {
        String value  = getParameter(args, 0);

        checkArgument(value, "Missing value in " + command);
        boolean v;
        if (StringUtil.equalsIgnoreCase(value, "true")) {
            v = true;
        } else {
            v = false;
        }
        blackBerryRobot.setCheckBox(v);
    }

    private void clickMenuItem(String command, String args) throws Throwable {
        String label = getParameter(args, 0);
        checkArgument(label, "Missing label in clickMenuItem");
        blackBerryRobot.simulateMenuClick(label);
    }

    private void waitForScreen(String command, String args) throws Throwable {
        String screenName = getParameter(args, 0);
        String delay      = getParameter(args, 1);

        checkArgument(screenName, "Missing screen name in " + command);
        checkArgument(delay, "Missing delay in " + command);

        int d = Integer.parseInt(delay);
        blackBerryRobot.waitForScreen(screenName, d * 1000);
    }

    private void checkVisibleSourceName(String command, String args) throws Throwable {
        String idx   = getParameter(args, 0);
        String name  = getParameter(args, 1);

        checkArgument(idx, "Missing source index in " + command);
        checkArgument(name, "Missing source name in " + command);

        int idxValue = Integer.parseInt(idx);
        blackBerryRobot.checkVisibleSourceName(idxValue, name);
    }

    private void checkVisibleSourceCount(String command, String args) throws Throwable {
        String count = getParameter(args, 0);

        checkArgument(count, "Missing count in " + command);
        int countValue = Integer.parseInt(count);
        blackBerryRobot.checkVisibleSourceCount(countValue);
    }

    private void setTimeZone(String command, String args) throws Throwable {
        String tz = getParameter(args, 0);

        checkArgument(tz, "Missing timezone in " + command);
        blackBerryRobot.setTimeZone(tz);
    }

    private void setDateTime(String command, String args) throws Throwable {
        String time = getParameter(args, 0);

        checkArgument(time, "Missing date time in " + command);
        blackBerryRobot.setDateTime(time);
    }

    private void writeStringWithKeypad(String command, String args) throws Throwable {
        String string = getParameter(args, 0);

        checkArgument(string, "Missing string in " + command);
        blackBerryRobot.writeStringWithKeypad(string);
    }

    private void saveAccount(String command, String args) throws Throwable {
        String url = getParameter(args, 0);
        String user  = getParameter(args, 1);
        String pwd  = getParameter(args, 2);

        if (url == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find url " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (user == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find user " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        if (pwd == null) {
            Log.error(TAG_LOG, "Syntax error in script, cannot find pwd " + command);
            throw new IllegalArgumentException("Syntax error in script");
        }

        blackBerryRobot.saveAccount(url, user, pwd);
    }
}
