/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import com.funambol.client.test.CalendarCommandRunner;
import com.funambol.util.DateUtil;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.StaticDataHelper;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Vector;

/**
 * It allows to interact with the BlackBerry Calendar application.
 */
public class BlackBerryNativeCalendarRobot {
    
    private static final String TAG_LOG = "BlackBerryNativeCalendarsRobot";

    private static final String CRLF = "\r\n";

    /**
     * Generate a test script which creates a new Event through the native
     * Calendar application.
     * 
     * @param eventMap The map containing the event fields (except recurrences)
     * @param eventRecMap The map containing the recurrence fields
     * @return The script string
     */
    public static String getCreateEventScript(Hashtable eventMap, Hashtable eventRecMap) {

        Log.trace(TAG_LOG, "getCreateEventScript");

        StringBuffer script = new StringBuffer();

        // Go to calendar application
        goToCalendar(script);

        // Select "New" menu option
        fireDefaultMenuOption(script);

        // Set device date time to be earlier than the current time
        // This prevents a popup message related to the event reminder to show
        script.append("SetDateTime(2004-01-01T12:00:00)").append(CRLF);

        // Set summary
        setStringField(script, CalendarCommandRunner.EVENT_FIELD_SUMMARY, eventMap);

        // Set location
        setStringField(script, CalendarCommandRunner.EVENT_FIELD_LOCATION, eventMap);

        // Set all day
        boolean allday = setBooleanField(script, CalendarCommandRunner.EVENT_FIELD_ALLDAY, eventMap);

        // Set start date
        setDateField(script, CalendarCommandRunner.EVENT_FIELD_START, allday, eventMap);

        // Set end date
        setDateField(script, CalendarCommandRunner.EVENT_FIELD_END, allday, eventMap);

        // Skip duration/timezone/freebusy fields
        if(allday) {
            script.append("KeyPress(KeyDown, 3)").append(CRLF);
        } else {
            script.append("KeyPress(KeyDown, 4)").append(CRLF);
        }

        // Set reminder
        setReminderField(script, eventMap);

        // Set recurrence
        setRecurrenceFields(script, eventRecMap, eventMap);

        // Skip class field
        script.append("KeyPress(KeyDown)").append(CRLF);

        // Set description
        setStringField(script, CalendarCommandRunner.EVENT_FIELD_DESCRIPTION, eventMap);

        // Save the new event
        fireDefaultMenuOption(script);

        // Set recurrence exceptions
        setRecurrenceExceptions(script, eventRecMap);

        // Return to the main app
        script.append("KeyPress(KeyBack)").append(CRLF);

        // Back to the first source
        script.append("KeyPress(KeyUp)").append(CRLF);

        return script.toString();
    }

    private static void goToCalendar(StringBuffer script) {
        Log.trace(TAG_LOG, "goToCalendar");
        script.append("KeyPress(KeyDown)").append(CRLF);
        script.append("ClickMenuItem(\"Go to Calendar\")").append(CRLF);
    }

    private static void fireDefaultMenuOption(StringBuffer script) {
        Log.trace(TAG_LOG, "fireDefaultMenuOption");
        script.append("KeyPress(KeyMenu)").append(CRLF);
        script.append("KeyPress(KeyFire)").append(CRLF);
    }

    private static void setStringField(StringBuffer script, String field, Hashtable eventMap) {
        Log.trace(TAG_LOG, "setStringField field: " + field);
        String value = (String)eventMap.get(field);
        Log.trace(TAG_LOG, "setStringField value: " + value);
        if(!StringUtil.isNullOrEmpty(value)) {
            script.append("WriteStringWithKeypad(" + value + ")").append(CRLF);
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static boolean setBooleanField(StringBuffer script, String field, Hashtable eventMap) {
        Log.trace(TAG_LOG, "setBooleanField field: " + field);
        boolean result = false;
        String value = (String)eventMap.get(field);
        Log.trace(TAG_LOG, "setBooleanField value: " + value);
        if("1".equals(value)) {
            script.append("KeyPress(KeyFire)").append(CRLF);
            result = true;
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
        return result;
    }

    private static void setDateField(StringBuffer script, String field, boolean allday, Hashtable eventMap) {
        Log.trace(TAG_LOG, "setDateField field: " + field);
        String value = (String)eventMap.get(field);
        Log.trace(TAG_LOG, "setDateField value: " + value);
        if(!StringUtil.isNullOrEmpty(value)) {
            writeDate(script, value, !allday);
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setReminderField(StringBuffer script, Hashtable eventMap) {
        Log.trace(TAG_LOG, "setReminderField");
        String value = (String)eventMap.get(CalendarCommandRunner.EVENT_FIELD_REMINDER);
        Log.trace(TAG_LOG, "setReminderField value: " + value);
        int interval = -1;
        if(!StringUtil.isNullOrEmpty(value)) {
            interval = Integer.parseInt(value);
        }
        script.append("KeyPress(KeyFire)").append(CRLF);
        script.append("KeyPress(KeyDown, 10)").append(CRLF);
        int[] supportedValues = new int[] {
                                           -1,    // None
                                            0,    // 0 Minutes
                                            5,    // 5 Minutes
                                            10,   // 10 Minutes
                                            15,   // 15 Minutes
                                            30,   // 30 Minutes
                                            45,   // 45 Minutes
                                            60,   // 1 Hour
                                            120,  // 2 Hours
                                            180,  // 3 Hours
                                            240,  // 4 Hours
                                            300,  // 5 Hours
                                            360,  // 6 Hours
                                            420,  // 7 Hours
                                            480,  // 8 Hours
                                            540,  // 9 Hours
                                            600,  // 10 Hours
                                            660,  // 11 Hours
                                            720,  // 12 Hours
                                            1440, // 1 day
                                            2880, // 2 days
                                            4320, // 3 days
                                            5760, // 4 days
                                            7200, // 5 days
                                            8640, // 6 days
                                            10080 // 1 week
                                          };
        boolean found = false;
        for(int i=0; i<supportedValues.length; i++) {
            if(interval == supportedValues[i]) {
                script.append("KeyPress(KeyFire)").append(CRLF);
                found = true;
                break;
            }
            script.append("KeyPress(KeyUp)").append(CRLF);
        }
        if(!found) {
            // select "None" if not found
            script.append("KeyPress(KeyDown, " + supportedValues.length + ")").append(CRLF);
            script.append("KeyPress(KeyFire)").append(CRLF);
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setRecurrenceFields(StringBuffer script, 
            Hashtable eventRecMap, Hashtable eventMap) {
        Log.trace(TAG_LOG, "setRecurrenceFields");
        String recType = (String)eventRecMap.get(
                BlackBerryCalendarsRobot.RECURRENCE_TYPE);
        Log.trace(TAG_LOG, "setRecurrenceFields recType: " + recType);
        if(!StringUtil.isNullOrEmpty(recType)) {
            int type = Integer.parseInt(recType);
            switch(type) {
                // Daily
                case 0: {
                    selectRecurrenceType (script, 1);
                    setRecurrenceInterval(script, eventRecMap);
                    setRecurrenceEndDate (script, eventRecMap);
                    break;
                }
                // Weekly
                case 1: {
                    selectRecurrenceType  (script, 2);
                    setRecurrenceInterval (script, eventRecMap);
                    setRecurrenceDayOfWeek(script, eventRecMap, eventMap);
                    setRecurrenceEndDate  (script, eventRecMap);
                    break;
                }
                // Montly/MontlyNth
                case 2:
                case 3: {
                    selectRecurrenceType (script, 3);
                    setRecurrenceInterval(script, eventRecMap);
                    setRelativeRecurrence(script, type);
                    setRecurrenceEndDate (script, eventRecMap);
                    break;
                }
                // Yearly/YearlyNth
                case 5:
                case 6: {
                    selectRecurrenceType (script, 4);
                    setRelativeRecurrence(script, type);
                    setRecurrenceEndDate (script, eventRecMap);
                    break;
                }
            }
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setRecurrenceExceptions(StringBuffer script,
            Hashtable eventRecMap) {
        Log.trace(TAG_LOG, "setRecurrenceExceptions");
        Vector exceptions = (Vector)eventRecMap.get(
                BlackBerryCalendarsRobot.EXCLUDE_DATE);
        if(exceptions == null) {
            return;
        }
        for(int i=0; i<exceptions.size(); i++) {
            // Goto the exception date
            script.append("KeyPress(KeyMenu)").append(CRLF);

            // We must find the "New" menu item, but this depends on the
            // platform. For this reason we generate conditional code
            int numUps = 5;
            if (StaticDataHelper.checkOS(StaticDataHelper.getOS(), 4, 6, 0)) {
                // We need one more because of the "New alarm" menu entry
                numUps = 6;
            }

            script.append("KeyPress(KeyUp, ").append(numUps).append(")").append(CRLF);
            script.append("KeyPress(KeyFire)").append(CRLF);
            String exception = (String)exceptions.elementAt(i);
            writeDate(script, exception, false);
            script.append("KeyPress(KeyFire)").append(CRLF);
            // Remove the occurence
            script.append("KeyPress(KeyMenu)").append(CRLF);
            // Go to the delete command
            script.append("KeyPress(KeyDown)").append(CRLF);
            // Delete
            script.append("KeyPress(KeyFire)").append(CRLF);
            script.append("KeyPress(KeyUp)").append(CRLF);
            script.append("KeyPress(KeyFire)").append(CRLF);
        }
    }

    private static void selectRecurrenceType(StringBuffer script, int count) {
        Log.trace(TAG_LOG, "selectRecurrenceType");
        script.append("KeyPress(KeyFire)").append(CRLF);
        script.append("KeyPress(KeyDown, " + count + ")").append(CRLF);
        script.append("KeyPress(KeyFire)").append(CRLF);
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setRecurrenceInterval(StringBuffer script, Hashtable eventRecMap) {
        Log.trace(TAG_LOG, "setRecurrenceInterval");
        String interval = (String)eventRecMap.get(
                BlackBerryCalendarsRobot.INTERVAL);
        Log.trace(TAG_LOG, "setRecurrenceInterval interval: " + interval);
        if(!StringUtil.isNullOrEmpty(interval)) {
            script.append("WriteStringWithKeypad(" + interval + ")").append(CRLF);
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setRecurrenceDayOfWeek(StringBuffer script, 
            Hashtable eventRecMap, Hashtable eventMap) {
        Log.trace(TAG_LOG, "setRecurrenceDayOfWeek");
        String dow = (String)eventRecMap.get(
                BlackBerryCalendarsRobot.DAY_OF_WEEK_MASK);
        Log.trace(TAG_LOG, "setRecurrenceInterval dow: " + dow);
        if(!StringUtil.isNullOrEmpty(dow)) {
            // Read start day of week
            String start = (String)eventMap.get(CalendarCommandRunner.EVENT_FIELD_START);
            int startDayOfWeek = -1;
            if(!StringUtil.isNullOrEmpty(start)) {
                Calendar date = DateUtil.parseDateTime(start);
                int day = date.get(Calendar.DAY_OF_WEEK);
                if(day == Calendar.SUNDAY) {
                    startDayOfWeek = 1;
                } else if(day == Calendar.MONDAY) {
                    startDayOfWeek = 2;
                } else if(day == Calendar.TUESDAY) {
                    startDayOfWeek = 4;
                } else if(day == Calendar.WEDNESDAY) {
                    startDayOfWeek = 8;
                } else if(day == Calendar.THURSDAY) {
                    startDayOfWeek = 16;
                } else if(day == Calendar.FRIDAY) {
                    startDayOfWeek = 32;
                } else if(day == Calendar.SATURDAY) {
                    startDayOfWeek = 64;
                }
            }
            int mask = Integer.parseInt(dow);
            int day = 1;
            for(int i=0; i<7; i++) {
                if(day != startDayOfWeek && (mask & day) == day) {
                    script.append("KeyPress(KeyFire)").append(CRLF);
                }
                if(i<6) {
                    script.append("KeyPress(KeyRight)").append(CRLF);
                }
                day *= 2;
            }
        } else {
            script.append("KeyPress(KeyDown, 6)").append(CRLF);
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setRelativeRecurrence(StringBuffer script, int type) {
        Log.trace(TAG_LOG, "setRelativeRecurrence");
        if(type == 3 || type == 6) {
            script.append("KeyPress(KeyFire)").append(CRLF);
        }
        script.append("KeyPress(KeyDown)").append(CRLF);
    }

    private static void setRecurrenceEndDate(StringBuffer script, Hashtable eventRecMap) {
        Log.trace(TAG_LOG, "setRecurrenceEndDate");
        String endDate = (String)eventRecMap.get(
                BlackBerryCalendarsRobot.PATTERN_END_DATE);
        Log.trace(TAG_LOG, "setRecurrenceEndDate endDate: " + endDate);
        if(!StringUtil.isNullOrEmpty(endDate)) {
            script.append("KeyPress(KeyFire)").append(CRLF);
            script.append("KeyPress(KeyDown)").append(CRLF);
            script.append("KeyPress(KeyFire)").append(CRLF);
            script.append("KeyPress(KeyDown)").append(CRLF);
            writeDate(script, endDate, false);
        }
    }

    private static void writeDate(StringBuffer script, String date, boolean writeTime) {
        Calendar cal = DateUtil.parseDateTime(date);
        // Day
        script.append("WriteStringWithKeypad(" + cal.get(
                Calendar.DAY_OF_MONTH) + ")").append(CRLF);
        script.append("KeyPress(KeyRight)").append(CRLF);
        // Month
        script.append("WriteStringWithKeypad(" + (cal.get(
                Calendar.MONTH)+1) + ")").append(CRLF);
        script.append("KeyPress(KeyRight)").append(CRLF);
        // Year
        script.append("WriteStringWithKeypad(" + cal.get(
                Calendar.YEAR) + ")").append(CRLF);
        if(writeTime) {
            script.append("KeyPress(KeyRight)").append(CRLF);
            // Hour
            script.append("WriteStringWithKeypad(" + cal.get(
                    Calendar.HOUR_OF_DAY) + ")").append(CRLF);
            script.append("KeyPress(KeyRight)").append(CRLF);
            // Minute
            script.append("WriteStringWithKeypad(" + cal.get(
                    Calendar.MINUTE) + ")").append(CRLF);
        }
    }
}
