/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import java.util.TimeZone;
import java.util.Enumeration;
import java.util.Vector;
import java.io.InputStream;
import java.io.OutputStream;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.Keypad;
import net.rim.device.api.ui.component.TextField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.ObjectChoiceField;

import net.rim.device.api.system.EventInjector;
import net.rim.device.api.system.PersistentStore;

import com.funambol.blackberry.Constants;
import com.funambol.blackberry.Utils;
import com.funambol.blackberry.ui.rim.RimHomeScreen;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.Configuration;
import com.funambol.client.test.BasicCommandRunner;

import com.funambol.client.test.BasicRobot;
import com.funambol.client.test.SyncMonitor;
import com.funambol.client.test.SyncMonitorListener;
import com.funambol.client.test.CheckSyncClient;

import com.funambol.util.StringUtil;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncListener;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.platform.FileAdapter;
import com.funambol.util.DateUtil;
import com.funambol.util.Log;
import java.util.Calendar;

/**
 * 
 */
public class BlackBerryRobot extends BasicRobot {
   
    private static final String TAG_LOG = "BlackBerryRobot";

    private static final int WAIT_DELAY = 1000;
    
    public BlackBerryRobot() {
    }
    
    public void waitForAuthToComplete(int minStart, int max, SyncMonitor syncMonitor) throws Throwable {
        waitForSyncToComplete(minStart, max, syncMonitor);
    }

    // TEMP TODO FIXME
    public void waitForSyncToComplete(int minStart, int max, SyncMonitor syncMonitor) throws Throwable {
        Log.setLogLevel(Log.TRACE);
        super.waitForSyncToComplete(minStart, max, syncMonitor);
    }

    public void setTimeZone(String tz) throws Throwable {
        Log.info(TAG_LOG, "Setting timezone to " + tz);
        TimeZone timezone = TimeZone.getTimeZone(tz);
        if (timezone == null) {
            throw new IllegalArgumentException("Cannot set the timezone to: " + tz);
        }
        Utils utils = new Utils();
        utils.setDeviceTimeZone(timezone);
    }

    public void setDateTime(String dateTime) throws Throwable {
        Log.info(TAG_LOG, "Setting date time to " + dateTime);
        Utils utils = new Utils();
        Calendar date = DateUtil.parseDateTime(dateTime);
        utils.setDeviceDateTime(date.getTime().getTime());
    }

    protected SyncSource getSyncSource(String sourceName) throws Exception {
        return null;
    }

    protected void saveSourceConfig(SyncSource source) {
    }

    protected void reapplySyncConfig(CheckSyncClient client) throws Throwable {
    }

    public void simulateMenuClick(String label) throws Throwable {
        //Get the current screen
        Log.debug(TAG_LOG, "simulating menu click: " + label);
        UiApplication app = UiApplication.getUiApplication();
        Screen screen = app.getActiveScreen();

        Menu menu = screen.getMenu(0);
        assertTrue(menu != null, "Cannot click on menu item for screens with no menu");

        int size = menu.getSize();
        int position = -1;
        int seps = 0;
        for(int i=0;i<size;++i) {
            MenuItem mItem = menu.getItem(i);
            if (!mItem.isSeparator()) {
                String text = mItem.toString(); 
                if (text.equals(label)) {
                    position = i;
                    break;
                }
            } else {
                seps++;
            }
        }
        // We shall skip separators because while scrolling down they are
        // automatically skipped
        position -= seps;
        assertTrue(position >= 0, "Cannot find menu item: " + label);

        // Now simulate all the actions to click on this menu item,
        // we could take a shortcut and run the MenuItem thread directly,
        // but it is better to simulate precisely what happens on the device
        pressAndReleaseKeyCode((char)Keypad.KEY_MENU);

        // Now scroll the wheel
        if (position > 0) {
            trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_DOWN, position);
        }

        // And finally click on the item
        trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_CLICK, 1);
    }

    public void resetConfig() throws Throwable {
        try {
            PersistentStore.destroyPersistentObject(Constants.CONFIG_KEY);
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot destroy configuration, continue anyway");
        }
    }

    public void resetSourceAnchor(String sourceName) throws Throwable {
        Log.debug(TAG_LOG, "resetting source anchor");
        UiApplication app = UiApplication.getUiApplication();

        Enumeration appSources = AppSyncSourceManager.getInstance().getWorkingSources();
        AppSyncSource appSource = null;
        while(appSources.hasMoreElements()) {
            AppSyncSource aSource = (AppSyncSource)appSources.nextElement();
            if (aSource.getName().equals(sourceName)) {
                appSource = aSource;
                break;
            }
        }

        assertTrue(appSource != null, "Cannot find source to reset");
        SyncSource source = appSource.getSyncSource();
        SourceConfig config = source.getConfig();
        config.setLastAnchor(0);
        config.setNextAnchor(0);
        Configuration.save(null);
    }

    public void checkVisibleSourceName(int idx, String name) throws Throwable {
        Log.debug(TAG_LOG, "checking visible source name");
        UiApplication app = UiApplication.getUiApplication();
        Screen screen = app.getActiveScreen();

        HomeScreenController controller = null;
        if (screen instanceof RimHomeScreen) {
            RimHomeScreen homeScreen = (RimHomeScreen)screen;
            controller = (HomeScreenController)homeScreen.getSynchronizationController();
        }

        assertTrue(controller != null, "Cannot check visible source properties on a screen which is not the home screen");
        Vector sources = controller.getVisibleItems();
        assertTrue(idx < sources.size(), "Invalid idx " + idx + " and size is: " + sources.size());
        AppSyncSource appSource = (AppSyncSource)sources.elementAt(idx);
        assertTrue(appSource.getName().equals(name), "Visible source name mismatch: is: " + appSource.getName()
                                                     + " expected: " + name);
    }

    public void checkVisibleSourceCount(int count) throws Throwable {
        Log.debug(TAG_LOG, "checking visible source count");
        UiApplication app = UiApplication.getUiApplication();
        Screen screen = app.getActiveScreen();

        HomeScreenController controller = null;
        if (screen instanceof RimHomeScreen) {
            RimHomeScreen homeScreen = (RimHomeScreen)screen;
            controller = (HomeScreenController)homeScreen.getSynchronizationController();
        }

        assertTrue(controller != null, "Cannot check visible source properties on a screen which is not the home screen");
        Vector sources = controller.getVisibleItems();
        assertTrue(sources.size() == count, "Number of visible sources mismatch: is " + sources.size()
                                            + " expected: " + count);
    }

    public void writeString(String value) throws Throwable {

        Log.debug(TAG_LOG, "simulating text field entering: " + value);
        UiApplication app = UiApplication.getUiApplication();
        synchronized(UiApplication.getEventLock()) {
            Screen screen = app.getActiveScreen();

            Field field = screen.getFieldWithFocus();
            assertTrue(field != null, "No field with focus, cannot set text");

            while (field instanceof Manager) {
                Manager manager = (Manager)field;
                field = manager.getFieldWithFocus();
            }

            if (field instanceof TextField) {
                TextField textField = (TextField) field;
                textField.setText(value);
            } else {
                Log.error(TAG_LOG, "Cannot set text on non TextField " + field);
                throw new ClientTestException("Cannot set text on non TextField " + field);
            }
        }
    }

    public void writeStringWithKeypad(String value) throws Throwable {
        Log.debug(TAG_LOG, "simulating text field entering with keypad: " + value);
        for(int i=0; i<value.length(); i++) {
            pressAndReleaseKey(value.charAt(i));
        }
    }

    public void setCheckBox(boolean enabled) throws Throwable {

        Log.debug(TAG_LOG, "simulating check box setting " + enabled);
        UiApplication app = UiApplication.getUiApplication();
        synchronized(UiApplication.getEventLock()) {
            Screen screen = app.getActiveScreen();

            Field field = screen.getFieldWithFocus();
            assertTrue(field != null, "No field with focus, cannot set text");

            while (field instanceof Manager) {
                Manager manager = (Manager)field;
                field = manager.getFieldWithFocus();
            }

            if (field instanceof CheckboxField) {
                CheckboxField checkBox = (CheckboxField) field;
                checkBox.setChecked(enabled);
            } else {
                Log.error(TAG_LOG, "Cannot set checkbox value " + field);
                throw new ClientTestException("Cannot set checkbox on non Checkbox " + field);
            }
        }
    }

    public void setChoiceField(int idx) throws Throwable {

        Log.debug(TAG_LOG, "simulating choice field setting " + idx);
        UiApplication app = UiApplication.getUiApplication();
        synchronized(UiApplication.getEventLock()) {
            Screen screen = app.getActiveScreen();

            Field field = screen.getFieldWithFocus();
            assertTrue(field != null, "No field with focus, cannot set text");

            while (field instanceof Manager) {
                Manager manager = (Manager)field;
                field = manager.getFieldWithFocus();
            }

            if (field instanceof ObjectChoiceField) {
                ObjectChoiceField choiceField = (ObjectChoiceField) field;
                choiceField.setSelectedIndex(idx);
            } else {
                Log.error(TAG_LOG, "Cannot set choice field value " + field);
                throw new ClientTestException("Cannot set choice field on non ChoiceField " + field);
            }
        }
    }

    public void saveAccount(String url, String user, String pwd) throws Throwable {
        Log.error(TAG_LOG, "save account not implemented");
        throw new ClientTestException("save account not implemented");
    }

    public void keyPress(String keyName, int count) throws Throwable {
        if(BasicCommandRunner.DOWN_KEY_NAME.equals(keyName)) {
            trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_DOWN, count);
        } else if(BasicCommandRunner.UP_KEY_NAME.equals(keyName)) {
            trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_UP, count);
        } else if(BasicCommandRunner.LEFT_KEY_NAME.equals(keyName)) {
            trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_LEFT, count);
        } else if(BasicCommandRunner.RIGHT_KEY_NAME.equals(keyName)) {
            trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_RIGHT, count);
        } else if(BasicCommandRunner.FIRE_KEY_NAME.equals(keyName)) {
            trackWheel(BlackBerryCommandRunner.TRACK_WHEEL_CLICK, count);
        } else if(BasicCommandRunner.MENU_KEY_NAME.equals(keyName)) {
            pressAndReleaseKeyCode((char)Keypad.KEY_MENU);
        } else if(BasicCommandRunner.BACK_KEY_NAME.equals(keyName)) {
            pressAndReleaseKeyCode((char)Keypad.KEY_ESCAPE);
        } else if(BasicCommandRunner.DEL_KEY_NAME.equals(keyName)) {
            pressAndReleaseKeyCode((char)Keypad.KEY_DELETE);
        } else {
            Log.error(TAG_LOG, "Unknown key: " + keyName);
            throw new ClientTestException("Unknown key: " + keyName);
        }
    }

    public void checkAlertMessage(String message) throws Throwable {

        UiApplication app = UiApplication.getUiApplication();
        Screen screen = app.getActiveScreen();

        String text = null;
        if (screen instanceof Dialog) {
            Dialog dialog = (Dialog)screen;
            RichTextField rtf = dialog.getLabel();
            text = rtf.getText();
            assertTrue(message, text, "Alert message mismatch");
        } else {
            throw new ClientTestException("The current screen is not an alert");
        }
    }

    public void resetDir(String dir) throws Throwable {
        FileAdapter fa = new FileAdapter(dir);
        Enumeration files = fa.list(false);
        while(files.hasMoreElements()) {
            String file = (String)files.nextElement();
            if (!dir.endsWith("/")) {
                dir = dir + "/";
            }
            file = dir + file;
            Log.debug(TAG_LOG, "Removing file: " + file);
            FileAdapter f = new FileAdapter(file);
            f.delete();
        }
    }

    public void copyFile(String sourceFile, String targetFile) throws Throwable {

        Log.debug(TAG_LOG, "Renaming file: " + sourceFile + " to: " + targetFile);
        FileAdapter sourceFA = new FileAdapter(sourceFile, true);
        InputStream is = sourceFA.openInputStream();
        FileAdapter targetFA = new FileAdapter(targetFile);
        OutputStream os = targetFA.openOutputStream();
        byte buf[] = new byte[4096];
        int len = is.read(buf);
        while(len > 0) {
            os.write(buf);
            len = is.read(buf);
        }
        os.close();
        targetFA.close();
        is.close();
        sourceFA.close();
    }

    public void renameFile(String sourceFile, String targetFile) throws Throwable {

        FileAdapter sourceFA = new FileAdapter(sourceFile);
        sourceFA.rename(targetFile);
        sourceFA.close();
    }

    public void deleteFile(String file) throws Throwable {

        Log.debug(TAG_LOG, "Deleting file: " + file);
        FileAdapter sourceFA = new FileAdapter(file);
        sourceFA.delete();
        sourceFA.close();
    }

    public void trackWheel(String operation, int repeat) throws Exception {
        EventInjector.Event event = null;
        if (repeat <= 0) {
            repeat = 1;
        }
        if (BlackBerryCommandRunner.TRACK_WHEEL_RIGHT.equals(operation)) {
            event = new EventInjector.NavigationEvent(EventInjector.NavigationEvent.NAVIGATION_MOVEMENT,
                                                      1, 0, 0);
        } else if (BlackBerryCommandRunner.TRACK_WHEEL_LEFT.equals(operation)) {
            event = new EventInjector.NavigationEvent(EventInjector.NavigationEvent.NAVIGATION_MOVEMENT,
                                                      -1, 0, 0);
        } else if (BlackBerryCommandRunner.TRACK_WHEEL_UP.equals(operation)) {
            event = new EventInjector.NavigationEvent(EventInjector.NavigationEvent.NAVIGATION_MOVEMENT,
                                                      0, -1, 0);
        } else if (BlackBerryCommandRunner.TRACK_WHEEL_DOWN.equals(operation)) {
            event = new EventInjector.NavigationEvent(EventInjector.NavigationEvent.NAVIGATION_MOVEMENT,
                                                      0, 1, 0);
        } else if (BlackBerryCommandRunner.TRACK_WHEEL_CLICK.equals(operation)) {
            event = new EventInjector.NavigationEvent(EventInjector.NavigationEvent.NAVIGATION_CLICK,
                                                      0, 0, 0);
            EventInjector.invokeEvent(event);
            wait(200);
            event = new EventInjector.NavigationEvent(EventInjector.NavigationEvent.NAVIGATION_UNCLICK,
                                                      0, 0, 0);
        } else {
            Log.error(TAG_LOG, "Unknown track wheel event " + operation);
            throw new ClientTestException("Unknwon track wheel event");
        }
        for(int i=0; i<repeat; i++) {
            EventInjector.invokeEvent(event);
            wait(500);
        }
    }

    public void waitForScreen(String screenName, int milliseconds) throws Exception {
        Log.debug(TAG_LOG, "screen: " + screenName);

        // If the app has not started yet, then we shall wait for the
        // UIContoller to start

        // Wait for the main app to start if it is not running already
        while(!Main.isMainAppStarted()) {
            Thread.sleep(WAIT_DELAY);
            milliseconds -= WAIT_DELAY;
            if(milliseconds < 0) {
                Log.error(TAG_LOG, "The application did not start");
                throw new ClientTestException("waitForScreen timeout (1)" +
                        "for screen: " + screenName);
            }
        }

        UiApplication app = UiApplication.getUiApplication();
        Screen screen = app.getActiveScreen();

        while(screen == null) {
            Thread.sleep(WAIT_DELAY);
            milliseconds -= WAIT_DELAY;
            if(milliseconds < 0) {
                Log.error(TAG_LOG, "The application did not bring any screen");
                throw new ClientTestException("waitForScreen timeout (2)" +
                        "for screen: " + screenName + " \nwas " + screen.getClass());
            }
            screen = app.getActiveScreen();
        }

        String fullName = screen.getClass().getName();
        String components[] = StringUtil.split(fullName, ".");
        String name = components[components.length - 1];
 
        while (!name.equals(screenName))
        {
            Log.debug(TAG_LOG, "Waiting for " + screenName + " " + milliseconds + " ...");
            Thread.sleep(WAIT_DELAY);
            milliseconds -= WAIT_DELAY;
            if(milliseconds < 0) {
                throw new ClientTestException("waitForScreen timeout (3)" +
                        "for screen: " + screenName + " \nwas " + screen.getClass());
            }
            screen = app.getActiveScreen();

            fullName = screen.getClass().getName();
            components = StringUtil.split(fullName, ".");
            name = components[components.length - 1];

            Log.info(TAG_LOG, "Current screen name=" + name);
        }
    }

    public void interruptSyncAfterPhase(String phaseName, int num, String reason, SyncMonitor syncMonitor) throws Throwable {
        Log.debug(TAG_LOG, "Preparing to interrupt sync after phase " + phaseName + "," + num);

        // Register the listeners to monitor the sync execution
        AppSyncSourceManager appSyncSourceManager = AppSyncSourceManager.getInstance();
        Enumeration workingSources = appSyncSourceManager.getWorkingSources();
        syncMonitor.cleanListeners();
        while(workingSources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)workingSources.nextElement();
            SyncSource    source    = appSource.getSyncSource();
            SyncListener  lis       = source.getListener();
            Log.info(TAG_LOG, "Registering monitoring listener for source " + source.getName());
            if (lis != null) {
                // Replace the listener with a monitoring one
                SyncMonitorListener monLis = new SyncMonitorListener(lis);
                source.setListener(monLis);
                syncMonitor.addListener(monLis);
                Log.info(TAG_LOG, "Monitoring listener registered");
            }
        }
        syncMonitor.interruptSyncAfterPhase(phaseName, num, reason);
    }

    public void checkItemsCount(String sourceName, int count) throws Throwable {

        // Search for the sync source
        Log.trace(TAG_LOG, "checkItemsCount for: " + sourceName + "," + count);
        int itemsCount = -1;
        AppSyncSourceManager appSyncSourceManager = AppSyncSourceManager.getInstance();
        Enumeration workingSources = appSyncSourceManager.getWorkingSources();
        while(workingSources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)workingSources.nextElement();
            SyncSource    source    = appSource.getSyncSource();
            if (sourceName.equals(source.getName())) {
                Log.trace(TAG_LOG, "Source found");
                itemsCount = 0;
                source.beginSync(SyncML.ALERT_CODE_SLOW);
                SyncItem item = source.getNextItem();
                while(item != null) {
                    itemsCount++;
                    source.setItemStatus(item.getKey(), 200);
                    item = source.getNextItem();
                }
            }
        }
        assertTrue(count, itemsCount, "Items count mismatch for source: " + sourceName);
    }

    private void pressAndReleaseKey(char keyCode) {
        EventInjector.KeyEvent event = new EventInjector.KeyEvent(EventInjector.KeyEvent.KEY_DOWN,
                                  keyCode, 0, 50);
        EventInjector.invokeEvent(event);
        wait(10);

        event = new EventInjector.KeyEvent(EventInjector.KeyEvent.KEY_UP,
                                  keyCode, 0, 50);
        EventInjector.invokeEvent(event);
        wait(10);
    }

    private void pressAndReleaseKeyCode(char keyCode) {
        EventInjector.KeyCodeEvent event = new EventInjector.KeyCodeEvent(EventInjector.KeyCodeEvent.KEY_DOWN,
                                  keyCode, 0, 50);
        EventInjector.invokeEvent(event);
        wait(100);

        event = new EventInjector.KeyCodeEvent(EventInjector.KeyCodeEvent.KEY_UP,
                                  keyCode, 0, 50);
        EventInjector.invokeEvent(event);
        wait(100);
    }

    public void wait(int delay) {
        try {
            Thread.sleep(delay);
        } catch(Exception ex) {
            Log.error(TAG_LOG, "Exception Occurred: " + ex);
        }
    }
}
