/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import java.util.Enumeration;

import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.Screen;

import com.funambol.blackberry.ui.SynchronizationScreen;
import com.funambol.blackberry.controller.SynchronizationController;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.util.StringUtil;
import com.funambol.syncml.spds.SyncReport;
import com.funambol.syncml.spds.SyncSource;

import com.funambol.client.test.SyncMonitor;

import com.funambol.util.StringUtil;
import com.funambol.util.Log;

public class BlackBerrySyncMonitor extends SyncMonitor {

    private static final String TAG_LOG = "BlackBerrySyncMonitor";

    public boolean isSyncing() {
        Log.debug(TAG_LOG, "Checking if a sync is in progress");
        UiApplication app = UiApplication.getUiApplication();
        Screen screen = app.getActiveScreen();

        SynchronizationController controller = null;
        if (screen instanceof SynchronizationScreen) {
            SynchronizationScreen sScreen = (SynchronizationScreen)screen;
            controller = sScreen.getSynchronizationController();
        }

        if (controller == null) {
            throw new IllegalArgumentException("Cannot wait for sync to terminate on a non syncrhonizing screen");
        }

        // We wait no more than minStart for sync client to start
        return controller.isSynchronizing();
    }

    public SyncReport getSyncReport(String source) {
        Enumeration appSources = AppSyncSourceManager.getInstance().getWorkingSources();
        AppSyncSource appSource = null;
        while(appSources.hasMoreElements()) {
            AppSyncSource aSource = (AppSyncSource)appSources.nextElement();
            if (aSource.getName().equals(source)) {
                appSource = aSource;
                break;
            }
        }
        if (appSource == null) {
            throw new IllegalArgumentException("Cannot check last sync exchanged because " + source + " cannot be found");
        }
        SyncReport sr = appSource.getUISyncSourceController().getLastSyncReport();
        return sr;
    }



} 
