/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.RichTextField;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.UiApplication;

import com.funambol.blackberry.ui.Screen;
import com.funambol.blackberry.ui.rim.RimHelper;
import com.funambol.blackberry.ui.rim.RimDisplayManager;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;

public class FirstScreen extends MainScreen implements Screen {

    private static final String TAG_LOG = "FirstScreen";

    private boolean started = false;
    private String scriptUrl;
    private BasicEditField scriptUrlField;

    public FirstScreen() {
        
        this.setFont(RimHelper.getDefaultFont());
        
        LabelField title = new LabelField("AutoTest start");

        title.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(title);

        LabelField serverUriLabel = new LabelField("Script Url");
        serverUriLabel.setFont(getFont().derive(Font.BOLD));
        scriptUrlField = new BasicEditField("", "http://");

        add(serverUriLabel);
        add(scriptUrlField);

        // Fire the auto run thread (start thread in 120 secs unless the user
        // starts changing the URL)
        AutoRunThread autoRunThread = new AutoRunThread(120);
        autoRunThread.start();
    }

    public void makeMenu(Menu menu, int instance) {
        menu.add(startMenu);
    }

    private void startTest(boolean force) {
        String url = scriptUrlField.getText();
        if ("http://".equals(url)) {
            // The user did not define his own script
            url = null;
        }

        if (url != null && !force) {
            // the user is editing the field, let him do it
            Log.info(TAG_LOG, "User editing the field, wait for manual start");
        } else {
            Log.info(TAG_LOG, "Starting test located at: " + url);
            Main.startTest(url);
            started = true;
        }
    }

    private final MenuItem startMenu = new MenuItem("start", 20000, 10) {
                                                              public void run() {
                                                                  startTest(true);
                                                              }
                                                          };

    private class AutoRunThread extends Thread {

        private int secs;

        public AutoRunThread(int secs) {
            this.secs = secs;
        }

        public void run() {
            try {
                Thread.sleep(secs * 1000);
            } catch (Exception e) {}

            if (!started) {
                startTest(false);
            }
        }
    }

}
