/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.autotest;

import java.util.Enumeration;

import net.rim.device.api.system.Application;
import net.rim.device.api.ui.Screen;
import net.rim.device.api.ui.container.MainScreen;

import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.ui.rim.RimDisplayManager;
import com.funambol.blackberry.autotest.AutoTestClient;

import com.funambol.syncml.spds.SyncListener;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.client.test.SyncMonitor;
import com.funambol.client.test.SyncMonitorListener;

import com.funambol.util.ConsoleAppender;
import com.funambol.util.Log;

/**
 * Provides the main entry point to the sync application
 * 
 */
public class Main extends Application {

    private static final String TAG_LOG = "Main";

    private static SyncClient client;
    private static AutoTestThread tt;

    private static boolean isMainAppStarted = false;
    private static FirstScreen firstScreen = null;

    /**
     * This is the entry point for SyncClient Application. The thread that calls
     * this method (typically the main thread in the application) becomes the
     * event-dispatching thread, which will execute all drawing and
     * event-handling code
     * 
     * @param s
     *            [] An array of strings passed on the command line as
     *            parameters for the <code>java</code> interpreter
     */
    public static void main(String args[]) throws Exception {

        // Create the Application instance here, otherwise without Application
        // instance many operations are forbidden (for example access to the
        // network)
        client = new AutoTestClient();

        // Init the Log
        Log.initLog(new ConsoleAppender(), Log.TRACE);
        Log.setLogLevel(Log.TRACE);

        // Show the first screen
        firstScreen = new FirstScreen();
        RimDisplayManager dm = new RimDisplayManager();
        dm.showScreen(firstScreen);
        
        // enter event loop
        client.enterEventDispatcher();

        //ApplicationPermissions requestedPermissions = new ApplicationPermissions();
        //requestedPermissions.addPermission( ApplicationPermissions.PERMISSION_EVENT_INJECTOR );
        //appPermissionsManager.invokePermissionsRequest( requestedPermissions );
    }

    public static void startTest(String scriptUrl) {

        RimDisplayManager dm = new RimDisplayManager();
        try {
            dm.hideScreen(firstScreen);
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot hide first screen", e);
        }

        Log.info(TAG_LOG, "Starting test with script at: " + scriptUrl);
        // Start the test
        tt = new AutoTestThread(scriptUrl);
        tt.start();
    }

    public static void startMainApp() throws Exception {
        // Start the application
        Log.info(TAG_LOG, "Starting main app");
        client.start();

        // Bring the app in foreground
        Log.info(TAG_LOG, "Bringing app to foreground");
        synchronized (SyncClient.getEventLock()) {
            Log.info(TAG_LOG, "Requesting foreground");
            SyncClient.getUiApplication().requestForeground();
        }
        isMainAppStarted = true;
    }

    public static boolean isMainAppStarted() {
        return isMainAppStarted;
    }
}
