/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import com.funambol.util.Log;

public class AppSyncSourceConfig {
    private static final String TAG_LOG = "AppSyncSourceConfig";

    private String uri;
    private boolean enabled;
    private boolean active;
    private int id;
    private int syncType = -1;
    private boolean deviceFullShown = false;
    private boolean sourceSynced;
    private boolean uploadContentViaHttp;

    public AppSyncSourceConfig(int id, boolean enabled, boolean active, String uri) {
        this(id, enabled, active, uri, false);
    }

    public AppSyncSourceConfig(int id, boolean enabled, boolean active, String uri,
                               boolean deviceFullShown) {
        this(id, enabled, active, uri, false, false);
    }

    public AppSyncSourceConfig(int id, boolean enabled, boolean active, String uri,
                               boolean deviceFullShown, boolean sourceSynced) {
        this.id = id;
        this.enabled = enabled;
        this.active  = active;
        this.uri = uri;
        this.deviceFullShown = deviceFullShown;
        this.sourceSynced = sourceSynced;
    }

    public boolean getEnabled() {
        return enabled;
    }

    public boolean getActive() {
        return active;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    public void setActive(boolean active) {
        this.active = active;
    }

    public String getUri() {
        return uri;
    }

    public void setUri(String uri) {
        this.uri = uri;
    }

    public boolean getDeviceFullShown() {
        return deviceFullShown;
    }

    public void setDeviceFullShown(boolean value) {
        this.deviceFullShown = value;
    }

    public void setSynced(boolean sourceSynced) {
        this.sourceSynced = sourceSynced;
    }

    public boolean getSynced() {
        return sourceSynced;
    }

    public int getSyncType() {
        // If this ss has its own specific sync mode then we use it,
        // otherwise we use the global one in the Configuration
        return syncType;
    }

    public void setSyncType(int syncType) {
        this.syncType = syncType;
    }

    public boolean getUploadContentViaHttp() {
        return uploadContentViaHttp;
    }

    public void setUploadContentViaHttp(boolean value) {
        Log.info(TAG_LOG, "Setting upload content via http to " + value);
        uploadContentViaHttp = value;
    }
}

