/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

import java.io.DataInputStream;
import java.io.ByteArrayInputStream;

import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMList;
import net.rim.blackberry.api.pdap.BlackBerryPIM;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.system.Application;

import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.blackberry.helper.HelperFactory;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.source.BBPIMSyncSource;
import com.funambol.blackberry.source.BBPIMSyncSourceWithUpdateFix;
import com.funambol.blackberry.source.PhotoTracker;
import com.funambol.blackberry.source.PhotoSyncSource;
import com.funambol.blackberry.source.TwoPhasesPhotoSyncSource;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.client.ConfigSyncSource;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.storage.StringKeyValueFileStore;
import com.funambol.syncml.client.CacheTracker;
import com.funambol.syncml.client.FileSyncSource;
import com.funambol.syncml.protocol.SyncML;

/**
 * This class is responsible for handling all the sources at the application
 * level. A source at the application level is a type of data for which a
 * SyncSource exits, but also a sync button on the screen and configuration
 * parameters.
 * This class creates and registers the sources. The list of sources is handled
 * and allows other parts of the application to be completely parametric wrt to
 * this value.
 * For new sources to be added to the application it is necessary to register
 * them in this manager. The only other part of the application that must be
 * changed when new sources are added is the Customization which holds default
 * parameters for each source.
 */
public class AppSyncSourceManager {

    private static final String TAG_LOG = "AppSyncSourceManager";

    public static final int     SYNC_FLAG_CONTACTS        = 1;
    public static final int     SYNC_FLAG_CALENDAR        = 2;
    public static final int     SYNC_FLAG_TASKS           = 4;
    public static final int     SYNC_FLAG_NOTES           = 8;
    public static final int     SYNC_FLAG_MAIL            = 16;
    public static final int     SYNC_FLAG_PHOTOS          = 32;
    public static final int     SYNC_FLAG_CONFIG          = 64;
    public static final int     SYNC_FLAG_ALL             = 255;

    public static final long    CONTACT_STORAGE_KEY       = 0xbc565a75eda5330L;
    public static final long    CALENDAR_STORAGE_KEY      = 0xbc565a75eda5340L;
    public static final long    NOTE_STOREAGE_KEY         = 0xbc565a75eda5350L;
    public static final long    TASK_STORAGE_KEY          = 0xbc565a75eda5360L;
    public static final long    PHOTO_STORAGE_KEY         = 0xbc565a75eda5380L;
    public static final long    CONFIG_STORAGE_KEY        = 0xbc565a75eda53A0L;

    public static final long    CONTACTS_ADDITS_STORAGE_KEY = 0xbc565a75eda5390L;
    public static final long    CALENDAR_ADDITS_STORAGE_KEY = 0xbc565a75eda5370L;
    public static final long    NOTE_ADDITS_STORAGE_KEY     = -1;
    public static final long    TASKS_ADDITS_STORAGE_KEY    = -1;

    // Configuration keys for the source configs
    public static final String  CONFIG_KEY_SYNC_CONTACT   = "SYNC_CONTACT_CONFIG";
    public static final String  CONFIG_KEY_SYNC_CALENDAR  = "SYNC_CAL_CONFIG";
    public static final String  CONFIG_KEY_SYNC_TASK      = "SYNC_TASK_CONFIG";
    public static final String  CONFIG_KEY_SYNC_NOTE      = "SYNC_NOTE_CONFIG";
    public static final String  CONFIG_KEY_SYNC_PHOTO     = "SYNC_PHOTO_CONFIG";
    public static final String  CONFIG_KEY_SYNC_CONFIG    = "SYNC_CONFIG_CONFIG";

    // Types
    public static final String  CONTACT_TYPE_VCARD        = SourceConfig.VCARD_TYPE;
    public static final String  CONTACT_TYPE_SIF          = "text/x-s4j-sifc";
    public static final String  CALENDAR_TYPE_SIF         = "text/x-s4j-sife";
    public static final String  CALENDAR_TYPE_ICALENDAR   = "text/calendar";
    public static final String  CALENDAR_TYPE_VCALENDAR   = "text/x-vcalendar";
    public static final String  NOTE_TYPE_SIF             = "text/x-s4j-sifn";
    public static final String  TASK_TYPE_SIF             = "text/x-s4j-sift";
    public static final String  TASK_TYPE_VCALENDAR       = "text/x-vcalendar";
    public static final String  BRIEFCASE_TYPE            = SourceConfig.FILE_OBJECT_TYPE;

    // Source config settings
    private static final String  NOTE_TYPE                 = NOTE_TYPE_SIF;
    private static final String  PHOTO_TYPE                = SourceConfig.FILE_OBJECT_TYPE;
    private static final String  CONFIG_TYPE               = SourceConfig.BRIEFCASE_TYPE;
    
    private static final String  CONTACT                   = "contact";
    private static final String  CALENDAR                  = "calendar";
    private static final String  NOTE                      = "note";
    private static final String  TASK                      = "task";
    private static final String  PHOTO                     = "photo";
    private static final String  CONFIG                    = "config";


    private static AppSyncSourceManager instance = null;
    private Hashtable idMap       = null;
    private Vector    sourcesList = null;
    private Configuration configuration;

    private AppSyncSourceManager() {
        idMap       = new Hashtable();
        sourcesList = new Vector();
        configuration = Configuration.getInstance();
    }

    public static AppSyncSourceManager getInstance() {
        if (instance == null) {
            instance = new AppSyncSourceManager();
        }
        return instance;
    }

    public void registerSource(AppSyncSource source) {
        sourcesList.addElement(source);
        idMap.put(new Integer(source.getId()), source);
    }

    public int numberOfRegisteredSources() {
        return idMap.size();
    }

    public int numberOfEnabledSources() {
        int count = 0;
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled()) {
                count++;
            }
        }
        return count;
    }

    public int numberOfWorkingSources() {
        int count = 0;
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isWorking()) {
                count++;
            }
        }
        return count;
    }

    public int numberOfEnabledAndWorkingSources() {
        int count = 0;
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled() && appSource.isWorking()) {
                count++;
            }
        }
        return count;
    }

    public void setUpSource(int id, int uiOrder) throws Exception {
        switch(id) {
            case SYNC_FLAG_CONTACTS:
                setUpContactsSource(uiOrder);
                break;
            case SYNC_FLAG_CALENDAR:
                setUpCalendarSource(uiOrder);
                break;
            case SYNC_FLAG_TASKS:
                setUpTasksSource(uiOrder);
                break;
            case SYNC_FLAG_NOTES:
                setUpNotesSource(uiOrder);
                break;
            case SYNC_FLAG_PHOTOS:
                setUpPhotosSource(uiOrder);
                break;
            default:
                throw new IllegalArgumentException("Unknwon source with id: " + id);
        }
    }

    public void setUpContactsSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "setUpContactSource");

        // Build the appropriate AppSyncSource and register it, even if the
        // corresponding SyncSource has not been built yet
        String name = StaticDataHelper.getLanguage("type_contacts");
        AppSyncSource contactSource = new AppSyncSource(name);
        contactSource.setId(SYNC_FLAG_CONTACTS);
        contactSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-contact"));
        contactSource.setDisabledLabel(StaticDataHelper.getLanguage("message_contacts-disabled"));
        contactSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_CONTACTS));
        contactSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_CONTACTS));
        contactSource.setUiSourceIndex(uiOrder);
        contactSource.setConfigStorageKey(CONFIG_KEY_SYNC_CONTACT);
        contactSource.setHasSettings(true);
        contactSource.setLastSyncStatus(Configuration.loadLastSyncStatus(contactSource));
        registerSource(contactSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_CONTACTS);

        boolean base64 = Customization.getInstance().getContactType() !=
                         AppSyncSourceManager.CONTACT_TYPE_VCARD;
        SourceConfig config = initializeSourceConfig(true, SYNC_FLAG_CONTACTS,
                    CONFIG_KEY_SYNC_CONTACT, CONTACT, Customization.getInstance().getContactType(),
                    uri, base64);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.CONTACT_LIST,
                                                       PIM.READ_WRITE);

        boolean updateFix = false;
        String version[] = StaticDataHelper.getOSVersion();
        if (version.length > 1 && version[0].equals("4") && version[1].equals("5")) {
            Log.debug(TAG_LOG, "Version is 4.5, use source with update fix");
            updateFix = true;
        } else { 
            Log.debug(TAG_LOG, "Version is NOT 4.5, use standard source"); 
        }
        SyncSource syncSource;
        if (updateFix) {
            syncSource = new BBPIMSyncSourceWithUpdateFix(config, list,
                               HelperFactory.getContactHelper(list),
                               CONTACT_STORAGE_KEY,
                               CONTACTS_ADDITS_STORAGE_KEY);
        } else {
            syncSource = new BBPIMSyncSource(config, list,
                                    HelperFactory.getContactHelper(list),
                                    CONTACT_STORAGE_KEY,
                                    CONTACTS_ADDITS_STORAGE_KEY);
        }

        syncSource.setConfig(config);
        // Reset the underlying sync source
        contactSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(contactSource);
        contactSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;
            BBPIMListener listener = new BBPIMListener(bbList, contactSource,configuration);
            bbSource.setPIMListener(listener);
            bbList.addListener(listener);
            contactSource.setNativeListener(listener);
        }
    }

    public void setUpCalendarSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "setUpCalendarSource");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_calendar");
        AppSyncSource calendarSource = new AppSyncSource(name);
        calendarSource.setId(SYNC_FLAG_CALENDAR);
        calendarSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-calendar"));
        calendarSource.setDisabledLabel(StaticDataHelper.getLanguage("message_calendar-disabled"));
        calendarSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_CALENDAR));
        calendarSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_CALENDAR));
        calendarSource.setUiSourceIndex(uiOrder);
        calendarSource.setConfigStorageKey(CONFIG_KEY_SYNC_CALENDAR);
        calendarSource.setHasSettings(true);
        calendarSource.setLastSyncStatus(Configuration.loadLastSyncStatus(calendarSource));
        registerSource(calendarSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_CALENDAR);

        String calendarType = Customization.getInstance().getCalendarType();
        boolean b64;
        if (calendarType.equals(AppSyncSourceManager.CALENDAR_TYPE_SIF)) {
            b64 = true;
        } else {
            b64 = false;
        }

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                                                     CONFIG_KEY_SYNC_CALENDAR,
                                                     CALENDAR, calendarType,
                                                     uri, b64);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.EVENT_LIST, PIM.READ_WRITE);

        EventMailListener listener = null;
        if (Customization.getInstance().invitesFiltering()) {
            listener = new EventMailListener(StaticDataHelper.getLanguage("message_blocked-invite"));
        }

        PIMItemHelper eventhelper = HelperFactory.getEventHelper(list, listener,
                Configuration.getRangePast(), Configuration.getRangeFuture());

        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list,
                                         eventhelper, CALENDAR_STORAGE_KEY, CALENDAR_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        // Reset the underlying sync source
        calendarSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(calendarSource);
        calendarSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;

            BBPIMListener pimListener = new BBPIMListener(bbList,calendarSource,configuration);
            bbSource.setPIMListener(pimListener);
            bbList.addListener(pimListener);
            calendarSource.setNativeListener(pimListener);
        }
    }

    public void setUpTasksSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "setUpTasksSource");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_tasks");

        AppSyncSource tasksSource = new AppSyncSource(name);
        tasksSource.setId(SYNC_FLAG_TASKS);
        tasksSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-task"));
        tasksSource.setDisabledLabel(StaticDataHelper.getLanguage("message_tasks-disabled"));
        tasksSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_TASKS));
        tasksSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_TASKS));
        tasksSource.setUiSourceIndex(uiOrder);
        tasksSource.setConfigStorageKey(CONFIG_KEY_SYNC_TASK);
        tasksSource.setHasSettings(true);
        tasksSource.setLastSyncStatus(Configuration.loadLastSyncStatus(tasksSource));
        registerSource(tasksSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_TASKS);

        String taskType = Customization.getInstance().getTaskType();
        boolean b64;
        if (taskType.equals(AppSyncSourceManager.CALENDAR_TYPE_SIF)) {
            b64 = true;
        } else {
            b64 = false;
        }

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_TASKS,
                    CONFIG_KEY_SYNC_TASK, TASK, Customization.getInstance().getTaskType(),
                    uri, b64);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(PIM.TODO_LIST,
                                                                 PIM.READ_WRITE);
        PIMItemHelper helper = HelperFactory.getTaskHelper(list);

        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list, helper, TASK_STORAGE_KEY, TASKS_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        tasksSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(tasksSource);
        tasksSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;
            BBPIMListener listener = new BBPIMListener(bbList, tasksSource,configuration);
            bbSource.setPIMListener(listener);
            bbList.addListener(listener);
            tasksSource.setNativeListener(listener);
        }
    }

    public void setUpNotesSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Creating Note sync source");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_notes");
        AppSyncSource notesSource = new AppSyncSource(name);
        notesSource.setId(SYNC_FLAG_NOTES);
        notesSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-note"));
        notesSource.setDisabledLabel(StaticDataHelper.getLanguage("message_notes-disabled"));
        notesSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_NOTES));
        notesSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_NOTES));
        notesSource.setUiSourceIndex(uiOrder);
        notesSource.setConfigStorageKey(CONFIG_KEY_SYNC_NOTE);
        notesSource.setHasSettings(true);
        notesSource.setLastSyncStatus(Configuration.loadLastSyncStatus(notesSource));
        registerSource(notesSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_NOTES);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_NOTES,
                    CONFIG_KEY_SYNC_NOTE, NOTE, NOTE_TYPE, uri, true);

        PIMList list;
        list = (PIMList) PIM.getInstance().openPIMList(BlackBerryPIM.MEMO_LIST, PIM.READ_WRITE);
        SyncSource syncSource;
        syncSource = new BBPIMSyncSource(config, list, HelperFactory.getNoteHelper(list),
                                         NOTE_STOREAGE_KEY, NOTE_ADDITS_STORAGE_KEY);

        syncSource.setConfig(config);

        notesSource.setSyncSource(syncSource);
        // Set the open application (used in the goto menu)
        ExternalAppManager appManager = new ExternalAppManager(notesSource);
        notesSource.setAppManager(appManager);
        // Register the event listener for this source
        if (list instanceof BlackBerryPIMList) {
            BlackBerryPIMList bbList = (BlackBerryPIMList)list;
            BBPIMListener listener = new BBPIMListener(bbList, notesSource,configuration);
            BBPIMSyncSource   bbSource = (BBPIMSyncSource)syncSource;
            bbSource.setPIMListener(listener);
            bbList.addListener(listener);
            notesSource.setNativeListener(listener);
        }
    }

    public void setUpPhotosSource(int uiOrder) throws Exception {
        Log.info(TAG_LOG, "Creating Photos sync source");

        // Build the appropriate AppSyncSource
        String name = StaticDataHelper.getLanguage("type_photos");
        PhotoAppSyncSource photosSource = new PhotoAppSyncSource(name);
        photosSource.setId(SYNC_FLAG_PHOTOS);
        photosSource.setEnabledLabel(StaticDataHelper.getLanguage("conf_enable-sync-photo"));
        photosSource.setDisabledLabel(StaticDataHelper.getLanguage("message_photos-disabled"));
        photosSource.setIconName(Customization.getInstance().getSourceIcon(SYNC_FLAG_PHOTOS));
        photosSource.setDisabledIconName(Customization.getInstance().getSourceDisabledIcon(SYNC_FLAG_PHOTOS));
        photosSource.setUiSourceIndex(uiOrder);
        photosSource.setConfigStorageKey(CONFIG_KEY_SYNC_PHOTO);
        photosSource.setHasSettings(true);
        photosSource.setLastSyncStatus(Configuration.loadLastSyncStatus(photosSource));
        photosSource.setWarningOnFirstSync(StaticDataHelper.getLanguage("dialog_first_picture_sync"));
        registerSource(photosSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_PHOTOS);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_PHOTOS,
                    CONFIG_KEY_SYNC_PHOTO, PHOTO, PHOTO_TYPE, uri, false);

        // Set (and lock) the sync mode for this source
        Configuration.setSourceSyncType(photosSource.getId(),
                                  SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW);

        //Configuration.setSourceSyncType(SYNC_FLAG_PHOTOS,
        //                          SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT);

        // Create the sync source (also the one needed for backward
        // compatibility)
        String extensions[] = { "jpg", "jpeg", "gif", "png" };
        TwoPhasesPhotoSyncSource syncSource = new TwoPhasesPhotoSyncSource(config, extensions,
                Configuration.getSyncConfig(), Customization.getInstance().getUploadUrl());

        PhotoSyncSource syncMLSrc = new PhotoSyncSource(config, extensions);

        photosSource.setSyncMLSource(syncMLSrc);
        photosSource.setTwoPhasesSyncSource(syncSource);

        if (Configuration.isUploadViaHttp(SYNC_FLAG_PHOTOS)) {
            photosSource.setSyncSource(syncSource);
        } else {
            photosSource.setSyncSource(syncMLSrc);
        }
        
        // Set the open application (used in the goto menu)
        //ExternalAppManager appManager = new ExternalAppManager(photosSource);
        //photosSource.setAppManager(appManager);
        String directory = Configuration.getPicturesDir();
        Application app = Application.getApplication();

        BBFileSystemListener listener = new BBFileSystemListener(photosSource, 
                                                directory,configuration);
        app.addFileSystemJournalListener(listener);
        photosSource.setNativeListener(listener);
    }

    public void setUpConfigSource() throws Exception {
        Log.info(TAG_LOG, "setUpConfigSource");

        // Build the appropriate AppSyncSource
        String name = "config";
        AppSyncSource configSource = new AppSyncSource(name);
        configSource.setId(SYNC_FLAG_CONFIG);
        configSource.setEnabledLabel(null);
        configSource.setDisabledLabel(null);
        configSource.setIconName(null);
        configSource.setDisabledIconName(null);
        configSource.setUiSourceIndex(0);
        configSource.setConfigStorageKey(CONFIG_KEY_SYNC_CONFIG);
        configSource.setHasSettings(false);
        configSource.setIsRefreshSupported(false);
        registerSource(configSource);

        String uri = Configuration.getSourceUri(SYNC_FLAG_CONFIG);

        SourceConfig config = initializeSourceConfig(true, AppSyncSourceManager.SYNC_FLAG_CONFIG,
                                                     CONFIG_KEY_SYNC_CONFIG,
                                                     CONFIG, CONFIG_TYPE,
                                                     uri, false);

        StringKeyValueFileStore cacheStore = new StringKeyValueFileStore("file:///store/home/user/configcache.txt");
        StringKeyValueFileStore configStore = new StringKeyValueFileStore("file:///store/home/user/configstore.txt");
        CacheTracker tracker = new CacheTracker(cacheStore);
 
        SyncSource syncSource;
        syncSource = new ConfigSyncSource(config, tracker, configStore);

        syncSource.setConfig(config);

        // Reset the underlying sync source
        configSource.setSyncSource(syncSource);
    }


    public AppSyncSource getSource(int id) {
        AppSyncSource source = (AppSyncSource)idMap.get(new Integer(id));
        return source;
    }

    /*
     * Disable a given source. After this method returns the source is
     * disabled for syncs and the setting is propagated into the configuration
     */
    public void disableSource(int id) {
        Configuration.disableSource(id);
    }

    public Enumeration getEnabledSources() {

        Log.trace(TAG_LOG, "getEnabledSources");
        Vector result = new Vector();
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled()) {
                result.addElement(appSource);
            }
        }
        return result.elements();
    }

    public Enumeration getWorkingSources() {

        Log.trace(TAG_LOG, "getWorkingSources");
        Vector result = new Vector();
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isWorking()) {
                result.addElement(appSource);
            }
        }
        return result.elements();
    }

    public Enumeration getEnabledAndWorkingSources() {
        Log.trace(TAG_LOG, "getEnabledAndWorkingSources");
        Vector result = new Vector();
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            if (appSource.isEnabled() && appSource.isWorking()) {
                result.addElement(appSource);
            }
        }
        return result.elements();
    }

    public Enumeration getRegisteredSources() {
        Log.trace(TAG_LOG, "getRegisteredSources");
        
        Vector result = new Vector();
        for(int i=0;i<sourcesList.size();++i) {
            AppSyncSource appSource = (AppSyncSource)sourcesList.elementAt(i);
            result.addElement(appSource);
        }
        return result.elements();
    }

    /**
     * @param read
     * @param last
     * @param next
     * @param flag
     * @param name
     *            TODO
     * @param type
     *            TODO
     * @param uri
     *            TODO
     */
    private SourceConfig initializeSourceConfig(boolean read, int flag, String configKey,
                                                String name, String type, String
                                                uri, boolean base64) {

        long last = 0;
        long next = 0;
        boolean restore = false;
        SourceConfig config;

        Log.info(TAG_LOG, "Initialize SourceConfig [" + name + "]");
        config = new SourceConfig(name, type, uri);

        if (base64) {
            config.setEncoding(SyncSource.ENCODING_B64);
        } else {
            config.setEncoding(SyncSource.ENCODING_NONE);
        }

        // Read from persistent store?
        if (read) {
            try {
                Log.debug(TAG_LOG, "Reading config from store " + configKey);

                byte[] byte_array = Configuration.getValue(configKey, new byte[0]);
                if (byte_array != null) {
                    Log.debug(TAG_LOG, "Data Found");
                    DataInputStream temp = new DataInputStream(new ByteArrayInputStream(byte_array));
                    config.deserialize(temp);
                    temp.close();
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Exception while initializating (reading) of SourceConfig ["
                           + name + "]");
            }
        }

        // Re-sets the max number of items in a single SyncML message during
        // slow syncs
        int maxItemsPerMessage = Customization.getInstance().getMaxItemsPerMessageInSlowSync();
        Log.info(TAG_LOG, "Setting max items in slow sync to: " + maxItemsPerMessage + " for source " + name);
        config.setMaxItemsPerMessageInSlowSync(maxItemsPerMessage);

        // Restore anchors?
        if (restore) {
            config.setLastAnchor(last);
            config.setNextAnchor(next);
        }
        // Save!
        Configuration.saveSourceConfig(configKey, config);
        return config;
    }
}


