/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Enumeration;
import java.util.Vector;

import net.rim.device.api.io.file.FileSystemJournalListener;
import net.rim.device.api.io.file.FileSystemJournal;
import net.rim.device.api.io.file.FileSystemJournalEntry;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.blackberry.source.ExtendedSyncSource;
import com.funambol.blackberry.source.PhotoSyncSource;
import com.funambol.syncml.client.FileSyncSource;

import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Log;

/**
 * Class for PIM listening on BB. This class listens for events on PIM data
 * and notifies the sources so they can track changes and if the application is
 * in push mode a sync is scheduled for the proper source.
 */
public class BBFileSystemListener implements FileSystemJournalListener,NativeListener
{
    protected AppSyncSource        source;
    private   AppSyncSourceManager appSyncSourceManager;
    private   Customization        customization;
    private   String               directory;
    private   Configuration        configuration;

    public BBFileSystemListener(AppSyncSource source, String directory, Configuration config)
    {
        this.source    = source;
        appSyncSourceManager = AppSyncSourceManager.getInstance();
        customization = Customization.getInstance();
        configuration = config;
        setDirectory(directory);
    }

    public void setDirectory(String directory) {

        if (directory.startsWith("file://")) {
            this.directory = directory.substring("file://".length());
        } else {
            this.directory = directory;
        }
    }

    /**
     * This method is invoked by the system on file system changes. The
     * FileSystemJournal allows us to detect what changed on the file system.
     * If a change was made to the directory this listener is monitoring, then a
     * synchronization is triggered.
     * Since this method is invoked in a different process, it cannot use any
     * static variable of this application, otherwise it will get un-initialized
     * values.
     */
    public void fileJournalChanged() {
        long nextUSN = FileSystemJournal.getNextUSN();
        long lastStoredUSN = configuration.getLastUSN();
        //String picturesDir = configuration.getPicturesDir();

        for (long lookUSN = nextUSN-1; lookUSN >= lastStoredUSN; lookUSN--) {
            FileSystemJournalEntry entry = FileSystemJournal.getEntry(lookUSN);

            if (entry == null) {
                // we didn't find an entry.
                break;
            }

            if (entry.getEvent() == FileSystemJournalEntry.FILE_ADDED   ||
                entry.getEvent() == FileSystemJournalEntry.FILE_CHANGED) {

               String path = entry.getPath();

               if (path != null && path.startsWith(directory) && filterFile(path)) {
                   programSync();
                   break;
               }
           } else if (entry.getEvent() == FileSystemJournalEntry.FILE_DELETED) {

               String path = entry.getOldPath();
               if (path != null && path.startsWith(directory) && filterFile(path)) {
                   programSync();
                   break;
               }
           } else if (entry.getEvent() == FileSystemJournalEntry.FILE_RENAMED) {
               // Check if the file got moved into the picture's dir
               String path = entry.getPath();

               if (path != null && path.startsWith(directory) && filterFile(path)) {
                   programSync();
                   break;
               }
               // Now check if the file got deleted from the picture's dir
               path = entry.getOldPath();
               if (path != null && path.startsWith(directory) && filterFile(path)) {
                   programSync();
                   break;
               }
           }
        }
        UISyncSourceController uiSourceController = source.getUISyncSourceController();
        // The source controller may be null if the source is currently
        // disabled/deactivated
        if (uiSourceController != null) {
            Controller controller = uiSourceController.getController();
            configuration.setLastUSN(nextUSN);
            configuration.save(controller);
        }
    }

    private void programSync() {
        if (customization.isC2SPushEnabled() && configuration.getC2SPush()) {
            // We always use the home screen controller to fire the sync because
            // we want the sources to be updated in case the screen is shown
            // We must check if this source is enabled before triggering a sync
            Enumeration sources = appSyncSourceManager.getEnabledAndWorkingSources();
            while (sources.hasMoreElements()) {
                AppSyncSource appSource = (AppSyncSource)sources.nextElement();
                if (appSource == source) {
                    // The source is enabled, fire a sync
                    UISyncSourceController uiSourceController = appSource.getUISyncSourceController();
                    Controller controller = uiSourceController.getController();
                    HomeScreenController hsc = controller.getHomeScreenController();

                    // If a sync for this source is already in progress, then
                    // we ignore this change as it is very likely it has been
                    // generated by the sync itself
                    if (!hsc.isSynchronizing() || hsc.getCurrentSource() != source) {
                        int delay = customization.getC2SPushDelay();
                        Vector ss = new Vector();
                        ss.addElement(appSource);
                        hsc.synchronize("", ss, delay, true);
                        break;
                    }
                }
            }
        }
    }

    private boolean filterFile(String name) {

        SyncSource ss = source.getSyncSource();
        if (ss instanceof FileSyncSource) {
            FileSyncSource fss = (FileSyncSource)ss;
            return fss.filterFile(name);
        }
        return true;
    }
}
 
