/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;
import java.util.Enumeration;

import net.rim.device.api.i18n.Locale;

import com.funambol.blackberry.BBVersion;

import com.funambol.util.Log;
import com.funambol.util.StringUtil;

public class BlackBerryLocalization {

    private static final String TAG_LOG = "BlackBerryLocalization";

    private static final String XML_LANGUAGE_FILE = "/xml/language";
    private static final String LOCALE_SEP = "-";
    // Define the locale to use as default
    private static final String DEFAULT_LOCALE = BBVersion.DEFAULT_LOCALE;
    private static final String DEFAULT_LANGUAGE_FILE = XML_LANGUAGE_FILE +
            LOCALE_SEP + BBVersion.DEFAULT_LOCALE + ".xml";

    private Hashtable languageHash = new Hashtable();


    public BlackBerryLocalization() throws Exception {
        loadLanguage();
    }

    /**
     * Return a localized string for the given key. The method shall never
     * return null. If the key is not found, the key itself shall be returned.
     * @param key the string value key
     */
    public String getLanguage(String key) {
        Object valueFromHash = languageHash.get(key);
        if (valueFromHash != null) {
            return (String) valueFromHash;
        }
        // This should never happen
        Log.error(TAG_LOG, "Language string missing for key: " + key);
        return "";
    }

    /**
     * Return a date in the given locale format.
     * @param date is a UTC date
     */
    public String getDate(long date) {
        // TODO FIXME: to  be implemented
        return null;
    }

    /**
     * Return a time in the given locale format.
     * @param date is a UTC date
     */
    public String getTime(long date) {
        // TODO FIXME: to  be implemented
        return null;
    }

    /**
     * Load the application language
     */
    private void loadLanguage() throws Exception {

        Log.info(TAG_LOG, "Loading default language: " + DEFAULT_LOCALE);

        //
        // Initialize the default language
        //
        if(!includeLanguageFile(DEFAULT_LANGUAGE_FILE, DEFAULT_LOCALE)) {
            Log.error(TAG_LOG, "Can't load default language file: " + DEFAULT_LANGUAGE_FILE);
            throw new Exception("Can't load default language: " + DEFAULT_LANGUAGE_FILE);
        }

        //
        // Search for customizations of the default language
        //
        // Example: "/xml/language_en_US_custom.xml"
        //
        String customDefaultLanguageFile = XML_LANGUAGE_FILE + LOCALE_SEP + DEFAULT_LOCALE
                                           + LOCALE_SEP + "custom.xml";
        includeLanguageFile(customDefaultLanguageFile, DEFAULT_LOCALE);
        
        //
        // Read the current Locale information: 
        //  - l_language (e.g. "en")
        //  - l_country  (e.g. "US")
        //
        Locale locale = Locale.getDefault();
        String l_language = locale.getLanguage();
        String l_country  = locale.getCountry();

        Log.info(TAG_LOG, "Device locale language: " + l_language);
        Log.info(TAG_LOG, "Device locale country: "  + l_country);

        //
        // Search for a language resource related to current Locale, giving the
        // following priorities:
        //
        // 1) "/xml/language_" + language + "_" + country + ".xml"
        // 2) "/xml/language_" + language + ".xml"
        //
        String languageFile = null;
        String localeInfo   = null;

        boolean localeFound = false;
        String  localeFoundString = null;

        //
        // Search for "/xml/language_" + language + "_" + country + ".xml"
        //
        if(!StringUtil.isNullOrEmpty(l_language) && !StringUtil.isNullOrEmpty(l_country)) {
            // Search for "/xml/language_" + l_language + "_" + l_country + ".xml"
            localeInfo = l_language + "_" + l_country;
            languageFile = XML_LANGUAGE_FILE + LOCALE_SEP + localeInfo + ".xml";
            if(includeLanguageFile(languageFile, localeInfo)) {
                localeFound = true;
                localeFoundString = localeInfo;
            }
        }

        //
        // Search for "/xml/language_" + language + ".xml"
        //
        if(!localeFound && !StringUtil.isNullOrEmpty(l_language)) {
            // Search for "/xml/language_" + l_language + ".xml"
            localeInfo = l_language;
            languageFile = XML_LANGUAGE_FILE + LOCALE_SEP + localeInfo + ".xml";
            if(includeLanguageFile(languageFile, localeInfo)) {
                localeFound = true;
                localeFoundString = localeInfo;
            }
        }

        //
        // Search for customizations of the locale language
        //
        // Example: "/xml/language_de_custom.xml"
        //
        if(localeFound && localeFoundString != null) {
            // Search for customizations of the locale language
            String customLocaleLanguageFile = XML_LANGUAGE_FILE + "_" + localeFoundString +
                    "_custom.xml";
            includeLanguageFile(customLocaleLanguageFile, localeFoundString);
        }
    }

    /**
     * Include the given language file, if available, into the languageHash.
     *
     * @param languageFile The language file name
     * @param locale the language locale (e.g. en_US, de)
     *
     * @param c the Class object used to retrieve resource files.
     * 
     * @return True if the given language file exists and has been filled into
     * the language Hashtable (languageHash)
     */
    private boolean includeLanguageFile(String languageFile,
            String locale) throws Exception {


        Class c = this.getClass();
        InputStream languageStream = c.getResourceAsStream(languageFile);
        if(languageStream != null) {
            Log.info(TAG_LOG, "Found language file: " + languageFile);
            try {
                if(DEFAULT_LANGUAGE_FILE.equals(languageFile)) {
                    if(languageHash.isEmpty()) {
                        // Default language must be initialized
                        languageHash = readLanguageStream(languageStream, locale);
                        Log.info(TAG_LOG, "Reading default language file: " +
                                languageFile + " successfully");
                        return true;
                    } else {
                        // The default language has been already loaded
                        Log.info("Default language already read.");
                        return false;
                    }
                }
                Hashtable language_ht = readLanguageStream(languageStream, locale);
                Log.info(TAG_LOG, "Reading language file: " + languageFile +
                        " successfully");

                // Merge with default language
                Enumeration keys = language_ht.keys();
                while(keys.hasMoreElements()) {
                    String key = (String)keys.nextElement();
                    languageHash.put(key, (String)language_ht.get(key));
                }
                return true;
            } catch (final Throwable e) {
                e.printStackTrace();
                Log.error(TAG_LOG, "Error reading language file: " + languageFile +
                        " Exception: " + e.toString());
                throw new Exception("Error reading language file: " + languageFile);
            }
        } else {
            Log.debug(TAG_LOG, "Language file not found: " + languageFile);
            return false;
        }
    }

    /**
     * Read the given InputStream and return the corresponding language
     * Hashtable.
     * 
     * @param stream
     * @param locale
     * @return
     * @throws Throwable
     */
    private Hashtable readLanguageStream(InputStream stream, String locale)
            throws Throwable {
        String language = read(stream);
        return new LanguageXMLParser().parseData(locale, language);
    }

        /**
     * Reads the content of the given input stream.
     * 
     * @param is the input stream
     */
    private String read(InputStream is) throws IOException {
        final StringBuffer sb = new StringBuffer();
        try {
            final byte[] buf = new byte[1024];
            int nbyte = -1;
            while ((nbyte = is.read(buf)) >= 0) {
                sb.append(new String(buf, 0, nbyte));
            }
        } finally {
            is.close();
        }
        return sb.toString();
    }

}
