/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;


import java.util.Hashtable;

import net.rim.device.api.util.Persistable;

import com.funambol.util.Log;

public class ConfigData extends Hashtable implements Persistable
{
    /**
     * Get the value 'key' from config data and return it if valid. Otherwise,
     * defval is returned
     */
    public boolean getValue(String key, boolean defval)
    {
        final String v = (String) get(key);
        if (v != null) {
            return "true".equals(v) ? true : false;
        } else {
            Log.error("Parameter not found: " + key);
            return defval;
        }
    }
    
    /**
     * Get the value 'key' from config data and return it if valid. Otherwise,
     * defval is returned
     */
    public int getValue(String key, int defval)
    {
        final String v = (String) get(key);
        if (v != null) {
            return Integer.parseInt(v);
        } else {
            Log.error("Parameter not found: " + key);
            return defval;
        }
    }

    /**
     * Get the value 'key' from config data and return it if valid. Otherwise,
     * defval is returned
     */
    public long getValue(String key, long defval)
    {
        final String v = (String) get(key);
        if (v != null) {
            return Long.parseLong(v);
        } else {
            Log.error("Parameter not found: " + key);
            return defval;
        }
    }
 
    
    /**
     * Get the value 'key' from config data and return it if valid. Otherwise,
     * defval is returned
     */
    public String getValue(String key, String defval)
    {
        final String v = (String) get(key);
        if (v != null) {
            return v;
        } else {
            Log.error("Parameter not found: " + key);
            return defval;
        }
    }

    public byte[] getValue(String key, byte[] value)
    {
        try {
            return (byte[])get(key);
        } catch(Exception e) {
        }
        return null;
    }
        
    /**
     * Set the value 'key' in the config data
     */   
    public void setValue(String key, boolean val)
    {
        put(key, val ? "true" : "false");
    }
    
    /**
     * Set the value 'key' in the config data
     */
    public void setValue(String key, int val)
    {
        put(key, new Integer(val).toString().trim());
    }

    /**
     * Set the value 'key' in the config data
     */
    public void setValue(String key, long val)
    {
        put(key, new Long(val).toString().trim());
    }
 
        
    /**
     * Set the value 'key' in the config data
     */
    public void setValue(String key, String val)
    {
        put(key, val);
    }
    
    /**
     * Set the value 'key' in the config data
     */
    public void setValue(String key, byte[] value)
    {
        put (key,value);
    }
}

