/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Date;
import java.util.Hashtable;
import java.util.Enumeration;

import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.DataOutputStream;
import java.io.DataInputStream;

import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.controller.UpdaterManager;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.DeviceConfig;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;
import com.funambol.util.AppProperties;
import com.funambol.syncml.protocol.SyncML;

/**
 * Configuration class for configuration details
 */
public class Configuration {

    private static final String TAG_LOG = "Configuration";

    // ------------------------------------------------------------ Constants

    /**
     * These constants specify the sync mode in one of two possible ways:
     * 1) manual
     * 2) scheduled
     */
    public static final int    SYNC_MODE_MANUAL                = 0;
    public static final int    SYNC_MODE_SCHEDULED             = SYNC_MODE_MANUAL + 1;
    private boolean     c2sPush;

    public static final int    CONF_OK                         = 0;
    public static final int    CONF_NOTSET                     = -1;
    public static final int    CONF_INVALID                    = -2;

    private static final String CONF_KEY_VERSION                = "VERSION";
    private static final String CONF_KEY_LOG_LEVEL              = "LOG_LEVEL";
    private static final String CONF_KEY_SYNC_URL               = "SYNC_URL";
    private static final String CONF_KEY_USERNAME               = "USERNAME";
    private static final String CONF_KEY_PASSWORD               = "PASSWORD";
    private static final String CONF_KEY_CRED_CHECK_PENDING     = "CRED_CHECK_PENDING";
    private static final String CONF_KEY_CRED_CHECK_REMEMBER    = "CRED_CHECK_REMEMBER";
    private static final String CONF_KEY_SYNC_SOURCE            = "SYNC_SOURCE";
    private static final String CONF_KEY_ACTIVE_SYNC_SOURCE     = "SYNC_SOURCE_ACTIVE";
    private static final String CONF_KEY_SYNC_URI               = "SYNC_SOURCE_URI";
    private static final String CONF_KEY_SOURCE_FULL            = "SYNC_SOURCE_FULL";
    private static final String CONF_KEY_SOURCE_SYNCED          = "SYNC_SOURCE_SYNCED";
    private static final String CONF_KEY_POLL_FLAG              = "POLL_PIM";
    private static final String CONF_KEY_POLL_TIME              = "POLL_PIM_TIME";
    private static final String CONF_KEY_POLL_TIMESTAMP         = "POLL_PIM_TIMESTAMP";
    private static final String CONF_KEY_SYNC_RANGE_PAST        = "RANGE_PAST";
    private static final String CONF_KEY_SYNC_RANGE_FUTURE      = "RANGE_FUTURE";
    private static final String CONF_KEY_SYNC_MODE              = "SYNC_MODE";
    private final String CONF_KEY_C2S_PUSH               = "C2S_PUSH";
    private static final String CONF_KEY_SYNC_TYPE              = "SYNC_TYPE";
    private static final String CONF_KEY_BLOCK_INCOMING_INVITES = "BLOCK_INVITES";
    private static final String CONF_KEY_LAST_SYNC_STATUS       = "LAST_SYNC_STATUS";
    private static final String CONF_KEY_FORCE_SERVER_CAPS_REQ  = "FORCE_SERVER_CAPS_REQ";
    private static final String CONF_KEY_SOURCE_UPLOAD_VIA_HTTP = "UPLOAD_VIA_HTTP";
    private static final String FIRST_RUN_TS_KEY                = "FIRST_RUN_TS";
    private static final String PICTURES_SYNC_OLD               = "PICTURES_SYNC_OLD";
    private static final String PICTURES_SYNC_DIR               = "PICTURES_DIR";
    private static final String CONF_CLIENT_NONCE               = "CLIENT_NONCE";
    private static final String CONF_LAST_AUTH_METHOD           = "LAST_AUTH_METHOD";

    private static final String UPDATE_URL = "UPDATE_URL";
    private static final String UPDATE_TYPE = "UPDATE_TYPE";
    private static final String AVAILABLE_VERSION ="AVAILABLE_VERSION";
    private static final String LAST_UPDATE_CHECK = "LAST_UPDATE_CHECK";
    private static final String CHECK_INTERVAL   = "CHECK_INTERVAL";
    private static final String REMINDER_INTERVAL = "REMINDER_INTERVAL";
    private static final String LAST_REMINDER = "LAST_REMINDER";
    private static final String SKIP_UPDATE = "SKIP_UPDATE";
    private final String ACTIVATION_DATE = "ACTIVATION_DATE";
    private static final String CONF_FILE_SYSTEM_USN            = "FILE_SYSTEM_USN";
    private static final String MAXMSGSIZE                      = "16000";
    private static final String CONFIG_VERSION                  = "7";
    private static String       OLD_CONFIG_VERSION              = "6";
    private static String       PROTOCOL;
    private static String       RETRY;

    // These constants are only needed for configuration migration
    private static final String CONF_KEY_SYNC_CONTACT_FLAG       = "SYNC_CONTACT";
    private static final String CONF_KEY_SYNC_CONTACT_URI        = "CONTACT_URI";
    private static final String CONF_KEY_SYNC_CAL_FLAG           = "SYNC_CAL";
    private static final String CONF_KEY_SYNC_CAL_URI            = "CAL_URI";
    private static final String CONF_KEY_SYNC_NOTE_FLAG          = "SYNC_NOTE";
    private static final String CONF_KEY_SYNC_NOTE_URI           = "NOTE_URI";
    private static final String CONF_KEY_SYNC_TASK_FLAG          = "SYNC_TASK";
    private static final String CONF_KEY_SYNC_TASK_URI           = "TASK_URI";

    private static int          logLevel                        = 0;              // disabled
    private static String       syncUrl;
    private static String       username;
    private static String       password;
    private static boolean      credentialsCheckPending         = true;
    private static boolean      credentialsCheckRemember        = false;
    private static int          syncMode                        = 0;
    private static int          pollingInterval;
    private static int          rangePast;
    private static int          rangeFuture;
    private static int          syncType;
    private static boolean      blockIncomingInvites            = false;
    private static boolean      initialized                     = false;
    private static boolean      loaded                          = false;
    private static Hashtable    appSourceConfigs                = new Hashtable();
    private static String       picturesSyncDir                 = null;
    //update parameters
    private static String       downloadUrl                     = " ";;
    private static String       updateType                      = " ";
    private static String       availableVersion                = " ";;
    private static long         lastUpdateCheck                 = 0L;
    private static long         checkInterval                   = 24 * 60 * 60 * 1000; // 24hours in millisecs
    private static long         reminderInterval;
    private static long         lastReminder;
    private static boolean      skip                            = false;
    private long                activationDate                  = 0;
    
    private static long         lastUSN                         = 0;
    private static long         pollingTimestamp                = 0;
    private static long         firstRunTimestamp               = 0;
    private static boolean      picturesSyncOld                 = false;
    // Authentication information
    // The base64 client nonce
    private static String      clientNonce                      = null;

    // These values don't need to be saved/restored
    private static int          origLogLevel                    = 0;
    private static boolean      dirtyAccount                    = false;
    private static boolean      dirtyUpdater                    = false;
    private static boolean      dirtySyncMode                   = false;
    private static boolean      dirtyMisc                       = false;
    private static Hashtable    dirtySource                     = new Hashtable();
    private static boolean      hidePhotoSource                 = false;
    private static boolean      forceServerCapsRequest          = false;
    
    private static Configuration instance;

    public Configuration (){

    }

    public static Configuration getInstance() {
        if (instance == null) {
            instance = new Configuration();
        }
        return instance;
    }

    private static void copyDefaults() {

        Log.info(TAG_LOG, "copyDefaults");

        // Jad has precedence over Customization as it allows a per user
        // settings
        AppProperties properties = new AppProperties(null);
        syncUrl  = properties.get(AppProperties.URL_ATTR);
        username = properties.get(AppProperties.USER_ATTR);
        password = properties.get(AppProperties.PASSWORD_ATTR);
        String checkIntervalprop = properties.get(AppProperties.AUTOUPDATE_CHECK_INTERVAL);
        String reminderIntervalprop = properties.get(AppProperties.AUTOUPDATE_REMINDER_INTERVAL);

        if (syncUrl == null) {
            syncUrl  = Customization.getInstance().getServerUriDefault();
        }

        if (username == null) {
            username = Customization.getInstance().getUserDefault();
        }

        if (password == null) {
            password = Customization.getInstance().getPasswordDefault();
        }
        
        if(checkIntervalprop == null){
            checkInterval = Customization.getInstance().getCheckUpdtIntervalDefault();
        }else{
            checkInterval = Long.parseLong(checkIntervalprop);
        }
        
        if(reminderIntervalprop == null){
            reminderInterval = Customization.getInstance().getReminderUpdtIntervalDefault();
        }else{
            reminderInterval = Long.parseLong(reminderIntervalprop);
        }

        credentialsCheckPending  = true;
        credentialsCheckRemember = false;

        syncMode = Customization.getInstance().getDefaultSyncMode();
        pollingInterval = Customization.getInstance().getDefaultPollingInterval();
        getInstance().setC2SPush(false);

        pollingTimestamp = 0;

        rangePast = Customization.getInstance().getDefaultRangePast();
        rangeFuture = Customization.getInstance().getDefaultRangeFuture();

        syncType = Customization.getInstance().getDefaultSyncType();
        blockIncomingInvites = Customization.getInstance().getDefaultBlockInvites();
        
        picturesSyncDir    = Customization.getInstance().getPicturesDir();

        Date nowDate = new Date();
        firstRunTimestamp = nowDate.getTime();

        lastUSN = 0;

        forceServerCapsRequest = false;
        clientNonce = "";
    }

    /**
     * This must be called once at the beginning of the application.
     **/
    public static void init(Class c) throws Exception {
        if (!initialized) {
            StaticDataHelper.init(c);
            copyDefaults();
        }
        initialized = true;
    }

    // ------------------------------------------------------------ Public
    /**
     * Load the current config from the persistent store.
     * 
     * @return: <li><b>CONF_OK</b>: if all the data were present in the store.
     *          <li><b>CONF_NOTSET</b>: if the store is not present.
     *          Configuration remains untouched. <li><b>CONF_INVALID</b>: if the
     *          store does not contain valid data. Configuration is reverted to
     *          default. Note: if a parameter is not present in the store, the
     *          current value is kept for it.
     */
    public static int load() {

        if (!loaded) {

            Log.info(TAG_LOG, "Loading Configuration data from persistent storage");
            final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
            try {
                final ConfigData contents = (ConfigData) pobj.getContents();
                if (contents == null) {
                    Log.error(TAG_LOG, "Configuration not set, using current values.");
                    copyDefaults();
                    return CONF_NOTSET;
                }
                if (contents.get(CONF_KEY_VERSION) == null) {
                    Log.error(TAG_LOG, "Configuration invalid: reverting to default.");
                    PersistentStore.destroyPersistentObject(Constants.CONFIG_KEY);
                    copyDefaults();
                    return CONF_INVALID;
                }

                OLD_CONFIG_VERSION = contents.getValue(Configuration.CONF_KEY_VERSION,
                                                       CONFIG_VERSION);

                // This must be loaded before migraring, because a migration may
                // change its value
                forceServerCapsRequest = contents.getValue(CONF_KEY_FORCE_SERVER_CAPS_REQ, false);

                // Migrate the configuration if needed
                hidePhotoSource = false;
                migrateConfig(contents);

                syncUrl = contents.getValue(CONF_KEY_SYNC_URL, syncUrl);
                if (Customization.getInstance().stripPortFromSyncAddress()) {
                    syncUrl = StringUtil.removePortFromUrl(syncUrl, "http");
                }

                username = contents.getValue(CONF_KEY_USERNAME, username);
                password = contents.getValue(CONF_KEY_PASSWORD, password);

                credentialsCheckPending  = contents.getValue(CONF_KEY_CRED_CHECK_PENDING,
                                                             credentialsCheckPending);
                credentialsCheckRemember = contents.getValue(CONF_KEY_CRED_CHECK_REMEMBER,
                                                             credentialsCheckRemember);

                Log.debug(TAG_LOG, "Init Registered Sources");
                initSource(contents);
                
                syncMode = contents.getValue(Configuration.CONF_KEY_SYNC_MODE, syncMode);


                pollingInterval = contents.getValue(Configuration.CONF_KEY_POLL_TIME,
                        pollingInterval);
                pollingTimestamp = contents.getValue(Configuration.CONF_KEY_POLL_TIMESTAMP,
                        pollingTimestamp);

                getInstance().setC2SPush(contents.getValue(getInstance().CONF_KEY_C2S_PUSH,
                        getInstance().getC2SPush()));

                rangePast = contents.getValue(Configuration.CONF_KEY_SYNC_RANGE_PAST, rangePast);
                rangeFuture = contents.getValue(Configuration.CONF_KEY_SYNC_RANGE_FUTURE,
                        rangeFuture);

                syncType = contents.getValue(Configuration.CONF_KEY_SYNC_TYPE, syncType);

                logLevel = contents.getValue(Configuration.CONF_KEY_LOG_LEVEL, logLevel);

                blockIncomingInvites = contents.getValue(
                        Configuration.CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);

                picturesSyncDir = contents.getValue(Configuration.PICTURES_SYNC_DIR,
                                                    picturesSyncDir);
                if (picturesSyncDir == null) {
                    // This is for backward compatibility.
                    picturesSyncDir = Customization.getInstance().getPicturesDir();
                }

                lastUSN = contents.getValue(Configuration.CONF_FILE_SYSTEM_USN, 0);

                downloadUrl = contents.getValue(Configuration.UPDATE_URL, downloadUrl);
                updateType = contents.getValue(Configuration.UPDATE_TYPE, updateType);
                availableVersion = contents.getValue(Configuration.AVAILABLE_VERSION, availableVersion);
                lastUpdateCheck = contents.getValue(Configuration.LAST_UPDATE_CHECK, lastUpdateCheck);
                checkInterval = contents.getValue(Configuration.CHECK_INTERVAL, checkInterval);
                reminderInterval = contents.getValue(Configuration.REMINDER_INTERVAL, reminderInterval);
                lastReminder = contents.getValue(Configuration.LAST_REMINDER, lastReminder);
                skip = contents.getValue(Configuration.SKIP_UPDATE, skip);
                getInstance().setActivationDate(contents.getValue(getInstance().ACTIVATION_DATE,
                            getInstance().getActivationDate()));

                picturesSyncOld = contents.getValue(Configuration.PICTURES_SYNC_OLD, false);

                Date nowDate = new Date();
                long now = nowDate.getTime();
                firstRunTimestamp = contents.getValue(Configuration.FIRST_RUN_TS_KEY, now);
                clientNonce = contents.getValue(Configuration.CONF_CLIENT_NONCE, "");

                Log.debug(TAG_LOG, "Init Registered Sources");
                initSource(contents);

                // From now on, the config is always cached
                loaded = true;

                return CONF_OK;
            } catch (final ClassCastException cce) {
                Log.error(TAG_LOG, "ClassCastException - Configuration invalid: reverting to default.", cce);
                PersistentStore.destroyPersistentObject(Constants.CONFIG_KEY);
                copyDefaults();
                return CONF_INVALID;
            }
        }

        return CONF_OK;
    }

    /**
     * Save the current config to the persistent store and apply changes to the
     * client.
     */
    public static int save(Controller controller) {
        int ret = saveConfiguration();
        if (controller != null) {
            // Notify the controller on the config changes so that
            // proper actions can be taken
            if (dirtyAccount) {
                controller.reapplyAccountConfiguration();
                dirtyAccount = false;
            }
            if (dirtyUpdater) {
                controller.reapplyUpdaterConfiguration();
                dirtyUpdater = false;
            }
            if (dirtySource != null) {
                Enumeration sourcesId = dirtySource.keys();
                while(sourcesId.hasMoreElements()) {
                    Integer id = (Integer) sourcesId.nextElement();
                    AppSyncSource appSource = AppSyncSourceManager.getInstance().getSource(id.intValue());
                    controller.reapplySourceConfiguration(appSource);

                    dirtySource.remove(id);
                }
            }
            if (dirtySyncMode) {
                controller.reapplySyncModeConfiguration();
                dirtySyncMode = false;
            }
            if (dirtyMisc) {
                controller.reapplyMiscConfiguration();
                dirtyMisc = false;
            }
        }
        return ret;
    }

    /**
     * Saves the current config to the persistent store without applying it to
     * the client.
     */
    public static int saveConfiguration() {
        Log.info(TAG_LOG, "Saving Configuration data to persistent storage");
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null) {
            contents = new ConfigData();
        }

        // This field can be used to check which version of the sw
        // saved the data. If not present, it is assumed it's an
        // old version and the saved config is discarded.
        contents.setValue(Configuration.CONF_KEY_VERSION, CONFIG_VERSION);
        contents.setValue(Configuration.CONF_KEY_LOG_LEVEL, logLevel);

        contents.setValue(Configuration.CONF_KEY_SYNC_URL, syncUrl);
        contents.setValue(Configuration.CONF_KEY_USERNAME, username);
        contents.setValue(Configuration.CONF_KEY_PASSWORD, password);

        contents.setValue(CONF_KEY_CRED_CHECK_PENDING, credentialsCheckPending);

        contents.setValue(CONF_KEY_CRED_CHECK_REMEMBER, credentialsCheckRemember);

        Enumeration sources = Customization.getInstance().getActiveSources();
        while (sources.hasMoreElements()) {
            Integer sourceId = (Integer)sources.nextElement();
            int id = sourceId.intValue();
            String enabledKey         = Configuration.CONF_KEY_SYNC_SOURCE + "_" + id;
            String activeKey  = Configuration.CONF_KEY_ACTIVE_SYNC_SOURCE + "_" + id;
            String uriKey             = Configuration.CONF_KEY_SYNC_URI    + "_" + id;
            String deviceFullShownKey = Configuration.CONF_KEY_SOURCE_FULL + "_" + id;
            String syncedKey          = Configuration.CONF_KEY_SOURCE_SYNCED + "_" + id;
            String httpUploadKey      = Configuration.CONF_KEY_SOURCE_UPLOAD_VIA_HTTP + "_" + id;

            boolean enabled              = isSourceEnabled(id);
            boolean active               = isSourceActive(id);
            String  uri                  = getSourceUri(id);
            boolean deviceFullShown      = getDeviceFullShown(id);
            boolean synced               = isSourceSynced(id);
            boolean uploadContentViaHttp = isUploadViaHttp(id);

            contents.setValue(enabledKey, enabled);
            contents.setValue(activeKey, active);
            contents.setValue(uriKey, uri);
            contents.setValue(deviceFullShownKey, deviceFullShown);
            contents.setValue(syncedKey, synced);
            contents.setValue(httpUploadKey, uploadContentViaHttp);
        }

        contents.setValue(Configuration.CONF_KEY_SYNC_MODE, syncMode);
        contents.setValue(Configuration.CONF_KEY_POLL_TIME, pollingInterval);
        contents.setValue(Configuration.CONF_KEY_POLL_TIMESTAMP, pollingTimestamp);
        contents.setValue(getInstance().CONF_KEY_C2S_PUSH, getInstance().getC2SPush());

        contents.setValue(Configuration.CONF_KEY_SYNC_RANGE_PAST, rangePast);
        contents.setValue(Configuration.CONF_KEY_SYNC_RANGE_FUTURE, rangeFuture);

        contents.setValue(Configuration.CONF_KEY_SYNC_TYPE, syncType);

        contents.setValue(Configuration.CONF_KEY_BLOCK_INCOMING_INVITES, blockIncomingInvites);

        contents.setValue(Configuration.PICTURES_SYNC_DIR, picturesSyncDir);

        contents.setValue(Configuration.CONF_FILE_SYSTEM_USN, lastUSN);

        contents.setValue(Configuration.FIRST_RUN_TS_KEY, firstRunTimestamp);
        contents.setValue(Configuration.CONF_KEY_FORCE_SERVER_CAPS_REQ, forceServerCapsRequest);
        contents.setValue(Configuration.PICTURES_SYNC_OLD, picturesSyncOld);

        contents.setValue(Configuration.UPDATE_URL, downloadUrl);
        contents.setValue(Configuration.UPDATE_TYPE, updateType);
        contents.setValue(Configuration.AVAILABLE_VERSION, availableVersion);
        contents.setValue(Configuration.LAST_UPDATE_CHECK, lastUpdateCheck);
        contents.setValue(Configuration.CHECK_INTERVAL, checkInterval);
        contents.setValue(Configuration.REMINDER_INTERVAL, reminderInterval);
        contents.setValue(Configuration.LAST_REMINDER, lastReminder);
        contents.setValue(Configuration.SKIP_UPDATE, skip);
        contents.setValue(getInstance().ACTIVATION_DATE, getInstance().getActivationDate());

        if (clientNonce != null) {
            contents.setValue(Configuration.CONF_CLIENT_NONCE, clientNonce);
        }

        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }

    public static int append(String key, byte[] value) {
        Log.trace(TAG_LOG, "append " + key);
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null) {
            contents = new ConfigData();
        }
        contents.setValue(key, value);
        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }

    public static int append(String key, String value) {
        Log.trace(TAG_LOG, "append " + key);
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        ConfigData contents = (ConfigData) pobj.getContents();
        if (contents == null) {
            contents = new ConfigData();
        }
        contents.setValue(key, value);
        synchronized (pobj) {
            pobj.setContents(contents);
            pobj.commit();
        }
        return CONF_OK;
    }

    public static byte[] getValue(String key, byte[] value) {
        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        try {
            final ConfigData contents = (ConfigData) pobj.getContents();
            if (contents == null) {
                return null;
            }
            return contents.getValue(key, value);
        } catch (final ClassCastException e) {
        }
        return null;
    }

    public static int getLogLevel() {
        return logLevel;
    }

    public static void setLogLevel(int logLevel) {
        dirtyMisc = true;
        Configuration.logLevel = logLevel;
    }

    public static boolean getBlockIncomingInvites() {
        return blockIncomingInvites;
    }

    public static void setBlockIncomingInvites(boolean blockIncomingInvites) {
        dirtyMisc = true;
        Configuration.blockIncomingInvites = blockIncomingInvites;
    }

    public static String getUsername() {
        return username;
    }

    public static void setUsername(String username) {
        dirtyAccount = true;
        Configuration.username = username;
    }

    public static String getPassword() {
        return password;
    }

    public static void setPassword(String password) {
        dirtyAccount = true;
        Configuration.password = password;
    }

    public static String getSyncUrl() {
        return syncUrl;
    }

    public static void setSyncUrl(String syncUrl) {
        dirtyAccount = true;
        Configuration.syncUrl = syncUrl;
    }

    public static int getSyncMode() {
        return syncMode;
    }

    public static void setSyncMode(int modeIndex) {
        dirtySyncMode = true;
        syncMode = modeIndex;
    }

    public static int getPollingInterval() {
        return pollingInterval;
    }

    public static long getPollingTimestamp() {
        return pollingTimestamp;
    }

    public boolean getC2SPush(){
        return c2sPush;
    }

    public static long getLastUSN() {
        return lastUSN;
    }

    public static void setLastUSN(long usn) {
        dirtyMisc = true;
        lastUSN = usn;
    }
    
    public static String getPicturesDir() {
        return picturesSyncDir;
    }

    public static void setPicturesDir(String dir) {
        dirtySource.put(new Integer(AppSyncSourceManager.SYNC_FLAG_PHOTOS),
                        new Boolean(true));
        picturesSyncDir = dir;
    }

    public static boolean getPicturesSyncOld() {
        return picturesSyncOld;
    }

    public static void setPicturesSyncOld(boolean value) {
        dirtySource.put(new Integer(AppSyncSourceManager.SYNC_FLAG_PHOTOS),
                        new Boolean(true));
        picturesSyncOld = value;
    }

    public static boolean getCredentialsCheckPending() {
        return credentialsCheckPending;
    }

    public static void setCredentialsCheckPending(boolean value) {
        dirtyUpdater = true;
        credentialsCheckPending = value;
    }

    public static void setPollingInterval(int interval) {
        dirtySyncMode = true;
        pollingInterval = interval;
    }

    public static void setPollingTimestamp(long timestamp) {
        dirtySyncMode = true;
        pollingTimestamp = timestamp;
    }

    public void setC2SPush (boolean value){
        dirtySyncMode = true;
        c2sPush = value;
        
    }

    public static long getFirstRunTimestamp() {
        return firstRunTimestamp;
    }

    public static int getRangePast() {
        return rangePast;
    }

    public static void setRangePast(int range) {
        dirtyMisc = true;
        rangePast = range;
    }
    
    public static int getRangeFuture() {
        return rangeFuture;
    }

    public static void setRangeFuture(int range) {
        dirtyMisc = true;
        rangeFuture = range;
    }

    public static boolean isInitialized() {
        return initialized;
    }

    public static String getDownloadUrl() {
        return downloadUrl;
    }

    public static void setDownloadUrl(String url) {
        dirtyUpdater = true;
        downloadUrl = url;
    }

    public static String getUpdateType() {
        return updateType;
    }

    public static void setUpdateType(String type) {
        dirtyUpdater = true;
        updateType = type;
    }

    public static String getAvailableVersion() {
        return availableVersion;
    }

    public static void setAvailableVersion(String version) {
        dirtyUpdater = true;
        availableVersion = version;
    }

    public static long getLastUpdateCheck() {
        return lastUpdateCheck;
    }

    public static void setLastUpdateCheck(long when) {
        dirtyUpdater = true;
        lastUpdateCheck = when;
    }

    public static long getCheckInterval() {
        return checkInterval;
    }

    public static void setCheckInterval(long interval) {
        dirtyUpdater = true;
        checkInterval = interval;
    }

    public static long getReminderInterval() {
        return reminderInterval;
    }

    public static void setReminderInterval(long interval) {
        dirtyUpdater = true;
        reminderInterval = interval;
    }

    public static long getLastReminder() {
        return lastReminder;
    }

    public static void setLastReminder(long when) {
        dirtyUpdater = true;
        lastReminder = when;
    }

    public static boolean getSkip() {
        return skip;
    }

    public static void setSkip(boolean skip) {
        dirtyUpdater = true;
        Configuration.skip = skip;
    }

    public long getActivationDate(){
        return activationDate;
    }

    public void setActivationDate (long expDate){
        activationDate = expDate;
    }

    private static DeviceConfig devconf;

    public static int getSyncTypeAlertCode() {
        int syncType = 0;
        switch (Configuration.syncType) {
            case StaticDataHelper.SYNC_TYPE_ONE_WAY_TO_CLIENT:
                syncType = SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER;
                break;

            case StaticDataHelper.SYNC_TYPE_ONE_WAY_TO_SERVER:
                syncType = SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT;
                break;
            case StaticDataHelper.SYNC_TYPE_TWO_WAY:
            default:
                syncType = SyncML.ALERT_CODE_FAST;
                break;
        }
        return syncType;
    }

    public static void setSyncType(int type) {
        syncType = type;
    }

    public static int getSyncType() {
        return syncType;
    }

    public static void setSourceUri(int id, String uri) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            Log.error(TAG_LOG, "Cannot set source uri as there is no config for source: " + id);
        } else {
            sc.setUri(uri);
            dirtySource.put(new Integer(id), new Boolean(true));
        }
    }

    public static void setDeviceFullShown(int id, boolean shown) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            Log.error(TAG_LOG, "Cannot set source device full shown as there is no config for source: " + id);
        } else {
            sc.setDeviceFullShown(shown);
            dirtySource.put(new Integer(id), new Boolean(true));
        }
    }


    public static void setSourceEnabled(int id, boolean enabled) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            Log.error(TAG_LOG, "Cannot enable source as there is no config for source: " + id);
        } else {
            sc.setEnabled(enabled);
            dirtySource.put(new Integer(id), new Boolean(true));
        }
    }

    public static void setSourceSynced(int id, boolean synced) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            Log.error(TAG_LOG, "Cannot set synced as there is no config for source: " + id);
        } else {
            sc.setSynced(synced);
            dirtySource.put(new Integer(id), new Boolean(true));
        }
    }

    public static void setSourceActive(int id, boolean active) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            Log.error(TAG_LOG, "Cannot activate source as there is no config for source: " + id);
        } else {
            sc.setActive(active);
            dirtySource.put(new Integer(id), new Boolean(true));
        }
    }

    public static void setUploadViaHttp(int id, boolean http) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            Log.error(TAG_LOG, "Cannot set upload via http as there is no config for source: " + id);
        } else {
            sc.setUploadContentViaHttp(http);
            dirtySource.put(new Integer(id), new Boolean(true));
        }
    }

    public static void setClientNonce(String nextNonce) {
        Configuration.clientNonce = nextNonce;
    }

    public static SyncConfig getSyncConfig() {
        SyncConfig syncConfig = new SyncConfig();

        // TODO set before this runs
        syncConfig.syncUrl = Configuration.syncUrl;
        syncConfig.lastServerUrl = Configuration.syncUrl;
        syncConfig.userName = Configuration.username;
        syncConfig.password = Configuration.password;
        syncConfig.clientNonce = clientNonce;
        
        // Remember to update the blackberry synclet pattern (server side) when changing the user agent
        syncConfig.userAgent = "Funambol BlackBerry Plug-in " + BBVersion.VERSION;
        syncConfig.deviceConfig = getDeviceConfig();
        syncConfig.forceCookies = false;

        return syncConfig;
    }

    /**
     * Helper function to build the DevConf
     * 
     * @return A DeviceConfig
     */
    private static DeviceConfig getDeviceConfig() {
        if (devconf != null) {
            return devconf;
        }
        devconf = new DeviceConfig();
        devconf.man = DeviceInfo.getManufacturerName();
        devconf.mod = DeviceInfo.getDeviceName();
        devconf.swv = Constants.VER;
        devconf.hwv = "" + DeviceInfo.getPlatformVersion();
        devconf.devID = "fbb-" + DeviceInfo.getDeviceId();
        devconf.setMaxMsgSize(64 * 1024);
        devconf.loSupport = false;
        devconf.utc = true;
        devconf.nocSupport = true;
        return devconf;
    }

    /**
     * Saves a syncsource config to storage
     * 
     * @param storageKey
     * @param config
     */
    public static void saveSourceConfig(final String storageKey, SourceConfig config) {
        Log.debug(TAG_LOG, "Storing SourceConfig [" + config.getName() + "]");
        try {
            ByteArrayOutputStream buff = new ByteArrayOutputStream(512);
            DataOutputStream temp = new DataOutputStream(buff);
            config.serialize(temp);
            append(storageKey, buff.toByteArray());
            temp.close();
        } catch (final Exception e) {
            Log.error(TAG_LOG, "Exception while storing SourceConfig [" + config.getName() + "] ", e);
        }
    }

    public static boolean isSourceEnabled(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            return sc.getEnabled();
        }
        // We shall grab the default value from the customization here
        return Customization.getInstance().isSourceEnabledByDefault(id);
    }

    public static boolean isSourceSynced(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            return sc.getSynced();
        }
        return false;
    }


    public static boolean isSourceActive(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            return sc.getActive();
        }
        // We shall grab the default value from the customization here
        return Customization.getInstance().isSourceActive(id);

    }

    public static boolean isUploadViaHttp(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            return sc.getUploadContentViaHttp();
        }
        return true;
    }

    public static String getSourceUri(int id) {
        String uri = null;
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            uri = sc.getUri();
        }
        if (uri == null) {
            // We shall grab the default value from the customization here
            uri = Customization.getInstance().getDefaultSourceUri(id);
            // This should never happen, but just in case...
            if (uri == null) {
                uri = "undefined";
            }
        }
        return uri;
    }

    public static boolean getDeviceFullShown(int id) {
        boolean shown = false;
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            shown = sc.getDeviceFullShown();
        }
        return shown;
    }

    public static int loadLastSyncStatus(AppSyncSource appSource) {
        int id = appSource.getId();
        int status = appSource.getLastSyncStatus();
        String lastSyncStatusKey = CONF_KEY_LAST_SYNC_STATUS + "_" + id;

        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        try {
            final ConfigData contents = (ConfigData) pobj.getContents();
            if (contents != null) {
                status = contents.getValue(lastSyncStatusKey, status);
            }
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot load last sync status for " + appSource.getName(), e);
        }
        return status;
    }

    public static void saveLastSyncStatus(AppSyncSource appSource) {
        int id = appSource.getId();
        int status = appSource.getLastSyncStatus();
        String lastSyncStatusKey = CONF_KEY_LAST_SYNC_STATUS + "_" + id;

        final PersistentObject pobj = PersistentStore.getPersistentObject(Constants.CONFIG_KEY);
        try {
            final ConfigData contents = (ConfigData) pobj.getContents();
            if (contents != null) {
                contents.setValue(lastSyncStatusKey, status);
            }
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot save last sync status for " + appSource.getName(), e);
        }
    }

    private static void setSourceConfig(Integer id, boolean enabled, boolean active, String uri,
                                        boolean deviceFullShown, boolean sourceSynced,
                                        boolean uploadViaHttp)
    {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(id);
        if (sc == null) {
            sc = new AppSyncSourceConfig(id.intValue(), enabled, active, uri, deviceFullShown, sourceSynced);
            appSourceConfigs.put(id, sc);
        } else {
            sc.setEnabled(enabled);
            sc.setActive(active);
            sc.setUri(uri);
            sc.setDeviceFullShown(deviceFullShown);
            sc.setSynced(sourceSynced);
        }
        sc.setUploadContentViaHttp(uploadViaHttp);
    }


    public static void disableSource(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        // The source config has never been set, so we use the default uri
        if (sc == null) {
            String uri = getSourceUri(id);
            sc = new AppSyncSourceConfig(id, false, true, uri);
            Log.debug(TAG_LOG, "AppSyncSourceConfig");
            appSourceConfigs.put(new Integer(id), sc);
        }
        dirtySource.put(new Integer(id), new Boolean(true));
        sc.setEnabled(false);
    }

    public static void deactivateSource(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        // The source config has never been set, so we use the default uri
        if (sc == null) {
            String uri = getSourceUri(id);
            sc = new AppSyncSourceConfig(id, false, false, uri);
            Log.debug(TAG_LOG, "AppSyncSourceConfig");
            appSourceConfigs.put(new Integer(id), sc);
        }
        dirtySource.put(new Integer(id), new Boolean(true));
        sc.setActive(false);
    }


    public static int getSourceSyncType(int id) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc != null) {
            int syncType = sc.getSyncType();
            if (syncType != -1) {
                return syncType;
            }
        }
        return getSyncTypeAlertCode();
    }

    public static void setSourceSyncType(int id, int type) {
        AppSyncSourceConfig sc = (AppSyncSourceConfig) appSourceConfigs.get(new Integer(id));
        if (sc == null) {
            String uri = getSourceUri(id);
            sc = new AppSyncSourceConfig(id, isSourceEnabled(id), isSourceActive(id), uri);
            appSourceConfigs.put(new Integer(id), sc);
        }
        Log.trace(TAG_LOG, "sc="+sc);

        sc.setSyncType(type);
        dirtySource.put(new Integer(id), new Boolean(true));
    }

    private static void initSource(final ConfigData contents) {
        Enumeration sources = Customization.getInstance().getActiveSources();
        while (sources.hasMoreElements()) {
            Integer id = (Integer) sources.nextElement();
            String enabledKey = CONF_KEY_SYNC_SOURCE + "_" + id;
            String activeKey  = Configuration.CONF_KEY_ACTIVE_SYNC_SOURCE + "_" + id;
            String uriKey = CONF_KEY_SYNC_URI + "_" + id;
            String deviceFullKey = CONF_KEY_SOURCE_FULL + "_" + id;
            String sourceSyncedKey  = CONF_KEY_SOURCE_SYNCED + "_" + id;
            String httpUploadKey    = Configuration.CONF_KEY_SOURCE_UPLOAD_VIA_HTTP + "_" + id;

            boolean enabled = isSourceEnabled(id.intValue());
            boolean active  = isSourceActive(id.intValue());

            if (id.intValue() == AppSyncSourceManager.SYNC_FLAG_PHOTOS && hidePhotoSource) {
                active = false;
            }

            String uri = getSourceUri(id.intValue());
            boolean deviceFullShown = getDeviceFullShown(id.intValue());
            boolean sourceSynced = contents.getValue(sourceSyncedKey, false);
            boolean uploadContentViaHttp = isUploadViaHttp(id.intValue());
            if (contents != null) {
                enabled = contents.getValue(enabledKey, enabled);
                active  = contents.getValue(activeKey, active);
                uri = contents.getValue(uriKey, uri);
                deviceFullShown = contents.getValue(deviceFullKey, deviceFullShown);
                uploadContentViaHttp = contents.getValue(httpUploadKey, uploadContentViaHttp);
            }
            // Set the source config
            setSourceConfig(id, enabled, active, uri, deviceFullShown, sourceSynced, uploadContentViaHttp);
        }
    }

    public static void setTempLogLevel(int tempLogLevel) {
        origLogLevel = logLevel;
        logLevel = tempLogLevel;
    }

    public static void restoreLogLevel() {
        logLevel = origLogLevel;
    }

    public static boolean getForceServerCapsRequest() {
        return forceServerCapsRequest;
    }

    public static void setForceServerCapsRequest(boolean value) {
        forceServerCapsRequest = value;
    }


    /**
     * This method migrates old configuration to the new one
     * Currently we support upgrading from 7.0, 7.1, 7.2, 8.0 to 8.2
     */
    private static void migrateConfig(ConfigData contents) {
        Log.debug(TAG_LOG, "migrate config: " + CONFIG_VERSION + " -- " + OLD_CONFIG_VERSION);

        String currentVersion = OLD_CONFIG_VERSION;

        if (currentVersion.equals("20060721") || currentVersion.equals("1")) {
            migrateConfigFrom1To2(contents);
            currentVersion = "2";
        }

        if (currentVersion.equals("2")) {
            migrateConfigFrom2To3(contents);
            currentVersion = "3";
        }

        if (currentVersion.equals("3")) {
            migrateConfigFrom3To4(contents);
            currentVersion = "4";
        }

        if (currentVersion.equals("4")) {
            migrateConfigFrom4To5(contents);
            currentVersion = "5";
        }

        if (currentVersion.equals("5")) {
            migrateConfigFrom5To6(contents);
            currentVersion = "6";
        }

        if (currentVersion.equals("6")) {
            migrateConfigFrom6To7(contents);
            currentVersion = "7";
        }
    }

    /**
     * Migration from 7.0 or 7.1 to 7.2
     */
    private static void migrateConfigFrom1To2(ConfigData contents) {

        Log.info(TAG_LOG, "Migrating from 1 to 2");

        // This is an old version, upgrading is necessary
        boolean syncContact  = true;
        boolean syncCalendar = true;
        boolean syncTask     = true;
        boolean syncNote     = true;
        String contactSourceUri = Customization.getInstance().getDefaultContactUri(); //"card";
        String calendarSourceUri = Customization.getInstance().getDefaultCalendarUri(); //"scal";
        String taskSourceUri = Customization.getInstance().getDefaultTaskUri(); //"staks";
        String noteSourceUri = Customization.getInstance().getDefaultNoteUri(); //"snote";
        
        String oldContactSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_CONTACT_URI,
                                     contactSourceUri);

        syncContact = contents.getValue(Configuration.CONF_KEY_SYNC_CONTACT_FLAG,
                                        syncContact);
        contactSourceUri = Customization.getInstance().getDefaultSourceUri(AppSyncSourceManager.SYNC_FLAG_CONTACTS);
        
        syncCalendar = contents.getValue(Configuration.CONF_KEY_SYNC_CAL_FLAG,
                                         syncCalendar);
        calendarSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_CAL_URI,
                                              calendarSourceUri);
        
        syncTask = contents.getValue(Configuration.CONF_KEY_SYNC_TASK_FLAG, syncTask);
        taskSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_TASK_URI,
                                          taskSourceUri);
        
        syncNote = contents.getValue(Configuration.CONF_KEY_SYNC_NOTE_FLAG, syncNote);
        noteSourceUri = contents.getValue(Configuration.CONF_KEY_SYNC_NOTE_URI,
                                          noteSourceUri);
        
        if(oldContactSourceUri.indexOf("scard")!= -1){
            contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                                 contactSourceUri);
        }else{
            contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                                 oldContactSourceUri);
        }
        contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                          contactSourceUri);

        contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                          syncCalendar);
        contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                          calendarSourceUri);

        contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_TASKS,
                          syncTask);
        contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_TASKS,
                          taskSourceUri);

        contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_NOTES,
                          syncNote);
        contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_NOTES,
                          noteSourceUri);

        /*
         * Check if upgraded application from an old version with no support for
         * email attributes.
         */
        AppSyncSource appSource = AppSyncSourceManager.getInstance().getSource(AppSyncSourceManager.SYNC_FLAG_CONTACTS);
        SyncSource ss = appSource.getSyncSource();
        SourceConfig config = ss.getConfig();

        config.setRemoteUri(contactSourceUri);
        config.setType(Customization.getInstance().getContactType());
        // Force a slow sync on first run
        config.setLastAnchor(0);
    }

    /**
     * This one step of the migration from 7.2 to 8.0 (account settings conf)
     */
    private static void migrateConfigFrom2To3(ConfigData contents) {

        Log.info(TAG_LOG, "Migrating from 2 to 3");
        // Credentials check parameters need to be migrated. If the upgrade
        // happens when user/passwd are already set, then we do not show the
        // account screen to the user
        String syncUrl  = contents.getValue(CONF_KEY_SYNC_URL, "");
        String username = contents.getValue(CONF_KEY_USERNAME, "");
        String password = contents.getValue(CONF_KEY_PASSWORD, "");

        if (!StringUtil.isNullOrEmpty(syncUrl) &&
            !StringUtil.isNullOrEmpty(username) &&
            !StringUtil.isNullOrEmpty(password)) {

            // The application was installed and all parameters were set,
            // so we do not show the account screen to the user
            contents.setValue(CONF_KEY_CRED_CHECK_PENDING, false);
        }
    }

    /**
     * This one step of the migration from 7.2 to 8.0
     */
    private static void migrateConfigFrom3To4(ConfigData contents) {

        Log.info(TAG_LOG, "Migrating from 3 to 4");

        boolean enablePolling = false;
        
        enablePolling = contents.getValue(Configuration.CONF_KEY_POLL_FLAG,
                                             enablePolling);

        if (enablePolling) {
            contents.setValue(Configuration.CONF_KEY_SYNC_MODE,
                                Configuration.SYNC_MODE_SCHEDULED);
        } else {
            contents.setValue(Configuration.CONF_KEY_SYNC_MODE,
                                Configuration.SYNC_MODE_MANUAL);
        }

        // Now migrate the polling interval as one item (6 hours)
        // value in minutes now
        int mins = 30;
        mins = contents.getValue(Configuration.CONF_KEY_POLL_TIME, mins);
        if (mins == 360) {
            mins = 240;
        }
        contents.setValue(Configuration.CONF_KEY_POLL_TIME, mins); 

        // The USN journal property was not present in previous version
        contents.setValue(Configuration.CONF_FILE_SYSTEM_USN, 0);

        // The pictures uri must be set and the pictures are enabled
        String uri = Customization.getInstance().getDefaultSourceUri(AppSyncSourceManager.SYNC_FLAG_PHOTOS);
        contents.setValue(CONF_KEY_SYNC_SOURCE + "_" + AppSyncSourceManager.SYNC_FLAG_PHOTOS,
                true);
        contents.setValue(CONF_KEY_SYNC_URI    + "_" + AppSyncSourceManager.SYNC_FLAG_PHOTOS,
                uri);
    }

    /**
     * This one step of the migration from 8.0 to 8.2
     */
    private static void migrateConfigFrom4To5(ConfigData contents) {

        Log.info(TAG_LOG, "Migrating from 4 to 5");

        // In version 5 we have introduced the pim poller timestamp which
        // indicates the specific timestamp on which a scheduled sync shall
        // be executed.
        // Set default value to 0.
        contents.setValue(Configuration.CONF_KEY_POLL_TIMESTAMP, 0);
    }

    /**
     * Migration from 8.2 to 8.5
     */
    private static void migrateConfigFrom5To6(ConfigData contents) {
        Log.info(TAG_LOG, "Migrating from 5 to 6");

        // In this version we added support for pictures sync. Since we don't
        // know if the server we are currently using supports pics sync, we
        // initially disable the source and prepare for caps to be asked.
        hidePhotoSource = true;
        setForceServerCapsRequest(true);
    }

    /**
     * Migration from 8.5 to 8.7 (Diablo to Gallardo)
     */
    private static void migrateConfigFrom6To7(ConfigData contents) {
        Log.info(TAG_LOG, "Migrating from 6 to 7");

        // In this version we added support for http based upload. Since we don't
        // know if the server we are currently using supports it, we
        // prepare to ask for caps.
        setForceServerCapsRequest(true);

        if (Customization.getInstance().getCalendarType().equals(AppSyncSourceManager.CALENDAR_TYPE_VCALENDAR)) {

            // We switched from SIF-E, SIF-T to vCalendar, but if the user does an
            // upgrade we must keep the old remote uri (scal and stask to avoid
            // slow sync on the server)
            String uriKey = CONF_KEY_SYNC_URI + "_" + AppSyncSourceManager.SYNC_FLAG_CALENDAR;
            String uri = contents.getValue(uriKey, (String)null);
            if (uri == null) {
                // There is not a specific value for the URI, we shall force the
                // old default to prevent slow sync
                Log.info(TAG_LOG, "Forcing scal as remote URI during upgrade");
                contents.setValue(uriKey, "scal");
            }
            // Change the configuration of the source
            try {
                String configKey = AppSyncSourceManager.CONFIG_KEY_SYNC_CALENDAR;
                Log.debug(TAG_LOG, "Reading config from store " + configKey);

                byte[] byte_array = Configuration.getValue(configKey, new byte[0]);
                if (byte_array != null) {
                    SourceConfig config = new SourceConfig();
                    Log.debug(TAG_LOG, "Data Found");
                    DataInputStream temp = new DataInputStream(new ByteArrayInputStream(byte_array));
                    config.deserialize(temp);
                    temp.close();
                    // Now change the mime type and the encoding
                    config.setType(AppSyncSourceManager.CALENDAR_TYPE_VCALENDAR);
                    config.setEncoding(SyncSource.ENCODING_NONE);
                    saveSourceConfig(configKey, config);
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Exception while migrating SourceConfig for event source", e);
            }
        }

        if (Customization.getInstance().getTaskType().equals(AppSyncSourceManager.TASK_TYPE_VCALENDAR)) {
            String uriKey = CONF_KEY_SYNC_URI + "_" + AppSyncSourceManager.SYNC_FLAG_TASKS;
            String uri = contents.getValue(uriKey, (String)null);
            if (uri == null) {
                // There is not a specific value for the URI, we shall force the
                // old default to prevent slow sync
                Log.info(TAG_LOG, "Forcing stask as remote URI during upgrade");
                contents.setValue(uriKey, "stask");
            }
            // Change the configuration of the source
            try {
                String configKey = AppSyncSourceManager.CONFIG_KEY_SYNC_TASK;
                Log.debug(TAG_LOG, "Reading config from store " + configKey);

                byte[] byte_array = Configuration.getValue(configKey, new byte[0]);
                if (byte_array != null) {
                    SourceConfig config = new SourceConfig();
                    Log.debug(TAG_LOG, "Data Found");
                    DataInputStream temp = new DataInputStream(new ByteArrayInputStream(byte_array));
                    config.deserialize(temp);
                    temp.close();
                    // Now change the mime type and the encoding
                    config.setType(AppSyncSourceManager.TASK_TYPE_VCALENDAR);
                    config.setEncoding(SyncSource.ENCODING_NONE);
                    saveSourceConfig(configKey, config);
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Exception while migrating SourceConfig for task source", e);
            }
        }
    }
}
