/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Hashtable;
import java.util.Enumeration;

import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Color;

public class Customization {

    // Application fullname
    private final String   APPLICATION_FULLNAME    = "Funambol BlackBerry Sync";
    private final String   APPLICATION_TITLE       = "Funambol BlackBerry Sync";

    // Settings customization
    private final String   SERVER_URI              = "http://my.funambol.com/sync";
    private final String   USERNAME                = "";
    private final String   PASSWORD                = "";

    // About customization
    private final String   ABOUT_COPYRIGHT_DEFAULT = "Copyright (C) 2010 Funambol, Inc.";
    private final String   ABOUT_SITE_DEFAULT      = "www.funambol.com";
    
    // The application preferred font
    private final String   PREFERRED_FONT          = "Verdana";
    private final boolean  USE_DEFAULT_FONT        = true;
    
    // Icons/images customization
    private final String   IMAGE_LOGO              = "icon_logo_64x64.png";
    private final String   IMAGE_CONTACTS          = "contacts.png";
    private final String   IMAGE_CALENDAR          = "calendar.png";
    private final String   IMAGE_TASKS             = "tasks.png";
    private final String   IMAGE_NOTES             = "notes.png";
    private final String   IMAGE_PHOTOS            = "photos.png";
    private final String   IMAGE_CONTACTS_DISABLED = "contacts_grey.png";
    private final String   IMAGE_CALENDAR_DISABLED = "calendar_grey.png";
    private final String   IMAGE_TASKS_DISABLED    = "tasks_grey.png";
    private final String   IMAGE_NOTES_DISABLED    = "notes_grey.png";
    private final String   IMAGE_PHOTOS_DISABLED   = "photos_grey.png";
    private final String   IMAGE_ALL               = "sync_logo.png";
    private final String   IMAGE_ALL_BG            = "bg_button_dark.png";
    private final String   IMAGE_ALL_BG_HIGHLIGHT  = "bg_button_darkblue.png";
    private final String   IMAGE_BG                = "bg_button_light.png";
    private final String   IMAGE_BG_HIGHLIGHT      = "bg_button_blue.png";
    private final String   IMAGE_ERROR             = "alert.png";
    private final String   IMAGE_OK                = "complete.png";
    private final String   IMAGE_CANCEL            = "cancel.png";
    private final String   IMAGE_WARNING           = "cancel.png";
    private final String   IMAGE_FOLDER            = "folder.png";

    // Events sync range customization
    private final boolean  EVENT_RANGE_LIMITED     = true;
    private final int      DEFAULT_RANGE_PAST      = 1;
    private final int      DEFAULT_RANGE_FUTURE    = 6;

    // Log customization
    protected final boolean  LOG_ENABLED                = true;
    protected final boolean  SEND_LOG_ENABLED           = false;
    protected final boolean  SEND_LOG_VIA_EMAIL_ENABLED = true;
    protected final boolean  LOG_IN_SETTINGS_SCREEN     = true;
    protected final String   LOG_EMAIL_ADDRESS          = "fbb-logs@funambol.com";
    protected final String   LOG_EMAIL_NAME             = "BlackBerry Logs";
    private final String     LOG_TITLE                  = APPLICATION_FULLNAME + " Log";
    private final String     LOG_FILE_NAME              = "synclog.txt";
    private final String     LOG_PATH                   = "file:///store/home/user";
    private final String     SEND_LOGS_URI              = "http://sync.emailsrvr.com/logs/postLogs.php";

    // Updater customization
    private final boolean  CHECK_FOR_UPDATE_ON_SYNC = false;
    private final boolean  ENABLE_UPDATER_MANAGER   = true;
    private final long     CHECK_UPDATE_INTERVAL   = (long)24*(long)60*(long)60*(long)1000; // 1 day in milliseconds
    private final long     REMINDER_UPDATE_INTERVAL= (long)2 *(long)60*(long)60*(long)1000; // 2 hours in seconds

    private final int      FONT_SIZE               = 14;
    private final int      FONT_STYLE              = Font.PLAIN;
    // A null value for the req uri implies using the default URI
    private final String   UPDATER_REQUEST_URI     = null;

    // Set enabled sources
    private final boolean  CONTACTS_SYNC_ENABLED   = true;
    private final boolean  CALENDARS_SYNC_ENABLED  = true;
    private final boolean  TASKS_SYNC_ENABLED      = true;
    private final boolean  NOTES_SYNC_ENABLED      = true;
    private final boolean  PICTURES_SYNC_ENABLED   = true;
    private final boolean  MAILS_SYNC_ENABLED      = false;
    private final boolean  CONF_SYNC_ENABLED       = true;

    // Set active sources
    private final boolean  CONTACTS_SYNC_ACTIVE    = true;
    private final boolean  CALENDARS_SYNC_ACTIVE   = true;
    private final boolean  TASKS_SYNC_ACTIVE       = true;
    private final boolean  NOTES_SYNC_ACTIVE       = true;
    private final boolean  PICTURES_SYNC_ACTIVE    = true;
    private final boolean  MAILS_SYNC_ACTIVE       = false;
    private final boolean  CONF_SYNC_ACTIVE        = true;

    // Sync sources customization
    private final String   CONTACTS_DEFAULT_URI    = "card";
    private final String   CALENDARS_DEFAULT_URI   = "event";
    private final String   TASKS_DEFAULT_URI       = "task";
    private final String   NOTES_DEFAULT_URI       = "snote";
    private final String   PICTURES_DEFAULT_URI    = "picture";
    private final String   MAILS_DEFAULT_URI       = "email";
    private final String   CONF_DEFAULT_URI        = "configuration";

    // This is the order in which sources will appear in the UI
    private final int SOURCES_ORDER[] = { AppSyncSourceManager.SYNC_FLAG_CONTACTS,
                                          AppSyncSourceManager.SYNC_FLAG_CALENDAR,
                                          AppSyncSourceManager.SYNC_FLAG_TASKS,
                                          AppSyncSourceManager.SYNC_FLAG_NOTES,
                                          AppSyncSourceManager.SYNC_FLAG_PHOTOS,
                                          AppSyncSourceManager.SYNC_FLAG_MAIL };
    
    // Max number of items per SyncML message sent to server during slow sync.
    // This is used to limit the number of items and to avoid client's timeouts
    // To be decreased with slow server/backend installations
    // Set to -1 to disable the limit
    private final int      MAX_ITEMS_PER_MESSAGE_IN_SLOW_SYNC = 20;

    // Set this parameter if a fix apn is to be used from your application
    private final String   FIXED_APN               = null;

    // Define the schedule choices
    private final int[] POLLING_PIM_INTERVAL_CHOICES = {
        5,
        15,
        30,
        60,
        120,
        240,
        480,
        720,
        1440
    };
    private final int  DEFAULT_POLLING_INTERVAL = POLLING_PIM_INTERVAL_CHOICES[1];

    // Specifies if the login screen must be presented when the application
    // starts and/or credentials are invalid
    private final boolean  LOGIN_SCREEN_REQUIRED        = true;

    // Specifies if the account screen shall check credentials on the server
    // when the user changes them (this is done via config synchronization)
    private final boolean  CHECK_CRED_IN_ACCOUNT_SCREEN = true;

    // Specifies if the port must be stripped out of the sync url
    private final boolean  STRIP_PORT_FROM_SYNC_ADDRESS = false;

    // Specifies if the sync url is editable in both the login and setting screens
    private final boolean  SYNC_URI_EDITABLE            = true;

    // Specifies if the attendees filtering is enabled or disabled
    private final boolean  INVITES_FILTERING            = false;

    // Specifies if slow sync needs to be confirmed by the user
    private final boolean  CONFIRM_SLOW_SYNC            = false;

    // Specifies if the user must be warned on large amount of deletes
    private final boolean  WARN_ON_DELETES              = false;

    // Specifies if photos must be synchronized in contact sync
    private final boolean  SYNC_CONTACT_PHOTO           = true;

    //Specifies if the C2S COP must be enabled in the client
    private final boolean ENABLE_C2S_PUSH               = true;

    // Sync items type
    private final String DEFAULT_CALENDAR_TYPE = AppSyncSourceManager.CALENDAR_TYPE_VCALENDAR;
    private final String DEFAULT_TASK_TYPE     = AppSyncSourceManager.TASK_TYPE_VCALENDAR;
    private final String DEFAULT_CONTACT_TYPE  = AppSyncSourceManager.CONTACT_TYPE_VCARD;

    // Shall we be smarter and check if a memory card is present? In such a
    // case this is where the BB stores pictures by default....
    private final String   DEFAULT_PICTURES_DIR        = "file:///store/home/user/pictures/";
    private final String   DEFAULT_FILE_BROWSER_DIR    = "file:///store/home/user/";
    
    private final boolean  SYNC_ALL_ON_MAIN_SCREEN     = false;
    private final boolean  SOURCE_URI_VISIBLE          = true;
    private final boolean  SYNC_DIRECTION_VISIBLE      = true;
    private final int      DEFAULT_C2S_PUSH_DELAY      = 60 * 1000;
    private final int      DEFAULT_SYNC_TYPE           = 0;
    private final int      DEFAULT_SYNC_MODE           = Configuration.SYNC_MODE_MANUAL;
    private final boolean  DEFAULT_BLOCK_INVITES       = false;
    private final boolean  BLOCK_INVITES_FIELD_VISIBLE = true;
    
    private final boolean  SYNC_COMPANY_DIRECTORY      = false;
    private final boolean  DISABLE_POLL_ON_8830        = false;
    private final boolean  SHOW_TITLE_WITH_VERSION     = false;
    private final boolean  ROLLOVER_ICON_ENABLED       = false;
    private final boolean  GOTO_MENU_ENABLED           = true;
    private final boolean  ENABLE_FILE_LOGGING         = true;
    private final boolean  SHOW_ABOUT_LICENCE          = true;
    private final boolean  SHOW_POWERED_BY             = false;
    private final boolean  HIDE_HELP_SCREEN            = false;
    private final boolean  REPORT_STATUS_IN_SYNC_ALL   = false;

    private final boolean  DEFAULT_ENCODE              = true;
    private final String   DEFAULT_ENCRYPTION_TYPE     = "b64";

    // The url parameter used by http uploader
    private final String   UPLOAD_URL_PARAM            = "sapi/media";
    
    //// ------------------- END OF CUSTOMIZABLE FIELDS --------------------////

    private static Customization instance = null;

    private Hashtable sourcesUri          = new Hashtable();
    private Hashtable activeSources       = new Hashtable();
    private Hashtable enabledSources      = new Hashtable();
    private Hashtable sourcesIcon         = new Hashtable();
    private Hashtable sourcesDisabledIcon = new Hashtable();

    private Customization() {
        initSourcesInfo();
    }

    public static Customization getInstance() {
        if (instance == null) {
            instance = new Customization();
        }
        return instance;
    }

    public String getApplicationFullname() {
        return APPLICATION_FULLNAME;
    }

    public String getApplicationTitle() {
        return APPLICATION_TITLE;
    }

    public boolean syncAllOnMainScreenRequired() {
        return SYNC_ALL_ON_MAIN_SCREEN;
    }
    
    public String getFixedApn() {
        return FIXED_APN;
    }

    public int[] getSourcesOrder() {
        return SOURCES_ORDER;
    }
    
    public boolean stripPortFromSyncAddress() {
        return STRIP_PORT_FROM_SYNC_ADDRESS;
    }

    public boolean loginScreenRequired() {
        return LOGIN_SCREEN_REQUIRED;
    }

    public boolean checkCredentialsInAccountScreen() {
        return CHECK_CRED_IN_ACCOUNT_SCREEN;
    }

    public boolean syncUriEditable() {
        return SYNC_URI_EDITABLE;
    }

    public boolean isRangePastVisible() {
        return !EVENT_RANGE_LIMITED;
    }
    
    public boolean isRangeFutureVisible() {
        return !EVENT_RANGE_LIMITED;
    }

    public boolean syncInRange() {
        return EVENT_RANGE_LIMITED;
    }
    
    public boolean isBlockInvitesFieldVisible() {
        return BLOCK_INVITES_FIELD_VISIBLE;
    }
    
    public boolean isSourceUriVisible() {
        return SOURCE_URI_VISIBLE;
    }

    public boolean isSyncDirectionVisible() {
        return SYNC_DIRECTION_VISIBLE;
    }

    public boolean invitesFiltering() {
        return INVITES_FILTERING;
    }

    public boolean isSourceActive(int id) {
        Boolean active = (Boolean)activeSources.get(new Integer(id));
        return active.booleanValue();
    }

    public boolean isSourceEnabledByDefault(int id) {
        Boolean enabled = (Boolean)enabledSources.get(new Integer(id));
        return enabled.booleanValue();
    }

    public String getSourceIcon(int id) {
        String icon = (String)sourcesIcon.get(new Integer(id));
        return icon;
    }

    public String getSourceDisabledIcon(int id) {
        String icon = (String)sourcesDisabledIcon.get(new Integer(id));
        return icon;
    }

    public boolean confirmSlowSync() {
        return CONFIRM_SLOW_SYNC;
    }

    public boolean warnOnDeletes() {
        return WARN_ON_DELETES;
    }

    public boolean syncContactPhoto() {
        return SYNC_CONTACT_PHOTO;
    }

    public String getContactType() {
        return DEFAULT_CONTACT_TYPE;
    }

    public String getCalendarType() {
        return DEFAULT_CALENDAR_TYPE;
    }
    
    public String getTaskType() {
        return DEFAULT_TASK_TYPE;
    }

    public boolean checkForRequiredUpdateOnSync() {
        return CHECK_FOR_UPDATE_ON_SYNC;
    }

    public boolean enableUpdaterManager() {
        return ENABLE_UPDATER_MANAGER;
    }

    public boolean syncCompanyDirectory() {
        return SYNC_COMPANY_DIRECTORY;
    }

    public boolean disablePollOn8830() {
        return DISABLE_POLL_ON_8830;
    }

    public boolean showTitleWithVersion() {
        return SHOW_TITLE_WITH_VERSION;
    }

    public boolean rolloverIcon() {
        return ROLLOVER_ICON_ENABLED;
    }

    public String getImageLogo() {
        return IMAGE_LOGO;
    }

    public String getSyncAllIcon() {
        return IMAGE_ALL;
    }

    public String getSyncAllBackground() {
        return IMAGE_ALL_BG;
    }

    public String getSyncAllHighlightedBackground() {
        return IMAGE_ALL_BG_HIGHLIGHT;
    }

    public String getButtonBackground() {
        return IMAGE_BG;
    }

    public String getButtonHighlightedBackground() {
        return IMAGE_BG_HIGHLIGHT;
    }

    public String getDefaultSourceUri(int id) {
        return (String)sourcesUri.get(new Integer(id));
    }

    public String getOkIcon() {
        return IMAGE_OK;
    }

    public String getErrorIcon() {
        return IMAGE_ERROR;
    }

    public String getCancelledIcon() {
        return IMAGE_CANCEL;
    }

    public String getWarningIcon() {
        return IMAGE_WARNING;
    }

    public boolean gotoMenuEnabled() {
        return GOTO_MENU_ENABLED;
    }
    public String getFolderIcon() {
        return IMAGE_FOLDER;
    }

    public boolean logEnabled() {
        return LOG_ENABLED;
    }

    public boolean isLogEnabledInSettingsScreen() {
        return LOG_IN_SETTINGS_SCREEN;
    }

    public boolean sendLogEnabled() {
        return SEND_LOG_ENABLED;
    }

    public boolean sendLogViaEmailEnabled() {
        return SEND_LOG_VIA_EMAIL_ENABLED;
    }

    public String getLogEmailAddress() {
        return LOG_EMAIL_ADDRESS;
    }

    public String getLogEmailName() {
        return LOG_EMAIL_NAME;
    }

    public String getServerUriDefault() {
        return SERVER_URI;
    }

    public String getUserDefault() {
        return USERNAME;
    }

    public String getPasswordDefault() {
        return PASSWORD;
    }

    public long getCheckUpdtIntervalDefault(){
        return CHECK_UPDATE_INTERVAL;
    }

    public long getReminderUpdtIntervalDefault(){
        return REMINDER_UPDATE_INTERVAL;
    }

    public String getLogAppTitle() {
        return LOG_TITLE;
    }

    public boolean enableFileLogging() {
        return ENABLE_FILE_LOGGING;
    }

    public String getLogFileDirectory() {
        return LOG_PATH;
    }

    public String getLogFileName() {
        return LOG_FILE_NAME;
    }

    public boolean useDefaultFont() {
        return USE_DEFAULT_FONT;
    }

    public String getPreferredFont() {
        return PREFERRED_FONT;
    }
    
    public String getAboutCopyright() {
        return ABOUT_COPYRIGHT_DEFAULT;
    }

    public String getAboutSite() {
        return ABOUT_SITE_DEFAULT;
    }

    public boolean showAboutLicence() {
        return SHOW_ABOUT_LICENCE;
    }

    public boolean showPoweredBy() {
        return SHOW_POWERED_BY;
    }
    

    public boolean reportStatusInSyncAll() {
        return REPORT_STATUS_IN_SYNC_ALL;
    }

    public String getPicturesDir() {
        return DEFAULT_PICTURES_DIR;
    }

    public String getFileBrowserDir() {
        return DEFAULT_FILE_BROWSER_DIR;
    }

    public int getDefaultPollingInterval() {
        return DEFAULT_POLLING_INTERVAL;
    }

    public String getDefaultContactUri() {
        return CONTACTS_DEFAULT_URI;
    }
    
    public String getDefaultCalendarUri() {
        return CALENDARS_DEFAULT_URI;
    }
    
    public String getDefaultTaskUri() {
        return TASKS_DEFAULT_URI;
    }
    
    public String getDefaultNoteUri() {
        return NOTES_DEFAULT_URI;
    }

    public String getDefaultConfigurationUri() {
        return CONF_DEFAULT_URI;
    }

    public boolean isC2SPushEnabled() {
        return ENABLE_C2S_PUSH;
    }
    
    public int getC2SPushDelay() {
        return DEFAULT_C2S_PUSH_DELAY;
    }

    public int getDefaultSyncMode() {
        return DEFAULT_SYNC_MODE;
    }
    
    public int[] getPollingPimIntervalChoices() {
        return POLLING_PIM_INTERVAL_CHOICES;
    }
    
    public int getMaxItemsPerMessageInSlowSync() {
        return MAX_ITEMS_PER_MESSAGE_IN_SLOW_SYNC;
    }
    
    public boolean getDefaultEncode() {
        return DEFAULT_ENCODE;
    }

    public String getDefaultEncryptionType() {
        return DEFAULT_ENCRYPTION_TYPE;
    }

    public int getDefaultSyncType() {
        return DEFAULT_SYNC_TYPE;
    }

    public int getDefaultRangePast() {
        return DEFAULT_RANGE_PAST;
    }

    public int getDefaultRangeFuture() {
        return DEFAULT_RANGE_FUTURE;
    }

    public boolean getDefaultBlockInvites() {
        return DEFAULT_BLOCK_INVITES;
    }

    public String getSendLogsUri() {
        return SEND_LOGS_URI;
    }

    public int getFontStyle() {
        return FONT_STYLE;
    }

    public int getFontSize() {
        return FONT_SIZE;
    }

    public int getSyncAllTextColor() {
        return Color.WHITE;
    }

    public boolean keepSyncAllBackgroundWhenDisabled() {
        return true;
    }

    public String getUpdaterRequestUri() {
        return UPDATER_REQUEST_URI;
    }

    public String getUploadUrl() {
        return UPLOAD_URL_PARAM;
    }

    /**
     * Returns an Enumeration of Integer where each item represents the id of an
     * active source.
     *
     * @return an enumeration of Integer
     */
    public Enumeration getActiveSources() {
        Enumeration keys = activeSources.keys();
        return keys;
    }

    private void initSourcesInfo() {
        int id = AppSyncSourceManager.SYNC_FLAG_CONTACTS;
        sourcesUri.put(new Integer(id), CONTACTS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(CONTACTS_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(CONTACTS_SYNC_ENABLED));
        sourcesIcon.put(new Integer(id), IMAGE_CONTACTS);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_CONTACTS_DISABLED);
        
        id = AppSyncSourceManager.SYNC_FLAG_CALENDAR;
        sourcesUri.put(new Integer(id), CALENDARS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(CALENDARS_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(CALENDARS_SYNC_ENABLED));
        sourcesIcon.put(new Integer(id), IMAGE_CALENDAR);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_CALENDAR_DISABLED);
        
        id = AppSyncSourceManager.SYNC_FLAG_TASKS;
        sourcesUri.put(new Integer(id), TASKS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(TASKS_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(TASKS_SYNC_ENABLED));
        sourcesIcon.put(new Integer(id), IMAGE_TASKS);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_TASKS_DISABLED);
        
        id = AppSyncSourceManager.SYNC_FLAG_NOTES;
        sourcesUri.put(new Integer(id), NOTES_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(NOTES_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(NOTES_SYNC_ENABLED));
        sourcesIcon.put(new Integer(id), IMAGE_NOTES);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_NOTES_DISABLED);
        
        id = AppSyncSourceManager.SYNC_FLAG_PHOTOS;
        sourcesUri.put(new Integer(id), PICTURES_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(PICTURES_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(PICTURES_SYNC_ENABLED));
        sourcesIcon.put(new Integer(id), IMAGE_PHOTOS);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_PHOTOS_DISABLED);

        id = AppSyncSourceManager.SYNC_FLAG_MAIL;
        sourcesUri.put(new Integer(id), MAILS_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(MAILS_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(MAILS_SYNC_ENABLED));
        sourcesIcon.put(new Integer(id), IMAGE_NOTES);
        sourcesDisabledIcon.put(new Integer(id), IMAGE_NOTES_DISABLED);

        id = AppSyncSourceManager.SYNC_FLAG_CONFIG;
        sourcesUri.put(new Integer(id), CONF_DEFAULT_URI);
        activeSources.put(new Integer(id), new Boolean(CONF_SYNC_ACTIVE));
        enabledSources.put(new Integer(id), new Boolean(CONF_SYNC_ENABLED));
        //sourcesIcon.put(new Integer(id), null);
        //sourcesDisabledIcon.put(new Integer(id), null);
    }
}

