/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.util.Hashtable;
import java.util.Vector;

import net.rim.device.api.xml.parsers.SAXParserFactory;
import net.rim.device.api.xml.parsers.SAXParser;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.funambol.util.Log;


/**
 * 
 */
public final class LanguageXMLParser 
{
    private static final String TAG_LOG = "LanguageXMLParser";

    private OneLevelHandler parserHelper;
    private SAXParser mainParser;

    public LanguageXMLParser() throws Throwable {
        this.parserHelper = new OneLevelHandler("");
        this.mainParser = SAXParserFactory.newInstance().newSAXParser();
    }

    public Hashtable parseData(String roottag, InputSource is) throws Throwable
    {
        parserHelper.reset();
        this.mainParser.parse(is,this.parserHelper);
        return this.parserHelper.getResult();
    }

    public Hashtable parseData(String roottag, InputStream is) throws Throwable
    {
        return this.parseData(roottag, new InputSource(is));
    }

    public Hashtable parseData(String roottag, String is) throws Throwable
    {
        return this.parseData(roottag, new InputSource(new ByteArrayInputStream(is.getBytes())));
    }

    static final class OneLevelHandler extends org.xml.sax.helpers.DefaultHandler
    {
        private Hashtable result;
        private StringBuffer stringData;
        private String currentKey;
        private boolean resourcesStarted;

        private static final String RESOURCES_TAG = "resources";
        private static final String STRING_TAG    = "string";

        public OneLevelHandler(String roottag) {
            super();
            this.result = new Hashtable();
            this.stringData = null;
            this.currentKey = null;
            resourcesStarted = false;
        }
        //-------------------------------------
        public Hashtable getResult() {
            return this.result;
        }
        public void reset() {
            this.result.clear();
            this.stringData = null;
            this.currentKey = null;
            this.resourcesStarted = false;
        }

        // The syntax is very simple
        // <resources>
        // (<string name="key">value</string>)*
        // </resources>

        //-------------------------------------
        public void startElement(String uri, String localName, String qName, Attributes properties)
        throws SAXException
        {
            if (RESOURCES_TAG.equals(qName)) {
                if (resourcesStarted) {
                    Log.error(TAG_LOG, "Syntax error in language file, multiple resources tag found ");
                }
                resourcesStarted = true;
            } else if (STRING_TAG.equals(qName)) {
                if (!resourcesStarted) {
                    Log.error(TAG_LOG, "Syntax error in language file, resources tag not found ");
                }
                if (properties != null) {
                    // <string name="KEY">VALUE</string>
                    currentKey = properties.getValue("name");
                    if (currentKey == null) {
                        Log.error(TAG_LOG, "Syntax error in language file, string tag without name attribute");
                    } else {
                        stringData = new StringBuffer();
                    }
                } else {
                    Log.error(TAG_LOG, "Syntax error in language file, ignoring string: " + localName);
                }
            } else {
                Log.error(TAG_LOG, "Syntax error in language file, unknown tag " + qName);
            }
        }
        //-------------------------------------
        public void endElement(String uri, String localName, String qName) throws SAXException
        {
             if (RESOURCES_TAG.equals(qName)) {
                if (!resourcesStarted) {
                    Log.error(TAG_LOG, "Syntax error in language file, resources tag not opened");
                }
                resourcesStarted = false;
            } else if (STRING_TAG.equals(qName)) {
                if (!resourcesStarted) {
                    Log.error(TAG_LOG, "Syntax error in language file, resources tag not opened ");
                }
                if (currentKey != null) {
                    // <string name="KEY">VALUE</string>
                    result.put(currentKey, decodeUtf8(stringData.toString()));
                } else {
                    Log.error(TAG_LOG, "Syntax error in language file, ignoring string: " + localName);
                }
            } else {
                Log.error(TAG_LOG, "Syntax error in language file, unknown tag " + qName);
            }
        }
        //-------------------------------------
        public void characters(char[] ch, int start, int length) throws SAXException {
            if (this.stringData != null) {
                this.stringData.append(ch,start,length);
            }
        }
        private String decodeUtf8(String src) {
            String dest = src;
            try {
                byte[]data = src.getBytes("UTF-8");
                String tempstr = new String(data, 0, data.length, "UTF-8");
                dest = tempstr;
            } catch (java.io.UnsupportedEncodingException e) {
                // Use original string - nothing to do
                Log.error(TAG_LOG, "decodeUtf8 Parse error: ", e);
            }
            return dest;
        }
    }
} 
