/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import com.funambol.blackberry.source.ExtendedSyncSource;
import com.funambol.blackberry.source.PhotoSyncSource;
import com.funambol.blackberry.source.TwoPhasesPhotoSyncSource;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.util.StaticDataHelper;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncSource;

import com.funambol.util.Log;

public class PhotoAppSyncSource extends AppSyncSource {

    private static final String TAG_LOG = "PhotoAppSyncSource";

    protected SyncSource syncMLSource;
    protected TwoPhasesPhotoSyncSource twoPhasesSyncSource;

    public PhotoAppSyncSource(String name, SyncSource source) {
        super(name, source);
    }

    public PhotoAppSyncSource(String name) {
        super(name);
    }

    public int prepareRefresh(int direction) throws Exception {
        int syncMode = 0;
        switch (direction) {
            case Constants.REFRESH_FROM_SERVER:
                throw new IllegalArgumentException("Invalid refresh direction " + direction);
            case Constants.REFRESH_TO_SERVER:
                syncMode = SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW;
                break;
            default:
                throw new IllegalArgumentException("Invalid refresh direction " + direction);
        }
        // Now clear the tracker so that everything is sent again to the server
        if(source instanceof ExtendedSyncSource) {
            ExtendedSyncSource ess = (ExtendedSyncSource)source;
            ess.clearTracker();
        } 
        return syncMode;
    }

    public boolean isRefreshSupported(int direction) {

        boolean supported;
        switch (direction) {
            case Constants.REFRESH_FROM_SERVER:
                supported = false;
                break;
            case Constants.REFRESH_TO_SERVER:
                supported = true;
                break;
            default:
                throw new IllegalArgumentException("Invalid refresh direction " + direction);
        }
        return supported;
    }

    public String getSyncConfirmationMessage() {
        boolean synced = Configuration.isSourceSynced(getId());
        if (!synced) {
            boolean syncOld = Configuration.getPicturesSyncOld();
            if (syncOld) {
                return StaticDataHelper.getLanguage("dialog_first_picture_sync");
            }
        }
        return null;
    }

    public void setRefreshSupported(boolean value) {
        throw new IllegalArgumentException("Photo sync source refresh hanlding cannot be changed");
    }

    public void setRefreshSupported(int direction, boolean value) {
        throw new IllegalArgumentException("Photo sync source refresh hanlding cannot be changed");
    }

    public void reapplyConfiguration() {

        String picturesDir = Configuration.getPicturesDir();
        // Set this property in the SS and in the system listener
        BBFileSystemListener listener = (BBFileSystemListener)getNativeListener();
        if (listener != null) {
            listener.setDirectory(picturesDir);
        }
        if(source instanceof ExtendedSyncSource) {
            ExtendedSyncSource ss = (ExtendedSyncSource)getSyncSource();
            ss.reapplyConfiguration();
        }
        // Change the upload mode if required
        if (Configuration.isUploadViaHttp(getId()) && twoPhasesSyncSource != null) {
            Log.info(TAG_LOG, "Switching to two phases sync source");
            setSyncSource(twoPhasesSyncSource);
        } else {
            Log.info(TAG_LOG, "Switching to pure syncml sync source");
            if (syncMLSource != null) {
                setSyncSource(syncMLSource);
            } else {
                Log.error(TAG_LOG, "Invalid syncml source, source will be disabled");
            }
        }
    }

    /**
     * Returns the message to show at the very first sync for this source.
     * For the pictures sync source we only want to show the msg if the user is
     * about to sync also old pictures.
     */
    public String getWarningOnFirstSync() {
        if (Configuration.getPicturesSyncOld()) {
            return warningOnFirstSync;
        } else {
            return null;
        }
    }

    public void setSyncMLSource(SyncSource source) {
        syncMLSource = source;
    }

    public void setTwoPhasesSyncSource(TwoPhasesPhotoSyncSource source) {
        twoPhasesSyncSource = source;
    }

    public void setUISyncSourceController(UISyncSourceController controller) {
        super.setUISyncSourceController(controller);
        Controller globalController = controller.getController();
        twoPhasesSyncSource.setController(globalController.getHomeScreenController());
    }
}


