/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Enumeration;

import net.rim.device.api.ui.UiApplication;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.LoginManager;
import com.funambol.blackberry.helper.EventHelper;
import com.funambol.blackberry.source.BBPIMSyncSource;
import com.funambol.blackberry.source.TwoPhasesPhotoSyncSource;
import com.funambol.blackberry.ui.rim.RimControllerDataFactory;

import com.funambol.util.BlackberryUtils;

import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Appender;
import com.funambol.util.FileAppender;
import com.funambol.util.ConsoleAppender;
import com.funambol.util.MemoryAppender;
import com.funambol.util.Log;
import com.funambol.util.MultipleAppender;
import com.funambol.util.appenders.BlackberryEventLogAppender;
import com.funambol.util.PluginObjectWrapperHanlder;
import com.funambol.storage.BlackberryRecordStore;
import com.funambol.util.ConnectionManager;
import com.funambol.syncml.spds.MappingStoreBuilder;
import com.funambol.syncml.spds.MappingManager;
import com.funambol.storage.StringKeyValueStore;
import com.funambol.storage.StringKeyValueRMSStore;

/**
 * Provides the user interface with the configuration settings, mail
 * synchronization capabilities (polling included), and contacts and calendar
 * events synchronization
 * 
 */
public class SyncClient extends UiApplication {

    private Controller              controller = null;

    private static Appender         logAppender  = null;
    private MemoryAppender          memoryAppender = null;

    /**
     * The default constructor.
     */
    public SyncClient() {
    }

    /**
     * Start the sync client application, but setting up everything which is
     * required.
     */
    public void start() throws Exception {

        // First of all we initialize the log
        initLog();

        /*
         * get configurations contents from the store. The class configuration
         * initializes also StaticDataHelper defaults.
         */
        Configuration.init(this.getClass());
        Configuration.load();
        if (!Configuration.isInitialized()) {
            throw new Exception(); // TODO
        }
        // The entire boot sequence is performed at log level DEBUG, then we
        // revert to the user settings
        Configuration.setTempLogLevel(Log.DEBUG);

        // Create syncsource configs
        initializeSources();

        //Set the Connection parameters if the customization file has one fixed 
        //apn  
        ConnectionManager.getInstance().setFixedApn(Customization.getInstance().getFixedApn());

        // Initialize the BlackBerry Record Store for this application
        PluginObjectWrapperHanlder wrapper = new PluginObjectWrapperHanlder();
        BlackberryRecordStore.init(wrapper, "Funambol_PIM_Sync_Client");
        // Initialize the MappingManagerStoreManager builder for this app
        // We must initialize the mapping store builder so that the
        // MappingManager will use a persistable store (in RMS)
        // At the moment we use an RMS store also for the BB because the 64K
        // limitation is not a problem for mappings
        MappingStoreBuilder mappingStoreBuilder = new BBSyncClientMappingStoreBuilder();
        MappingManager.setStoreBuilder(mappingStoreBuilder);

        // Create the global controller. This can be done only after the
        // AppSyncSources are created
        controller = new Controller(this, new RimControllerDataFactory());

        // The controller must be registered in each source
        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            // TODO FIXME, this must change. It is only necessary to stop the
            // sync. Once the SyncManager will be stoppable the sync source
            // will not need the controller any longer
            SyncSource source = appSource.getSyncSource();
            if (source != null) {
                if(source instanceof BBPIMSyncSource) {
                    BBPIMSyncSource bbPIMSyncSource = (BBPIMSyncSource)source;
                    bbPIMSyncSource.setController(controller.getHomeScreenController());
                } else if(source instanceof TwoPhasesPhotoSyncSource) {
                    TwoPhasesPhotoSyncSource twoPhasesPhotoSyncSource = (TwoPhasesPhotoSyncSource)source;
                    twoPhasesPhotoSyncSource.setController(controller.getHomeScreenController());
                }
            }
        }

        // Restore the user defined log level
        Configuration.restoreLogLevel();
        controller.reapplyConfiguration();

        /*
         * The UI engine must be accessed holding the event lock object shared
         * with the application's main thread, otherwise an
         * IllegalStateException is thrown (the event lock is held here where
         * the screen is pushed in the foreground)
         */

        Log.debug("[SyncClient]eventLock");
        boolean homeScreen = true;
        synchronized (getEventLock()) {
            if (Customization.getInstance().loginScreenRequired()) {
                new LoginManager(controller).login();
                homeScreen = false;
            } else {
                if (Configuration.getCredentialsCheckPending()) {
                    controller.getScreenDisplayController().showAccountScreen(true);
                    homeScreen = false;
                }
            }
            if (homeScreen) {
                controller.getScreenDisplayController().showHomeScreen(true);
            }
        }
        Log.debug("[SyncClient]End");

        // TODO FIXME
        //controller.checkForUpdate();
    }

    protected void initLog() {
        if (Customization.getInstance().enableFileLogging()) {

            String logFilePath = Customization.getInstance().getLogFileDirectory();
            String logFileName = Customization.getInstance().getLogFileName();
            Appender fileAppender = new FileAppender(logFilePath, logFileName);

            Appender appender;

            if (BlackberryUtils.isSimulator()) {
                // On the simulator we use the console appender and the file
                // appender so it is very convient to get the logs
                MultipleAppender appenders = new MultipleAppender();
                ConsoleAppender consoleAppender = new ConsoleAppender();
                appenders.addAppender(consoleAppender);
                appenders.addAppender(fileAppender);
                appender = appenders;
            } else {
                // On real devices we log to file only
                appender = fileAppender;
            }

            Log.initLog(appender);
            Log.setLogLevel(Log.TRACE);
        } else {
            MultipleAppender appenders = new MultipleAppender();
            String appName = Customization.getInstance().getLogAppTitle();
            Appender eventAppender = new BlackberryEventLogAppender(appName);
            eventAppender.initLogFile();
            appenders.addAppender(eventAppender);
            memoryAppender = new MemoryAppender();
            appenders.addAppender(memoryAppender);
            logAppender = appenders;
            Log.initLog(logAppender, Log.TRACE);
        }
    }

    private void initializeSources() {

        // First of all initialize all the sources that are displayable
        int sources[] = Customization.getInstance().getSourcesOrder();
        int uiOrder = 0;
        for (int i=0;i<sources.length;++i) {
            int sourceId = sources[i];
            switch (sourceId) {
                case AppSyncSourceManager.SYNC_FLAG_CONTACTS:
                    uiOrder = initContactsSource(uiOrder);
                    break;
                case AppSyncSourceManager.SYNC_FLAG_CALENDAR:
                    uiOrder = initCalendarSource(uiOrder);
                    break;
                case AppSyncSourceManager.SYNC_FLAG_TASKS:
                    uiOrder = initTasksSource(uiOrder);
                    break;
                case AppSyncSourceManager.SYNC_FLAG_NOTES:
                    uiOrder = initNotesSource(uiOrder);
                    break;
                case AppSyncSourceManager.SYNC_FLAG_PHOTOS:
                    uiOrder = initPhotosSource(uiOrder);
                    break;
                default:
                    Log.error("[SyncClient.initializeSources] Unknown source id: " + sourceId);
            }
        }
        // Now initialize other sources (non displayable)
        initConfigSource();
    }

    private int initContactsSource(int uiOrder) {

        /**
         * Contacts
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_CONTACTS)) {
            try {
                AppSyncSourceManager.getInstance().setUpContactsSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error("Coud not create contacts sync source, disabling contacts "
                          + e.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_CONTACTS);
            }
        }
        return uiOrder;
    }

    private int initCalendarSource(int uiOrder) {

        /**
         * Calendar
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_CALENDAR)) {
            try {
                AppSyncSourceManager.getInstance().setUpCalendarSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error("Coud not create calendar sync source, disabling calendar "
                          + e.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_CALENDAR);
            }
        }
        return uiOrder;
    }

    private int initTasksSource(int uiOrder) {

        /**
         * Tasks
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_TASKS)) {
            try {
                AppSyncSourceManager.getInstance().setUpTasksSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error("Coud not create tasks sync source, disabling tasks "
                          + e.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_TASKS);
            }
        }
        return uiOrder;
    }

    private int initNotesSource(int uiOrder) {

        /**
         * Note
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_NOTES)) {
            try {
                AppSyncSourceManager.getInstance().setUpNotesSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error("Coud not create notes sync source, disabling notes"
                          + e.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_NOTES);
            }
        }
        return uiOrder;
    }

    private int initPhotosSource(int uiOrder) {

        /**
         * Photos
         */
        if (Customization.getInstance().isSourceActive(AppSyncSourceManager.SYNC_FLAG_PHOTOS)) {
            try {
                AppSyncSourceManager.getInstance().setUpPhotosSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error("Coud not create photos sync source, disabling photos"
                          + e.toString());
                AppSyncSourceManager.getInstance().disableSource(
                                            AppSyncSourceManager.SYNC_FLAG_PHOTOS);
            }
        }
        return uiOrder;
    }

    private void initConfigSource() {

        /**
         * Config
         */
        try {
            AppSyncSourceManager.getInstance().setUpConfigSource();
        } catch (Throwable e) {
            Log.error("Coud not create config sync source, disabling config"
                          + e.toString());
            AppSyncSourceManager.getInstance().disableSource(
                                        AppSyncSourceManager.SYNC_FLAG_CONFIG);
        }
    }

    public SourceConfig getSourceConfig(int type) {
        AppSyncSource   appSource = AppSyncSourceManager.getInstance().getSource(type);
        SyncSource source = appSource.getSyncSource();
        if (source != null) {
            return source.getConfig();
        }
        return null;
    }

    protected SourceConfig setSourceConfig(int type, SourceConfig config) {
        AppSyncSource   appSource = AppSyncSourceManager.getInstance().getSource(type);
        SyncSource source = appSource.getSyncSource();
        if (source != null) {
            source.setConfig(config);
        } else {
            Log.error("Unable to set source config for type " + type);
        }
        return null;
    }

    public void setEnableBlockInvites(boolean enabled) {
        AppSyncSource   appSource = AppSyncSourceManager.getInstance()
                                    .getSource(AppSyncSourceManager.SYNC_FLAG_CALENDAR);
        if (appSource != null) {
            BBPIMSyncSource ss = (BBPIMSyncSource) appSource.getSyncSource();
            if (ss != null) {
                ((EventHelper) ss.getHelper()).setIncomingMailListenerStatus(enabled);
            }
        }
    }

    public String getLogContent() {
        // Just for safety we check if the memoryAppender is initialized.
        // It it is not, then we have a logical error, but at least we avoid a
        // NPE
        if (memoryAppender != null) {
            return memoryAppender.getLogData();
        } else {
            return "";
        }
    }

    public static Appender getAppender() {
        return logAppender;
    }

    private class BBSyncClientMappingStoreBuilder extends MappingStoreBuilder {

        public BBSyncClientMappingStoreBuilder() {
        }

        public StringKeyValueStore createNewStore(String name) {
            return new StringKeyValueRMSStore(name);
        }
    }

}
