/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Enumeration;
import java.util.Vector;
import net.rim.device.api.i18n.DateFormat;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.SyncEngine;
import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.ui.AccountScreen;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.ConnectionListener;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.BasicSyncListener;
import com.funambol.syncml.spds.SyncListener;
import com.funambol.storage.StringKeyValueFileStore;
import com.funambol.syncml.client.ChangesTracker;
import com.funambol.syncml.client.CacheTracker;
import com.funambol.syncml.client.ConfigSyncSource;

/**
 * This class is the controller (in the MVC model) for the AccountScreen.
 */
public class AccountScreenController extends SynchronizationController {

    private AccountScreen        accountScreen    = null;
    private AppSyncSource        configAppSource  = null;
    private boolean              failed           = false;
    private boolean              sourceStarted    = false;
    private UISyncSource         uiSource         = null;
    private Exception            exp              = null;
    private String               originalUrl      = null;
    private String               originalUser     = null;
    private String               originalPassword = null;
    private Configuration        config           = null;

    AccountScreenController(Controller controller, AccountScreen accountScreen, SyncClient client) {
        super(controller, client);
        config = Configuration.getInstance();
        this.accountScreen = accountScreen;
        // Save the original values so that we can revert changes at any time
        originalUrl = Configuration.getSyncUrl() != null ?  Configuration.getSyncUrl() : ""; 
        originalUser = Configuration.getUsername() != null ?  Configuration.getUsername() : "";
        originalPassword = Configuration.getPassword() != null ?  Configuration.getPassword() : "";
    }

    public Controller getController() {
        return controller;
    }

    public void saveAndCheck(String serverUri, String username, String password) {

        Log.trace("[AccountScreenController.saveAndCheck]");

        // Load all the default settings and overwrite the parameters edited
        // here
        Configuration.load();

        //Reset all Update parameters
        UpdaterManager.getInstance().resetUpdateParams();
        

        // Preliminary check
        if (StringUtil.isNullOrEmpty(username) || StringUtil.isNullOrEmpty(password)
            || StringUtil.isNullOrEmpty(serverUri)) {


            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-user-password"));

            return;
        } else if ((serverUri.indexOf("http") != 0) && (serverUri.indexOf("https") != 0)) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-server"));
            return;
        }

        // Okay, save the configuration
        Configuration.setSyncUrl(serverUri);
        Configuration.setUsername(username);
        Configuration.setPassword(password);

        // Save the new values as original ones 
        originalUrl      = Configuration.getSyncUrl(); 
        originalUser     = Configuration.getUsername();
        originalPassword = Configuration.getPassword();

        if (Configuration.save(controller) != Configuration.CONF_OK) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-save"));
            return;
        }

        if (Customization.getInstance().checkCredentialsInAccountScreen()) {
            // Now we must perform a sync of the configuration to authenticate and
            // verify the credentials
            configAppSource = AppSyncSourceManager.getInstance().getSource(
                                                AppSyncSourceManager.SYNC_FLAG_CONFIG);
            if (configAppSource == null) {
                Log.error("No suitable ConfigSyncSource, cannot verify credentials");
            } else {
                // Disable the save command
                failed = false;
                sourceStarted = false;
                exp = null;
                accountScreen.disableSave();

                Vector sources = new Vector();
                sources.addElement(configAppSource);

                // We want to perform this sync at log level debug because if
                // this is the first sync the user has no means to set a log
                // level and we may need to check what's going on in case of
                // errors
                try {
                    // Remember that at this point there is a credentials check
                    // pending. This info will be cleared once the check
                    // succeeds
                    Configuration.setCredentialsCheckPending(true);
                    Configuration.setTempLogLevel(Log.TRACE);
                    Configuration.save(controller);

                    controller.reapplyMiscConfiguration();
                    synchronize(Constants.SYNC_TYPE_MANUAL, sources);
                } catch (Exception e) {
                    Log.error("[AccountScreenController] Config sync failed " + e.toString());
                    failed = true;
                    syncEnded();
                } finally {
                    // Restore the original log level
                    Configuration.restoreLogLevel();
                    controller.reapplyMiscConfiguration();
                }
            }
        } else {
            // We simulate a successful sync so that the controller unlock the
            // account screen and accepts the credentials
            failed = false;
            sourceStarted = true;
            syncEnded();
        }
    }

    public void sourceFailed(AppSyncSource appSource, SyncException e) {
        super.sourceFailed(appSource, e);
        // In order to guarantee compatibility with servers without a
        // ConfigSyncSource, we must allow users to access the home screen even
        // if the server sends a 404 status for this sync
        if (e.getCode() != SyncException.NOT_FOUND_URI_ERROR) {
            exp = e;
            failed = true;
        } else {
            // Since this is not a Funambol server, we disable picture source
            controller.reapplyServerCaps(null);
        }
    }

    public void sourceStarted(AppSyncSource appSource) {
        super.sourceStarted(appSource);
        sourceStarted = true;
    }

    public void syncEnded() {
        super.syncEnded();

        accountScreen.enableSave();
        if (failed && sourceStarted) {
            Log.info("[AccountScreenController] Cannot access home screen");
            accountScreen.checkFailed();
            // Clear the configuration for no pending credentials check
            Configuration.setCredentialsCheckPending(true);
            Configuration.save(controller);
            // Now we show an error to the user, depending on the error we got
            String msg;
            if (exp instanceof SyncException) {
                SyncException se = (SyncException)exp;
                switch (se.getCode()) {
                    case SyncException.AUTH_ERROR:
                        msg = StaticDataHelper.getLanguage("status_invalid-credentials");
                        break;
                    case SyncException.FORBIDDEN_ERROR:
                        msg = StaticDataHelper.getLanguage("status_forbidden-error");
                        break;
                    case SyncException.CONN_NOT_FOUND:
                        msg = StaticDataHelper.getLanguage("status_invalid-url");
                        break;
                    case SyncException.READ_SERVER_RESPONSE_ERROR:
                    case SyncException.WRITE_SERVER_REQUEST_ERROR:
                    case SyncException.SERVER_CONNECTION_REQUEST_ERROR:
                        msg = StaticDataHelper.getLanguage("status_network-error");
                        break;
                    case SyncException.CONNECTION_BLOCKED_BY_USER:
                        msg = StaticDataHelper.getLanguage("status_connection-blocked");
                        break;
                    case SyncException.CANCELLED:
                        // In this case we shall simply go back to the account
                        // screen, so we just return from this method
                        return;
                    default:
                        msg = StaticDataHelper.getLanguage("status_generic-error");
                        break;
                }
            } else {
                msg = StaticDataHelper.getLanguage("status_generic-error");
            }
            // We should never fall into this case, unless we miss some strings
            // in the language table
            if (msg == null) {
                msg = "Generic error";
            }
            // Show an error to the user
            controller.getDialogController().showMessage(msg);

        } else if (!sourceStarted) {
            // Sync the source did not start, an appropriate error was displayed
            // by the syncrhonizationController
            Log.info("[AccountScreenController] Cannot access home screen");
            // Clear the configuration for no pending credentials check
            Configuration.setCredentialsCheckPending(true);
            Configuration.save(controller);
            accountScreen.checkFailed();
        } else {
            // The user is authenticated, hide the login and open the main view
            // screen
            Log.info("[AccountScreenController] Opening home screen ");
            accountScreen.checkSucceeded();
            controller.getScreenDisplayController().hideAccountScreen();
            controller.getScreenDisplayController().showHomeScreen();
            // Clear the configuration for no pending credentials check
            Configuration.setCredentialsCheckPending(false);
            Configuration.save(controller);
        }
    }

    public boolean hasChanges(String serverUri, String username, String password) {

        if (Customization.getInstance().syncUriEditable()) {
            if (!originalUrl.equals(serverUri)) {
                return true;
            }
        }

        if (!originalUser.equals(username) ||
            !originalPassword.equals(password)) {
            return true;
        }
        return false;
    }

    public void resetValues() {
        if (Customization.getInstance().syncUriEditable()) {
            accountScreen.setSyncUrl(originalUrl);
        }
        accountScreen.setUsername(originalUser);
        accountScreen.setPassword(originalPassword);
    }

    /**
     * Apply the current Configuration values
     */
    public void applyCurrentValues() {

        originalUrl = Configuration.getSyncUrl();
        originalUser = Configuration.getUsername();
        originalPassword = Configuration.getPassword();
        
        accountScreen.setSyncUrl(originalUrl);
        accountScreen.setUsername(originalUser);
        accountScreen.setPassword(originalPassword);
    }



    public void endSync(Vector sources, boolean hadErrors) {
        // Errors are handled in the syncEnded method
        setCancel(false);
    }

    public void hide() {
        controller.toBackground();
    }
}
