/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Enumeration;
import java.util.Vector;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.util.StaticDataHelper;

public class DialogController {

    private DisplayManager displayManager;

    public DialogController(DisplayManager manager) {
        this.displayManager = manager;
    }

    public boolean promptNext(String message) {
        return displayManager.promptNext(message);
    }

    public boolean askYesNoQuestion(String message, boolean defaultyes) {
        return displayManager.askYesNoQuestion(message, defaultyes, -1);
    }

    public boolean askAcceptDenyQuestion(String message, boolean defaultyes) {
        return displayManager.askAcceptDenyQuestion(message, defaultyes, -1);
    }

    public boolean askYesNoQuestion(String message, boolean defaultyes, int timeToWait) {
        return displayManager.askYesNoQuestion(message, defaultyes, timeToWait);
    }

    public int promptSelection(String message, String[] options, int[] values, int defaultValue,
            boolean needAnswer, long timeToWait) {

        return displayManager.promptSelection(message, options, values, defaultValue, needAnswer,
                DisplayManager.NO_LIMIT);
    }

    /**
     * 
     * @return The selected direction.
     */
    public int showRefreshDirectionDialog() {
        String[] opt = new String[3];
        int[] val = new int[3];

        opt[0] = StaticDataHelper.getLanguage("dialog_refresh_from");
        val[0] = Constants.REFRESH_FROM_SERVER;

        opt[1] = StaticDataHelper.getLanguage("dialog_refresh_to");
        val[1] = Constants.REFRESH_TO_SERVER;

        opt[2] = StaticDataHelper.getLanguage("dialog_cancel");
        val[2] = -1;

        return promptSelection(StaticDataHelper.getLanguage("dialog_refresh_which") + "\n"
                + StaticDataHelper.getLanguage("dialog_refresh_warn2"), opt, val, -1, true,
                DisplayManager.NO_LIMIT);
    }

    /**
     * 
     * @return The mask for the selected category(ies).
     */
    public int showRefreshTypeDialog(int direction) {

        // Count the number of enabled and refreshable sources
        Vector opts = new Vector();
        Vector vals = new Vector();
        int numEnabledSources = 0;
        Enumeration enabledSources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        while(enabledSources.hasMoreElements()) {
            AppSyncSource source = (AppSyncSource)enabledSources.nextElement();
            if (source.isRefreshSupported(direction)) {
                opts.addElement(source.getName());
                vals.addElement(new Integer(source.getId()));
                ++numEnabledSources;
            }
        }
        int numChoices;
        if (numEnabledSources > 1) {
            numChoices = numEnabledSources + 2;
        } else {
            numChoices = numEnabledSources + 1;
        }
        String[] options = new String[numChoices];
        int[] values = new int[numChoices];
        int i = 0;
        for(i=0;i<numEnabledSources;++i) {
            options[i] = (String)opts.elementAt(i);
            Integer v = (Integer)vals.elementAt(i);
            values[i]  = v.intValue();
        }

        if (numEnabledSources > 1) {
            options[i] = StaticDataHelper.getLanguage("type_all-enabled");
            values[i]  = AppSyncSourceManager.SYNC_FLAG_ALL;
            ++i;
        }

        options[i] = StaticDataHelper.getLanguage("dialog_cancel");
        values[i]  = 0;

        return promptSelection(StaticDataHelper.getLanguage("dialog_refresh")
                + " "
                + (direction == Constants.REFRESH_FROM_SERVER ? StaticDataHelper
                        .getLanguage("dialog_refresh_from") : StaticDataHelper
                        .getLanguage("dialog_refresh_to")), options, values, -1, true,
                DisplayManager.NO_LIMIT);
    }

    public void showMessage(String language) {
        displayManager.showMessage(language);
    }

    public void showMessage(String message, long delay){
        displayManager.showMessage(message, delay);
    }

}
