/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import com.funambol.blackberry.ui.AboutScreen;
import com.funambol.blackberry.ui.ConfigurationScreen;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.blackberry.ui.AccountScreen;
import com.funambol.blackberry.ui.Screen;
import com.funambol.util.Log;

public class ScreenDisplayController {

    private final DisplayManager      displayManager;

    private final HomeScreen          homeScreen;

    private final ConfigurationScreen configScreen;

    private final AboutScreen         aboutScreen;

    private final AccountScreen       accountScreen;

    ScreenDisplayController(DisplayManager manager, HomeScreen homeScreen,
            ConfigurationScreen configScreen, AboutScreen aboutScreen,
            AccountScreen accountScreen) {
        this.displayManager = manager;
        this.homeScreen = homeScreen;
        this.configScreen = configScreen;
        this.aboutScreen = aboutScreen;
        this.accountScreen = accountScreen;
    }

    // Show functions
    public void showScreen(Screen scr) {
        showScreen(scr, false);
    }

    public void showScreen(Screen scr, boolean donotwait) {
        try {
            displayManager.showScreen(scr, donotwait);
        } catch (Exception e) {
            Log.error("Could not show screen " + scr.getClass().getName());
        }
    }

    public void showAboutScreen() {
        showScreen(aboutScreen);
    }

    public void showConfigScreen() {
        showScreen(configScreen);
    }

    public void showHomeScreen() {
        showHomeScreen(false);
    }

    public void showHomeScreen(boolean donotwait) {
        showScreen(homeScreen, donotwait);
    }

    public void showAccountScreen() {
        showAccountScreen(false);
    }

    public void showAccountScreen(boolean donotwait) {
        showScreen(accountScreen, donotwait);
    }

    // Hide functions
    public void hideScreen(Screen scr) {
        try {
            displayManager.hideScreen(scr);
        } catch (Exception e) {
            Log.error("Could not hide screen " + scr.getClass().getName());
        }
    }

    public void hideAboutScreen() {
        hideScreen(aboutScreen);
    }

    public void hideConfigScreen() {
        hideScreen(configScreen);
    }

    public void hideHomeScreen() {
        hideScreen(homeScreen);
    }

    public void hideAccountScreen() {
        hideScreen(accountScreen);
    }
}
