/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Vector;
import java.util.Enumeration;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;

import net.rim.device.api.system.ApplicationManager;

import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.source.Poller;
import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.SyncEngine;
import com.funambol.blackberry.SyncEngineListener;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.AppSyncRequest;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.BBPIMListener;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.source.BBPIMSyncSource;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncListener;
import com.funambol.syncml.spds.SyncReport;
import com.funambol.push.SyncScheduler;
import com.funambol.util.ConnectionListener;
import com.funambol.util.ConnectionConfig;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.ConnectionManager;

/**
 * This class provides a basic controller that can be used by any other
 * controller that needs synchronization support. This controller is just
 * a building block, it does not control any UI component. But it shall be
 * extended by controllers that need synchronization capabilities (e.g.
 * HomeScreenController and AccountScreenController).
 */
public class SynchronizationController implements ConnectionListener, SyncEngineListener {

    private static final String TAG_LOG = "SynchronizationController";

    protected Controller controller;

    protected SyncClient client;

    protected SyncEngine engine;

    private boolean    doCancel        = false;

    private String     syncType        = null;

    private AppSyncSource currentSource = null;

    protected boolean    showTCPAlert;

    protected boolean    logConnectivityError;

    private SyncScheduler  syncScheduler;

    private int            scheduledAttempt = 0;

    private Poller         retryPoller = null;

    private AppSyncRequest appSyncRequestArr[] = new AppSyncRequest[1];
    private RequestHandler reqHandler;
    private Vector         deviceFullSources = null;

    private UpdaterManager updMan = UpdaterManager.getInstance();

    private Object           lock      = new Object();
    private int processID;

    SynchronizationController(Controller controller, SyncClient client) {
        this.controller = controller;
        this.client = client;
        this.engine = new SyncEngine();
        this.syncScheduler = new SyncScheduler(engine);
        engine.setListener(this);

        // The request handler is a daemon serving external requests
        reqHandler = new RequestHandler();
        reqHandler.start();

        updMan.setController(controller);
        processID = client.getProcessId();
        if(Customization.getInstance().enableUpdaterManager()){
            updateDialogThread();
        }



    }

    public void updateDialogThread(){
         Thread th = new Thread() {
            public void run() {
             while(true){
                    try{
                        synchronized (lock) {
                            lock.wait();
                        }
                        if(updMan.needsToUpdate())
                        {
                            Log.info(TAG_LOG, "Needs to upgrade application");
                            updMan.promptExpiredUpdate();
                        }
                    }catch(Exception ex){
                    }
                }
            }
        };
        th.start();

    }

    /**
     * Returns true iff a synchronization is in progress
     */
    public boolean isSynchronizing() {
        return engine.isSynchronizing();
    }

    /**
     * Returns the sync source currently being synchronized. If a sync is not
     * in progress, then null is returned. Please note that this method is not
     * completely equivalent to isSynchronizing. At the beginning of a sync,
     * isSynchronizing returns true, but getCurrentSource may return null until
     * the source is prepared for the synchronization.
     */
    public SyncSource getCurrentSource() {
        return engine.getCurrentSource();
    }

    /**
     * Try to cancel the current sync. This works for cooperative sources that
     * check the synchronizationController status.
     */
    public void cancelSync() {
        setCancel(true);

        if (isSynchronizing() && currentSource != null) {
            UISyncSourceController uiSourceController = currentSource.getUISyncSourceController();
            uiSourceController.startCancelling();
            engine.cancelSync();
        }
    }

    /**
     * Perform a refresh for a set of sources and a given direction. The method
     * gets blocked until the sync terminates.
     *
     * @param syncMask the set of sources to sync
     * @param direction the refresh direction
     */
    public synchronized void refresh(int syncMask, int direction) {

        Log.info(TAG_LOG, "refresh for " + syncMask + " and direction " + direction);

        if (isSynchronizing()) {
            Log.info(TAG_LOG, "A sync is already running, cannot refresh");
            return;
        }

        AppSyncRequest appSyncRequest = new AppSyncRequest(null,0);
        Enumeration sources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        int numSourcesToReset = 0;
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) sources.nextElement();
            if ((syncMask & appSource.getId()) != 0) {
                if (appSource.isRefreshSupported(direction)) {
                    boolean includeSource = false;

                    String warning = appSource.getWarningOnFirstSync();
                    boolean synced = Configuration.isSourceSynced(appSource.getId());
                    if (!synced && warning != null) {

                        DialogController dialControll = controller.getDialogController();
                        String labels[] = { StaticDataHelper.getLanguage("dialog_sync_now"),
                                            StaticDataHelper.getLanguage("dialog_try_later") };
                        int    values[] = { 0, 1};
                        int selection = dialControll.promptSelection(warning, labels, values, 0, true, DisplayManager.NO_LIMIT);

                        if (selection == 0) {
                            includeSource = true;
                        }
                        // We don't want this alert to show up again, so we consider
                        // this source as synced
                        Configuration.setSourceSynced(appSource.getId(), true);
                    } else {
                        includeSource = true;
                    }

                    if (includeSource) {
                        try {
                            int syncMode = appSource.prepareRefresh(direction);
                            SyncSource source = appSource.getSyncSource();
                            source.getConfig().setSyncMode(syncMode);
                            appSyncRequest.addRequestContent(appSource);
                            numSourcesToReset++;
                        } catch (Exception e) {
                            Log.error(TAG_LOG, "Cannot perform refresh on source " + appSource.getName()
                                    + " because: " + e.toString());
                        }
                    }
                } else {
                    Log.info(TAG_LOG, "Source " + appSource.getName() + " does not support refresh " + direction);
                }
            }
        }

        if (numSourcesToReset > 0) {
            syncScheduler.addRequest(appSyncRequest);
        }
    }

    public void syncEnded() {
        syncType = null;

        if( Customization.getInstance().enableUpdaterManager() &&
           !Customization.getInstance().checkForRequiredUpdateOnSync())
        {
            updMan.setController(controller);
            updMan.check();
        }
    }

    /**
     * Triggers a synchronization for the given syncSources. The caller can
     * specify its type (manual, scheduled, push) to change the error handling
     * behavior
     *
     * @param syncType the caller type (SYNC_TYPE_MANUAL, SYNC_TYPE_SCHEDULED)
     * @param syncSources is a vector of AppSyncSource to be synced
     *
     */
    public void synchronize(String syncType, Vector syncSources) {
        synchronize(syncType, syncSources, 0);
    }

    /**
     * Schedules a synchronization for the given syncSources. The sync is
     * scheduled in "delay" milliseconds from now. The caller can
     * specify its type (manual, scheduled, push) to change the error handling
     * behavior
     *
     * @param syncType the caller type (SYNC_TYPE_MANUAL, SYNC_TYPE_SCHEDULED)
     * @param syncSources is a vector of AppSyncSource to be synced
     * @param delay the interval at which the sync shall be performed (relative
     *              to now)
     *
     */
    public void synchronize(String syncType, Vector syncSources, int delay) {
        synchronize(syncType, syncSources, delay, false);
    }

    /**
     * Schedules a synchronization for the given syncSources. The sync is
     * scheduled in "delay" milliseconds from now. The caller can
     * specify its type (manual, scheduled, push) to change the error handling
     * behavior.
     * The caller can also specify it the sync request is generated outside of
     * the application. In such a case the handling is special and the
     * synchronization is actually performed by the Sync Client process. This
     * calls notifies the SyncClient to schedule a sync at the given interval.
     * This is useful when syncs are triggered on external events, such as
     * modification of PIM (c2s push).
     *
     * Since the method can be called from an external process (when triggered
     * from the BBPIMListener as result of an external modification) the method
     * cannot use any static variable or the code will throw NPE.
     *
     * @param syncType the caller type (SYNC_TYPE_MANUAL, SYNC_TYPE_SCHEDULED)
     * @param syncSources is a vector of AppSyncSource to be synced
     * @param delay the interval at which the sync shall be performed (relative
     *              to now)
     * @param fromOutside specifies if the request is generated outside of the
     * application
     */
    public void synchronize(String syncType, Vector syncSources,
                                         int delay, boolean fromOutside) {

        if(Customization.getInstance().enableUpdaterManager() &&
           updMan.needsToUpdate())
        {
            //Log.info(TAG_LOG, "Needs to upgrade application");
            if(updMan.dialogShown()){
                ApplicationManager manager = ApplicationManager.getApplicationManager();
                manager.requestForeground(processID);
            }else{
                synchronized (lock) {
                    lock.notify();
                }
            }
            return;
        }


        sync(syncType, syncSources, delay, fromOutside);

    }



    /**
     * Schedules a synchronization for the given syncSources. The sync is
     * scheduled in "delay" milliseconds from now. The caller can
     * specify its type (manual, scheduled, push) to change the error handling
     * behavior.
     * The caller can also specify it the sync request is generated outside of
     * the application. In such a case the handling is special and the
     * synchronization is actually performed by the Sync Client process. This
     * calls notifies the SyncClient to schedule a sync at the given interval.
     * This is useful when syncs are triggered on external events, such as
     * modification of PIM (c2s push).
     *
     * Since the method can be called from an external process (when triggered
     * from the BBPIMListener as result of an external modification) the method
     * cannot use any static variable or the code will throw NPE.
     *
     * @param syncType the caller type (SYNC_TYPE_MANUAL, SYNC_TYPE_SCHEDULED)
     * @param syncSources is a vector of AppSyncSource to be synced
     * @param delay the interval at which the sync shall be performed (relative
     *              to now)
     * @param fromOutside specifies if the request is generated outside of the
     * application
     */
    private synchronized void sync(String syncType, Vector syncSources,
                                         int delay, boolean fromOutside) {

        Log.info(TAG_LOG, "sync " + syncType);

        if (isSynchronizing()) {
            Log.info(TAG_LOG, "A sync is already in progress");
            return;
        }


        // Search if at least one of the selected sources has a warning on the
        // first sync
        Vector filteredSources = new Vector();
        for(int i=0;i<syncSources.size();++i) {
            AppSyncSource appSource = (AppSyncSource)syncSources.elementAt(i);
            String warning = appSource.getWarningOnFirstSync();
            boolean synced = Configuration.isSourceSynced(appSource.getId());
            if (!synced && warning != null) {

                DialogController dialControll = controller.getDialogController();
                String labels[] = { StaticDataHelper.getLanguage("dialog_sync_now"),
                                    StaticDataHelper.getLanguage("dialog_try_later") };
                int    values[] = { 0, 1};
                int selection = dialControll.promptSelection(warning, labels, values, 0, true, DisplayManager.NO_LIMIT);

                if (selection == 0) {
                    // Sync it now
                    filteredSources.addElement(appSource);
                }
                // We don't want this alert to show up again, so we consider
                // this source as synced
                Configuration.setSourceSynced(appSource.getId(), true);
            } else {
                filteredSources.addElement(appSource);
                Configuration.setSourceSynced(appSource.getId(), true);
            }
        }

        // On non manual sync we filter out all the sources that have exceeded
        // their quota (unless this is the very first time in this session [aka
        // since the last reboot])
        if (!Constants.SYNC_TYPE_MANUAL.equals(syncType)) {
            Vector refilteredSources = new Vector();
            for(int i=0;i<filteredSources.size();++i) {
                AppSyncSource appSource = (AppSyncSource)syncSources.elementAt(i);
                if (!appSource.getSyncedInSession() ||
                    appSource.getLastSyncStatus() != SyncListener.DEVICE_FULL_ERROR)
                {
                    refilteredSources.addElement(appSource);
                } else {
                    Log.info(TAG_LOG, "Skipping source " + appSource.getName() + " because server quota reached");
                }
            }
            filteredSources = refilteredSources;
        }

        // If no sources left, we simply return and do not update/change
        // anything
        if (filteredSources.size() == 0) {
            return;
        }

        this.syncType = syncType;
        int sourceSyncType = 0;
        AppSyncRequest appSyncRequest = new AppSyncRequest(null, delay);
        Enumeration sources = filteredSources.elements();
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) sources.nextElement();
            // Remember that this source got synced in this session
            appSource.setSyncedInSession(true);
            // Prepare the sync request
            SyncSource source = appSource.getSyncSource();
            sourceSyncType = Configuration.getSourceSyncType(appSource.getId());
            appSyncRequest.addRequestContent(appSource);
            // If this source has no config set, then we cannot force a sync
            // mode, but this is a logical error
            if (source.getConfig() != null) {
                source.getConfig().setSyncMode(sourceSyncType);
            } else {
                Log.error(TAG_LOG, "Source has no config, cannot set sync mode");
            }
        }

        if (fromOutside) {
            synchronized(appSyncRequestArr) {
                appSyncRequestArr[0] = appSyncRequest;
                appSyncRequestArr.notify();
            }
        } else {
            syncScheduler.addRequest(appSyncRequest);
        }
    }

    public boolean confirmDeletes(Enumeration sourceNameList) {

        String sourceNames = getListOfSourceNames(sourceNameList).toLowerCase();

        Log.info(TAG_LOG, "Prompting user for delete confirmation");

        String message = StaticDataHelper.getLanguage("dialog_delete1") + " " + sourceNames
                + StaticDataHelper.getLanguage("dialog_delete2");

        boolean result = controller.getDialogController().askYesNoQuestion(message, false);

        if (result) {
            Log.info(TAG_LOG, "Continuing with sync");
        } else {
            Log.info(TAG_LOG, "User opted to cancel sync - " + sourceNames);
        }

        return result;
    }

    private String getListOfSourceNames(Enumeration sourceNameList) {
        StringBuffer sourceNames = new StringBuffer();

        int x = 0;
        if (!sourceNameList.hasMoreElements()) {
            return "";
        }
        AppSyncSource appSource = (AppSyncSource)sourceNameList.nextElement();

        while(appSource != null) {

            String name = appSource.getName();

            if (x > 0) {
                // If this is not the last one, then we add a comma
                if (sourceNameList.hasMoreElements()) {
                    sourceNames.append(", ");
                } else {
                    // If this is the last one, we add the "and" word
                    sourceNames.append(StaticDataHelper.getLanguage("dialog_and").toLowerCase());
                }
            }

            sourceNames.append(name);
            if (sourceNameList.hasMoreElements()) {
                appSource = (AppSyncSource)sourceNameList.nextElement();
            } else {
                appSource = null;
            }
            x++;
        }

        return sourceNames.toString();
    }



    // ConnectionListener implementation

    public boolean isConnectionConfigurationAllowed(final String apn) {
        String message = StaticDataHelper.getLanguage("message_APN-question-1") + " " + apn + " "
                + StaticDataHelper.getLanguage("message_APN-question-2");
        return controller.getDialogController().askAcceptDenyQuestion(message, true);
    }

    public void noCredentials() {
        controller.getScreenDisplayController().showAccountScreen();
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_login-required"),DisplayManager.NO_LIMIT);

    }

    public void noSources(boolean fromCancel) {
        Log.trace(TAG_LOG, "noSources");
        if (!fromCancel) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_nothing-to-sync"),DisplayManager.NO_LIMIT);
        }
    }

    public void noConnection() {
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_radio-off"),5000);

    }

    public void noSignal() {
        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_no-signal"),5000);

    }

    public void setCancel(boolean value) {
        doCancel = value;
    }

    /**
     * Check if the current sync should be cancelled
     *
     * @return
     */
    public boolean isCancelled() {
        return doCancel;
    }

    public void beginSync() {
        clearErrors();
        if (ConnectionConfig.isUserConfirmationNeeded()) {
            Log.debug(TAG_LOG, "Setting connection listener for this application");
            ConnectionManager.getInstance().setConnectionListener(this);
        } else {
            Log.debug(TAG_LOG, "Using Default BasicConnectionListener");
        }
        setCancel(false);
    }


    public boolean syncStarted(Vector sources) {
        Log.trace(TAG_LOG, "syncStarted");
        if (Customization.getInstance().enableUpdaterManager() &&
            Customization.getInstance().checkForRequiredUpdateOnSync())
        {
            boolean isRequired = controller.checkForUpdate(false);
            if (isRequired) {
                updMan.setController(controller);
                return false;
            }
        }
        return true;
    }

    public void endSync(Vector sources, boolean hadErrors) {

        setCancel(false);

        // Disable the retry poller if not null
        if(retryPoller != null) {
            retryPoller.disable();
            retryPoller = null;
        }

        // If we had a CONNECTION BLOCKED BY THE USER error (user does not allow
        // any network configuration) then we show an error because the user
        // had already interacted with the app for this sync
        if (hadErrors && showTCPAlert) {
            controller.toForeground();
            Log.debug(TAG_LOG, "showing tcp settings alert!");
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_enter-TCP-settings"),DisplayManager.NO_LIMIT);
        }

        // if we had a connectivity error, we bring the app to foreground
        // iff the user started the sync by hand
        if (logConnectivityError) {
            if (Constants.SYNC_TYPE_MANUAL.equals(syncType)) {
                Log.error(TAG_LOG, "Manual sync: Connection attempts failed.");
                controller.toForeground();
                controller.error(StaticDataHelper.getLanguage("message_connectivity"));
            } else {
                if (!retry(sources)) {
                    // Do not show any error to the user, just log the fact
                    Log.error(TAG_LOG, "Automatic sync(" + syncType + ") failed after retrying");
                }
            }
        }

        // if we had at least one device full error, we must show a popup error
        if (deviceFullSources != null) {
            StringBuffer sourceNames = new StringBuffer("");
            for(int i=0;i<deviceFullSources.size();++i) {
                AppSyncSource appSource = (AppSyncSource)deviceFullSources.elementAt(i);
                Log.error(TAG_LOG, "Manual sync: server full for source " + appSource.getName());

                boolean shown = Configuration.getDeviceFullShown(appSource.getId());
                // The popup is always shown on manual syncs and only the first
                // time if the sync is automatic
                if (!shown || Constants.SYNC_TYPE_MANUAL.equals(syncType)) {
                    if (sourceNames.length() > 0) {
                        sourceNames.append(",");
                    }
                    sourceNames.append(appSource.getName().toLowerCase());
                    // Remember that we have shown the error to the user. We
                    // won't show this popup again until a sync succeeds
                    Configuration.setDeviceFullShown(appSource.getId(), true);
                }
            }
            if (sourceNames.length() > 0) {
                controller.toForeground();
                String msg = StaticDataHelper.getLanguage("dialog_server_full");
                msg = StringUtil.replaceAll(msg, "__source__", sourceNames.toString());
                controller.getDialogController().showMessage(msg,30000);
            }
        }

        // We reset these errors because this sync is over (if we are retrying,
        // we must consider the new one with no errors)
        logConnectivityError = false;
        showTCPAlert = false;
        deviceFullSources = null;
    }

    public void sourceStarted(AppSyncSource appSource) {
        currentSource = appSource;
        UISyncSourceController sourceController = appSource.getUISyncSourceController();
        sourceController.setSelected(true);

        if (currentSource.getSyncSource().getConfig().getSyncMode()==SyncML.ALERT_CODE_REFRESH_FROM_SERVER) {
            refreshClientData(appSource, sourceController);
        }
    }

    public void sourceEnded(AppSyncSource appSource) {
        saveSourceConfig(appSource);
        UISyncSourceController sourceController = appSource.getUISyncSourceController();
        sourceController.setSelected(true);
        currentSource = null;
        // Since we have had a successfull sync, we get ready to show the device
        // full error if it happens again
        Configuration.setDeviceFullShown(appSource.getId(), false);
    }

    public void sourceFailed(AppSyncSource appSource, SyncException e) {

        Log.trace(TAG_LOG, "SynchronizationController.sourceFailed");

        int code = e.getCode();
        if (   code == SyncException.READ_SERVER_RESPONSE_ERROR
            || code == SyncException.WRITE_SERVER_REQUEST_ERROR
            || code == SyncException.CONN_NOT_FOUND) {

            logConnectivityError = true;
        } else if (code == SyncException.CONNECTION_BLOCKED_BY_USER) {
            showTCPAlert = true;
        } else if (code == SyncException.DEVICE_FULL) {
            if (deviceFullSources == null) {
                deviceFullSources = new Vector();
            }
            deviceFullSources.addElement(appSource);
        }
    }

    public String getRemoteUri(AppSyncSource appSource) {
        SourceConfig config = appSource.getSyncSource().getConfig();
        return config.getRemoteUri();
    }

    public void serverOperationFailed() {

        controller.getDialogController().showMessage(
                StaticDataHelper.getLanguage("message_not-send-to-server"),5000);
        controller.error(StaticDataHelper.getLanguage("message_not-send-to-server"));

    }

    public Controller getController() {
        return controller;
    }

    public void clearErrors() {
        controller.clearErrors();
        showTCPAlert = false;
        logConnectivityError = false;
        deviceFullSources = null;
    }

    public void error(String message) {
        controller.error(message);
    }

    private void saveSourceConfig(AppSyncSource appSource) {

        SyncSource source = appSource.getSyncSource();
        String sscKey = appSource.getConfigStorageKey();
        if (sscKey != null) {
            Configuration.saveSourceConfig(sscKey, source.getConfig());
        } else {
            Log.error(TAG_LOG, "Unable to save source - no key!");
        }
    }


    public void connectionOpened() {
        Log.debug(TAG_LOG, "Connection opened");
    }

    public void requestWritten() {
        Log.debug(TAG_LOG, "Request written");
    }

    public void responseReceived() {
        Log.debug(TAG_LOG, "Response received");
    }

    public void connectionClosed() {
        Log.debug(TAG_LOG, "Connection closed");
    }

    public void connectionConfigurationChanged() {
        Log.debug(TAG_LOG, "Configuration changed");
    }

    private boolean retry(Vector sources) {

        boolean willRetry = false;

        if (retryPoller != null) {
            retryPoller.disable();
        }

        if (scheduledAttempt < 3) {
            scheduledAttempt++;
            Log.error(TAG_LOG, "Scheduled sync: Connection attempt failed. " + "Try again between "
                    + Constants.RETRY_POLL_TIME + " minutes");

            retryPoller = new Poller(this, Constants.RETRY_POLL_TIME, true, false);
            retryPoller.start();
            willRetry = true;
        } else {
            retryPoller = null;
            scheduledAttempt = 0;
        }
        return willRetry;
    }

    private class RequestHandler extends Thread {

        private boolean stop = false;

        public RequestHandler() {
        }

        public void run() {
            Log.info(TAG_LOG, "Starting request handler");
            while (!stop) {
                try {
                    synchronized (appSyncRequestArr) {
                        appSyncRequestArr.wait();
                        syncScheduler.addRequest(appSyncRequestArr[0]);
                    }
                } catch (Exception e) {
                    // All handled exceptions are trapped below, this is just a
                    // safety net for runtime exception because we don't want
                    // this thread to die.
                    Log.error(TAG_LOG, "Exception while performing a programmed sync " + e.toString());
                }
            }
        }
    }

    private void refreshClientData(AppSyncSource appSource, UISyncSourceController controller) {
        // Removing items before the sync starts is only required for PIM
        // sources, where removing items is very slow.
        if (appSource.getSyncSource() instanceof BBPIMSyncSource) {
            BBPIMSyncSource bpss = (BBPIMSyncSource) appSource.getSyncSource();
            try {
                bpss.deleteAllItems();
            } catch (SyncException se) {
                if (se.getCode() == SyncException.CANCELLED) {

                    // The sync has not started yet, so we must synthetize a
                    // report here
                    SyncReport report = new SyncReport(bpss);
                    report.setSyncStatus(SyncListener.CANCELLED);
                    controller.endSession(report);
                } else {
                    Log.error(TAG_LOG, "Cannot reset client data", se);
                    // We allow the sync to go on because the SyncSource will
                    // try to remove items once more and handle errors there
                }
            } catch (Exception ex) {
                Log.error(TAG_LOG, "Cannot reset client data", ex);
                // We allow the sync to go on because the SyncSource will
                // try to remove items once more and handle errors there
            }
        }
    }
}