/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Enumeration;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.Contact;

import net.rim.blackberry.api.pdap.BlackBerryContact;
import net.rim.blackberry.api.pdap.BlackBerryContactList;

import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;

public abstract class ContactHelper extends PIMItemHelper
{    
    ContactHelper(PIMList list)
    {
        super(list);
    }

    public PIMItem findItem(String itemKey) throws PIMException
    {
        PIMList list = getPIMList();
        if (list != null)
        {
            Log.info("[ContactHelper.findItem] begin");
            Enumeration items = list.items();
            PIMItem item = null;
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                if (itemKey.equals(this.getUID(item)))
                {
                    Log.info("[ContactHelper.findItem] End");
                    return item;
                }
            }
        }
        return null;
    }
    
    
    public boolean deleteItem(PIMItem item) throws PIMException
    {
        BlackBerryContactList list = getContactList();
        if (list != null &&
                item != null &&
                item instanceof Contact &&
                isInSyncRange(item))
        {
            list.removeContact((Contact)item);    
            return true;
        }
        return false;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#createPIMItem(PIMList pimlist)
     */
    public PIMItem createItem() throws PIMException
    {
        BlackBerryContactList list = getContactList();
        if (list != null)
        {
            return list.createContact();
        }
        return null;
    }
    

    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof BlackBerryContact)
        {
            try
            {
                String uid = item.getString(BlackBerryContact.UID,0);
                return uid;
            }
            catch(final Exception e)
            {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Contact]";
                Log.error(msg);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Contact] is not a BlackBerryContact.";
        Log.error(msg);
        throw new PIMException(msg);
    }
    
    protected BlackBerryContactList getContactList()
    {
        if (getPIMList() instanceof BlackBerryContactList)
            return (BlackBerryContactList)getPIMList();
        return null;
    }
    
    
    public boolean isInSyncRange(PIMItem _item) throws PIMException
    {
        return true;
    }

}
