/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Date;
import java.util.Calendar;

import javax.microedition.pim.Event;
import javax.microedition.pim.RepeatRule;
import javax.microedition.pim.EventList;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.helper.EventMailListener;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;
import com.funambol.util.DateUtil;
import com.funambol.util.StringUtil;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.source.AdditionalItemsAttributes;

public abstract class EventHelper extends PIMItemHelper
{

    private static final String TAG_LOG = "EventHelper";

    public static final long        SECOND_FACTOR       = 1000;
    public static final long        MINUTE_FACTOR       = 60*EventHelper.SECOND_FACTOR;
    public static final long        HOUR_FACTOR         = 60*EventHelper.MINUTE_FACTOR;
    public static final long        DAY_FACTOR          = 24*EventHelper.HOUR_FACTOR;
    public static final long        MONTH_FACTOR        = 30*EventHelper.DAY_FACTOR;
    
    protected EventMailListener mailListener            = null;

    protected long limitPast;
    protected long limitFuture;

    private String settingFieldName = null;

    /*
     * These are the keys that can be stored in a hashtable that the fillEvent
     * method interprets to create a BB event. The set of values works for all
     * events formats (SIF-E, VCal, ICal)
     */
    protected static final String             ALARM                       = "ReminderMinutesBeforeStart";
    protected static final String             ALARM_ABS_TIME              = "ReminderStart";
    protected static final String             ALARM_ENABLE                = "ReminderSet";
    protected static final String             ALLDAY                      = "AllDayEvent";
    protected static final String             BUSY_STATUS                 = "BusyStatus";
    protected static final String             END                         = "End";
    protected static final String             IS_RECURRING                = "IsRecurring";
    protected static final String             LOCATION                    = "Location";
    protected static final String             NOTE                        = "Body";
    protected static final String             SENSITIVITY                 = "Sensitivity";
    protected static final String             START                       = "Start";
    protected static final String             SUMMARY                     = "Subject";
    protected static final String             ATTENDEES                   = "Attendees";
    protected static final String             ATTENDEE                    = "Attendee";
    protected static final String             ATTENDEE_NAME               = "AttendeeName";
    protected static final String             ATTENDEE_EMAIL              = "AttendeeEmail";
    protected static final String             ATTENDEE_STATUS             = "AttendeeStatus";
    protected static final String             ATTENDEE_TYPE               = "AttendeeType";
    protected static final String             APPOINTMENT_TYPE_KEY        = "appointment";
    
    protected static final int                RECURS_DAILY                = 0;
    protected static final int                RECURS_WEEKLY               = 1;
    protected static final int                RECURS_MONTHLY              = 2;
    protected static final int                RECURS_MONTHLY_NTH          = 3;
    protected static final int                RECURS_YEARLY               = 5;
    protected static final int                RECURS_YEARLY_NTH           = 6;

    protected static final String             RECURRENCE_TYPE             = "RecurrenceType";
    protected static final String             INTERVAL                    = "Interval";
    protected static final String             MONTH_OF_YEAR               = "MonthOfYear";
    protected static final String             DAY_OF_MONTH                = "DayOfMonth";
    protected static final String             DAY_OF_WEEK_MASK            = "DayOfWeekMask";
    protected static final String             INSTANCE                    = "Instance";
    protected static final String             PATTERN_START_DATE          = "PatternStartDate";
    protected static final String             NO_END_DATE                 = "NoEndDate";
    protected static final String             PATTERN_END_DATE            = "PatternEndDate";
    protected static final String             OCCURRENCES                 = "Occurrences";
    protected static final String             EXCEPTIONS                  = "Exceptions";
    protected static final String             EXCLUDE_DATE                = "ExcludeDate";
    protected static final String             INCLUDE_DATE                = "IncludeDate";

    private static final int                  MONTHS[][]                  = {{1,RepeatRule.JANUARY}
                                                                             ,{2,RepeatRule.FEBRUARY}
                                                                             ,{3,RepeatRule.MARCH}
                                                                             ,{4,RepeatRule.APRIL}
                                                                             ,{5,RepeatRule.MAY}
                                                                             ,{6,RepeatRule.JUNE}
                                                                             ,{7,RepeatRule.JULY}
                                                                             ,{8,RepeatRule.AUGUST}
                                                                             ,{9,RepeatRule.SEPTEMBER}
                                                                             ,{10,RepeatRule.OCTOBER}
                                                                             ,{11,RepeatRule.NOVEMBER}
                                                                             ,{12,RepeatRule.DECEMBER}
                                                                          };
    private static final int                WEEKS[][]                     = {{1,RepeatRule.FIRST}
                                                                             ,{2,RepeatRule.SECOND}
                                                                             ,{3,RepeatRule.THIRD}
                                                                             ,{4,RepeatRule.FOURTH}
                                                                             ,{5,RepeatRule.FIFTH}
                                                                             ,{-5,RepeatRule.FIFTHLAST}
                                                                             ,{-4,RepeatRule.FOURTHLAST}
                                                                             ,{-3,RepeatRule.THIRDLAST}
                                                                             ,{-2,RepeatRule.SECONDLAST}
                                                                             ,{-1,RepeatRule.LAST}
                                                                          };
    private static final int                DAYS[][]                      = {{1,RepeatRule.SUNDAY}
                                                                             ,{2,RepeatRule.MONDAY}
                                                                             ,{4,RepeatRule.TUESDAY}
                                                                             ,{8,RepeatRule.WEDNESDAY}
                                                                             ,{16,RepeatRule.THURSDAY}
                                                                             ,{32,RepeatRule.FRIDAY}
                                                                             ,{64,RepeatRule.SATURDAY}
                                                                          };

        
    protected static final int[] SUPPORTED_FIELDS =
    {
        BlackBerryEvent.START
        ,BlackBerryEvent.END
        ,BlackBerryEvent.ALLDAY
        ,BlackBerryEvent.SUMMARY
        ,BlackBerryEvent.LOCATION
        ,BlackBerryEvent.NOTE
        ,BlackBerryEvent.ALARM
        ,BlackBerryEvent.CLASS
        ,BlackBerryEvent.FREE_BUSY
        ,BlackBerryEvent.ATTENDEES
    };
 
    
    public void init()
    {
        if (mailListener != null) {
            mailListener.attach();
            mailListener.unblockAllOutgoingSubjects();
        }
    }
    
    public void setIncomingMailListenerStatus(boolean enabled)
    {
        if (mailListener != null) {
            if (enabled)
                mailListener.enableBlockIncoming();
            else
                mailListener.disableBlockIncoming();
        }
    }
    
    public EventHelper(PIMList list, EventMailListener listener, int pastlimit, int futurelimit)
    {
        super(list);
        this.limitPast = pastlimit;
        this.limitFuture = futurelimit;
        
        // Creating EventMailListener to filter emails from events
        mailListener = listener;
        
        if (mailListener != null) {
            mailListener.setHelper(this);
            mailListener.enableBlockOutgoing();
            if (Configuration.getBlockIncomingInvites()) {
                mailListener.enableBlockIncoming();
            }
        }
    }
    
    public EventList getEventList()
    {
        PIMList list = getPIMList();
        if (list != null && list instanceof EventList)
        {
            return (EventList)list;
        }
        return null;
    }
    
    public void handleUpdatedItem(PIMItem item)
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)item;
            String subject = getItemString(event, BlackBerryEvent.SUMMARY, 0);
            if (subject == null) {
                subject = "";
            }
            mailListener.blockIncomingEvent(subject, event);
        }
    }
    
    public AdditionalItemsAttributes handleAddedItem(PIMItem item)
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)item;
            String subject = getItemString(event, BlackBerryEvent.SUMMARY, 0);
            if (subject == null) {
                subject = "";
            }
            mailListener.blockIncomingEvent(subject, event);
        }

        return null;
    }

    public void saveNewPIMItem(PIMItem pitem) throws PIMException
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)pitem;
            
            if (isSupportedField(BlackBerryEvent.ATTENDEES))
            {
                int size = event.countValues(BlackBerryEvent.ATTENDEES);
                if (size > 0)
                {
                    String subject = getItemString(event, BlackBerryEvent.SUMMARY, 0);
                    if (subject == null) {
                        subject = "";
                    }
                    mailListener.blockOutgoingEvent(subject, EventMailListener.NEW);
                }
            }
        }
        
        pitem.commit();
    }

    public void saveUpdatedPIMItem(PIMItem pitem) throws PIMException
    {
        if (mailListener != null) {
            BlackBerryEvent event = (BlackBerryEvent)pitem;
            
            if (isSupportedField(BlackBerryEvent.ATTENDEES))
            {
                int size = event.countValues(BlackBerryEvent.ATTENDEES);
                if (size > 0)
                {
                    String subject = getItemString(event, BlackBerryEvent.SUMMARY, 0);
                    if (subject == null) {
                        subject = "";
                    }
                    mailListener.blockOutgoingEvent(subject, EventMailListener.UPD);
                }
            }
        }
            
        pitem.commit();
    }
    
    
    public boolean deleteItem(PIMItem item) throws PIMException
    {
        EventList list = getEventList();
        if (list != null && item != null && item instanceof Event) {
            Event event = (Event)item;
            if (mailListener != null) {
                String subject = getItemString(event, BlackBerryEvent.SUMMARY, 0);
                if (subject == null) {
                    subject = "";
                }
                mailListener.blockOutgoingEvent(subject, EventMailListener.DEL);
            }
            list.removeEvent((Event)item);
            return true;
        }
        return false;
    }
    
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#createPIMItem(PIMList pimlist)
     */
    public PIMItem createItem() throws PIMException
    {
        EventList list = getEventList();
        if (list != null) {
            return list.createEvent();
        }
        return null;
    }
    
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof BlackBerryEvent)
        {
            try {
                return item.getString(BlackBerryEvent.UID,0);
            } catch(final Exception e) {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Calendar]";
                Log.error(msg);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Calendar] is not a BlackBerryEvent.";
        Log.error(msg);
        throw new PIMException(msg);
    }

    public boolean isInSyncRange(PIMItem item) throws PIMException
    {
        if(Customization.getInstance().syncInRange()){
            long NOW = System.currentTimeMillis();
            Calendar start = getFieldDate(item,BlackBerryEvent.START,false,false,false);
            Calendar end = null;
            RepeatRule rrule = ((BlackBerryEvent)item).getRepeat();
            if (rrule!=null) {
                end = SIFRepeatRuleHelper.getEndDate(rrule);
                if (end == null) {
                    end = Calendar.getInstance();
                    end.setTime(new Date(NOW));
                }
            } else {
                end = getFieldDate(item,BlackBerryEvent.END,true,false,false);
            }
            if (start!=null && end!=null) {
                if (end.getTime().getTime()< (NOW-this.limitPast*EventHelper.MONTH_FACTOR)
                    || start.getTime().getTime() > (NOW+this.limitFuture*EventHelper.MONTH_FACTOR))
                {
                    return false;
                }
            }
            return true;
        }else{
            return true;
        }
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(item,BlackBerryEvent.UID,SIFEventHelper.SUPPORTED_FIELDS);
        ((Event)item).setRepeat(null);
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item, BlackBerryEvent.UID,
                              SUPPORTED_FIELDS);
    }
 


    /**
     * This method fills an event, starting from a list of properties.
     * @param event the event to be filled
     * @param eventMap a map containing key/value for all the fields
     */
    protected Event fillEvent(Event event, Hashtable eventMap) throws PIMException {

        String fieldname = null;
        
        EventList list = getEventList();
        
        if (list == null) {
            return null;
        }
        
        try
        {
            long dateStart = 0;
            long dateStartForAlarm=0;
            long rruleOffset = 0;
            boolean allday = false;
            String field = null;
            RepeatRule rrule = null;

            fieldname = ALLDAY;
            //Log.debug("Setting event field ["+fieldname+"]");
            field = PIMItemHelper.getValue(eventMap, ALLDAY, false);
            if ("1".equals(field)) {
                // When the all day property is applied, the RIM's APIs move the
                // dates around, so we need to adjust them here to compensate
                allday = true;
            }
            
            fieldname = IS_RECURRING;
            //Log.debug("Setting event field ["+fieldname+"]");
            field = PIMItemHelper.getValue(eventMap, IS_RECURRING, false);// e.g. 1
            if (field.length()>0) {
                if (!"false".equalsIgnoreCase(field) && !"0".equals(field)) {
                    rrule = new RepeatRule();
                    setRepeatRule(eventMap,rrule,false,allday);
                }
            }

            if (rrule == null) {
                // Setting the ALLDAY property may change START and END (see
                // BlackBerryEvent for more information). Because of this, and because the
                // behavior of what gets moved is different for various OS
                // versions, we need to set the property before setting start/end (for non recurring events)
                fieldname = ALLDAY;
                //Log.debug("Setting event field ["+fieldname+"]");
                if (isSupportedField(BlackBerryEvent.ALLDAY)) {
                    event.addBoolean(BlackBerryEvent.ALLDAY, BlackBerryEvent.ATTR_NONE, allday);
                }
            }

            fieldname = START;
            //Log.debug("Setting event field ["+fieldname+"]");
            if (isSupportedField(BlackBerryEvent.START)) {
                field = PIMItemHelper.getValue(eventMap, START, false);
                if (field.length() > 0) {
                    if(allday) {
                        // For all day events we must set the start date to be a
                        // value "safe" value so that RIM api's adjustments will
                        // not change the day. But we need to preserve the
                        // original value (start of day) for alarm computation.
                        Calendar calStart = PIMItemHelper.parseDateTime(field);
                        String dataStart  = PIMItemHelper.formatAllDay(calStart);
                        Calendar cal      = PIMItemHelper.parseDate(dataStart);
                        // Save the original value for alarm computation
                        dateStartForAlarm = PIMItemHelper.getStartOfDay(cal).getTime().getTime();
                        // Compute a safe time to set the START
                        cal = DateUtil.getSafeTimeForAllDay(cal);
                        dateStart = cal.getTime().getTime();
                    } else {
                        dateStart = PIMItemHelper.parseDateTime(field).getTime().getTime();
                    }

                    if (dateStart < 0) {
                        /*
                        * if the year is before 1970, the year is set to
                        * 1970 for the BlackBerry device
                        */
                        dateStart = PIMItemHelper.adjustDateToEpoch(dateStart);
                    }
                    if (rrule != null) {
                        // In case of recurring events, the start date may be
                        // adjusted by the adjustDateFromRRule, therefore we
                        // must save the date Start for alarm
                        dateStartForAlarm=PIMItemHelper.parseDateTime(field).getTime().getTime();//
                        
                    }
                    long newDateStart = adjustDateFromRRule(rrule, dateStart);
                    rruleOffset = newDateStart-dateStart;
                    dateStart = newDateStart;

                    Log.debug("The Date that was set to StartDate ["+new Date(dateStart)+"]");
                    event.addDate(BlackBerryEvent.START, BlackBerryEvent.ATTR_NONE, dateStart);
                }
            }

            fieldname = END;
            //Log.debug("Setting event field ["+fieldname+"]");
            if (isSupportedField(BlackBerryEvent.END)) {
                field = PIMItemHelper.getValue(eventMap, END, true);
                if (field.length() > 0) {
                    long dateEnd = 0;
                    if(allday) {
                        Calendar calEnd = PIMItemHelper.parseDateTime(field);
                        String dataEnd  = PIMItemHelper.formatAllDay(calEnd);
                        Calendar cal    = PIMItemHelper.parseDateTime(dataEnd);
                        cal = DateUtil.getSafeTimeForAllDay(cal);
                        dateEnd = cal.getTime().getTime();
                        // All day event have an exclusive end date, so we must
                        // add one extra day
                        dateEnd += EventHelper.DAY_FACTOR;
                    } else {
                        dateEnd = PIMItemHelper.parseDateTime(field).getTime().getTime();
                    }
                        
                    if (dateEnd < 0) {
                        dateEnd = PIMItemHelper.adjustDateToEpoch(dateEnd);
                    }
                    // adjust the end date if the start date has been changed
                    dateEnd += rruleOffset;
                    event.addDate(BlackBerryEvent.END, BlackBerryEvent.ATTR_NONE, dateEnd);
                    Log.info("The Date that was set to EndDate ["+new Date(dateEnd)+"]");
                }
            }

            // Set the repeat rule after setting the start/end dates
            event.setRepeat(rrule);
            if (rrule != null) {
                // For recurring events we must set the all day after we set the
                // repeat rule, otherwise the events will be changed
                fieldname = ALLDAY;
                if (isSupportedField(BlackBerryEvent.ALLDAY)) {
                    event.addBoolean(BlackBerryEvent.ALLDAY, BlackBerryEvent.ATTR_NONE, allday);
                }
            }

            fieldname = SUMMARY;
            if (isSupportedField(BlackBerryEvent.SUMMARY)) {
                field = PIMItemHelper.getValue(eventMap, SUMMARY, true);
                if (event.countValues(BlackBerryEvent.SUMMARY)==0) {
                    event.addString(BlackBerryEvent.SUMMARY, BlackBerryEvent.ATTR_NONE, field);
                } else {
                    event.setString(BlackBerryEvent.SUMMARY, 0, BlackBerryEvent.ATTR_NONE, field);
                }
            }

            fieldname = LOCATION;
            if (isSupportedField(BlackBerryEvent.LOCATION)) {
                field = PIMItemHelper.getValue(eventMap, LOCATION, true);
                if (event.countValues(BlackBerryEvent.LOCATION)==0) {
                    event.addString(BlackBerryEvent.LOCATION, BlackBerryEvent.ATTR_NONE, field);
                } else {
                    event.setString(BlackBerryEvent.LOCATION, 0, BlackBerryEvent.ATTR_NONE, field);
                }
            }

            fieldname = NOTE;
            if (isSupportedField(BlackBerryEvent.NOTE)) {
                field = PIMItemHelper.getValue(eventMap, NOTE, true);
                // We must replace all \r\n sequences into single \n to avoid
                // empty lines in the BB UI
                field = StringUtil.replaceAll(field, "\r\n", "\n");
                event.addString(BlackBerryEvent.NOTE, BlackBerryEvent.ATTR_NONE, field);
            }
                
            fieldname = ALARM;
            if (isSupportedField(BlackBerryEvent.ALARM)) {
                field = PIMItemHelper.getValue(eventMap, ALARM_ENABLE, false);
                if ("1".equals(field)) {
                    field = PIMItemHelper.getValue(eventMap, ALARM, false);
                    if (field == null || field.length() == 0) {
                        field = PIMItemHelper.getValue(eventMap, ALARM_ABS_TIME, false);
                        if (field == null || field.length() == 0) {
                            field = "0";
                        } else {
                            Log.trace(TAG_LOG, "Alarm expressed with absolute time: " + field);
                            // Compute the number of minutes before the start
                            long alarmTime = PIMItemHelper.parseDateTime(field).getTime().getTime();
                            long delta = 0;
                            if (allday || rrule!=null){
                                delta = dateStartForAlarm - alarmTime;
                                Log.trace(TAG_LOG, "delta rrule allday=" + delta);
                            } else {
                                delta = dateStart - alarmTime;
                                Log.trace(TAG_LOG, "delta=" + delta);
                            }
                            
                            if (delta < 0) {
                                field = "0";
                            } else {
                                // Convert msecs to mins
                                delta = delta / (60 * 1000);
                                field = "" + delta;
                                Log.trace(TAG_LOG, "field=" + field);
                            }
                        }
                    }
                    int remindTime = (Integer.parseInt(field) * 60);
                    remindTime = remindTime < 0 ? -1*remindTime : remindTime;
                    event.addInt(BlackBerryEvent.ALARM, BlackBerryEvent.ATTR_NONE, remindTime);
                }
            }
            
            // bugfix 869a
            fieldname = SENSITIVITY;
            if (isSupportedField(BlackBerryEvent.CLASS)) {
                int sensitivity = 0;
                field = PIMItemHelper.getValue(eventMap, SENSITIVITY, false);
                
                if (field.length()==0) {
                    field = "0";
                }
                switch (Integer.parseInt(field))
                {
                    case 0:// olNormal
                        sensitivity = Event.CLASS_PUBLIC;
                        break;
                    case 2:// olPrivate
                        sensitivity = Event.CLASS_PRIVATE;
                        break;
                    case 3:// olConfidential
                        sensitivity = Event.CLASS_CONFIDENTIAL;
                        break;
                }
                event.addInt(BlackBerryEvent.CLASS, Event.ATTR_NONE, sensitivity);
            }

            // bugfix 869b BusyStatus
            fieldname = BUSY_STATUS;
            if (isSupportedField(BlackBerryEvent.FREE_BUSY)) {
                field = PIMItemHelper.getValue(eventMap, BUSY_STATUS, false);
                if (field.length()==0) {
                    field = ""+BlackBerryEvent.FB_BUSY;
                }
                int status = 0;
                switch (Integer.parseInt(field))
                {
                    case 0:// olFree
                        status = BlackBerryEvent.FB_FREE;
                        break;
                    case 1:// olTentative
                        status = BlackBerryEvent.FB_TENTATIVE;
                        break;
                    case 2:// olBusy
                        status = BlackBerryEvent.FB_BUSY;
                        break;
                    case 3:// olOutOfOffice
                        status = BlackBerryEvent.FB_OUT_OF_OFFICE;
                        break;
                }
                event.addInt(BlackBerryEvent.FREE_BUSY, BlackBerryEvent.ATTR_NONE, status);
            }

            fieldname = ATTENDEE;
            if (isSupportedField(BlackBerryEvent.ATTENDEES)) {
                Vector attendees = PIMItemHelper.getVectorValue(eventMap,ATTENDEE);
                int countAtts = attendees.size();
                for (int i=0;i<countAtts;i++) {
                    Hashtable attendee = (Hashtable)attendees.elementAt(i);
                    event.addString(BlackBerryEvent.ATTENDEES,BlackBerryEvent.ATTR_NONE,
                                    (String)attendee.get(ATTENDEE_EMAIL));
                }
            }
            return event;
        } catch (final Throwable e) {
            if (fieldname.equals(IS_RECURRING)) {
                fieldname = SIFRepeatRuleHelper.popSettingFieldName();
            }
            Log.error(TAG_LOG, "SIFEventHelper.addEvent() on field ["+fieldname+"]", e);
            throw new PIMException("Exception while setting value to Event field ["+fieldname+"]");
        }
    }

    private int parseInstance(int instance) throws PIMException
    {
        boolean found = false;
        int weekth = 0;
        for(int i=0;i<WEEKS.length;i++) {
            if (instance == WEEKS[i][0]) {
                weekth = WEEKS[i][1];
                found = true;
                break;
            }
        }
        if (!found) {
            throw new PIMException("Invalid value for SIF.Instance field ["+instance+"]");
        }
        return weekth;
    }
    
    private int parseMonthOfYear(int monthOfYear)
    {
        int month = 0;
        for(int i=0;i<MONTHS.length;i++) {
            if (monthOfYear == MONTHS[i][0]) {
                month = MONTHS[i][1];
                break;
            }
        }
        return month;
    }
    
    private int parseDayOfWeekMask(int dayofweekmask)
    {
        int mask = 0;
        for(int i=0;i<DAYS.length;i++) {
            if ((dayofweekmask & DAYS[i][0]) == DAYS[i][0]) {
                mask = mask + DAYS[i][1];
            }
        }
        return mask;
    }
    
    private String popSettingFieldName()
    {
        String ret = settingFieldName;
        settingFieldName = null;
        return ret;
    }

    private int getRecurrenceType(Hashtable eventMap)
    {
        final String recurrenceType = PIMItemHelper.getValue(eventMap, RECURRENCE_TYPE, false);// 1-6
        if (!"".equals(recurrenceType)) {
            return Integer.parseInt(recurrenceType);
        }
        return 0;
    }
    
    private boolean isWeeklyRecurrence(int recurType)
    {
        switch (recurType)
        {
            case RECURS_WEEKLY:
                return true;
        }
        return false;
    }

    private boolean isMonthlyNthRecurrence(int recurType)
    {
        switch (recurType)
        {
            case RECURS_MONTHLY_NTH:
                return true;
        }
        return false;
    }

    private boolean isYearlyNthRecurrence(int recurType)
    {
        switch (recurType)
        {
            case RECURS_YEARLY_NTH:
                return true;
        }
        return false;
    }

    public void setRepeatRule(Hashtable eventMap, RepeatRule rule,
                                     boolean extraGMTToLocal) throws PIMException
    {
        setRepeatRule(eventMap, rule, extraGMTToLocal, false);
    }
 

    public void setRepeatRule(Hashtable eventMap, RepeatRule rule,
                                     boolean extraGMTToLocal,
                                     boolean allDay) throws PIMException
    {
        // This is currently not supported by the RIM API
        // <PatternStartDate>
        // 20060706T220000Z
        // final String patternStartDate = PIMItemHelper.getValue(eventMap, PATTERN_START_DATE, false);

        // <RecurrenceType>
        settingFieldName = RECURRENCE_TYPE;
        final String recurrenceType = PIMItemHelper.getValue(eventMap, RECURRENCE_TYPE, false);// 1-6
        int intervalFactor = 1;
        int type = 0;
        int olRecType = -1;

        if (!"".equals(recurrenceType)) {
            olRecType = Integer.parseInt(recurrenceType);
        }

        Log.trace("olRecType=" + olRecType);

        if (olRecType != -1) {
            switch (olRecType)
            {
                case RECURS_DAILY:
                    type = RepeatRule.DAILY;
                    break;
                case RECURS_WEEKLY:
                    type = RepeatRule.WEEKLY;
                    break;
                case RECURS_MONTHLY:
                    type = RepeatRule.MONTHLY;
                    break;
                case RECURS_MONTHLY_NTH:
                    type = RepeatRule.MONTHLY;
                    break;
                case RECURS_YEARLY:
                    type = RepeatRule.YEARLY;
                    //type = RepeatRule.MONTHLY;
                    //intervalFactor = 12;
                    break;
                case RECURS_YEARLY_NTH:
                {
                    // On OS >= 4.2 there is a bug that prevents these events
                    // to be stored properly.
                    // With this patch we transform these events from yearly to
                    // monthly every 12 months
                    boolean patch = false;
                    String version[] = StaticDataHelper.getOSVersion();

                    if (version.length > 1) {
                        int major = Integer.parseInt(version[0]);
                        int minor = Integer.parseInt(version[1]);
                        if (major == 4 && minor >= 2) {
                            patch = true;
                        }
                    }

                    if (patch) {
                        Log.trace("Applying patch for recurring events on relative dates");
                        type = RepeatRule.MONTHLY;
                        intervalFactor = 12;
                        eventMap.remove(MONTH_OF_YEAR);
                        eventMap.put(MONTH_OF_YEAR, "");
                    } else {
                        type = RepeatRule.YEARLY;
                    }
                    break;
                }
                default:
                    Log.error("Unknown recurrence type: " + olRecType);
                    break;
            }
            Log.trace("Setting FREQUENCY to: " + type);
            rule.setInt(RepeatRule.FREQUENCY, type);
        }

        // <Interval>
        settingFieldName = INTERVAL;
        final String interval = PIMItemHelper.getValue(eventMap, INTERVAL, false);
        if (!"".equals(interval) && !"0".equals(interval)) {
            Log.trace("Setting INTERVAL to: " + Integer.parseInt(interval)*intervalFactor);
            rule.setInt(RepeatRule.INTERVAL, Integer.parseInt(interval)*intervalFactor);
        } else if (intervalFactor != 1) {
            // Even if there is no interval, but the interval factor got
            // adjusted, we must set the INTERVAL accordingly
            Log.trace("Setting INTERVAL to: " + intervalFactor);
            rule.setInt(RepeatRule.INTERVAL, intervalFactor);
        }

        // <DayOfWeekMask>
        settingFieldName = DAY_OF_WEEK_MASK;
        final String dayOfWeekMaskAsString = PIMItemHelper.getValue(eventMap, DAY_OF_WEEK_MASK, false);
        if (!"".equals(dayOfWeekMaskAsString)) {
            final int dayOfWeekMask = Integer.parseInt(dayOfWeekMaskAsString);
            int BBDow = parseDayOfWeekMask(dayOfWeekMask);
            if (BBDow != 0) {
                Log.trace("Setting DAY IN WEEK: " + BBDow);
                rule.setInt(RepeatRule.DAY_IN_WEEK, BBDow);
            }
        }

        // <Instance> or WEEK_IN_MONTH
        settingFieldName = INSTANCE;
        final String instance = PIMItemHelper.getValue(eventMap, INSTANCE, false);// e.g. 2
        if (!"".equals(instance) && !"0".equals(instance)) {
            Log.trace("Setting WEEK IN MONTH: " + instance);
            rule.setInt(RepeatRule.WEEK_IN_MONTH, parseInstance(Integer.parseInt(instance)));
        } else {
            // <DayOfMonth> 1-31
            settingFieldName = DAY_OF_MONTH;
            final String dayOfMonth = PIMItemHelper.getValue(eventMap, DAY_OF_MONTH, false);
            if (!"".equals(dayOfMonth) && Integer.parseInt(dayOfMonth) != 0) {
                Log.trace("Setting DAY IN MONTH: " + dayOfMonth);
                rule.setInt(RepeatRule.DAY_IN_MONTH, Integer.parseInt(dayOfMonth));
            }
        }
        
        // <MonthOfYear>
        settingFieldName = MONTH_OF_YEAR;
        final String monthOfYear = PIMItemHelper.getValue(eventMap, MONTH_OF_YEAR, false);
        int month = 0;
        if (!"".equals(monthOfYear)) {
            month = parseMonthOfYear(Integer.parseInt(monthOfYear));
            if (month!=0) {
                // There is a bug in OS 4.5+ where setting the month
                // for events that recur yearly causes a calendar corruption,
                // for events recurring in January only.
                // In this case we avoid setting the occurrence date, and just
                // use the DATE as occurrence date. This causes some info to be
                // dropped, but a better solution would require us to store
                // extra information.
                String version[] = StaticDataHelper.getOSVersion();
                boolean patch = false;
                if (month == RepeatRule.JANUARY && olRecType == RECURS_YEARLY && version.length > 1) {
                    int major = Integer.parseInt(version[0]);
                    int minor = Integer.parseInt(version[1]);
                    if (major == 4 && minor >= 5) {
                        Log.trace("RepeatRule patch will be applied, avoid setting MONTH_IN_YEAR");
                        patch = true;
                    }
                }
               
                // If we don't have to apply the patch, then we set all the
                // info, otherwise we just use the date as occurring date
                if (!patch) {
                    Log.trace("Setting MONTH IN YEAR: " + month + " " + monthOfYear);
                    rule.setInt(RepeatRule.MONTH_IN_YEAR, month);
                }
            }
        }

        // This is not managed in input
        // <NoEndDate>
        // final String noEndDate = PIMItemHelper.getValue(eventMap, NO_END_DATE, false);

        // <Occurences>
        settingFieldName = OCCURRENCES;
        final String occurrences = PIMItemHelper.getValue(eventMap, OCCURRENCES, false);
        if (!"".equals(occurrences)) {
            rule.setInt(RepeatRule.COUNT, Integer.parseInt(occurrences));
        } else {
            // The field value is currently limited to RepeatRule.END (RIM API)
            // <PatternEndDate>
            // 20111106T230000Z
            settingFieldName = PATTERN_END_DATE;
            final String patternEndDate = PIMItemHelper.getValue(eventMap, PATTERN_END_DATE, false);
            if (!"".equals(patternEndDate)) {
                if (allDay) {
                    Calendar date = PIMItemHelper.parseDateTime(patternEndDate);
                    date = DateUtil.getSafeTimeForAllDay(date);
                    long utcTime = date.getTime().getTime();
                    rule.setDate(RepeatRule.END, utcTime);
                } else {
                    long localTime = PIMItemHelper.parseDateTime(patternEndDate).getTime().getTime();
                    //localTime -=  PIMItemHelper.DAY_FACTOR;
                    if (extraGMTToLocal) {
                        localTime = PIMItemHelper.convertGMTToLocal(localTime);
                    }
                    rule.setDate(RepeatRule.END, localTime);
                }
            }
        }

        setExceptionDates(eventMap, allDay, rule);
    }

    private void setExceptionDates(Hashtable eventMap, boolean allDay, RepeatRule rule) {
        // <Exceptions>.<ExcludeDate>
        settingFieldName = EXCLUDE_DATE;
        Vector excludeDates = PIMItemHelper.getVectorValue(eventMap, EXCLUDE_DATE);
        for(int i=excludeDates.size()-1;0<=i;i--) {
            String tmp = (String)excludeDates.elementAt(i);
            if (tmp.length() > 0) {
                Log.trace(TAG_LOG, "Setting exception to: " + tmp);
                rule.addExceptDate(PIMItemHelper.parseDateTime(tmp).getTime().getTime());
            }
        }
        
        excludeDates = null;
    }
}
