/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.pim.Event;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;

import net.rim.blackberry.api.mail.BodyPart;
import net.rim.blackberry.api.mail.Folder;
import net.rim.blackberry.api.mail.Message;
import net.rim.blackberry.api.mail.MessagingException;
import net.rim.blackberry.api.mail.Multipart;
import net.rim.blackberry.api.mail.SendListener;
import net.rim.blackberry.api.mail.ServiceConfiguration;
import net.rim.blackberry.api.mail.Session;
import net.rim.blackberry.api.mail.Store;
import net.rim.blackberry.api.mail.TextBodyPart;
import net.rim.blackberry.api.mail.event.FolderEvent;
import net.rim.blackberry.api.mail.event.FolderListener;
import net.rim.blackberry.api.mail.event.StoreEvent;
import net.rim.blackberry.api.mail.event.StoreListener;
import net.rim.device.api.i18n.DateFormat;
import net.rim.device.api.servicebook.ServiceBook;
import net.rim.device.api.servicebook.ServiceRecord;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.SyncClient;
import com.funambol.util.Appender;
import com.funambol.util.Log;

/**
 * 
 */
public class EventMailListener implements SendListener, FolderListener, StoreListener {

    private static final String TAG_LOG = "EventMailListener";

    public static final String NEW    = "";

    public static final String UPD    = "Updated: ";

    public static final String DEL    = "Cancelled: ";

    private Hashtable          blockOutgoingSubjects;

    private Hashtable          blockIncomingEvents;

    private boolean            blockOutgoingEnabled;

    private boolean            blockIncomingEnabled;

    private EventHelper        helper = null;

    private String             inviteReplacementText;

    private String             eventDetailsText;

    private String             lastBlockedEmail;

    public EventMailListener(String repl) {
        this.inviteReplacementText = repl;

        blockOutgoingEnabled = false;
        blockOutgoingSubjects = new Hashtable();
        blockIncomingEvents = new Hashtable();

        blockIncomingEnabled = false;
        attach();
    }

    public void setHelper(EventHelper helper) {
        this.helper = helper;
    }

    private void reload() {

        Log.initLog(SyncClient.getAppender(), Log.DISABLED);

        // We must reload the config, because this function is called from a
        // different context, and the config might not be loaded cause its all
        // static
        Configuration.load();

        Log.setLogLevel(Configuration.getLogLevel());
        if (Configuration.getBlockIncomingInvites()) {
            Log.info(TAG_LOG, "Incoming invite block enabled");
            enableBlockIncoming();
        } else {
            Log.info(TAG_LOG, "Incoming invite block disabled");
            disableBlockIncoming();
        }

        Log.info(TAG_LOG, "Outgoing invite block enabled");
        enableBlockOutgoing();
    }

    public void batchOperation(StoreEvent e) {

        reload();

        Log.info(TAG_LOG, "batchOperation");
    }

    public void messagesAdded(FolderEvent e) {

        reload();

        Log.trace(TAG_LOG, "Messages added");

        Message m = e.getMessage();

        if (m == null) {
            Log.trace(TAG_LOG, "Message is null!");
            return;
        }

        if (Log.getLogLevel() >= Log.TRACE) {
            StringBuffer details = new StringBuffer();
            details.append("[EventMailListener.messagesAdded] Mail details: \n");
            details.append("Subject: ").append(m.getSubject()).append("\n");
            details.append("ContentType: ").append(m.getContentType()).append("\n");
            details.append("BodyText: ").append(m.getBodyText()).append("\n");
            details.append("IsInbound: ").append(m.isInbound()).append("\n");

            Log.trace(TAG_LOG, details.toString());
        }

        if (m.isInbound()) {

            Log.info(TAG_LOG, "Incoming message received - " + m.getSubject());

            Object parts = m.getContent();

            if (blockIncomingEnabled) {

                Log.info(TAG_LOG, "Message blocking enabled");

                if (parts == null) {
                    Log.trace(TAG_LOG, "Message has no parts - returning");
                    return;
                }

                if (parts instanceof Multipart) {

                    Multipart mp = (Multipart) parts;
                    int count = mp.getCount();

                    if (Log.getLogLevel() == Log.TRACE) {
                        StringBuffer details = new StringBuffer();
                        details.append("[EventMailListener.messagesAdded] MultiPart Info : \n")
                                .append("contentType: ").append(mp.getContentType()).append("\n")
                                .append("toString: ").append(mp.toString()).append("\n").append(
                                        "Count: ").append(count).append("\n").append("Parent: ")
                                .append(mp.getParent().toString()).append("\n");

                        Log.info(TAG_LOG, details.toString());
                    }

                    if (count == 0) {
                        Log
                                .info("[EventMailListener.messagesAdded] Multipart Message has no parts - likely an event");

                        if (isIncomingEventBlocked(m.getSubject())) {
                            Log.info(TAG_LOG, "Attempting to delete event from incoming email: "
                                    + m.getSubject());
                            if (manageBlockedIncomingEvent(m.getSubject())) {
                                clearBlockedIncomingEvents();

                                String text = "\n" + inviteReplacementText + "\n\n"
                                        + eventDetailsText;

                                BodyPart txt = new TextBodyPart(mp, text);
                                mp.addBodyPart(txt);
                                try {
                                    m.setContent(mp);
                                } catch (MessagingException ex) {
                                    Log
                                            .info("[EventMailListener.messagesAdded] Unable to add new part");
                                }
                            } else {
                                Log.info(TAG_LOG, "Did not remove event");
                            }
                        }
                    } else {
                        Log
                                .info("[EventMailListener.messagesAdded] Multipart Message has parts - I dont know what to do!");
                    }
                }
            }
        } else {
            Log.trace(TAG_LOG, "Message is not inbound");
        }
        Log.trace(TAG_LOG, "Complete");
    }

    public void messagesRemoved(FolderEvent e) {
        // Log.info("--Messages Removed");
    }

    public boolean sendMessage(Message message) {

        reload();

        String subject = message.getSubject();

        Log.info(TAG_LOG, "Device is attempting to send message with subject "
                        + subject);
        
        boolean send = true;

        if (blockOutgoingEnabled) {
            Log.info(TAG_LOG, "Outgoing message blocking is enabled");

            boolean block = isOutgoingSubjectBlocked(subject);

            if (block) {
                lastBlockedEmail = subject;

                Log.info(TAG_LOG, "BLOCKING outgoing email for subject: "
                        + subject);

                if (!message.getFolder().deleteMessage(message)) {
                    Log.error(TAG_LOG, "Unable to delete outgoing email: " + subject);
                }
                
                send = false;
            }
        }

        Log.info(TAG_LOG, "sendMessage Complete");

        return send;
    }

    public void enableBlockOutgoing() {
        blockOutgoingEnabled = true;
    }

    public void disableBlockOutgoing() {
        blockOutgoingEnabled = false;
    }

    public void enableBlockIncoming() {
        blockIncomingEnabled = true;
    }

    public void disableBlockIncoming() {
        blockIncomingEnabled = false;
    }

    public void blockOutgoingEvent(String subject, String type) {
        blockOutgoingSubject(subject);
    }

    public void unblockOutgoingEvent(String subject, String type) {
        unblockOutgoingSubject(subject);
    }

    public void unblockAllOutgoingSubjects() {
        synchronized (blockOutgoingSubjects) {
            blockOutgoingSubjects.clear();
        }
    }

    /*
     * private boolean isIncomingTypeBlocked(String type) { if (type == null)
     * return false; boolean ret = false; synchronized(blockIncomingTypes) { if
     * (blockIncomingTypes.contains(type)) { ret = true; } } return ret; }
     */
    /*
     * private boolean isIncomingExtensionBlocked(String filename) { if
     * (filename == null) return false; boolean ret = false;
     * synchronized(blockIncomingExtensions) { int index =
     * filename.indexOf("."); if (index > 0) { String ext =
     * filename.substring(index); if (blockIncomingExtensions.contains(ext)) {
     * ret = true; } } } return ret; }
     */

    private boolean isOutgoingSubjectBlocked(String subject) {
        if (subject == null)
            return false;

        boolean ret = false;
        synchronized (blockOutgoingSubjects) {
            Log.debug(TAG_LOG, "Checking if '" + subject + "' is blocked from being sent");
            Enumeration subjects = blockOutgoingSubjects.keys();
            while (subjects.hasMoreElements()) {
                String blocked = (String) subjects.nextElement();
                Log.trace(TAG_LOG, "Checking if '" + subject + "' ends with '" + blocked + "'");
                if (subject.endsWith(blocked)) {
                    Long blockedUntil = (Long)(blockOutgoingSubjects.get(blocked));
                    if (blockedUntil.longValue() > System.currentTimeMillis()) {
                        ret = true;
                        break;
                    } else {
                        blockOutgoingSubjects.remove(blocked);
                    }
                }
            }

            Log.debug(TAG_LOG, "No more subjects to check");
        }

        if (ret) {
            Log.debug(TAG_LOG, "Message is blocked");
        } else {
            Log.debug(TAG_LOG, "Message is not blocked");
        }

        return ret;
    }

    private boolean isIncomingEventBlocked(String subject) {
        Log.info(TAG_LOG, "Subject = " + subject);
        if (subject == null) {
            Log.info(TAG_LOG, "null subject");
            return false;
        }

        boolean ret = false;
        synchronized (blockIncomingEvents) {
            Enumeration subjects = blockIncomingEvents.keys();
            while (subjects.hasMoreElements()) {
                String blocked = (String) subjects.nextElement();
                Log.trace(TAG_LOG, "Checking if '" + subject + "' ends with '" + blocked + "'");
                if (subject.endsWith(blocked)) {
                    ret = true;
                    Log
                            .info("[EventMailListener.isIncomingEventBlocked] Incoming invite *is* blocked: "
                                    + subject);
                    break;
                }
            }

            Log.info(TAG_LOG, "No more subjects to check");
        }
        return ret;
    }

    private boolean manageBlockedIncomingEvent(String subject) {
        lastBlockedEmail = subject;

        Log
                .info("[EventMailListener.manageBlockedIncomingEvent] Managing blocked event "
                        + subject);

        boolean ret = false;
        synchronized (blockIncomingEvents) {

            PIMItem item = null;
            Enumeration subjects = blockIncomingEvents.keys();
            while (subjects.hasMoreElements()) {
                String blocked = (String) subjects.nextElement();
                if (subject.endsWith(blocked)) {
                    item = (PIMItem) blockIncomingEvents.get(blocked);
                    break;
                }
            }

            eventDetailsText = new String("");

            if (item != null) {
                Calendar temp;
                if (helper.isSupportedField(Event.SUMMARY)) {
                    try {
                        eventDetailsText += "Subject: " + item.getString(Event.SUMMARY, 0) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if (helper.isSupportedField(Event.LOCATION)) {
                    try {
                        eventDetailsText += "Location: " + item.getString(Event.LOCATION, 0) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if ((temp = helper.getFieldDate(item, Event.START, false, false, false)) != null) {
                    try {
                        eventDetailsText += "Start: "
                                + DateFormat.getInstance(
                                        DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT).format(
                                        new Date(PIMItemHelper.convertGMTToLocal(PIMItemHelper
                                                .getTimestamp(temp)))) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if ((temp = helper.getFieldDate(item, Event.END, false, false, false)) != null) {
                    try {
                        eventDetailsText += "End: "
                                + DateFormat.getInstance(
                                        DateFormat.DATE_MEDIUM | DateFormat.TIME_SHORT).format(
                                        new Date(PIMItemHelper.convertGMTToLocal(PIMItemHelper
                                                .getTimestamp(temp)))) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if (helper.isSupportedField(Event.NOTE)) {
                    try {
                        eventDetailsText += "Notes: " + item.getString(Event.NOTE, 0) + "\n";
                    } catch (final Exception e) {
                    }
                }
                if (((Event) item).getRepeat() != null)
                    eventDetailsText += "This event is recurring." + "\n";

                Log
                        .info("[EventMailListener.manageBlockedIncomingEvent] Attemptng to delete event");
                try {
                    helper.deleteItem(item);
                    ret = true;
                } catch (PIMException e) {
                    String msg = helper.handlePIMException(e);
                    Log.error(TAG_LOG, "Error removing event from email: " + msg, e);
                }
            } else {
                Log.info(TAG_LOG, "PIMItem is null - cannot handle block");
            }
        }
        return ret;
    }

    private void clearBlockedIncomingEvents() {
        synchronized (blockIncomingEvents) {
            Log.info(TAG_LOG, "Clearing blocked incoming event list");
            blockIncomingEvents.clear();
        }
    }

    private void blockOutgoingSubject(String subject) {
        synchronized (blockOutgoingSubjects) {

            boolean found = false;
            Enumeration subjects = blockOutgoingSubjects.keys();
            while (subjects.hasMoreElements()) {
                String blocked = (String) subjects.nextElement();
                if (subject.endsWith(blocked)) {
                    Long blockedUntil = (Long)(blockOutgoingSubjects.get(blocked));
                    if (blockedUntil.longValue() > System.currentTimeMillis()) {
                        found = true;
                        break;
                    } else {
                        blockOutgoingSubjects.remove(blocked);
                    }
                }
            }

            if (!found) {
                Log.debug(TAG_LOG, "Registering outgoing subject to be blocked: " + subject);
                blockOutgoingSubjects.put(subject, new Long(System.currentTimeMillis() + (5 * 60000))); // 5 Minutes
            }
        }
    }

    private void unblockOutgoingSubject(String subject) {
        synchronized (blockOutgoingSubjects) {
            Log.debug(TAG_LOG, "Unblocking outgoing subject: " + subject);

            boolean found = false;
            String toRemove = null;
            Enumeration subjects = blockOutgoingSubjects.keys();
            while (subjects.hasMoreElements()) {
                String blocked = (String) subjects.nextElement();
                if (subject.endsWith(blocked)) {
                    found = true;
                    toRemove = blocked;
                    break;
                }
            }

            if (found) {
                blockOutgoingSubjects.remove(toRemove);
            }
        }
    }

    public void blockIncomingEvent(String subject, PIMItem item) {
        synchronized (blockIncomingEvents) {
            clearBlockedIncomingEvents();
            Log.debug(TAG_LOG, "Registering incoming subject to be blocked: " + subject);
            blockIncomingEvents.put(subject, item);
        }
    }

    public void unblockIncomingEvent(String subject) {
        synchronized (blockIncomingEvents) {
            if (!blockIncomingEvents.contains(subject)) {
                Log.debug(TAG_LOG, "Unblocking incoming subject: " + subject);
                blockIncomingEvents.remove(subject);
            }
        }
    }

    public void attach() {
        reload();

        Log.info(TAG_LOG, "Mail listener attaching");
        Store mailStore = null;

        ServiceBook sb = ServiceBook.getSB();
        ServiceRecord[] srs = sb.getRecords();

        for (int cnt = srs.length - 1; cnt >= 0; --cnt) {

            // identify the service record associated with a mail message
            // service via a CID of 'CMIME'
            if (srs[cnt].getCid().equals("CMIME")) {
                ServiceConfiguration sc = new ServiceConfiguration(srs[cnt]);
                mailStore = null;
                Session sess = Session.getInstance(sc);
                if (sess != null) {
                    mailStore = sess.getStore();
                }

                if (mailStore == null) {
                    Log.error(TAG_LOG, "Unable to get mail store for " + srs[cnt].getName());
                } else {
                    attachSendListener(mailStore);
                    attachFolderListener(mailStore);
                    attachStoreListener(mailStore);
                }
            }
        }

        mailStore = null;
        Session sess = Session.getDefaultInstance();
        if (sess != null) {
            mailStore = sess.getStore();
        }

        if (mailStore == null) {
            Log.error(TAG_LOG, "Unable to get default mail store");
        } else {
            attachSendListener(mailStore);
            attachFolderListener(mailStore);
            attachStoreListener(mailStore);
        }

        Log.info(TAG_LOG, "Mail listeners attach complete");
    }

    private void attachSendListener(Store mailStore) {
        Log.trace(TAG_LOG, "Attaching send listener");
        // Remove the listeners in case we're attaching for a second time
        try {
            mailStore.removeSendListener(this);
        } catch (Exception e) {
        }
        mailStore.addSendListener(this);
    }

    private void attachFolderListener(Store mailStore) {
        Log.trace(TAG_LOG, "Attaching folder listener");
        try {
            mailStore.removeFolderListener(this);
        } catch (Exception e) {
        }
        mailStore.addFolderListener(this);

        // And to each folder
        Folder[] folders;
        folders = mailStore.list();
        for (int foldercnt = folders.length - 1; foldercnt >= 0; --foldercnt) {
            Folder f = folders[foldercnt];
            attachFolderListenerRecursive(f);
        }
    }

    private void attachFolderListenerRecursive(Folder f) {
        Log.trace(TAG_LOG, "Attaching folder listener to: " + f.getFullName());
        try {
            f.removeFolderListener(this);
        } catch (Exception e) {
        }
        f.addFolderListener(this);

        Folder[] farray = f.list();
        for (int fcnt = farray.length - 1; fcnt >= 0; --fcnt) {
            attachFolderListenerRecursive(farray[fcnt]);
        }
    }

    private void attachStoreListener(Store mailStore) {
        Log.trace(TAG_LOG, "Attaching store listener");
        try {
            mailStore.removeStoreListener(this);
        } catch (Exception e) {
        }
        mailStore.addStoreListener(this);
    }

    /**
     * @return the lastSentSubject
     */
    public String getLastBlockedEmail() {
        return lastBlockedEmail;
    }

    /**
     * @return the lastSentSubject
     */
    public void cearLastBlockedEmail() {
        lastBlockedEmail = null;
    }

}
