/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.ToDo;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.blackberry.source.AdditionalItemsAttributes;
import com.funambol.blackberry.source.CalendarAdditionalAttributes;
import com.funambol.common.pim.icalendar.ICalendarParserListener;
import com.funambol.common.pim.icalendar.ICalendarFormatter;
import com.funambol.common.pim.icalendar.ICalendarSyntaxParser;
import com.funambol.common.pim.icalendar.VAlarm;
import com.funambol.common.pim.icalendar.ICalendar;
import com.funambol.common.pim.vcalendar.CalendarUtils;
import com.funambol.common.pim.PimUtils;
import com.funambol.util.StringUtil;
import com.funambol.util.Log;

/**
 * This class extends the EventHelper to help on iCalendar items handling.
 * It redefines the iCalendar parser & formatter in order to save additional
 * fields not directly supported by the basic JSR75 implementation. These fields
 * are saved into the BlackBerry PIMItem object if possible, otherwise into an
 * AdditionalItemsAttributes object.
 */
public class ICalTaskHelper extends TaskHelper {
    
    public static final int  BB_STATUS = PIMItem.EXTENDED_FIELD_MIN_VALUE + 9;
    public static final int  BB_ALARM  = 100;

    private static final int[] SUPPORTED_FIELDS = {
        BlackBerryToDo.SUMMARY,
        BlackBerryToDo.NOTE,
        BlackBerryToDo.COMPLETED,
        BlackBerryToDo.DUE,
        BlackBerryToDo.COMPLETION_DATE,
        BlackBerryToDo.PRIORITY,
        BlackBerryToDo.CLASS,
        ICalTaskHelper.BB_STATUS,
        ICalTaskHelper.BB_ALARM
    };

    private CalendarAdditionalAttributes additionalAttributes = null;

    /**
     * This class extends the default ICalendarParserListener in order to save
     * additional fields not directly supported by the basic JSR75
     * implementation.
     */
    private class BBTaskParserListener extends ICalendarParserListener {

        public BBTaskParserListener(ToDo event) {
            super(event);
        }

        protected void setTZID(String value) {
            additionalAttributes.setTZID(value);
        }

        protected void setTZOffset(long offset) {
            additionalAttributes.setTZOffset(offset);
        }

        protected void setTaskAlarm(VAlarm alarm) {
            Log.trace("[BBTaskParserListener.setTaskAlarm]");
            alarm.setCalStartAbsoluteTime(alarmStartRelatedTime);
            int interval = (int)alarm.getAlarmInterval()/1000;
            if(interval != CalendarUtils.UNDEFINED_TIME) {
                if (pimList.isSupportedField(ICalTaskHelper.BB_ALARM)) {
                    pimItem.addInt(ICalTaskHelper.BB_ALARM, 0, interval);
                }
                else {
                    additionalAttributes.setAlarmInterval(interval);
                }
            }
            else {
                Log.error("[BBTaskParserListener.setTaskAlarm] Failed to get alarm interval");
            }
        }
        protected void setStatus(String value) {
            Log.trace("[BBTaskParserListener.setStatus]");
            if (pimList.isSupportedField(ICalTaskHelper.BB_STATUS)) {
                int status = 0; // 
                if(StringUtil.equalsIgnoreCase(value, ICalendar.STATUS_NEEDS_ACTION)) {
                    status = 0; // STATUS_NOT_STARTED
                } else if(StringUtil.equalsIgnoreCase(value, ICalendar.STATUS_IN_PROCESS)) {
                    status = 1; // STATUS_IN_PROGRESS
                } else if(StringUtil.equalsIgnoreCase(value, ICalendar.STATUS_COMPLETED)) {
                    status = 2; // STATUS_COMPLETED
                } else if(StringUtil.equalsIgnoreCase(value, ICalendar.STATUS_CANCELLED)) {
                    status = 4; // STATUS_DEFERRED
                }
                pimItem.addInt(ICalTaskHelper.BB_ALARM, 0, status);
            }
            else {
                super.setStatus(value);
            }
        }
    }

    /**
     * This class extends the default ICalendarFormatter in order to format
     * additional fields not directly supported by the basic JSR75
     * implementation.
     */
    private class BBTaskFormatter extends ICalendarFormatter {

        private CalendarAdditionalAttributes additionals;

        public BBTaskFormatter(String defaultCharset, CalendarAdditionalAttributes add) {
            super(defaultCharset);
            additionals = add;
        }
        protected String getTZID(PIMItem pimItem) {
            String tzid = additionals.getTZID();
            Log.trace("[BBTaskFormatter.getTZID] Value: " + tzid);
            return tzid;
        }
        protected long getTZOffset(PIMItem pimItem) {
            long tzoffset = additionals.getTZOffset();
            Log.trace("[BBTaskFormatter.getTZOffset] Value: " + tzoffset);
            return tzoffset;
        }
        protected int getTaskAlarmInterval(PIMItem pimItem) {
            if (isSupported(pimItem, ICalTaskHelper.BB_ALARM) &&
                pimItem.countValues(ICalTaskHelper.BB_ALARM) > 0) {
                return pimItem.getInt(ICalTaskHelper.BB_ALARM, 0);
            }
            else {
                return additionalAttributes.getAlarmInterval();
            }
        }
        protected void formatStatus(PIMItem pimItem, OutputStream os) throws PIMException {
            Log.trace("[BBTaskFormatter.formatStatus]");
            if (isSupported(pimItem, ICalTaskHelper.BB_STATUS)) {
                int status = pimItem.getInt(ICalTaskHelper.BB_STATUS, 0);
                String status_value = ICalendar.STATUS_IN_PROCESS;
                if(status == 0) {
                    status_value = ICalendar.STATUS_NEEDS_ACTION;
                } else if(status == 1) {
                    status_value = ICalendar.STATUS_IN_PROCESS;
                } else if(status == 2) {
                    status_value = ICalendar.STATUS_COMPLETED;
                } else if(status == 4) {
                    status_value = ICalendar.STATUS_CANCELLED;
                }
                PimUtils.println(os, ICalendar.STATUS + ":" + status_value);
            } else {
                super.formatStatus(pimItem, os, true);
            }
        }
    }

    /**
     * Constructor
     */
    public ICalTaskHelper(PIMList list) {
        super(list);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#itemToString(PIMItem item, AdditionalItemsAttributes additionals)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException {
        if (item instanceof ToDo) {
            ByteArrayOutputStream ostream = new ByteArrayOutputStream();
            BBTaskFormatter formatter =  new BBTaskFormatter(ICalendar.UTF8, (CalendarAdditionalAttributes)additionals);
            formatter.format(item, ostream, true);
            return ostream.toString();
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#fillItem(PIMItem item, String pimString, AdditionalItemsAttributes additionals)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString, AdditionalItemsAttributes additionals) throws PIMException {
        
        Log.trace("[ICalTaskHelper.fillItem]");
        if(additionals == null){
            Log.debug("[ICalTaskHelper.fillItem] Additionals null, creating new one");
            additionalAttributes = new CalendarAdditionalAttributes();
        } else {
            additionalAttributes = (CalendarAdditionalAttributes) additionals;
        }
        if (item instanceof ToDo) {
            ByteArrayInputStream is = new ByteArrayInputStream(pimString.getBytes());
            try {
                ToDo newTask = (ToDo)item;
                BBTaskParserListener listener = new BBTaskParserListener(newTask);
                ICalendarSyntaxParser parser = new ICalendarSyntaxParser(is);
                parser.setListener(listener);
                parser.parse();
            } catch (Exception pe) {
                Log.debug("[ICalTaskHelper.fillItem] Exception while parsing the item " + pe.toString());
                throw new PIMException(pe.toString());
            }
        }
        return additionalAttributes;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem _item) throws PIMException {
        PIMItemHelper.clearPIMItem(_item, BlackBerryToDo.UID, SUPPORTED_FIELDS);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#copyUnsupportedFields(PIMItem origItem, PIMItem item)
     */
    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException {
        copyUnsupportedFields(origItem, item, BlackBerryToDo.UID, SUPPORTED_FIELDS);
    }

    public AdditionalItemsAttributes createAdditionals(PIMItem newItem){
        Log.trace("[ICalTaskHelper.createAdditionals]");
        if (newItem instanceof ToDo) {
            additionalAttributes = new CalendarAdditionalAttributes();
            return additionalAttributes;
        }
        return null;
    }
}
    

   
