/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.TimeZone;
import java.util.Vector;
import java.util.Enumeration;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;

import javax.microedition.pim.RepeatRule;
import net.rim.device.api.util.StringUtilities;

import com.funambol.common.pim.PimUtils;
import com.funambol.util.SimpleXMLParser;
import com.funambol.util.XmlUtil;
import com.funambol.util.Log;

import com.funambol.blackberry.source.AdditionalItemsAttributes;

public abstract class PIMItemHelper 
{
    public static final String      EMPTY_STRING        = "";
    
    protected boolean deleteForUpdate;
    
    private PIMList pimlist;
    
    public PIMItemHelper(PIMList list)
    {
        pimlist = list;
        this.deleteForUpdate = false;
    }
    
    protected PIMList getPIMList()
    {
        return pimlist;
    }
    
    public void init()
    {
        
    }
    
    public void setDeleteForUpdate(boolean flag)
    {
        this.deleteForUpdate = flag;
    }
    
    public boolean getDeleteForUpdate()
    {
        return this.deleteForUpdate;
    }

    /**
     * Invoked at the end of the synchronization (in endSync) so that
     * the helper can perform its cleanup operations
     */
    public void tearDown() {
    }
    
    /**
     * This function will try to convert a PIMItem to appropriate SIF format.
     * @param item The PIMItem will be converted
     * @param pimList The PIMList will be used for checking if a Property is supported
     * @return An XML - SIF string or null.
     */
    public abstract String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException;
    
    /**
     * This function will try to get the UID string of provided PIMItem.
     * @param item The PIMItem
     * @return A string which is the UID of the provided PIMItem or null
     */
    public abstract String getUID(PIMItem item) throws PIMException;
    
    /**
     * The function will try to create a new-blank PIMItem
     * @return A new-blank PIMItem or null
     */
    public abstract PIMItem createItem() throws PIMException;
    
    /**
     * This function will parse a XML - SIF string and set the values to provided PIMItem.
     * @param item A PIMItem 
     * @param pimString The XML-SIF format string
     */
    public abstract AdditionalItemsAttributes fillItem(PIMItem item, String pimString, AdditionalItemsAttributes additionals) throws PIMException;

    /**
     * This function will clear off all properties of a PIMItem with a value.
     * @param _item The PIMItem that will have all properties cleared off.
     */
    public abstract void clearItem(PIMItem _item) throws PIMException;

    /**
     * This function copy fields from origItem to item. The method copies only
     * the fields which are not supported by the synchronization.
     *
     * @param origItem The source of the copy
     * @param item The target of the copy
     */
    public abstract void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException;
 
    
    /**
     * Remove the given item from the list
     * @param item        
     */
    public abstract boolean deleteItem(PIMItem item) throws PIMException;

    /**
     * Determine if the item is within the specific sync range
     * @param item
     * @return true if in range
     * @throws PIMException
     */
    public abstract boolean isInSyncRange(PIMItem item) throws PIMException;

    
    public Enumeration getItemsList() throws PIMException
    {
        return pimlist.items();
    }

    public PIMItem findItem(String itemKey) throws PIMException
    {
        if (pimlist != null)
        {
            PIMItem item = null;
            Enumeration items = pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                if (itemKey.equals(this.getUID(item)))
                {
                    return item;
                }
            }
        }
        return null;
    }

    
    public Hashtable findItems(Vector itemKeys) throws PIMException
    {
        Hashtable ret = new Hashtable();
        
        if (pimlist != null)
        {
            PIMItem item = null;
            Enumeration items = pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                String uid = this.getUID(item);
                if (itemKeys.contains(uid))
                {
                    ret.put(uid,item);
                    itemKeys.removeElement(uid);
                }
            }
        }
        
        return ret;
    }

    public Hashtable deleteItems(Hashtable items) throws PIMException
    {
        Hashtable ret = new Hashtable();
        Enumeration keys = items.keys();
        while (keys.hasMoreElements())
        {
            String key = (String)keys.nextElement();
            if (!deleteItem((PIMItem)items.get(key)))
            {
                ret.put(key, items.get(key));
            }
        }
        return ret;
    }
    
    public PIMItem deleteItemFromKey(String itemKey) throws PIMException
    {
        PIMItem item = findItem(itemKey);
        deleteItem(item);
        return item; 
    }
    
    public Vector deleteItemsFromKeys(Vector itemKeys) throws PIMException
    {
        Hashtable items = findItems(itemKeys);
        items = deleteItems(items);
        
        Vector ret = new Vector();
        
        Enumeration keys = items.keys();
        while (keys.hasMoreElements())
        {
            String key = (String)keys.nextElement();
            ret.addElement(key);
        }

        return ret;
    }


    public void deleteAllItems() throws PIMException
    {
        if (pimlist != null)
        {
            PIMItem item = null;
            Enumeration items = pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                this.deleteItem(item);
            }
        }
    }

    public void saveNewPIMItem(PIMItem pitem) throws PIMException
    {
        pitem.commit();
    }
    
    public void saveUpdatedPIMItem(PIMItem pitem) throws PIMException
    {
        pitem.commit();
    }
    
    public AdditionalItemsAttributes handleAddedItem(PIMItem item)
    {
        return null;
    }

    public AdditionalItemsAttributes updateAdditionals(PIMItem item, AdditionalItemsAttributes additionals)
    {
        return null;
    }

    public AdditionalItemsAttributes createAdditionals(PIMItem item)
    {
        return null;
    }
    
    public void handleUpdatedItem(PIMItem item)
    {
        
    }
    
    public void handleDeletedItem(PIMItem item)
    {
        
    }
   
    public static void clearPIMItem(PIMItem _item, int _UID, int[] fieldIDs) throws PIMException
    {
        int oldvalues[] = _item.getFields();
        for(int i=0;i<oldvalues.length;i++)
        {
            if (_UID != oldvalues[i])
            {
                for (int t=0;t<fieldIDs.length;t++)
                {
                    if (oldvalues[i] == fieldIDs[t])
                    {
                        int count = _item.countValues(oldvalues[i]);
                        Log.debug("Field ID ["+oldvalues[i]+"] has ["+count+"] values");
                        for (int j=count-1;0<=j;j--)
                        {
                            // This operation on OS 4.5 may throw an
                            // IndexOutOfBoundsException for apparently no
                            // good reason. On OS 4.5 we replace items by
                            // deleting them and recreating them. If this
                            // happens on other versions of the OS, then we need
                            // to apply this strategy on other versions as well
                            Log.trace("Removing value ["+j+"] of Field ID ["+oldvalues[i]+"]");
                            _item.removeValue(oldvalues[i],j);
                            Log.trace("Value removed");
                        }
                        count = _item.countValues(oldvalues[i]);
                        Log.debug("After deletion Field ID ["+oldvalues[i]+"] has ["+count+"] values");
                        break;
                    }
                }
            }
        }
        String oldCategories[] = _item.getCategories();
        for(int i=0;i<oldCategories.length;i++)
        {
            _item.removeFromCategory(oldCategories[i]);
            Log.debug("Remove old category ["+oldCategories[i]+"]");
        }
        oldvalues = null;
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item,
                                      int uid, int supportedFields[]) throws PIMException
    {
        int oldvalues[] = origItem.getFields();
        for(int i=0;i<oldvalues.length;i++)
        {
            boolean supported = false;
            if (oldvalues[i] != uid) {
                for(int j=0;j<supportedFields.length;++j) {
                    if (supportedFields[j] == oldvalues[i]) {
                        supported = true;
                        break;
                    }
                }
            }

            if (!supported) {
                Log.debug("An item being updated has unsupported fields, copy it over");
                int fieldType = origItem.getPIMList().getFieldDataType(oldvalues[i]);
                int count     = origItem.countValues(oldvalues[i]);
                Log.debug("count = " + count);
                switch (fieldType) {
                    case PIMItem.INT:
                        Log.debug("INT");
                        for(int c=0;c<count;++c) {
                            int value = origItem.getInt(oldvalues[i], c);
                            item.addInt(oldvalues[i], c, value);
                        }
                        break;
                    case PIMItem.STRING:
                        Log.debug("STRING");
                        for(int c=0;c<count;++c) {
                            String value = origItem.getString(oldvalues[i], c);
                            item.addString(oldvalues[i], c, value);
                        }
                        break;
                    case PIMItem.DATE:
                        Log.debug("DATE");
                        for(int c=0;c<count;++c) {
                            long value = origItem.getDate(oldvalues[i], c);
                            item.addDate(oldvalues[i], c, value);
                        }
                        break;
                    case PIMItem.STRING_ARRAY:
                        Log.debug("STRING_ARRAY");
                        for(int c=0;c<count;++c) {
                            String value[] = origItem.getStringArray(oldvalues[i], c);
                            item.addStringArray(oldvalues[i], c, value);
                        }
                        break;
                    case PIMItem.BINARY:
                        Log.debug("BINARY");
                        for(int c=0;c<count;++c) {
                            byte value[] = origItem.getBinary(oldvalues[i], c);
                            item.addBinary(oldvalues[i], c, value, 0, value.length);
                        }
                        break;
                    case PIMItem.BOOLEAN:
                        Log.debug("BOOLEAN");
                        for(int c=0;c<count;++c) {
                            boolean value = origItem.getBoolean(oldvalues[i], c);
                            item.addBoolean(oldvalues[i], c, value);
                        }
                        break;
                    default:
                        Log.error("Unsupported PIMItem data type: " + fieldType);
                        break;
                }
            }
        }
    }


    
    /**
     * This method checks if the field and attibute are supported by the PIMList
     * @param int field
     * @param int attribute
     * @return boolean:returns true if both the field and attribute associated with it r supported
     */
    public boolean isSupportedAttributedField(int field, int attribute)
    {
        return isSupportedField(field) && pimlist.isSupportedAttribute(field, attribute);
    }
    
    /**
     * Checks if the type of field is supported by the PIMList
     * @param int type
     * @return boolean returns true if the contact list supports this field
     */
    public boolean isSupportedField(int type)
    {
        return pimlist.isSupportedField(type);
    }

    /**
     * Gets a field value as String. The method checks that the field is
     * supported and a value corresponding to index is defined.
     * @return the value or null if not available
     */
    public String getItemString(PIMItem item, int field, int index) {
        String result = null;
        if (isSupportedField(field) && item.countValues(field) > index) {
            result = item.getString(field, index);
        }
        return result;
    }

    /**
     * Gets a field value as boolean. The method checks that the field is
     * supported and a value corresponding to index is defined.
     * @return the value or the default value if not available
     */
    public boolean getItemBoolean(PIMItem item, int field, int index, boolean defValue) {
        boolean result = defValue;
        if (isSupportedField(field) && item.countValues(field) > index) {
            result = item.getBoolean(field, index);
        }
        return result;
    }
    
    public static Vector getVectorValue(Hashtable eventMap, String key)
    {
        final Object val = eventMap.get(key);
        final Vector value = val == null ? new Vector(0) : (Vector)val;
        return value;
    }
    
    public static String getValue(Hashtable itemMap, String key, boolean unescape)
    {
        final Object val = itemMap.get(key);
        if (val==null)
        {
            return PIMItemHelper.EMPTY_STRING;
        }
        return unescape ? XmlUtil.unescapeXml(val.toString().trim()) : val.toString().trim();
    }
    
    /**
     * This function will add Category data to provided PIMItem.
     * 
     * @param pimMap The hashtable that was created from parsing the SIF data.
     * @param list This is the PIMList that the PIMItem belongs to.
     * @param item The PIMItem that will have Category data added to it.
     * @param categoryTag The string which is SIF tag for Categories.
     */
    public static void addCategories(Hashtable pimMap, PIMList list, PIMItem item, String categoryTag) throws PIMException
    {
        String field = PIMItemHelper.getValue(pimMap, categoryTag, false);// Categories
        PimUtils pimUtils = new PimUtils("UTF-8");
        pimUtils.addCategories(field, list, item, true);
    }
    
    /**
     * This function will try to extract the Categories out of a PIMItem.
     * 
     * @param _item The PIMItem will have categories extracted from.
     * @return An empty string or a tring in following format "<category>; <category>"
     */
    public static String extractCategories(PIMItem _item)
    {
        final String[] categories = _item.getCategories();
        final StringBuffer categoriesBuffer = new StringBuffer();
        String categoriesContents = null;
        if (categories.length > 0)
        {
            for (int k = 0; k < categories.length; k++)
            {
                categoriesBuffer.append(categories[k]);
                if (categories.length > 1 && k < categories.length - 1)
                {
                    categoriesBuffer.append("; ");// the separator (semicolon + blank)
                }
            }
            // e.g. Business; Marketing
            categoriesContents = categoriesBuffer.toString();
        }
        else
        {
            // no categories at all!
            categoriesContents = "";
        }
        return categoriesContents;
    }

    /**
     * This function parses a SIF format String to a Hashtable of fields.
     * @param contactString Information of a PIM in SIF format
     * @return A Hashtable with all fields from the above string
     */
    private static SimpleXMLParser parser; 
    public static Hashtable buildMapFromXML(String rootTag, String sifString, boolean resetparser
                                            , Vector multiValueTags, Vector hashtableValueTags, Vector multiHashtableBalueTags)
    {
        try
        {
            if (PIMItemHelper.parser == null)
            {
                PIMItemHelper.parser = new SimpleXMLParser();
            }
            if (resetparser)
            {
                PIMItemHelper.parser.reset();
            }
            // Multi simple value tags
            if (multiValueTags!=null)
            {
                for(int i=multiValueTags.size()-1;0<=i;i--)
                {
                    PIMItemHelper.parser.addMultiValueTag((String)multiValueTags.elementAt(i));
                }
            }
            // Hashtable value tags
            if (hashtableValueTags!=null)
            {
                for(int i=hashtableValueTags.size()-1;0<=i;i--)
                {
                    PIMItemHelper.parser.addHashtableValueTag((String)hashtableValueTags.elementAt(i));
                }
            }
            // multi Hashtable value tags
            if (multiHashtableBalueTags!=null)
            {
                for(int i=multiHashtableBalueTags.size()-1;0<=i;i--)
                {
                    PIMItemHelper.parser.addMultiHashtableValueTag((String)multiHashtableBalueTags.elementAt(i));
                }
            }

            return PIMItemHelper.parser.parseData(rootTag,sifString);
        }
        catch(Throwable e)
        {
            Log.error("Exception while parsing SIF-XML data ["+e.toString()+"]");
            Log.error("SIF-XML data ["+sifString+"]");
        }
        return null;
    }


    /**
     * This method appends the event information String Type supported by
     * blackberry to a string buffer.
     * @param StringBuffer: StringBuffer containing event information.
     * @param item: PIMItem
     * @param int: type of field
     * @param String: Name of field
     */
    public boolean appendXMLFieldBoolean(StringBuffer pimBuffer, PIMItem item, int field, String tag)
    {
        boolean value = false;
        if (pimlist.isSupportedField(field))
        {
            try
            {
                value = item.getBoolean(field, 0);
            }
            catch (final Exception e)
            {}
            if (value)
            {
                XmlUtil.addElementNewLine(pimBuffer, tag, "1",false);
            }
            else
            {
                XmlUtil.addElementNewLine(pimBuffer, tag, "0",false);
            }
        }
        return value;
    }
    
    
    public Calendar getFieldDate(PIMItem item, int field, boolean endday, boolean allday, boolean extraToGMT)
    {
        Log.trace("[PIMItemHelper.getFieldDate]");

        long value = 0;
        long oneDay = (allday && endday) ? (EventHelper.DAY_FACTOR) : 0;
                
        if (pimlist.isSupportedField(field))
        {
            try
            {
                value = item.getDate(field, 0);
                Log.trace("value=" + value);

                // round up if we detect 29 or 59 minute in time
                value += ( ( (value/EventHelper.MINUTE_FACTOR) %60) %30 == 29) ? EventHelper.MINUTE_FACTOR : 0;

                final Calendar cal = Calendar.getInstance();
                value = PIMItemHelper.convertLocalToGMT(value);
                Log.trace("value=" + value);
                if (extraToGMT)
                {
                    value = PIMItemHelper.convertLocalToGMT(value);
                    Log.trace("value=" + value);
                }
                value -= oneDay;
                Log.trace("value=" + value);
                cal.setTime(new Date(value));
                return cal;
            } catch (final Exception e) {
                Log.info("PIMItemHelper.getFieldDate(): unknown exception during " +
                         "retrieving and converting Date value ["+ e.toString() +"]");
            }
        }
        return null;
    }
    
    public String handlePIMException(PIMException e)
    {
        String error = "";
        Log.debug("PIMItemHelper: Handling PIM Exception");
        if (e.getMessage() != null)
        {
                Log.info("PIMException message: " + e.getMessage());
        }
        switch (e.getReason())
        {
            case PIMException.FEATURE_NOT_SUPPORTED:
                error = "feature not supported";
                break;
            case PIMException.GENERAL_ERROR:
                error = "general error";
                break;
            case PIMException.LIST_CLOSED:
                error = "list closed";
                break;
            case PIMException.LIST_NOT_ACCESSIBLE:
                error = "list not accessible";
                break;
            case PIMException.MAX_CATEGORIES_EXCEEDED:
                error = "max categories exceeded";
                break;
            case PIMException.UNSUPPORTED_VERSION:
                error = "unsupported version";
                break;
            case PIMException.UPDATE_ERROR:
                error = "update error";
                break;
            default:
                error = "unknown error";
                break;
        }
        Log.info("PIMException reason: " + error);
        return error;
    }
    
    
    
    
    // If the year is prior to 1970, set it to 1970.
    public static long adjustDateToEpoch(long date)
    {
        final Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(date));
        if (cal.get(Calendar.YEAR) < 1970)
        {
            cal.set(Calendar.YEAR, 1970);
        }
        return cal.getTime().getTime();
    }
    
    /**
     * Get time (a {@code long} value that holds the number of milliseconds
     * since midnight GMT, January 1, 1970) from date in "yyyy-MM-dd" String
     * format
     * @param field Date in "yyyy-MM-dd" String format
     * @return time at 00:00:00 from date
     */
    public static Calendar parseDate(String field) {
        return parseDate(field, false);
    }

    public static Calendar parseDate(String field, boolean gmt)
    {
        int day = 0;
        int month = 0;
        int year = 0;
        
        Calendar date = null;
        
        if (field.length() == 10)
        {
            year = Integer.parseInt(field.substring(0, 4));
            month = Integer.parseInt(field.substring(5, 7));
            day = Integer.parseInt(field.substring(8, 10));
        }
        else
        {
            year = Integer.parseInt(field.substring(0, 4));
            month = Integer.parseInt(field.substring(4, 6));
            day = Integer.parseInt(field.substring(6, 8));
        }
       
        if (gmt) {
            date = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        } else {
            date = Calendar.getInstance(TimeZone.getDefault());
        }
        
        date.set(Calendar.DAY_OF_MONTH, day);
        date.set(Calendar.MONTH, month - 1);
        year = year > 1970 ? year : 1970;
        date.set(Calendar.YEAR, year);
        date.set(Calendar.HOUR_OF_DAY, 0);
        date.set(Calendar.MINUTE, 0);
        date.set(Calendar.SECOND, 0);
         
        return date;
    }
    
    /**
     * Get time from date in "yyyyMMddTHHmmssZ" or "yyyyMMdd" or "yyyyMMddTHHmmss" String format
     * @param field date in "yyyyMMddTHHmmssZ" String format
     * @return A 'long' time in milliseconds from the Epoch
     */
    public static Calendar parseDateTime(String field)
    {
        int day = 0;
        int month = 0;
        int year = 0;
        int hour = 0;
        int minute = 0;
        int second = 0;
                
        Calendar date = null;
        
        if (field.length() <= 10) {
            return PIMItemHelper.parseDate(field);
        }

        year = Integer.parseInt(field.substring(0, 4));
        year = year > 1970 ? year : 1970;
        month = Integer.parseInt(field.substring(4, 6));
        day = Integer.parseInt(field.substring(6, 8));
        if (field.charAt(8)=='T')
        {
            hour = Integer.parseInt(field.substring(9, 11));
            minute = Integer.parseInt(field.substring(11, 13));
            second = Integer.parseInt(field.substring(13, 15));
        }
                
        if (field.charAt(field.length()-1) == 'Z')
        {
                date = Calendar.getInstance(TimeZone.getTimeZone("GMT")); //GMT
        }
        else
        {
                date = Calendar.getInstance(TimeZone.getDefault());
        }
        date.set(Calendar.DAY_OF_MONTH, day);
        date.set(Calendar.MONTH, month - 1);
        date.set(Calendar.YEAR, year);
        date.set(Calendar.HOUR_OF_DAY, hour);
        date.set(Calendar.MINUTE, minute);
        date.set(Calendar.SECOND, second);
        return date;
    }
    
    public static Calendar getStartOfDay(Calendar date)
    {
        date.set(Calendar.HOUR_OF_DAY, 00);
        date.set(Calendar.MINUTE, 00);
        date.set(Calendar.SECOND, 00);// 00 from 59 to 00 bugfix
        return date;
    }
    
    public static Calendar getEndOfDay(Calendar date)
    {
        date.set(Calendar.HOUR_OF_DAY, 23);// 00
        date.set(Calendar.MINUTE, 59);// 00
        date.set(Calendar.SECOND, 00);// 00 from 59 to 00 bugfix
        return date;
    }

    public static long getTimestamp(Calendar date)
    {
        return date.getTime().getTime();
    }
    
    public static long convertLocalToGMT(long localtime)
    {
        long GMTtime = localtime - PIMItemHelper.getTimeZoneOffSet(new Date(localtime));
        return GMTtime;
    }
    
    /**
     * This function was build to fix a BUG in get/set DateTime in BlackBerryToDo 4.2.1.
     * This function might return a wrong value round the DayLightSaving time stamp.
     * Using this is just a work around for the BUG in BlackBerry API 4.2.1
     * @param gmttime <description>
     * @return <description>
     */
    public static long convertGMTToLocal(long gmttime)
    {
        long Localtime = gmttime + PIMItemHelper.getTimeZoneOffSet(new Date(gmttime));
        return Localtime;
    }
    
    /**
     * @param date a local-date to read offset
     * @return timezone offset
     */
    public static long getTimeZoneOffSet(Date date)
    {
        long offset = 0;
        TimeZone zn = TimeZone.getDefault();
        Calendar local = Calendar.getInstance();
        local.setTime(date);
        /*
        * the offset to add to GMT to get local time, modified in case of
        * daylight savings
        */
        int time = (int)(local.get(Calendar.HOUR_OF_DAY)*EventHelper.HOUR_FACTOR + local.get(Calendar.MINUTE)*EventHelper.MINUTE_FACTOR  + local.get(Calendar.SECOND)*EventHelper.SECOND_FACTOR);
        offset = zn.getOffset(1 // era AD
                            ,local.get(Calendar.YEAR)
                            ,local.get(Calendar.MONTH)
                            ,local.get(Calendar.DAY_OF_MONTH)
                            ,local.get(Calendar.DAY_OF_WEEK)
                            ,time);
        return offset;
    }
    
    public static long getDefaultTimeZoneOffset()
    {
        long offset = 0;
        TimeZone zn = TimeZone.getDefault();
        Calendar local = Calendar.getInstance();
        local.setTime(new Date(System.currentTimeMillis()));
        /*
        * the offset to add to GMT to get local time, modified in case of
        * daylight savings
        */
        int time = (int)(local.get(Calendar.HOUR_OF_DAY)*EventHelper.HOUR_FACTOR + local.get(Calendar.MINUTE)*EventHelper.MINUTE_FACTOR  + local.get(Calendar.SECOND)*EventHelper.SECOND_FACTOR);
        offset = zn.getOffset(1 // era AD
                            ,local.get(Calendar.YEAR)
                            ,local.get(Calendar.MONTH)
                            ,local.get(Calendar.DAY_OF_MONTH)
                            ,local.get(Calendar.DAY_OF_WEEK)
                            ,time);
        return offset;
    }
    
    public static String formatAllDay(Calendar allday)
    {
        String ret = "";
        int tt =0;
        ret += allday.get(Calendar.YEAR) + "-";
        
        tt = allday.get(Calendar.MONTH)+1;
        ret += (tt<10 ? "0"+tt : ""+tt )+ "-";
        
        tt = allday.get(Calendar.DAY_OF_MONTH);
        ret += tt<10 ? "0"+tt : ""+tt;
        return ret;
    }
    
    public static String formatDateTime(Calendar datetime)
    {
        String ret = "";
        int tt = 0;
        ret += datetime.get(Calendar.YEAR);
        
        tt = datetime.get(Calendar.MONTH)+1;
        ret += tt<10 ? "0"+tt : ""+tt;
        
        tt = datetime.get(Calendar.DAY_OF_MONTH);
        ret += (tt<10 ? "0"+tt : ""+tt) + "T";
        
        tt = datetime.get(Calendar.HOUR_OF_DAY);
        ret += tt<10 ? "0"+tt : ""+tt;
        
        tt = datetime.get(Calendar.MINUTE);
        ret += tt<10 ? "0"+tt : ""+tt;
        
        tt = datetime.get(Calendar.SECOND);
        ret += (tt<10 ? "0"+tt : ""+tt);
        return ret;
    }

    public static String formatDateTimeUTC(Calendar dateTime) {
        String baseDate = formatDateTime(dateTime);
        return baseDate + "Z";
    }

    /**
     * Shift a date to the first rrule occurence
     * @param pitem
     */
    public long adjustDateFromRRule(RepeatRule rrule, long date) {

        Log.trace("[fixDateFromRRule] date: " + date);
        if(rrule == null) {
            return date;
        }

        final int WEEK_DAYS[][] = {{RepeatRule.SUNDAY,Calendar.SUNDAY},
                                   {RepeatRule.MONDAY,Calendar.MONDAY},
                                   {RepeatRule.TUESDAY,Calendar.TUESDAY},
                                   {RepeatRule.WEDNESDAY,Calendar.WEDNESDAY},
                                   {RepeatRule.THURSDAY,Calendar.THURSDAY},
                                   {RepeatRule.FRIDAY,Calendar.FRIDAY},
                                   {RepeatRule.SATURDAY,Calendar.SATURDAY}
                                  };

        int[] fieldsArray = rrule.getFields();
        int daysMask = 0;

        // verify that this is a WEEKLY recurrence
        for (int i=0; i<fieldsArray.length; i++) {
            switch (fieldsArray[i]) {
                case RepeatRule.FREQUENCY:
                    if(rrule.getInt(RepeatRule.FREQUENCY) != RepeatRule.WEEKLY) {
                        // Only WEEKLY recurrences shall be adjusted
                        Log.trace("[fixDateFromRRule] The recurrence type is not WEEKLY");
                        return date;
                    }
                    break;
                case RepeatRule.DAY_IN_WEEK:
                    daysMask = rrule.getInt(RepeatRule.DAY_IN_WEEK);
                    Log.trace("[fixDateFromRRule] DAY_IN_WEEK mask: " + daysMask);
                    break;
            }
        }
        if(daysMask == 0) {
            return date;
        }

        Calendar cal = Calendar.getInstance();
        cal.setTime(new Date(date));
        int startDay = cal.get(Calendar.DAY_OF_WEEK);
        Log.trace("[fixDateFromRRule] Date day of week: " + startDay);

        for(int i=0; i<WEEK_DAYS.length; i++) {
            if (((daysMask & WEEK_DAYS[i][0]) == WEEK_DAYS[i][0])) {
                if(startDay == WEEK_DAYS[i][1]) {
                    // The date is already included in the recurrence pattern
                    Log.trace("[fixDateFromRRule] The date is already included in the recurrence pattern.");
                    return date;
                }
            }
        }

        // Get the first available occurence in the next 6 days
        Enumeration e = rrule.dates(date,
                                    date + EventHelper.DAY_FACTOR,
                                    date + EventHelper.DAY_FACTOR*7);
        long newDate = ((Date) e.nextElement()).getTime();
        Log.trace("[fixDateFromRRule] Get the shifted date: " + newDate);
        return newDate;
    }
} 
