/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;
import javax.microedition.pim.RepeatRule;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.helper.SIFRepeatRuleHelper;
import com.funambol.blackberry.source.AdditionalItemsAttributes;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.XmlUtil;
import com.funambol.util.DateUtil;
import com.funambol.util.Log;

public class SIFEventHelper extends EventHelper
{
    /**
     * The constructor
     */
    public SIFEventHelper(PIMList list, EventMailListener listener, int pastlimit, int futurelimit) {
        super(list, listener, pastlimit, futurelimit);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException
    {
        PIMList pimlist = getPIMList();
        
        if (item instanceof BlackBerryEvent)
        {
            BlackBerryEvent event = (BlackBerryEvent)item;
            final StringBuffer eventBuffer = new StringBuffer();
            eventBuffer.append(XmlUtil.createOpenTagNewLine(SIFEventHelper.APPOINTMENT_TYPE_KEY));
                
            boolean allday = appendXMLFieldBoolean(eventBuffer,event, BlackBerryEvent.ALLDAY, SIFEventHelper.ALLDAY); // 20000928
            //String patternStartDate = SIFEventHelper.appendXMLFieldDate(pimlist,eventBuffer, event, BlackBerryEvent.START, SIFEventHelper.START, allday); //106 "Start"
            
            String patternStartDate = "";
            Calendar StartDate = getFieldDate(event, BlackBerryEvent.START, false, allday, false);
            if (StartDate != null) {
                if (allday) {
                    patternStartDate = PIMItemHelper.formatAllDay(StartDate);
                }
                else {
                    patternStartDate = PIMItemHelper.formatDateTimeUTC(StartDate);
                }
            }
            Log.info("SIFEventHelper.toString() - START - my format ["+ patternStartDate +"]");
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.START, patternStartDate, false);
            

            //SIFEventHelper.appendXMLFieldDate(pimlist,eventBuffer, event, BlackBerryEvent.END, SIFEventHelper.END, allday);// 102 "End"
            
            String patternEndDate = "";
            Calendar EndDate = getFieldDate(event, BlackBerryEvent.END, true, allday, false);
            if (EndDate != null) {
                if (allday) {
                    patternEndDate = PIMItemHelper.formatAllDay(EndDate);
                }
                else {
                    patternEndDate = PIMItemHelper.formatDateTimeUTC(EndDate);
                }
            }
            Log.info("SIFEventHelper.toString() - END - my format ["+ patternEndDate +"]");
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.END, patternEndDate, false);


            appendXMLField(pimlist,eventBuffer,event, BlackBerryEvent.SUMMARY,SIFEventHelper.SUMMARY); //107
            appendXMLField(pimlist,eventBuffer,event, BlackBerryEvent.LOCATION,SIFEventHelper.LOCATION); //103
            appendXMLField(pimlist,eventBuffer,event, BlackBerryEvent.NOTE,SIFEventHelper.NOTE);// 104 <Body>
            appendXMLFieldAlarmMB(pimlist,eventBuffer, event); // 100
            appendXMLSensitivity(pimlist,eventBuffer, event); // bugfix 869a
            Log.trace("Appending BusyStatus");
            appendXMLBusyStatus(pimlist,eventBuffer, event); // bugfix 869b
            Log.trace("Appending Attendees");
            appendXMLAttendees(pimlist,eventBuffer, event);
            
            Log.trace("Appending Recurrence");
            // enhancement 868 (recurrence)

            Calendar patStartDateCal;
            if (allday) {
                // All day events are expressed in their day and we should not
                // apply any GMT-local conversion, otherwise we may end up in
                // the wrong day. This is because the RIM's API perform days
                // rounding to all day events
                patStartDateCal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
                long patStartDateLong = event.getDate(BlackBerryEvent.START, 0);
                patStartDateCal.setTime(new Date(patStartDateLong));
                // For all day events the pattern start date shall start at midnight
                patStartDateCal = PIMItemHelper.getStartOfDay(patStartDateCal);
            } else {
                patStartDateCal = Calendar.getInstance();
                long patStartDateLong = event.getDate(BlackBerryEvent.START, 0);
                patStartDateCal.setTime(new Date(patStartDateLong));
            }
            // Pattern start date and end date must be expressed in local time
            // to avoid TZ issues (bug #7322)
            patternStartDate = PIMItemHelper.formatDateTime(patStartDateCal);

            appendXMLRecurrence(pimlist,eventBuffer, event, patternStartDate, allday);
            eventBuffer.append(XmlUtil.createCloseTagNewLine(SIFEventHelper.APPOINTMENT_TYPE_KEY));
            Log.debug("MY LOG - event to string: "+eventBuffer.toString());
            return eventBuffer.toString();
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString,
                                              AdditionalItemsAttributes additionals) throws PIMException
    {
        Log.debug("pimString ["+pimString+"]");

        EventList list = getEventList();
        
        if (list != null)
        {
            Vector multiValueTags = new Vector(2);
            multiValueTags.addElement(SIFRepeatRuleHelper.EXCLUDE_DATE);
            multiValueTags.addElement(SIFRepeatRuleHelper.INCLUDE_DATE);

            Vector multiHashtableValueTags = new Vector(1);
            multiHashtableValueTags.addElement(SIFEventHelper.ATTENDEE);

            Hashtable eventMap = PIMItemHelper.buildMapFromXML(SIFEventHelper.APPOINTMENT_TYPE_KEY,
                                                               pimString,true,multiValueTags,
                                                               null,multiHashtableValueTags);
            if (eventMap != null) {
                fillEvent((Event)item,eventMap);
            } else {
                Log.info("Hashtable properties is null.");
            }
        }
        //return null because Additionals attributes does not used
        return null;
    }
    
   
    private String formatGMT(long date) {
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        cal.setTime(new Date(date));
        return PIMItemHelper.formatDateTimeUTC(cal);
    }

    private void appendXMLRecurrence(PIMList list, StringBuffer eventBuffer,
                                     Event event, String patternStartDate,
                                     boolean allDay) throws PIMException
    {
        // enhancement 868 (recurrence)
        final RepeatRule rule = event.getRepeat();
        if (rule == null) {
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.IS_RECURRING, "0",false);
        } else if (patternStartDate!=null) {
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.IS_RECURRING, "1",false);
            eventBuffer.append(SIFRepeatRuleHelper.toString(rule,patternStartDate,patternStartDate,
                                                            allDay));
        } else {
            Log.error("Cannot find start date for a recurring event");
            throw new PIMException("An Event must have a StartDate to have Recurrence.");
        }
    }
    

    private void appendXMLBusyStatus(PIMList list, StringBuffer eventBuffer, BlackBerryEvent event)
    {
        if (list.isSupportedField(BlackBerryEvent.FREE_BUSY))// 20000929
        {
            int busyStatus = 2; //olBusy
            try
            {
                switch (event.getInt(BlackBerryEvent.FREE_BUSY, 0))
                {
                    case BlackBerryEvent.FB_FREE:
                        busyStatus = 0;// olFree
                        break;
                    case BlackBerryEvent.FB_TENTATIVE:
                        busyStatus = 1;// olTentative
                        break;
                    case BlackBerryEvent.FB_BUSY:
                        busyStatus = 2;// olBusy
                        break;
                    case BlackBerryEvent.FB_OUT_OF_OFFICE:
                        busyStatus = 3;// olOutOfOffice
                        break;
                }
            }
            catch (final Exception e)
            {
                Log.debug("Default value was used because of exception while retrieving SIF-XML field ["+SIFEventHelper.BUSY_STATUS+"]");
            }
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.BUSY_STATUS, ""+busyStatus,false);
        }
    }
    
    /*
     * bugfix 869a (but Event.CLASS is unfortunately unsupported: to not pollute
     * the data coming to the server from an other client, it is important that
     * this tag is not added to the SIF-E to the server!)
     */
    private void appendXMLSensitivity(PIMList list,StringBuffer eventBuffer, BlackBerryEvent event)
    {
        /*
        * using unsupported Event.CLASS field causes an
        * UnsupportedFieldException
        */
        if (list.isSupportedField(Event.CLASS))
        {
            int c = 0;
            try
            {
                switch (event.getInt(Event.CLASS, 0))
                {
                    case Event.CLASS_PUBLIC:// 202
                        c = 0;// <Sensitivity>0</Sensitivity>
                        break;
                    case Event.CLASS_PRIVATE:// 201
                        c = 2;// <Sensitivity>2</Sensitivity>
                        break;
                    case Event.CLASS_CONFIDENTIAL:
                        c = 3;
                        break;
                }
            }
            catch (final Exception e)
            {
                Log.debug("Default value was used because of exception while retrieving IF-XML field ["+SIFEventHelper.SENSITIVITY+"]");
            }
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.SENSITIVITY, ""+c,false);
        }
    }
    
    /**
     * This method appends the event information int Type supported by
     * blackberry to a string buffer.
     * @param PIMList: An instance of PIMList
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @return void.
     */
    private void appendXMLFieldAlarmMB(PIMList list, StringBuffer eventBuffer, BlackBerryEvent event)
    {
        String value = null;
        if (list.isSupportedField(BlackBerryEvent.ALARM))
        {
            int reminder = 0;
            value = "";
            try
            {
                reminder = event.getInt(BlackBerryEvent.ALARM, 0) / 60;
                value = String.valueOf(reminder);
            }
            catch (final Exception e)
            {
                Log.debug("Default value was used because of exception while retrieving SIF-XML field ["+SIFEventHelper.ALARM+"]");
            }
            XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ALARM, value,false);
            if (reminder > 0)
            {
                XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ALARM_ENABLE, "1",false);
            }
            else
            {
                XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ALARM_ENABLE, "0",false);
            }
        }
        else
        {
            Log.debug("BlackBerryEvent does not support BlackBerryEvent.ALARM");
        }
    }
    
    private void appendXMLAttendees(PIMList list, StringBuffer eventBuffer, Event event) throws PIMException
    {
        if (list.isSupportedField(BlackBerryEvent.ATTENDEES))
        {
            int size = event.countValues(BlackBerryEvent.ATTENDEES);
            if (size>0)
            {
                eventBuffer.append(XmlUtil.createOpenTagNewLine(SIFEventHelper.ATTENDEES));
                for (int i=0;i<size;i++)
                {
                    eventBuffer.append(XmlUtil.createOpenTagNewLine(SIFEventHelper.ATTENDEE));
                    XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ATTENDEE_NAME, "", false);
                    XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ATTENDEE_STATUS, "", false);
                    XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ATTENDEE_TYPE, "", false);
                    XmlUtil.addElementNewLine(eventBuffer, SIFEventHelper.ATTENDEE_EMAIL, event.getString(BlackBerryEvent.ATTENDEES,i), true);
                    eventBuffer.append(XmlUtil.createCloseTagNewLine(SIFEventHelper.ATTENDEE));
                }
                eventBuffer.append(XmlUtil.createCloseTagNewLine(SIFEventHelper.ATTENDEES));
            }
        }
    }
        
    /**
     * This method appends the event information String Type supported by blackberry to a string buffer.
     * @param PIMList: An instance of PIMList
     * @param StringBuffer: StringBuffer containing event information.
     * @param Event: Blackberry event
     * @param int: type of field
     * @param String: Name of field
     */
    private void appendXMLField(PIMList list, StringBuffer eventBuffer, Event event, int field, String tag)
    {
        String value = null;
        if (list.isSupportedField(field))
        {
            value = "";
            try
            {
                value = event.getString(field, 0);
            }
            catch (final Exception e)
            {
                Log.debug("Default value was used because of exception while retriving SIF-XML field ["+tag+"]");
            }
            XmlUtil.addElementNewLine(eventBuffer, tag, value,true);
        }
        value =null;
    }
}

