/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.Date;

import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.ToDo;
import javax.microedition.pim.ToDoList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.RepeatRule;

import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.util.XmlUtil;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;
import com.funambol.blackberry.source.AdditionalItemsAttributes;

public class SIFTaskHelper extends TaskHelper
{
    private static final String TAG_LOG = "SIFTaskHelper";

    public SIFTaskHelper(PIMList list)
    {
        super(list);
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof ToDo)
        {
            try
            {
                return item.getString(ToDo.UID,0);
            }
            catch(final Exception e)
            {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Task]";
                Log.error(TAG_LOG, msg, e);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Task] is not a BlackBerryToDo/ToDo.";
        Log.error(TAG_LOG, msg);
        throw new PIMException(msg);
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException
    {
        String fieldname = null;
        
        PIMList pimlist = getPIMList();
        
        try
        {
            if (item instanceof BlackBerryToDo)
            {
                BlackBerryToDo task = (BlackBerryToDo)item;
                StringBuffer temp = new StringBuffer(XmlUtil.createOpenTagNewLine(SIFTaskHelper.TASK_TYPE_KEY));

                fieldname = SIFTaskHelper.SUBJECT;
                if (isSupportedField(BlackBerryToDo.SUMMARY) && item.countValues(BlackBerryToDo.SUMMARY)>0)
                {
                    XmlUtil.addElementNewLine(temp,SIFTaskHelper.SUBJECT,task.getString(BlackBerryToDo.SUMMARY,0),true);
                }

                fieldname = SIFTaskHelper.BODY;
                if (isSupportedField(BlackBerryToDo.NOTE) && item.countValues(BlackBerryToDo.NOTE)>0)
                {
                    XmlUtil.addElementNewLine(temp,SIFTaskHelper.BODY,task.getString(BlackBerryToDo.NOTE,0),true);
                }

                fieldname = SIFTaskHelper.COMPLETE;
                appendXMLFieldBoolean(temp,task,BlackBerryToDo.COMPLETED,SIFTaskHelper.COMPLETE);
                fieldname = SIFTaskHelper.SENSITIVITY;
                appendXMLSensitivity(pimlist,temp,task);
                fieldname = SIFTaskHelper.IMPORTANCE;
                appendXMLPriority(pimlist,temp,task);
                fieldname = SIFTaskHelper.DATE_COMPLETED;

                Calendar completedDate = getDate(task, BlackBerryToDo.COMPLETION_DATE);
                String formattedCompletedDate = completedDate != null ? PIMItemHelper.formatDateTime(completedDate) : "";
                Log.debug(TAG_LOG, "COMPLETION_DATE : my format ["+ formattedCompletedDate +"]");
                XmlUtil.addElementNewLine(temp, SIFTaskHelper.DATE_COMPLETED, formattedCompletedDate, false);
                
                fieldname = SIFTaskHelper.DUE_DATE;
                Calendar dueDate = getDate(task, BlackBerryToDo.DUE);
                String formattedDueDate = dueDate != null ?  PIMItemHelper.formatDateTime(dueDate) : "";
                Log.debug(TAG_LOG, "DUE : my format ["+ formattedDueDate +"]");
                XmlUtil.addElementNewLine(temp, SIFTaskHelper.DUE_DATE, formattedDueDate, false);
                
                fieldname = SIFTaskHelper.STATUS;
                appendXMLStatus(pimlist,temp,task);
                fieldname = SIFTaskHelper.IS_RECURRING;
                formattedDueDate = dueDate != null ? PIMItemHelper.formatAllDay(dueDate) : null;
                appendXMLRecurrence(pimlist,temp, task, formattedDueDate, formattedDueDate);
                fieldname = SIFTaskHelper.CATEGORIES;
                XmlUtil.addElementNewLine(temp, SIFTaskHelper.CATEGORIES, PIMItemHelper.extractCategories(task),true);
                
                if (isSupportedField(SIFTaskHelper.BB_ALARM)) {
                    fieldname = SIFTaskHelper.REMINDER_TIME;
                    long value = task.getInt(SIFTaskHelper.BB_ALARM,0);
                    if (value>0)
                    {
                        XmlUtil.addElementNewLine(temp,SIFTaskHelper.REMINDER_SET,"1",false);
                        XmlUtil.addElementNewLine(temp,SIFTaskHelper.REMINDER_TIME,""+value,false);
                    }
                    else
                    {
                        XmlUtil.addElementNewLine(temp,SIFTaskHelper.REMINDER_SET,"0",false);
                    }
                } else {
                    Log.debug(TAG_LOG, "BlackBerryToDo does not support BlackBerryToDo.ALARM");
                }
                
                temp.append(XmlUtil.createCloseTagNewLine(SIFTaskHelper.TASK_TYPE_KEY));
                task = null;
                return temp.toString();
            }
            return null;
        }
        catch(final Throwable e)
        {
            Log.error(TAG_LOG, "on field ["+fieldname+"]. Exception -> ", e);
            throw new PIMException("Exception while converting Task to SIF format ["+fieldname+"]");
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString,
                                              AdditionalItemsAttributes additionals)
    throws PIMException
    {
        Log.debug(TAG_LOG, "pimString ["+pimString+"]");
        
        ToDoList list = getToDoList();
        
        if (list != null)
        {
            Vector multiValueTags = new Vector(2);
            multiValueTags.addElement(SIFRepeatRuleHelper.EXCLUDE_DATE);
            multiValueTags.addElement(SIFRepeatRuleHelper.INCLUDE_DATE);
            
            
            Hashtable taskMap = PIMItemHelper.buildMapFromXML(SIFTaskHelper.TASK_TYPE_KEY,
                                                              pimString,true,
                                                              multiValueTags,null,null);
            if (taskMap != null)
            {
                fillTask((BlackBerryToDo)item,taskMap);
            }
            else
            {
                Log.debug(TAG_LOG, "Hashtable properties is null.");
            }
        }
        //return null because Additionals attributes does not used
        return null;
    }


   
   
    private Calendar getDate(BlackBerryToDo task, int field) {
        Calendar date = null;
        try {
            if (isSupportedField(field)) {
                if (task.countValues(field) > 0) {
                    // The DUE date is local time and we send it as local time
                    // as we assume that the server and the client are in the same
                    // timezone. If we send it as GMT time we get in troubles as per
                    // bug 7374 (which boils down to the client not supporting
                    // timezone)
                    long dateVal = task.getDate(field, 0);
                    dateVal      = PIMItemHelper.convertLocalToGMT(dateVal);
                    date = Calendar.getInstance();
                    date.setTime(new Date(dateVal));
                }
            }
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot get field date from task: " + field, e);
        }
        return date;
    }

    
    
    private void appendXMLRecurrence(PIMList list, StringBuffer taskBuffer,
                                     BlackBerryToDo task, String patternStartDate,
                                     String dueDate) throws PIMException
    {
        // enhancement 868 (recurrence)
        RepeatRule rule = task.getRepeat();
        if (rule == null)
        {
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.IS_RECURRING, "0",false);
        }
        else if (dueDate!=null)
        {
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.IS_RECURRING, "1",false);
            taskBuffer.append(SIFRepeatRuleHelper.toString(rule,patternStartDate,dueDate));
            rule = null;
        }
        else
        {
            throw new PIMException("A task must have a DueDate to have Recurrence.");
        }
    }
    
    private void appendXMLStatus(PIMList list,StringBuffer taskBuffer, BlackBerryToDo task)
    {
        if (list.isSupportedField(SIFTaskHelper.BB_STATUS))
        {
            int c = SIFTaskHelper.SIF_BB_Status[0][0];
            try
            {
                int status = task.getInt(SIFTaskHelper.BB_STATUS, 0);
                Log.debug(TAG_LOG, "BlackBerry status ["+status+"]");
                
                for (int i=0;i<SIFTaskHelper.SIF_BB_Status.length;i++) {
                    if (status == SIFTaskHelper.SIF_BB_Status[i][1]) {
                        c = SIFTaskHelper.SIF_BB_Status[i][0];
                    }
                }
            }
            catch (final Exception e)
            {}
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.STATUS, ""+c,false);
        }
        else {
            Log.info(TAG_LOG, "BlackBerryToDo does not support BlackBerryToDo.STATUS or SIFTaskHelper.STATUS");
        }
    }
    
    private void appendXMLPriority(PIMList list,StringBuffer taskBuffer, BlackBerryToDo task)
    {
        if (list.isSupportedField(BlackBerryToDo.PRIORITY))
        {
            int c = SIFTaskHelper.SIF_BB_Important[0][0];
            try
            {
                int priority = task.getInt(BlackBerryToDo.PRIORITY, 0);
                for (int i=0;i<SIFTaskHelper.SIF_BB_Important.length;i++)
                {
                    if (priority == SIFTaskHelper.SIF_BB_Important[i][1])
                    {
                        c = SIFTaskHelper.SIF_BB_Important[i][0];
                    }
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Default value was used because of exception while retrieving SIF-XML field ["
                                   +SIFTaskHelper.IMPORTANCE+"]", e);
            }
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.IMPORTANCE, ""+c,false);
        } else {
            Log.info(TAG_LOG, "BlackBerryToDo does not support BlackBerryToDo.PRIORITY or SIFTaskHelper.IMPORTANCE");
        }
    }
    
    private void appendXMLSensitivity(PIMList list,StringBuffer taskBuffer, BlackBerryToDo task)
    {
        /*
        * using unsupported Event.CLASS field causes an
        * UnsupportedFieldException
        */
        if (list.isSupportedField(BlackBerryToDo.CLASS))
        {
            int c = SIFTaskHelper.SIF_BB_Sensitivity[0][0];
            try
            {
                int sensitivity = task.getInt(BlackBerryToDo.CLASS, 0);
                Log.debug(TAG_LOG, "BlackBerry Sensitivity/Class ["+sensitivity+"]");
                
                for (int i=0;i<SIFTaskHelper.SIF_BB_Sensitivity.length;i++) {
                    if (sensitivity == SIFTaskHelper.SIF_BB_Sensitivity[i][1]) {
                        c = SIFTaskHelper.SIF_BB_Sensitivity[i][0];
                    }
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Default value was used because of exception while retrieving SIF-XML field ["
                                    +SIFTaskHelper.SENSITIVITY+"]", e);
            }
            XmlUtil.addElementNewLine(taskBuffer, SIFTaskHelper.SENSITIVITY, ""+c,false);
        } else {
            Log.info("BlackBerryToDo does not support BlackBerryToDo.CLASS or SIFTaskHelper.SENSITIVITY");
        }
    }
} 
