/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.Date;
import java.util.Hashtable;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.ToDoList;
import javax.microedition.pim.ToDo;
import javax.microedition.pim.RepeatRule;

import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.util.StringUtil;
import com.funambol.util.Log;

public abstract class TaskHelper extends PIMItemHelper
{    
    private static final String         TAG_LOG = "TaskHelper";
    public static final String          TASK_TYPE_KEY       = "task";

    public static final String          BODY                = "Body";
    public static final String          SUBJECT             = "Subject";
    public static final String          COMPLETE            = "Complete";
    public static final String          SENSITIVITY         = "Sensitivity";
    public static final String          IMPORTANCE          = "Importance";
    public static final String          DATE_COMPLETED      = "DateCompleted";
    public static final String          DUE_DATE            = "DueDate";
    public static final String          REMINDER_SET        = "ReminderSet";
    public static final String          REMINDER_TIME       = "ReminderTime";
    public static final String          IS_RECURRING        = "IsRecurring";
    public static final String          CATEGORIES          = "Categories";
    
    public static final String          STATUS              = "Status";
    public static final int             BB_STATUS           = PIMItem.EXTENDED_FIELD_MIN_VALUE + 9;
    public static final int             BB_ALARM            = 100;

    // This is the value of the JSR75 field for REMINDER. This constant is
    // available in SDK for OS >= 4.6 but not defined for previous versions
    // (although the field is present)
    public static final int             REMINDER_FIELD      = 20000927;
    
    protected static final int                RECURS_DAILY                = 0;
    protected static final int                RECURS_WEEKLY               = 1;
    protected static final int                RECURS_MONTHLY              = 2;
    protected static final int                RECURS_MONTHLY_NTH          = 3;
    protected static final int                RECURS_YEARLY               = 5;
    protected static final int                RECURS_YEARLY_NTH           = 6;

    protected static final String             RECURRENCE_TYPE             = "RecurrenceType";
    protected static final String             INTERVAL                    = "Interval";
    protected static final String             MONTH_OF_YEAR               = "MonthOfYear";
    protected static final String             DAY_OF_MONTH                = "DayOfMonth";
    protected static final String             DAY_OF_WEEK_MASK            = "DayOfWeekMask";
    protected static final String             INSTANCE                    = "Instance";
    protected static final String             PATTERN_START_DATE          = "PatternStartDate";
    protected static final String             NO_END_DATE                 = "NoEndDate";
    protected static final String             PATTERN_END_DATE            = "PatternEndDate";
    protected static final String             OCCURRENCES                 = "Occurrences";
    protected static final String             EXCEPTIONS                  = "Exceptions";
    protected static final String             EXCLUDE_DATE                = "ExcludeDate";
    protected static final String             INCLUDE_DATE                = "IncludeDate";

    private static final int                  MONTHS[][]                  = {{1,RepeatRule.JANUARY}
                                                                             ,{2,RepeatRule.FEBRUARY}
                                                                             ,{3,RepeatRule.MARCH}
                                                                             ,{4,RepeatRule.APRIL}
                                                                             ,{5,RepeatRule.MAY}
                                                                             ,{6,RepeatRule.JUNE}
                                                                             ,{7,RepeatRule.JULY}
                                                                             ,{8,RepeatRule.AUGUST}
                                                                             ,{9,RepeatRule.SEPTEMBER}
                                                                             ,{10,RepeatRule.OCTOBER}
                                                                             ,{11,RepeatRule.NOVEMBER}
                                                                             ,{12,RepeatRule.DECEMBER}
                                                                          };
    private static final int                WEEKS[][]                     = {{1,RepeatRule.FIRST}
                                                                             ,{2,RepeatRule.SECOND}
                                                                             ,{3,RepeatRule.THIRD}
                                                                             ,{4,RepeatRule.FOURTH}
                                                                             ,{5,RepeatRule.FIFTH}
                                                                             ,{-5,RepeatRule.FIFTHLAST}
                                                                             ,{-4,RepeatRule.FOURTHLAST}
                                                                             ,{-3,RepeatRule.THIRDLAST}
                                                                             ,{-2,RepeatRule.SECONDLAST}
                                                                             ,{-1,RepeatRule.LAST}
                                                                          };
    private static final int                DAYS[][]                      = {{1,RepeatRule.SUNDAY}
                                                                             ,{2,RepeatRule.MONDAY}
                                                                             ,{4,RepeatRule.TUESDAY}
                                                                             ,{8,RepeatRule.WEDNESDAY}
                                                                             ,{16,RepeatRule.THURSDAY}
                                                                             ,{32,RepeatRule.FRIDAY}
                                                                             ,{64,RepeatRule.SATURDAY}
                                                                          };

    
    //private static final String             DATETIME_FORMAT_ALLDAY                  = "yyyy-MM-dd";
    //private static final String             DATETIME_FORMAT_UTC                     = "yyyyMMdd'T'HHmmss'Z'";
    
    protected static final int            SIF_BB_Important[][] = {{0,9}// LOW
                                                                 ,{1,5} // NORMAL
                                                                 ,{2,1} // HIGH
                                                            };
    protected static final int            SIF_BB_Status[][]   = {{0,0}// STATUS_NOT_STARTED
                                                                ,{1,1} // STATUS_IN_PROGRESS
                                                                ,{2,2} // STATUS_COMPLETED
                                                                ,{3,3} // STATUS_WAITING
                                                                ,{4,4} // STATUS_DEFERRED
                                                            };
    protected static final int            SIF_BB_Sensitivity[][]   = {{0,BlackBerryToDo.CLASS_PUBLIC}// NORMAL - PUBLIC
                                                                     ,{1,BlackBerryToDo.CLASS_PRIVATE} // PERSONAL
                                                                     ,{2,BlackBerryToDo.CLASS_PRIVATE} // PRIVATE
                                                                     ,{3,BlackBerryToDo.CLASS_CONFIDENTIAL} // CONFIDENTIAL
                                                            };

    protected static final int[] SUPPORTED_FIELDS =
    {
        BlackBerryToDo.SUMMARY
        ,BlackBerryToDo.NOTE
        ,BlackBerryToDo.COMPLETED
        ,BlackBerryToDo.DUE
        ,BlackBerryToDo.COMPLETION_DATE
        ,BB_STATUS
        ,BlackBerryToDo.PRIORITY
        ,BlackBerryToDo.CLASS
        ,REMINDER_FIELD
    };

    public TaskHelper(PIMList list) 
    {
        super(list);
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#createPIMItem(PIMList pimlist)
     */
    public PIMItem createItem() throws PIMException
    {
        ToDoList list = getToDoList();
        if (list != null)
        {
            return list.createToDo();
        }
        return null;
    }
    
    protected ToDoList getToDoList()
    {
        PIMList list = getPIMList();
        if (list != null && list instanceof ToDoList)
        {
            return (ToDoList)list;
        }
        return null;
    }
    
    
    public boolean isInSyncRange(PIMItem _item) throws PIMException
    {
        /*
        long NOW = System.currentTimeMillis();
        Calendar start = PIMItemHelper.getFieldDate(pimlist,_item,BlackBerryToDo.DUE,false,false,true);
        if (start!=null)
        {
            if (start.getTime().getTime() > (NOW+this.limitFuture*PIMItemHelper.MONTH_FACTOR))
            {
                return false;
            }
            else
            {
                RepeatRule rrule = ((BlackBerryToDo)_item).getRepeat();
                if (rrule==null)
                {
                    if (start.getTime().getTime() < (NOW-this.limitPast*PIMItemHelper.MONTH_FACTOR))
                    {
                        return false;
                    }                    
                }
                else
                {
                    Calendar end = SIFRepeatRuleHelper.getEndDate(rrule,true);
                    if (end == null)
                    {
                        end = Calendar.getInstance();
                        end.setTime(new Date(NOW));
                    }
                    if (end.getTime().getTime() < (NOW-this.limitPast*PIMItemHelper.MONTH_FACTOR))
                    {
                        return false;
                    }
                }
            }
        }*/
        return true;
    }
    
    public boolean deleteItem(PIMItem item) throws PIMException
    {
        ToDoList list = getToDoList();
        if (list != null &&
                item != null &&
                item instanceof ToDo &&
                isInSyncRange(item))
        {
            list.removeToDo((ToDo)item);
            return true;
        }
        return false;
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item, BlackBerryToDo.UID,
                              SUPPORTED_FIELDS);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem _item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(_item,BlackBerryToDo.UID,SUPPORTED_FIELDS);
        //((BlackBerryToDo)_item).setRepeat(null);
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#getUID(PIMItem item)
     */
    public String getUID(PIMItem item) throws PIMException
    {
        if (item instanceof ToDo)
        {
            try
            {
                return item.getString(ToDo.UID,0);
            }
            catch(final Exception e)
            {
                final String msg = "Exception while retrieving UID for PIMItem of SyncSource [Task]";
                Log.error(msg);
                throw new PIMException(msg);
            }
        }
        final String msg = "This PIMItem of SyncSource [Task] is not a BlackBerryToDo/ToDo.";
        Log.error(msg);
        throw new PIMException(msg);
    }

    protected ToDo fillTask(BlackBerryToDo task, Hashtable taskMap) throws PIMException
    {
        ToDoList list = getToDoList();
        if (list == null) {
            return null;
        }
        
        String fieldname = null;
        try
        {
            String field = null;
            
            //BlackBerryToDo.SUMMARY
            fieldname = SUBJECT;
            if (isSupportedField(BlackBerryToDo.SUMMARY)) {
                field = PIMItemHelper.getValue(taskMap, SUBJECT, true);
                if (task.countValues(BlackBerryToDo.SUMMARY)==0) {
                    task.addString(BlackBerryToDo.SUMMARY, BlackBerryToDo.ATTR_NONE, field);
                } else {
                    task.setString(BlackBerryToDo.SUMMARY, 0, BlackBerryToDo.ATTR_NONE, field);
                }
            }
            
            //BlackBerryToDo.NOTE
            fieldname = BODY;
            if (isSupportedField(BlackBerryToDo.NOTE)) {
                field = PIMItemHelper.getValue(taskMap, BODY, true);
                field = StringUtil.replaceAll(field, "\r\n", "\n");
                // We must replace all \r\n sequences into single \n to avoid
                // empty lines in the BB UI
                if (task.countValues(BlackBerryToDo.NOTE)==0) {
                    task.addString(BlackBerryToDo.NOTE, BlackBerryToDo.ATTR_NONE, field);
                } else {
                    task.setString(BlackBerryToDo.NOTE, 0, BlackBerryToDo.ATTR_NONE, field);
                }
            }

            //BlackBerryToDo.DUE
            fieldname = DUE_DATE;
            if (isSupportedField(BlackBerryToDo.DUE)) {
                field = PIMItemHelper.getValue(taskMap, DUE_DATE, false);
                if (field.length() > 0) {
                    setDate(task, field, BlackBerryToDo.DUE);
                }
            }

            //BlackBerryToDo.COMPLETION_DATE
            fieldname = DATE_COMPLETED;
            if (isSupportedField(BlackBerryToDo.COMPLETION_DATE)) {
                field = PIMItemHelper.getValue(taskMap, DATE_COMPLETED, false);
                if (field.length() > 0) {
                    setDate(task, field, BlackBerryToDo.COMPLETION_DATE);
                }
            }

            fieldname = IMPORTANCE;
            parsePriority(task,taskMap);
            fieldname = SENSITIVITY;
            parseSensitivity(task,taskMap);
            fieldname = CATEGORIES;
            PIMItemHelper.addCategories(taskMap,list,task,CATEGORIES);

            //BlackBerryToDo.COMPLETE
            boolean completed = false;
            fieldname = COMPLETE;
            if (isSupportedField(BlackBerryToDo.COMPLETED)) {
                field = PIMItemHelper.getValue(taskMap, COMPLETE, false);
                if (field.length()>0) {
                    if (!"false".equalsIgnoreCase(field) && !"0".equals(field)) {
                        completed = true;
                    }
                    task.addBoolean(BlackBerryToDo.COMPLETED,BlackBerryToDo.ATTR_NONE,completed);
                }
            }

            //BlackBerryToDo.REMINDER
            if (isSupportedField(REMINDER_FIELD)) {
                field = PIMItemHelper.getValue(taskMap, REMINDER_SET, false);
                if ("1".equals(field)) {
                    field = PIMItemHelper.getValue(taskMap, REMINDER_TIME, false);
                    if (field.length() > 0) {
                        setDate(task, field, REMINDER_FIELD);
                    }
                }
            } else {
                Log.info(TAG_LOG, "Ignoring unsupported REMINDER field");
            }
                        
            // RepeatRule
            if (!completed) {
                fieldname = IS_RECURRING;
                field = PIMItemHelper.getValue(taskMap, IS_RECURRING, false);// e.g. 1
                if (field.length()>0) {
                    if (!"false".equalsIgnoreCase(field) && !"0".equals(field)) {
                        //System.out.println("Before creating a new RepeatRule");
                        RepeatRule rrule = new RepeatRule();
                        //System.out.println("Before setting data to the new RepeatRule");
                        SIFRepeatRuleHelper.setRepeatRule(taskMap,rrule,true);
                        //System.out.println("Before setting the new RepeatRule to task ");
                        fieldname = OCCURRENCES;
                        final String occurrences = PIMItemHelper.getValue(taskMap, OCCURRENCES, false);
                        if (!"".equals(occurrences))
                        {
                        	//rrule.setInt(RepeatRule.FREQUENCY, RepeatRule.WEEKLY);
                        	//rrule.setInt(RepeatRule.COUNT, Integer.parseInt(occurrences));
                        }
                        Log.info(TAG_LOG, "OCCORRENCES in taskhelper " + occurrences);
                        task.setRepeat(rrule);
                        //System.out.println("After setting the new RepeatRule to task ");
                    }
                }
            } else {
                Log.info(TAG_LOG, "Can not set Recurrence and Completed on task.");
            }

            
            
            // Status or not
            if (!completed) {
                fieldname = STATUS;
                parseStatus(list,task,taskMap);
            }

            return task;
        } catch (final Throwable e) {
            if (fieldname.equals(IS_RECURRING)) {
                fieldname = SIFRepeatRuleHelper.popSettingFieldName();
            }
            Log.error(TAG_LOG, "Error converting field ["+fieldname+"]", e);
            throw new PIMException("Exception while setting value to Task field ["+fieldname+"]");
        }
    }

    private void setDate(BlackBerryToDo task, String value, int field) {
        Calendar date = PIMItemHelper.parseDateTime(value);

        long dateStart = date.getTime().getTime();
        // This is tricky because of a BB bug. What happens:
        // 1) addDate shifts the date, converting the given value
        //    from local to GMT
        // 2) the value is stored
        // 3) the BB UI shows it as local time (takes the stored and
        //    does not convert it back)
        // Because of this, we play a trick and on local dates, we
        // counter shift by the timeone offset, in order to
        // neutralize the addDate conversion
        TimeZone tz = date.getTimeZone();
        if (tz != null && !"GMT".equals(tz.getID())) {
            // The offset is here added so that the addDate can
            // subtract it again, bringing back to the original date
            dateStart = PIMItemHelper.convertGMTToLocal(dateStart);
        }
        task.addDate(field, BlackBerryToDo.ATTR_NONE, dateStart);
    }

    private void parseStatus(ToDoList list, BlackBerryToDo task, Hashtable taskMap)
    {
        if (list.isSupportedField(BB_STATUS))
        {
            int status = SIF_BB_Status[0][1];
            try
            {
                int sifStatus = Integer.parseInt(PIMItemHelper.getValue(taskMap, STATUS, false));
                for (int i=0;i<SIF_BB_Status.length;i++) {
                    if (sifStatus == SIF_BB_Status[i][0]) {
                        status = SIF_BB_Status[i][1];
                    }
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Exception while matching BB Status", e);
            }
            task.addInt(BB_STATUS,BlackBerryToDo.ATTR_NONE,status);
        }
    }
    
    private void parsePriority(BlackBerryToDo task, Hashtable taskMap)
    {
        if (isSupportedField(BlackBerryToDo.PRIORITY)) {
            int priority = SIF_BB_Important[1][1];
            try
            {
                int sifPrior = Integer.parseInt(PIMItemHelper.getValue(taskMap, IMPORTANCE, false));
                for (int i=0;i<SIF_BB_Important.length;i++) {
                    if (sifPrior == SIF_BB_Important[i][0]) {
                        priority = SIF_BB_Important[i][1];
                    }
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Cannot parse priority, using normal priority", e);
            }
            task.addInt(BlackBerryToDo.PRIORITY,BlackBerryToDo.ATTR_NONE,priority);
        }
    }
    
    private void parseSensitivity(BlackBerryToDo task, Hashtable taskMap)
    {
        if (isSupportedField(BlackBerryToDo.CLASS))
        {
            int sesitivity = SIF_BB_Sensitivity[0][0];
            try
            {
                int sif_sensitivity = Integer.parseInt(PIMItemHelper.getValue(taskMap, SENSITIVITY, false));
                for (int i=0;i<SIF_BB_Sensitivity.length;i++)
                {
                    if (sif_sensitivity == SIF_BB_Sensitivity[i][0])
                    {
                        sesitivity = SIF_BB_Sensitivity[i][1];
                    }
                }
            }
            catch (final Exception e)
            {}
            task.addInt(BlackBerryToDo.CLASS,BlackBerryToDo.ATTR_NONE,sesitivity);
        }
    }

}
