/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;
import javax.microedition.pim.RepeatRule;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.helper.SIFRepeatRuleHelper;
import com.funambol.blackberry.source.AdditionalItemsAttributes;

import com.funambol.common.pim.vcalendar.BasicVCalendarParserListener;
import com.funambol.common.pim.vcalendar.BasicVCalendar;
import com.funambol.common.pim.ParserProperty;
import com.funambol.common.pim.xvcalendar.XVCalendar;
import com.funambol.common.pim.xvcalendar.XVCalendarSyntaxParserListener;
import com.funambol.common.pim.xvcalendar.XVCalendarSyntaxParser;
import com.funambol.common.pim.xvcalendar.XVCalendarFormatter;
import com.funambol.common.pim.xvcalendar.ParseException;
import com.funambol.common.pim.xvcalendar.rrule.RRule10SyntaxParser;
import com.funambol.common.pim.xvcalendar.rrule.RRule10SyntaxParserListener;
import com.funambol.common.pim.PimUtils;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.XmlUtil;
import com.funambol.util.DateUtil;
import com.funambol.util.StringUtil;
import com.funambol.util.Log;

public class VCalEventHelper extends EventHelper {
    private static final String TAG_LOG = "VCalEventHelper";

    /** YEAR - constant from Calendar class. */
    private static final int YEAR = Calendar.YEAR;
    /** MONTH - constant from Calendar class. */
    private static final int MONTH = Calendar.MONTH;
    /** DAY_OF_MONTH - constant from Calendar class. */
    private static final int DAY_OF_MONTH = Calendar.DAY_OF_MONTH;
    /** HOUR_OF_DAY - constant from Calendar class. */
    private static final int HOUR_OF_DAY = Calendar.HOUR_OF_DAY;
    /** MINUTE - constant from Calendar class. */
    private static final int MINUTE = Calendar.MINUTE;
    /** SECOND - constant from Calendar class. */
    private static final int SECOND = Calendar.SECOND;

    private static final String SUNDAY = "SU";
    private static final String MONDAY = "MO";
    private static final String TUESDAY = "TU";
    private static final String WEDNESDAY = "WE";
    private static final String THURSDAY = "TH";
    private static final String FRIDAY = "FR";
    private static final String SATURDAY = "SA";
    private static final String LASTDAY = "LD";

    /**
     * The constructor
     */
    public VCalEventHelper(PIMList list, EventMailListener listener,
            int pastlimit, int futurelimit) {
        super(list, listener, pastlimit, futurelimit);
    }

    public String itemToString(PIMItem item,
            AdditionalItemsAttributes additionals) throws PIMException {
        PIMList pimlist = getPIMList();

        if (item instanceof BlackBerryEvent) {
            BlackBerryEvent event = (BlackBerryEvent) item;
            BBVCalFormatter formatter = new BBVCalFormatter();
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            formatter.format(item, os, true);
            return os.toString();
        } else {
            throw new PIMException("Unknown item class type");
        }
    }

    protected class BBVCalFormatter extends XVCalendarFormatter {

        protected boolean isAllDay(PIMItem item) {
            return getItemBoolean(item, BlackBerryEvent.ALLDAY, 0, false);
        }

        protected void formatFunambolAllday(PIMItem pimItem, OutputStream os, boolean allFields)
                throws PIMException {
            BlackBerryEvent event = (BlackBerryEvent) pimItem;

            boolean printField = allFields || isAllDay(pimItem);

            if (printField) {
                if (isAllDay(pimItem)) {
                    PimUtils.println(os, XVCalendar.X_FUNAMBOL_ALLDAY + ":" + "1");
                } else {
                    PimUtils.println(os, XVCalendar.X_FUNAMBOL_ALLDAY + ":" + "0");
                }
            }
        }

        public void formatDTStart(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {
            String date = "";
            boolean allDay = isAllDay(pimItem);
            Calendar startDate = getFieldDate(pimItem, BlackBerryEvent.START,
                    false, allDay, false);
            if (startDate != null) {
                if (allDay) {
                    // date = PIMItemHelper.formatAllDay(startDate);
                    date = PIMItemHelper.formatDateTime(startDate);
                } else {
                    date = PIMItemHelper.formatDateTimeUTC(startDate);
                }
            }
            Log.debug(TAG_LOG, "DTStart=" + date);
            PimUtils.println(os, XVCalendar.DTSTART + ":" + date);
        }

        public void formatDTEnd(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {
            String date = "";
            boolean allDay = isAllDay(pimItem);
            Calendar endDate = getFieldDate(pimItem, BlackBerryEvent.END,
                    false, allDay, false);
            if (endDate != null) {
                if (allDay) {
                    Log.debug(TAG_LOG, "DTEnd="
                            + PIMItemHelper.formatDateTime(endDate));
                    // date = PIMItemHelper.formatAllDay(endDate);
                    endDate = getFieldDate(pimItem, BlackBerryEvent.END, false,
                            allDay, false);
                    endDate.set(Calendar.DAY_OF_MONTH, endDate
                            .get(Calendar.DAY_OF_MONTH) - 1);
                    endDate.set(Calendar.HOUR, 23);
                    endDate.set(Calendar.MINUTE, 59);
                    date = PIMItemHelper.formatDateTime(endDate);
                } else {
                    date = PIMItemHelper.formatDateTimeUTC(endDate);

                }
            }
            Log.debug(TAG_LOG, "DTEnd=" + date);
            PimUtils.println(os, XVCalendar.DTEND + ":" + date);
        }

        public void formatRRule(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {

            Log.trace(TAG_LOG, "Formatting rrule");
            VCalRepeatRuleHelper rruleHelper = new VCalRepeatRuleHelper();
            boolean allDay = isAllDay(pimItem);
            Calendar endDate = getFieldDate(pimItem, BlackBerryEvent.END, true,
                    allDay, false);
            rruleHelper.formatRRule(pimItem, os, allFields, allDay, endDate);
        }

        public void formatAlarm(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {
            String date = "";
            try {
                boolean allDay = isAllDay(pimItem);
                Calendar startDate = getFieldDate(pimItem,
                        BlackBerryEvent.START, false, allDay, false);
                long dateStart = startDate.getTime().getTime();
                Log.debug(TAG_LOG, "date start" + Long.toString(dateStart));
                if (isSupportedField(BlackBerryEvent.ALARM)) {
                    long alarmTime = pimItem.getInt(BlackBerryEvent.ALARM, 0);
                    Log.debug(TAG_LOG, "AALARM time start"
                            + Long.toString(alarmTime));
                    if (alarmTime != 0) {
                        alarmTime = alarmTime * 1000;
                        long delta = dateStart - alarmTime;
                        delta = delta < 0 ? -1 * delta : delta;
                        Calendar alarmDate = Calendar.getInstance();
                        alarmDate.setTime(new Date(delta));
                        if (allDay) {
                            date = PIMItemHelper.formatDateTime(alarmDate);
                        } else {
                            date = PIMItemHelper.formatDateTimeUTC(alarmDate);
                        }
                    }
                }
            } catch (final Exception e) {
                final String msg = "Exception while retrieving Reminder: "+e.toString();
                Log.error(msg);
            } finally {
                Log.debug(TAG_LOG, "AALARM start" + date);
                PimUtils.println(os, "AALARM" + ":" + date + ";;0;");
            }
        }
        // This shall be generated with the recurrence
        /*
         * String patternEndDate = ""; Calendar endDate = getFieldDate(event,
         * BlackBerryEvent.END, true, allday, false); if (endDate != null) { if
         * (allday) { patternEndDate = PIMItemHelper.formatAllDay(EndDate); }
         * else { patternEndDate = PIMItemHelper.formatDateTimeUTC(EndDate); } }
         * Log.trace(TAG_LOG, "patternEndDate=" + patternEndDate);
         * formatSimpleField(PATTERN_END_DATE, patternEndDate, false);
         */

        /*
         * Calendar patStartDateCal; if (allday) { // All day events are
         * expressed in their day and we should not // apply any GMT-local
         * conversion, otherwise we may end up in // the wrong day. This is
         * because the RIM's API perform days // rounding to all day events
         * patStartDateCal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
         * long patStartDateLong = event.getDate(BlackBerryEvent.START, 0);
         * patStartDateCal.setTime(new Date(patStartDateLong)); // For all day
         * events the pattern start date shall start at midnight patStartDateCal
         * = PIMItemHelper.getStartOfDay(patStartDateCal); } else {
         * patStartDateCal = Calendar.getInstance(); long patStartDateLong =
         * event.getDate(BlackBerryEvent.START, 0); patStartDateCal.setTime(new
         * Date(patStartDateLong)); } // Pattern start date and end date must be
         * expressed in local time // to avoid TZ issues (bug #7322)
         * patternStartDate = PIMItemHelper.formatDateTime(patStartDateCal);
         * 
         * appendXMLRecurrence(pimlist,eventBuffer, event, patternStartDate,
         * allday);
         */

    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList
     * pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString,
            AdditionalItemsAttributes additionals) throws PIMException {

        EventList list = getEventList();

        if (list != null) {
            Hashtable eventMap = buildMapFromVCal(pimString, item);
            if (eventMap != null) {
                fillEvent((Event) item, eventMap);
            } else {
                Log.info("Hashtable properties is null.");
            }
        }
        // return null because Additionals attributes are not used
        return null;
    }

    private Hashtable buildMapFromVCal(String vCal, PIMItem item) {

        Hashtable hash = null;
        Vector exceptionDates = new Vector(2);
        exceptionDates.addElement(SIFRepeatRuleHelper.EXCLUDE_DATE);
        exceptionDates.addElement(SIFRepeatRuleHelper.INCLUDE_DATE);

        Vector attendees = new Vector(1);
        attendees.addElement(ATTENDEE);

        try {
            BBVCalParserListener listener = new BBVCalParserListener(item);
            ByteArrayInputStream is = new ByteArrayInputStream(vCal.getBytes());
            XVCalendarSyntaxParser parser = new XVCalendarSyntaxParser(is);
            parser.setListener(listener);
            parser.parse();
            hash = listener.getHash();
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot parse vcal event: " + vCal, e);
        }
        return hash;
    }

    private String formatGMT(long date) {
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        cal.setTime(new Date(date));
        return PIMItemHelper.formatDateTimeUTC(cal);
    }

    /*
     * private void appendXMLRecurrence(PIMList list, StringBuffer eventBuffer,
     * Event event, String patternStartDate, boolean allDay) throws PIMException
     * { // enhancement 868 (recurrence) final RepeatRule rule =
     * event.getRepeat(); if (rule == null) {
     * XmlUtil.addElementNewLine(eventBuffer, IS_RECURRING, "0",false); } else
     * if (patternStartDate!=null) { XmlUtil.addElementNewLine(eventBuffer,
     * IS_RECURRING, "1",false);
     * eventBuffer.append(SIFRepeatRuleHelper.toString(
     * rule,patternStartDate,patternStartDate, allDay)); } else {
     * Log.error("Cannot find start date for a recurring event"); throw new
     * PIMException("An Event must have a StartDate to have Recurrence."); } }
     * 
     * 
     * private void appendXMLBusyStatus(PIMList list, StringBuffer eventBuffer,
     * BlackBerryEvent event) { if
     * (list.isSupportedField(BlackBerryEvent.FREE_BUSY))// 20000929 { int
     * busyStatus = 2; //olBusy try { switch
     * (event.getInt(BlackBerryEvent.FREE_BUSY, 0)) { case
     * BlackBerryEvent.FB_FREE: busyStatus = 0;// olFree break; case
     * BlackBerryEvent.FB_TENTATIVE: busyStatus = 1;// olTentative break; case
     * BlackBerryEvent.FB_BUSY: busyStatus = 2;// olBusy break; case
     * BlackBerryEvent.FB_OUT_OF_OFFICE: busyStatus = 3;// olOutOfOffice break;
     * } } catch (final Exception e) {Log.debug(
     * "Default value was used because of exception while retrieving SIF-XML field ["
     * +BUSY_STATUS+"]"); } XmlUtil.addElementNewLine(eventBuffer, BUSY_STATUS,
     * ""+busyStatus,false); } }
     */

    /**
     * This method appends the event information int Type supported by
     * blackberry to a string buffer.
     * 
     * @param PIMList
     *            : An instance of PIMList
     * @param StringBuffer
     *            : StringBuffer containing event information.
     * @param Event
     *            : Blackberry event
     * @return void.
     */
    /*
     * private void appendXMLFieldAlarmMB(PIMList list, StringBuffer
     * eventBuffer, BlackBerryEvent event) { String value = null; if
     * (list.isSupportedField(BlackBerryEvent.ALARM)) { int reminder = 0; value
     * = ""; try { reminder = event.getInt(BlackBerryEvent.ALARM, 0) / 60; value
     * = String.valueOf(reminder); } catch (final Exception e) {Log.debug(
     * "Default value was used because of exception while retrieving SIF-XML field ["
     * +ALARM+"]"); } XmlUtil.addElementNewLine(eventBuffer, ALARM,
     * value,false); if (reminder > 0) { XmlUtil.addElementNewLine(eventBuffer,
     * ALARM_ENABLE, "1",false); } else { XmlUtil.addElementNewLine(eventBuffer,
     * ALARM_ENABLE, "0",false); } } else {
     * Log.debug("BlackBerryEvent does not support BlackBerryEvent.ALARM"); } }
     */

    /*
     * private void appendXMLAttendees(PIMList list, StringBuffer eventBuffer,
     * Event event) throws PIMException { if
     * (list.isSupportedField(BlackBerryEvent.ATTENDEES)) { int size =
     * event.countValues(BlackBerryEvent.ATTENDEES); if (size>0) {
     * eventBuffer.append(XmlUtil.createOpenTagNewLine(ATTENDEES)); for (int
     * i=0;i<size;i++) {
     * eventBuffer.append(XmlUtil.createOpenTagNewLine(ATTENDEE));
     * XmlUtil.addElementNewLine(eventBuffer, ATTENDEE_NAME, "", false);
     * XmlUtil.addElementNewLine(eventBuffer, ATTENDEE_STATUS, "", false);
     * XmlUtil.addElementNewLine(eventBuffer, ATTENDEE_TYPE, "", false);
     * XmlUtil.addElementNewLine(eventBuffer, ATTENDEE_EMAIL,
     * event.getString(BlackBerryEvent.ATTENDEES,i), true);
     * eventBuffer.append(XmlUtil.createCloseTagNewLine(ATTENDEE)); }
     * eventBuffer.append(XmlUtil.createCloseTagNewLine(ATTENDEES)); } } }
     */

    /**
     * This method appends the event information String Type supported by
     * blackberry to a string buffer.
     * 
     * @param PIMList
     *            : An instance of PIMList
     * @param StringBuffer
     *            : StringBuffer containing event information.
     * @param Event
     *            : Blackberry event
     * @param int: type of field
     * @param String
     *            : Name of field
     */
    private void appendXMLField(PIMList list, StringBuffer eventBuffer,
            Event event, int field, String tag) {
        String value = null;
        if (list.isSupportedField(field)) {
            value = "";
            try {
                value = event.getString(field, 0);
            } catch (final Exception e) {
                Log
                        .debug("Default value was used because of exception while retriving SIF-XML field ["
                                + tag + "]");
            }
            XmlUtil.addElementNewLine(eventBuffer, tag, value, true);
        }
        value = null;
    }

    public class BBVCalParserListener extends BasicVCalendarParserListener
            implements XVCalendarSyntaxParserListener,
            RRule10SyntaxParserListener {

        private final String LOG_TAG = "BBVCalParserListener";

        private static final String defaultCharset = BasicVCalendar.UTF8;

        /** Check whether this is an allday event */
        protected boolean allDay = false;

        private Hashtable hash;

        /**
         * The construtor accepts a <code>PIMItem</code> object that will be
         * populated of all the common vCalendar properties.
         * 
         * @param pimItem
         *            the PIMItem object
         */
        public BBVCalParserListener(PIMItem item) {
            super(item);
        }

        public void addEventProperty(ParserProperty property) {

            Log.trace(LOG_TAG, "addEventProperty: " + property.getName());

            String name = property.getName();
            String value = getClearValue(property);

            if (StringUtil.equalsIgnoreCase(XVCalendar.DTSTART, name)) {
                hash.put(START, value);
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.DTEND, name)) {
                hash.put(END, value);
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.CATEGORIES, name)) {
                Log.info(TAG_LOG, "CATEGORIES not supported yet");
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.AALARM, name)) {
                hash.put(ALARM_ENABLE, "1");
                int sepIdx = value.indexOf(";");
                if (sepIdx >= 0) {
                    value = value.substring(0, sepIdx);
                }
                hash.put(ALARM_ABS_TIME, value);

            } else if (StringUtil.equalsIgnoreCase(XVCalendar.RRULE, name)) {
                Log.info(TAG_LOG, "RRULE = " + value);
                ByteArrayInputStream rruleIs = new ByteArrayInputStream(value
                        .getBytes());
                RRule10SyntaxParser rruleParser = new RRule10SyntaxParser(
                        rruleIs);
                rruleParser.setListener(this);
                try {
                    rruleParser.parse();
                } catch (Exception e) {
                    Log.error(TAG_LOG,
                            "Cannot parser recurrence rule " + value, e);
                }
            } else if (StringUtil.equalsIgnoreCase(
                    XVCalendar.X_FUNAMBOL_ALLDAY, name)) {
                if ("1".equals(value)) {
                    hash.put(ALLDAY, "1");
                }
            } else if (StringUtil
                    .equalsIgnoreCase(BasicVCalendar.SUMMARY, name)) {
                hash.put(SUMMARY, value);
            } else if (StringUtil.equalsIgnoreCase(BasicVCalendar.CLASS, name)) {

                String sensitivity = null;
                if (StringUtil.equalsIgnoreCase("PUBLIC", value)) {
                    sensitivity = "0";
                } else if (StringUtil.equalsIgnoreCase("PRIVATE", value)) {
                    sensitivity = "2";
                } else if (StringUtil.equalsIgnoreCase("CONFIDENTIAL", value)) {
                    sensitivity = "3";
                }

                if (sensitivity == null) {
                    Log.error(TAG_LOG, "Unsupported class value: " + value);
                } else {
                    hash.put(SENSITIVITY, sensitivity);
                }
            } else if (StringUtil.equalsIgnoreCase(BasicVCalendar.DESCRIPTION,
                    name)) {
                hash.put(NOTE, value);
            } else if (StringUtil.equalsIgnoreCase(BasicVCalendar.UID, name)) {
                Log.info(TAG_LOG, "UID not supported");
            } else if (StringUtil.equalsIgnoreCase(
                    BasicVCalendar.LAST_MODIFIED, name)) {
                Log.info(TAG_LOG, "LAST_MODIFIED not supported");
            } else if (StringUtil.equalsIgnoreCase(BasicVCalendar.LOCATION,
                    name)) {
                hash.put(LOCATION, value);
            } else if (StringUtil.equalsIgnoreCase(BasicVCalendar.ATTENDEE,
                    name)) {
                Log.info(TAG_LOG, "Attendees not supported");
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.EXDATE, name)) {
                String exDates[] = StringUtil.split(value, ";");
                Vector dates = new Vector(exDates.length);
                for (int i = 0; i < exDates.length; ++i) {
                    dates.addElement(exDates[i].trim());
                }
                hash.put(EXCLUDE_DATE, dates);
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.XRULE, name)) {
                Log.info(TAG_LOG, "ExRule not supported yet");
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.RNUM, name)) {
                Log.info(TAG_LOG, "RNum not supported yet");
            } else if (StringUtil.equalsIgnoreCase(XVCalendar.RDATE, name)) {
                Log.info(TAG_LOG, "RDate not supported yet");
            } else {
                Log.error(LOG_TAG, "Unsupported property: " + name);
            }
        }

        public void addProperty(ParserProperty property) {
        }

        public void addToDoProperty(ParserProperty property) {
        }

        public void start() {
        }

        public void end() {
        }

        public void startEvent() throws ParseException {
            hash = new Hashtable();
        }

        public void endEvent() throws ParseException {
            Log.info(TAG_LOG, "event :" + hash.toString());
        }

        public void startToDo() throws ParseException {
        }

        public void endToDo() throws ParseException {
        }

        // //////////////// RRule10 listener implementation //////////////
        public void setEndDate(String endDate) {
            hash.put(PATTERN_END_DATE, endDate);
            Log.info(TAG_LOG, "endDate :" + endDate);
        }

        public void setType(String type) {
            int t = -1;
            if ("D".equals(type)) {
                t = 0;
            } else if ("W".equals(type)) {
                t = 1;
            } else if ("MD".equals(type)) {
                t = 2;
            } else if ("MP".equals(type)) {
                t = 3;
            } else if ("YM".equals(type)) {
                t = 5;
            } else if ("YD".equals(type)) {
                t = 6;
            }
            if (t >= 0) {
                hash.put(IS_RECURRING, "1");
                hash.put(RECURRENCE_TYPE, "" + t);
            }
            Log.info(TAG_LOG, "type :" + type);
        }

        public void setInterval(String interval) {
            Log.info(TAG_LOG, "interval :" + interval);
            hash.put(INTERVAL, interval);
        }

        public void setDuration(String duration) {
            Log.info(TAG_LOG, "Duration :" + duration);
            hash.put(OCCURRENCES, duration);
        }

        public void setDays(String days) {
            int t = 0;
            int bit_mask = 0;
            if (days.length() > 0) {
                String[] daysStr = StringUtil.split(days, ",");
                for (int j = 0; j < daysStr.length; j++) {
                    String day = daysStr[j];
                    if (SUNDAY.equals(day)) { // Sunday 1
                        t = 1;
                    } else if (MONDAY.equals(day)) { // Monday 2
                        t = 2;
                    } else if (TUESDAY.equals(day)) { // Tuesday 4
                        t = 4;
                    } else if (WEDNESDAY.equals(day)) { // Wednesday 8
                        t = 8;
                    } else if (THURSDAY.equals(day)) { // Thursday 16
                        t = 16;
                    } else if (FRIDAY.equals(day)) { // Friday 32
                        t = 32;
                    } else if (SATURDAY.equals(day)) { // Saturday 64
                        t = 64;
                    } else if (LASTDAY.equals(day)) {
                        Log.error(TAG_LOG, "LD not implemented");
                        throw new IllegalArgumentException("LD not implemented");
                    } else {
                        Log.error(TAG_LOG, "Unsupported day: " + day);
                        throw new IllegalArgumentException("Unsupported day "
                                + day);
                    }
                    if (bit_mask != 0)
                        bit_mask = bit_mask | t;
                    else
                        bit_mask = t;
                }
                hash.put(DAY_OF_WEEK_MASK, Integer.toString(bit_mask));
            }
            Log.info(TAG_LOG, "Days :" + Integer.toString(bit_mask));
        }

        public void setDaysNumber(String days) {
            Log.error(TAG_LOG, "setDaysNumber not implemented");
            // TODO FIXME: the days can be a list with with any number of days,
            // but at the moment we only support one day, because JSR75 only
            // allows for one day
            String[] d = StringUtil.split(days, ",");
            if (d.length > 1) {
                Log.error(TAG_LOG,
                        "Unsupported multiple days in MD recurrence. Keeping only the first day "
                                + days);
            }
            hash.put(EventHelper.DAY_OF_MONTH, d[0]);
        }

        public void setMonths(String monthlist) {
            // Yearly by month (YM) only
            // At the moment our parser is limited to one occurrence, so we are
            // ok, but the vCal spec seems to indicate more occurrences are
            // allowed
            Log.info(TAG_LOG, "Monthlist :" + monthlist);
            hash.put(MONTH_OF_YEAR, monthlist);
        }

        public void setOccurrences(String occurrences) {
            // Occurrences are only for "MP" recurrences
            // At the moment our parser is limited to one occurrence, so we are
            // ok, but the vCal spec seems to indicate more occurrences are
            // allowed
            String occ = StringUtil.replaceAll(occurrences, "+", "");
            Log.info(TAG_LOG, "Occurrences :" + occ);
            hash.put(INSTANCE, occ);
        }

        public Hashtable getHash() {
            return hash;
        }
    }
}
