/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
package com.funambol.blackberry.helper;

import java.io.UnsupportedEncodingException;
import java.util.Vector;

import java.util.Calendar;
import java.util.Enumeration;
import java.util.TimeZone;
import java.util.Date;
import java.util.Vector;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;
import javax.microedition.pim.RepeatRule;
import javax.microedition.pim.ToDo;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.common.pim.PimUtils;
import com.funambol.common.pim.xvcalendar.XVCalendar;

import com.funambol.util.DateUtil;
import com.funambol.util.Log;

public class VCalRepeatRuleHelper {

    private static final String TAG_LOG = "VCalRepeatRuleHelper";

    /** Repeat rule daily frequency char representation. */
    private static final char DAILY = 'D';

    /** Repeat rule weekly frequency char representation. */
    private static final char WEEKLY = 'W';

    /** Repeat rule monthly frequency char representation. */
    private static final char MONTHLY = 'M';

    /** Repeat rule yearly frequency char representation. */
    private static final char YEARLY = 'Y';

    /** Repeat rule day-in-month char representation. */
    private static final char DAY_IN_MONTH = 'D';

    /** Repeat rule week-in-month char representation. */
    private static final char WEEK_IN_MONTH = 'P';

    /** Repeat rule day-in-year char representation. */
    private static final char DAY_IN_YEAR = 'D';

    /** Repeat rule month-in-year char representation. */
    private static final char MONTH_IN_YEAR = 'M';

    private PIMItem pimItem;

    /** List of the weekday masks from RepeatRule */
    private static final int[] DAYS_OF_WEEK = { RepeatRule.SUNDAY,
            RepeatRule.MONDAY, RepeatRule.TUESDAY, RepeatRule.WEDNESDAY,
            RepeatRule.THURSDAY, RepeatRule.FRIDAY, RepeatRule.SATURDAY };

    /**
     * List of vCalendar weekday codes. This sequence is parallel to
     * DAYS_OF_WEEK.
     */
    private static final String[] DAYS_OF_WEEK_CODES = { "SU", "MO", "TU",
            "WE", "TH", "FR", "SA" };

    /** List of the week in month masks from RepeatRule */
    private static final int[] WEEKS_OF_MONTH = { RepeatRule.FIRST,
            RepeatRule.SECOND, RepeatRule.THIRD, RepeatRule.FOURTH,
            RepeatRule.FIFTH, RepeatRule.LAST, RepeatRule.SECONDLAST,
            RepeatRule.THIRDLAST, RepeatRule.FOURTHLAST, RepeatRule.FIFTHLAST };

    /**
     * List of vCalendar week of month codes. This sequence is parallel to
     * WEEKS_OF_MONTH.
     */
    private static final String[] WEEKS_OF_MONTH_CODES = { "1+", "2+", "3+",
            "4+", "5+", "1-", "2-", "3-", "4-", "5-", };

    /** List of month in year masks from RepeatRule */
    private static final int[] MONTHS_IN_YEAR = { 0, RepeatRule.JANUARY,
            RepeatRule.FEBRUARY, RepeatRule.MARCH, RepeatRule.APRIL,
            RepeatRule.MAY, RepeatRule.JUNE, RepeatRule.JULY,
            RepeatRule.AUGUST, RepeatRule.SEPTEMBER, RepeatRule.OCTOBER,
            RepeatRule.NOVEMBER, RepeatRule.DECEMBER };

    public VCalRepeatRuleHelper() {
    }

    public void formatRRule(PIMItem pimItem, OutputStream os,
            boolean allFields, boolean allDay, Calendar endDate)
            throws PIMException {
        String rrule;
        String patternEndDate = "";
        if (endDate != null) {
            if (allDay) {
                patternEndDate = PIMItemHelper.formatAllDay(endDate);
            } else {
                patternEndDate = PIMItemHelper.formatDateTimeUTC(endDate);
            }
        }
        Log.trace(TAG_LOG, "patternEndDate=" + patternEndDate);
        RepeatRule rule=null;
        if(pimItem instanceof Event) {
            Event event = (Event) pimItem;
            rule = event.getRepeat();
        } else if(pimItem instanceof ToDo) {
            BlackBerryToDo task = (BlackBerryToDo) pimItem;
            rule = task.getRepeat();
        }

        if (rule != null) {
            String s = encodeRepeatRule(rule, 0);
            Log.info(TAG_LOG, "RULE :" + s);
            PimUtils.println(os, XVCalendar.RRULE + ":" + s);
            StringBuffer exdate = new StringBuffer();
            for (Enumeration e = rule.getExceptDates(); e.hasMoreElements();) {
                Date date = (Date) e.nextElement();
                exdate.append(DateUtil.formatDateTimeUTC(date));
                if (e.hasMoreElements()) {
                    // If more exceptions are to come, we need a semicolon
                    exdate.append(";");
                }
            }
            if (exdate.length() > 0) {
                String ex = exdate.toString();
                Log.info(TAG_LOG, "EXDATE:" + ex);
                PimUtils.println(os, "EXDATE: " + ex);
            }
        } else if (allFields) {
            PimUtils.println(os, XVCalendar.RRULE + ":");
        }
    }

    public boolean isInSyncRange(PIMItem item) {
        return true;
    }

    private String encodeRepeatRule(RepeatRule rule, int startFreq) {
        StringBuffer sb = new StringBuffer();
        int[] fields = rule.getFields();
        sort(fields);
        if (!contains(fields, RepeatRule.FREQUENCY)) {
            return null;
        }
        int frequency;
        if (startFreq != 0) {
            frequency = startFreq;
        } else {
            frequency = rule.getInt(RepeatRule.FREQUENCY);
        }
        int interval = 1; // default value according to JSR75 spec
        if (contains(fields, RepeatRule.INTERVAL) && (startFreq == 0)) {
            interval = rule.getInt(RepeatRule.INTERVAL);
        }
        String encodedCount = " #0"; // 
        if (contains(fields, RepeatRule.COUNT) && (startFreq == 0)) {
            encodedCount = " #" + rule.getInt(RepeatRule.COUNT);
            Log.debug(TAG_LOG, "COUNT REPEAT RULE : " + encodedCount);
        }
        // enddate - ISO 8601 clause 5.4.1
        String encodedEndDate = "";
        if (contains(fields, RepeatRule.END)) {
            encodedCount = "";
            encodedEndDate = " " + DateUtil.formatDateTimeUTC(rule
                    .getDate(RepeatRule.END));
            Log.debug(TAG_LOG, "END REPEAT RULE : " + encodedEndDate);
        }
        switch (frequency) {
        case RepeatRule.DAILY: {
            // D<interval> [<duration>]
            sb.append(DAILY);
            sb.append(interval);
            sb.append(encodedCount);
            break;
        }
        case RepeatRule.WEEKLY: {
            // W<interval> <weekday> [<duration>]
            sb.append(WEEKLY);
            sb.append(interval);
            if (contains(fields, RepeatRule.DAY_IN_WEEK)) {
                sb.append(encodeRepeatRuleDaysInWeek(rule));
            }
            sb.append(encodedCount);
            break;
        }
        case RepeatRule.MONTHLY: {
            sb.append(MONTHLY);
            if (contains(fields, RepeatRule.DAY_IN_MONTH)) {
                // MD<interval> <daynumber> [<duration>]
                sb.append(DAY_IN_MONTH);
                sb.append(interval);
                sb.append(" ");
                sb.append(rule.getInt(RepeatRule.DAY_IN_MONTH));
                sb.append(encodedCount);
            } else if (contains(fields, RepeatRule.WEEK_IN_MONTH)) {
                sb.append(WEEK_IN_MONTH);
                sb.append(interval);
                sb.append(encodeRepeatRuleWeeksInMonth(fields, rule));
                if (contains(fields, RepeatRule.DAY_IN_WEEK)) {
                    sb.append(encodeRepeatRuleDaysInWeek(rule));
                }
                sb.append(encodedCount);
            }
            break;
        }
        case RepeatRule.YEARLY: {
            for (int k=0; k<fields.length; k++){
                Log.debug(TAG_LOG, "Yearly field: " + Integer.toBinaryString(fields[k]));
            }
            if (contains(fields, RepeatRule.WEEK_IN_MONTH) &&
                contains(fields, RepeatRule.DAY_IN_WEEK)) {
                // We express this occurrence using a monthly one, because
                // the day of the year depends on how many days there are in
                // that year
                Log.debug(TAG_LOG, "Transforming an yearly event into a monthly one");
                sb.append(MONTHLY).append(WEEK_IN_MONTH).append(12 * interval);
                sb.append(encodeRepeatRuleWeeksInMonth(fields, rule));
                sb.append(encodeRepeatRuleDaysInWeek(rule));
                sb.append(encodedCount);
            } else {
                sb.append(YEARLY);
                if (contains(fields, RepeatRule.DAY_IN_YEAR)) {
                    sb.append(DAY_IN_YEAR);
                    sb.append(interval);
                    sb.append(" ");
                    sb.append(rule.getInt(RepeatRule.DAY_IN_YEAR));
                    sb.append(encodedCount);
                } else if (contains(fields, RepeatRule.MONTH_IN_YEAR)) {
                    sb.append(MONTH_IN_YEAR);
                    sb.append(interval);
                    sb.append(encodeRepeatRuleMonthsInYear(fields, rule));
                    sb.append(encodedCount);
                    // The day in month is kept equal to the start date, so
                    // there is no need to format it
                } else if (contains(fields, RepeatRule.INTERVAL)) {//add for recurrence yearly without relative date 
                    sb.append(MONTH_IN_YEAR);
                    sb.append(interval);
                    sb.append(encodedCount);
                }
            }
            break;
        }
        default:
            return null;
        }
        if (startFreq == 0) {
            sb.append(encodedEndDate);
        }
        return sb.toString();
    }

    /**
     * Returns a string representation of a weekly rule.
     * 
     * @param rule
     *            data to be encoded
     * @return encoded rule
     */
    private String encodeRepeatRuleDaysInWeek(RepeatRule rule) {
        StringBuffer sb = new StringBuffer();
        int daysInWeek = rule.getInt(RepeatRule.DAY_IN_WEEK);
        for (int i = 0; i < DAYS_OF_WEEK.length; i++) {
            if ((daysInWeek & DAYS_OF_WEEK[i]) != 0) {
                sb.append(" ");
                sb.append(DAYS_OF_WEEK_CODES[i]);
            }
        }
        return sb.toString();
    }

    /**
     * Returns a string representation of a monthly rule with a weekly
     * parameter.
     * 
     * @param fields
     *            data to be processed
     * @param rule
     *            to encode
     * @return encoded rule
     */
    private String encodeRepeatRuleWeeksInMonth(int[] fields, RepeatRule rule) {
        StringBuffer sb = new StringBuffer();
        int weeksInMonth = rule.getInt(RepeatRule.WEEK_IN_MONTH);
        for (int i = 0; i < WEEKS_OF_MONTH.length; i++) {
            if ((weeksInMonth & WEEKS_OF_MONTH[i]) != 0) {
                sb.append(" ");
                sb.append(WEEKS_OF_MONTH_CODES[i]);
            }
        }
        return sb.toString();
    }

    /**
     * Returns a string representation of a yearly rule with a monthly
     * parameter.
     * 
     * @param fields
     *            data to be processed
     * @param rule
     *            to encode
     * @return encoded rule
     */
    private String encodeRepeatRuleMonthsInYear(int[] fields, RepeatRule rule) {
        StringBuffer sb = new StringBuffer();
        int monthsInYear = rule.getInt(RepeatRule.MONTH_IN_YEAR);
        for (int i = 0; i < MONTHS_IN_YEAR.length; i++) {
            if ((monthsInYear & MONTHS_IN_YEAR[i]) != 0) {
                sb.append(" ");
                sb.append(i);
            }
        }
        return sb.toString();
    }

    /**
     * Sorts an array of integers.
     * 
     * @param a
     *            the list of integers
     */
    private void sort(int[] a) {
        // insertion sort
        for (int j = 1; j < a.length; j++) {
            int v = a[j];
            int i = j - 1;
            while (i >= 0 && a[i] > v) {
                a[i + 1] = a[i];
                i--;
            }
            a[i + 1] = v;
        }
    }

    /**
     * Checks to see if a sorted array of integers contains a given integer.
     * 
     * @param a
     *            input array to be checked
     * @param value
     *            to be checked int the array
     * @return <code>true</code> if the value is found int the array
     */
    private boolean contains(int[] a, int value) {
        // binary chop search
        int lowerBound = 0;
        int upperBound = a.length - 1;
        while (upperBound - lowerBound >= 0) {
            int i = lowerBound + (upperBound - lowerBound) / 2;
            int v = a[i];
            if (v > value) {
                // look between lowerBound and i
                upperBound = i - 1;
            } else if (v < value) {
                // look between i and upperBound
                lowerBound = i + 1;
            } else {
                return true;
            }
        }
        return false;
    }
}
