/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Calendar;
import java.util.TimeZone;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;

import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;
import javax.microedition.pim.Event;
import javax.microedition.pim.EventList;
import javax.microedition.pim.RepeatRule;
import javax.microedition.pim.ToDo;
import javax.microedition.pim.ToDoList;

import net.rim.blackberry.api.pdap.BlackBerryEvent;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.blackberry.api.pdap.BlackBerryToDo;

import com.funambol.blackberry.Customization;
import com.funambol.blackberry.helper.SIFRepeatRuleHelper;
import com.funambol.blackberry.helper.VCalEventHelper.BBVCalFormatter;
import com.funambol.blackberry.helper.VCalEventHelper.BBVCalParserListener;
import com.funambol.blackberry.source.AdditionalItemsAttributes;
import com.funambol.blackberry.source.CalendarAdditionalAttributes;

import com.funambol.common.pim.vcalendar.BasicVCalendarParserListener;
import com.funambol.common.pim.vcalendar.BasicVCalendar;
import com.funambol.common.pim.ParserProperty;
import com.funambol.common.pim.xvcalendar.XVCalendar;
import com.funambol.common.pim.xvcalendar.XVCalendarSyntaxParserListener;
import com.funambol.common.pim.xvcalendar.XVCalendarSyntaxParser;
import com.funambol.common.pim.xvcalendar.XVCalendarFormatter;
import com.funambol.common.pim.xvcalendar.ParseException;
import com.funambol.common.pim.xvcalendar.rrule.RRule10SyntaxParser;
import com.funambol.common.pim.xvcalendar.rrule.RRule10SyntaxParserListener;
import com.funambol.common.pim.PimUtils;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.XmlUtil;
import com.funambol.util.DateUtil;
import com.funambol.util.StringUtil;
import com.funambol.util.Log;

public class VCalTaskHelper extends TaskHelper {
    private static final String TAG_LOG = "VCalTaskHelper";

    private static final String ACCEPTED = "ACCEPTED";
    private static final String NEEDS_ACTION = "NEEDS ACTION"; // in progress
    private static final String DECLINED = "DECLINED";
    private static final String DELEGATED = "DELEGATED";
    private static final String TENTATIVE = "TENTATIVE";
    private static final String COMPLETED = "COMPLETED";
    private static final String IN_PROCESS = "CONFIRMED";

    private static final String SUNDAY = "SU";
    private static final String MONDAY = "MO";
    private static final String TUESDAY = "TU";
    private static final String WEDNESDAY = "WE";
    private static final String THURSDAY = "TH";
    private static final String FRIDAY = "FR";
    private static final String SATURDAY = "SA";
    private static final String LASTDAY = "LD";

    /**
     * The constructor
     */
    public VCalTaskHelper(PIMList list) {
        super(list);
    }

    public String itemToString(PIMItem item,
            AdditionalItemsAttributes additionals) throws PIMException {

        if (item instanceof ToDo) {
            BBVCalFormatter formatter = new BBVCalFormatter();
            ByteArrayOutputStream os = new ByteArrayOutputStream();
            formatter.format(item, os, true);
            Log.debug(TAG_LOG, "Todo created as:" + os.toString());
            return os.toString();
        } else {
            throw new PIMException("Unknown item class type");
        }

    }

    protected class BBVCalFormatter extends XVCalendarFormatter {

        protected boolean isAllDay(PIMItem item) {
            return getItemBoolean(item, BlackBerryEvent.ALLDAY, 0, false);
        }

        protected void formatFunambolAllday(PIMItem pimItem, OutputStream os)
                throws PIMException {
            BlackBerryToDo task = (BlackBerryToDo) pimItem;
            if (isAllDay(task)) {
                PimUtils.println(os, XVCalendar.X_FUNAMBOL_ALLDAY + ":" + "1");
            }
        }

        public void formatAlarm(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {
            BlackBerryToDo task = (BlackBerryToDo) pimItem;
            Calendar reminder;
            String formattedDueDate = "";
            boolean allDay = isAllDay(pimItem);
            if (isSupportedField(REMINDER_FIELD) &&
                pimItem.countValues(REMINDER_FIELD) > 0) {

                    reminder = getFieldDate(task, REMINDER_FIELD, false, allDay, false);
                    formattedDueDate = reminder != null ? PIMItemHelper.formatDateTime(reminder) : "";
            }
            if (formattedDueDate.length() > 0 || allFields) {
                Log.debug(TAG_LOG, "AALARM" + formattedDueDate);
                PimUtils.println(os, "AALARM" + ":" + formattedDueDate + ";;0;");
            }
        }

        protected void formatStatus(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {
            BlackBerryToDo task = (BlackBerryToDo) pimItem;
            int c = VCalTaskHelper.SIF_BB_Status[0][0];
            String statusDescription = VCalTaskHelper.ACCEPTED;
            try {
                if (isSupportedField(BB_STATUS) &&
                    pimItem.countValues(BB_STATUS) > 0) {

                    int status = task.getInt(BB_STATUS, 0);
                    Log.debug(TAG_LOG, "BlackBerry status [" + status + "]");

                    for (int i = 0; i < VCalTaskHelper.SIF_BB_Status.length; i++) {
                        if (status == VCalTaskHelper.SIF_BB_Status[i][1]) {
                            c = VCalTaskHelper.SIF_BB_Status[i][0];
                        }
                    }
                    if (c == 0) {
                        statusDescription = VCalTaskHelper.ACCEPTED;
                    } else if (c == 1) {
                        statusDescription = VCalTaskHelper.IN_PROCESS;
                    } else if (c == 2) {
                        statusDescription = VCalTaskHelper.COMPLETED;
                    } else if (c == 3) {
                        statusDescription = VCalTaskHelper.NEEDS_ACTION;
                    } else if (c == 4) {
                        statusDescription = VCalTaskHelper.DECLINED;
                    }
                }
            } catch (final Exception e) {
                Log.error(TAG_LOG, "Cannot format status", e);
            }
            Log.debug(TAG_LOG, "STATUS : " + statusDescription);
            PimUtils.println(os, "PERCENT-COMPLETE:0");
            PimUtils.println(os, "STATUS:" + statusDescription);
        }

        public void formatDue(PIMItem pimItem, OutputStream os, boolean allFields) throws PIMException {
            BlackBerryToDo task = (BlackBerryToDo) pimItem;
            boolean allDay = isAllDay(pimItem);
            String formattedDueDate = "";
            String formattedEndDate = "";
            // The field is supported on all BB, no need to check for this
            if (isSupportedField(BlackBerryToDo.DUE) &&
                pimItem.countValues(BlackBerryToDo.DUE) > 0) {

                Calendar dueDate = getFieldDate(task, BlackBerryToDo.DUE, false, allDay, false);
                if (dueDate != null) {
                    formattedDueDate = dueDate != null ? PIMItemHelper
                        .formatDateTime(dueDate) : "";
                    formattedEndDate = formattedDueDate.substring(0,
                        formattedDueDate.length() - 6)
                        + "235900";
                }

                if (dueDate != null || allFields) {
                    Log.debug(TAG_LOG, "DUE Date: [" + formattedDueDate + ","
                              + formattedEndDate + "]");
                    // PimUtils.println(os, "DTSTART:" + formattedDueDate);
                    PimUtils.println(os, "DUE:" + formattedEndDate);
                }
            }
        }

        public void formatRRule(PIMItem pimItem, OutputStream os,
                boolean allFields) throws PIMException {
            Log.trace(TAG_LOG, "Formatting rrule");
            VCalRepeatRuleHelper rruleHelper = new VCalRepeatRuleHelper();
            boolean allDay = isAllDay(pimItem);
            Calendar endDate = null;
            if (isSupportedField(BlackBerryToDo.DUE) &&
                pimItem.countValues(BlackBerryToDo.DUE) > 0) {

                endDate = getFieldDate(pimItem, BlackBerryToDo.DUE, false, allDay, false);
            }
            rruleHelper.formatRRule(pimItem, os, allFields, allDay, endDate);
        }
    }

    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString,
            AdditionalItemsAttributes additionals) throws PIMException {

        Log.debug(TAG_LOG, "TASK pimString [" + pimString + "]");

        ToDoList list = getToDoList();

        if (list != null) {
            Hashtable taskMap = buildMapFromVCal(pimString, item);
            if (taskMap != null) {
                fillTask((BlackBerryToDo) item, taskMap);
                Log.debug(TAG_LOG, "Task saved");
            } else {
                Log.debug(TAG_LOG, "Hashtable properties is null.");
            }
        }
        // return null because Additionals attributes are not used
        return null;
    }

    private Hashtable buildMapFromVCal(String vCal, PIMItem item) {

        Log.debug(TAG_LOG, "BuildMap vCal [" + vCal + "]");

        Hashtable hash = null;
        Vector exceptionDates = new Vector(2);
        exceptionDates.addElement(SIFRepeatRuleHelper.EXCLUDE_DATE);
        exceptionDates.addElement(SIFRepeatRuleHelper.INCLUDE_DATE);

        try {
            BBVCalParserListener listener = new BBVCalParserListener(item);
            ByteArrayInputStream is = new ByteArrayInputStream(vCal.getBytes());
            XVCalendarSyntaxParser parser = new XVCalendarSyntaxParser(is);
            parser.setListener(listener);
            parser.parse();
            hash = listener.getHash();
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot parse vcal event: " + vCal, e);
        }
        return hash;
    }

    private String formatGMT(long date) {
        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("GMT"));
        cal.setTime(new Date(date));
        return PIMItemHelper.formatDateTimeUTC(cal);
    }

    public class BBVCalParserListener extends BasicVCalendarParserListener
            implements XVCalendarSyntaxParserListener,
            RRule10SyntaxParserListener {

        private final String LOG_TAG = "BBVCalParserListener";

        private static final String defaultCharset = BasicVCalendar.UTF8;

        /** Check whether this is an allday event */
        protected boolean allDay = false;

        private Hashtable hash;

        /**
         * The construtor accepts a <code>PIMItem</code> object that will be
         * populated of all the common vCalendar properties.
         * 
         * @param pimItem
         *            the PIMItem object
         */
        public BBVCalParserListener(PIMItem item) {
            super(item);
        }

        public void addEventProperty(ParserProperty property) {
            Log.trace(LOG_TAG, "addEventProperty: " + property.getName());
        }

        public void addProperty(ParserProperty property) {
        }

        public void addToDoProperty(ParserProperty property) {

            Log.trace(LOG_TAG, "addTODOProperty: " + property.getName());

            String name = property.getName();
            String value = getClearValue(property);

            if (StringUtil.equalsIgnoreCase("SUMMARY", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                hash.put(SUBJECT, value);
            } else if (StringUtil.equalsIgnoreCase("DESCRIPTION", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                hash.put(BODY, value);
            } else if (StringUtil.equalsIgnoreCase("CATEGORIES", name)) {

                if (!StringUtil.isNullOrEmpty(value)) {
                    hash.put(CATEGORIES, value);
                    Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                            + " => " + value);
                }
                //else
                //    hash.put(CATEGORIES, "personal"); // temporary . If the category
                                                    // is empty an error is
                                                    // generated

            } else if (StringUtil.equalsIgnoreCase(XVCalendar.AALARM, name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                hash.put(REMINDER_SET, "1");
                int sepIdx = value.indexOf(";");
                if (sepIdx >= 0) {
                    value = value.substring(0, sepIdx);
                }
                hash.put(REMINDER_TIME, value);
            } else if (StringUtil.equalsIgnoreCase("CLASS", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
            } else if (StringUtil.equalsIgnoreCase("DTSTART", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                // hash.put(DUE_DATE, value);
            } else if (StringUtil.equalsIgnoreCase("DUE", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                hash.put(DUE_DATE, value);
            } else if (StringUtil.equalsIgnoreCase("COMPLETION_DATE", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                hash.put(DATE_COMPLETED, value);
            } else if (StringUtil.equalsIgnoreCase("PRIORITY", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                String priority_value = "0";
                if (value.equalsIgnoreCase("1")) {
                    priority_value = "2";
                } else if (value.equalsIgnoreCase("2")) {
                    priority_value = "1";
                } else if (value.equalsIgnoreCase("3")) {
                    priority_value = "0";
                }
                hash.put(IMPORTANCE, priority_value);
            } else if (StringUtil.equalsIgnoreCase("ORGANIZER", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
            } else if (StringUtil.equalsIgnoreCase("PERCENT-COMPLETE", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
            } else if (StringUtil.equalsIgnoreCase("RRULE", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                ByteArrayInputStream rruleIs = new ByteArrayInputStream(value
                        .getBytes());
                RRule10SyntaxParser rruleParser = new RRule10SyntaxParser(
                        rruleIs);
                rruleParser.setListener(this);
                try {
                    rruleParser.parse();
                } catch (Exception e) {
                    Log.error(TAG_LOG,
                            "Cannot parser recurrence rule " + value, e);
                }
            } else if (StringUtil.equalsIgnoreCase("COMPLETED", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
            } else if (StringUtil.equalsIgnoreCase("STATUS", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
                String sif_value = "0";
                if (value.equalsIgnoreCase("ACCEPTED")) {
                    sif_value = "0";
                } else if (value.equalsIgnoreCase("CONFIRMED")) {
                    sif_value = "1";
                } else if (value.equalsIgnoreCase("COMPLETED")) {
                    sif_value = "2";
                } else if (value.equalsIgnoreCase("NEEDS ACTION")) {
                    sif_value = "3";
                } else if (value.equalsIgnoreCase("DECLINED")) {
                    sif_value = "4";
                }

                hash.put(STATUS, sif_value);
            } else if (StringUtil.equalsIgnoreCase("X-FUNAMBOL-FOLDER", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
            } else if (StringUtil.equalsIgnoreCase("X-FUNAMBOL-ALLDAY", name)) {
                Log.trace(LOG_TAG, "addTODOProperty: " + property.getName()
                        + " => " + value);
            } else if (StringUtil.equalsIgnoreCase(BasicVCalendar.CLASS, name)) {

                String sensitivity = null;
                if (StringUtil.equalsIgnoreCase("PUBLIC", value)) {
                    sensitivity = "0";
                } else if (StringUtil.equalsIgnoreCase("PRIVATE", value)) {
                    sensitivity = "2";
                } else if (StringUtil.equalsIgnoreCase("CONFIDENTIAL", value)) {
                    sensitivity = "3";
                }
                if (sensitivity == null) {
                    Log.error(TAG_LOG, "Unsupported class value: " + value);
                } else {
                    hash.put(SENSITIVITY, sensitivity);
                }
            } else {
                Log.error(LOG_TAG, "Unsupported property: " + name);
            }
        }

        public void start() {
        }

        public void end() {
        }

        public void startEvent() throws ParseException {
        }

        public void endEvent() throws ParseException {
        }

        public void startToDo() throws ParseException {
            hash = new Hashtable();
        }

        public void endToDo() throws ParseException {
            Log.info(TAG_LOG, "todo :" + hash.toString());
        }

        // //////////////// RRule10 listener implementation //////////////
        public void setEndDate(String endDate) {
            Log.info(TAG_LOG, "endDate :" + endDate);
            hash.put(PATTERN_END_DATE, endDate);
        }

        public void setType(String type) {
            int t = -1;
            if ("D".equals(type)) {
                t = 0;
            } else if ("W".equals(type)) {
                t = 1;
            } else if ("MD".equals(type)) {
                t = 2;
            } else if ("MP".equals(type)) {
                t = 3;
            } else if ("YM".equals(type)) {
                t = 5;
            } else if ("YD".equals(type)) {
                t = 6;
            }
            if (t >= 0) {
                hash.put(IS_RECURRING, "1");
                hash.put(RECURRENCE_TYPE, "" + t);
            }
            Log.info(TAG_LOG, "type :" + type);
        }

        public void setInterval(String interval) {
            Log.info(TAG_LOG, "interval :" + interval);
            hash.put(INTERVAL, interval);
        }

        public void setDuration(String duration) {
            Log.info(TAG_LOG, "Duration :" + duration);
            hash.put(OCCURRENCES, duration);
        }

        public void setDays(String days) {
            int t = 0;
            int bit_mask = 0;
            if (days.length() > 0) {
                String[] daysStr = StringUtil.split(days, ",");
                for (int j = 0; j < daysStr.length; j++) {
                    String day = daysStr[j];
                    if (SUNDAY.equals(day)) { // Sunday 1
                        t = 1;
                    } else if (MONDAY.equals(day)) { // Monday 2
                        t = 2;
                    } else if (TUESDAY.equals(day)) { // Tuesday 4
                        t = 4;
                    } else if (WEDNESDAY.equals(day)) { // Wednesday 8
                        t = 8;
                    } else if (THURSDAY.equals(day)) { // Thursday 16
                        t = 16;
                    } else if (FRIDAY.equals(day)) { // Friday 32
                        t = 32;
                    } else if (SATURDAY.equals(day)) { // Saturday 64
                        t = 64;
                    } else if (LASTDAY.equals(day)) {
                        Log.error(TAG_LOG, "LD not implemented");
                        throw new IllegalArgumentException("LD not implemented");
                    } else {
                        Log.error(TAG_LOG, "Unsupported day: " + day);
                        throw new IllegalArgumentException("Unsupported day "
                                + day);
                    }
                    if (bit_mask != 0)
                        bit_mask = bit_mask | t;
                    else
                        bit_mask = t;
                }
                hash.put(DAY_OF_WEEK_MASK, Integer.toString(bit_mask));
            }
            Log.info(TAG_LOG, "Days :" + Integer.toString(bit_mask));
        }

        public void setDaysNumber(String days) {
            Log.error(TAG_LOG, "setDaysNumber not implemented");
            // TODO FIXME: the days can be a list with with any number of days,
            // but at the moment we only support one day, because JSR75 only
            // allows for one day
            String[] d = StringUtil.split(days, ",");
            if (d.length > 1) {
                Log.error(TAG_LOG,
                        "Unsupported multiple days in MD recurrence. Keeping only the first day "
                                + days);
            }
            hash.put(EventHelper.DAY_OF_MONTH, d[0]);
        }

        public void setMonths(String monthlist) {
            // Yearly by month (YM) only
            // At the moment our parser is limited to one occurrence, so we are
            // ok, but the vCal spec seems to indicate more occurrences are
            // allowed
            Log.info(TAG_LOG, "Monthlist :" + monthlist);
            hash.put(MONTH_OF_YEAR, monthlist);
        }

        public void setOccurrences(String occurrences) {
            // Occurrences are only for "MP" recurrences
            // At the moment our parser is limited to one occurrence, so we are
            // ok, but the vCal spec seems to indicate more occurrences are
            // allowed
            String occ = StringUtil.replaceAll(occurrences, "+", "");
            Log.info(TAG_LOG, "Occurrences :" + occ);
            hash.put(INSTANCE, occ);
        }

        public Hashtable getHash() {
            return hash;
        }
    }
}
