/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.helper;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStream;


import javax.microedition.pim.PIM;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;
import javax.microedition.pim.PIMException;

import javax.microedition.pim.Contact;
import javax.microedition.pim.ContactList;

import net.rim.device.api.io.Base64InputStream;
import net.rim.device.api.system.EncodedImage;

import com.funambol.blackberry.source.AdditionalItemsAttributes;

import com.funambol.blackberry.source.ContactAdditionalAttributes;
import com.funambol.common.pim.*;
import com.funambol.common.pim.vcard.*;

import com.funambol.util.Log;
import com.funambol.util.StringUtil;

/**
 * This class is the helper for BB vCard sync source. It is responsible for
 * adding/updating changes from the server and reading items from the local store.
 * The class also provides utility methods used by the BBPimSyncSource.
 * VCard parsing and formatting is carried out by classes implemented in j2me
 * apis.
 */

public class VCardContactHelper extends ContactHelper
{
    private static final String TAG_LOG = "VCardContactHelper";

    private static final int[] SUPPORTED_FIELDS =
    {
        Contact.NAME
        ,Contact.EMAIL
        ,Contact.NOTE
        ,Contact.TITLE
        ,Contact.ORG
        ,Contact.URL
        ,Contact.TEL
        ,Contact.ADDR
        ,Contact.PHOTO
        ,Contact.BIRTHDAY
    };


    private ContactAdditionalAttributes contactAddits = null;

    private final boolean ENABLED_ADD_NEW_CATECORY = true;

    /**
     * The initialization constructor
     * @param ContactList A BlackBerry contacts list
     * @param Contact A BlackBerry contact
     * @param boolean To modify or not to modify: that is the question...
     */
    public VCardContactHelper(PIMList list)
    {
        super(list);
    }

    /**
     * This class extends the default VCardFormatter to reimplement the method
     * that autodetects the mime type of a picture. Standard j2me APIs do not
     * allow for such a feature, so we rely on RIM specific APIs to carry out
     * the task.
     */
    private class BBVCardFormatter extends VCardFormatter {

        private ContactAdditionalAttributes additionals;

        public BBVCardFormatter() {
            super();
        }

        public BBVCardFormatter(String defaultCharset) {
            super(defaultCharset);
        }

        public BBVCardFormatter(String defaultCharset, ContactAdditionalAttributes additionals) {
            super(defaultCharset);
            this.additionals = additionals;
        }


        // Is this really necessary? How does the BB stores photos? If they are
        // always converted to jpeg we are safe, otherwise we need this code
        // to detect the type
        protected String getPhotoType(byte[] photo) {
            try {
                byte[] photoDecoded = Base64InputStream.decode(photo, 0, photo.length);
                EncodedImage image = EncodedImage.createEncodedImage(photoDecoded, 0, -1);

                String imageType = image.getMIMEType();
                int index = imageType.indexOf("/");
                String type = imageType.substring(index+1);

                return type;
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot detect image type ", e);
                return super.getPhotoType(photo);
            }
        }

        protected void formatEmail(Contact contact, OutputStream os, boolean allFields)
                throws PIMException{
            if(additionals == null){
               Log.debug(TAG_LOG, "format emails using super method");
               super.formatEmail(contact, os, allFields);
            }else{
                Log.debug(TAG_LOG, "format emails using reimplemented method");
                String other = additionals.getEmail(ContactAdditionalAttributes.EMAIL_INTENET);
                String home  = additionals.getEmail(ContactAdditionalAttributes.EMAIL_HOME);
                String work  = additionals.getEmail(ContactAdditionalAttributes.EMAIL_WORK);


                if (allFields || other.length() > 0) {
                    formatField(EMAIL_OTHER_TAG, other, os);
                }
                if (allFields || home.length() > 0) {
                    formatField(EMAIL_HOME_TAG, home, os);
                }
                if (allFields || work.length() > 0) {
                    formatField(EMAIL_WORK_TAG, work, os);
                }
            }
        }

        protected void formatFaxTag (StringBuffer out, String tag){
            Log.debug(TAG_LOG, "formatting fax type");
            int faxType;
            if (additionals != null) {
                faxType = additionals.getFaxType();
            } else {
                faxType = ContactAdditionalAttributes.FAX_NONE;
            }
            if(faxType == ContactAdditionalAttributes.FAX_HOME){
                formatTag(out, VCardFormatter.FAX_HOME_TAG);
            }else if(faxType == ContactAdditionalAttributes.FAX_OTHER){
                formatTag(out, VCardFormatter.FAX_OTHER_TAG);
            }else{
                formatTag(out, VCardFormatter.FAX_WORK_TAG);
            }
        }

        protected void formatTel(Contact contact, OutputStream os, boolean allFields)
            throws PIMException
        {
            // Print the telephones
            String home   = "";
            String work   = "";
            String pager  = "";
            String fax    = "";
            String mobile = "";
            String other  = "";
            /* It's needs for devices OS > 5.0 where there are
             * MOBILE2 & FAX_HOME fields where BB have some issue
             * http://supportforums.blackberry.com/t5/Java-Development/Undocumented-Contacts-fields-in-version-4-7-1-and-5-0
                         /m-p/383862;jsessionid=6B22E5675660A84B5BF18DBDB0C21ED7
             */
            boolean mobileFormatted = false;
            boolean faxFormatted = false;

            for (int i = 0; i < contact.countValues(Contact.TEL); i++)
            {
                final String value = contact.getString(Contact.TEL, i);
                if (value != null) {
                    final int type = contact.getAttributes(Contact.TEL, i);
                    switch (type) {
                        case Contact.ATTR_HOME:
                            home = pimUtils.escape(value,false);
                            break;
                        case Contact.ATTR_WORK:
                            work = pimUtils.escape(value,false);
                            break;
                        case Contact.ATTR_PAGER:
                            pager = pimUtils.escape(value,false);
                            break;
                        case Contact.ATTR_FAX:
                            if(!faxFormatted){
                                fax = pimUtils.escape(value,false);
                                faxFormatted = true;
                            }
                            break;
                        case Contact.ATTR_MOBILE:
                            if(!mobileFormatted){
                                mobile = pimUtils.escape(value,false);
                                mobileFormatted = true;
                            }
                            break;
                        case Contact.ATTR_OTHER:
                            other = pimUtils.escape(value,false);
                            break;
                        default:
                            Log.error(TAG_LOG, "Unsupported telephone type");
                    }
                }
            }
             // Now print everything
            if (allFields || home.length() > 0) {
                StringBuffer value = new StringBuffer();
                formatTag(value, TEL_HOME_TAG);
                value.append(home);
                pimUtils.println(os, value.toString());
            }
            if (allFields || work.length() > 0) {
                StringBuffer value = new StringBuffer();
                formatTag(value, TEL_WORK_TAG);
                value.append(work);
                pimUtils.println(os, value.toString());
            }
            if (allFields || pager.length() > 0) {
                StringBuffer value = new StringBuffer();
                formatTag(value, TEL_PAGER_TAG);
                value.append(pager);
                pimUtils.println(os, value.toString());
            }
            if (allFields || fax.length() > 0) {
                StringBuffer value = new StringBuffer();
                //formatTag(value, FAX_WORK_TAG);
                formatFaxTag(value, FAX_WORK_TAG);
                value.append(fax);
                pimUtils.println(os, value.toString());
            }
            if (allFields || mobile.length() > 0) {
                StringBuffer value = new StringBuffer();
                formatTag(value, TEL_MOBILE_TAG);
                value.append(mobile);
                pimUtils.println(os, value.toString());
            }
            if (allFields || other.length() > 0) {
                StringBuffer value = new StringBuffer();
                formatTag(value, TEL_OTHER_TAG);
                value.append(other);
                pimUtils.println(os, value.toString());
            }
        }

        protected int[] getSupportedFields() {
            return SUPPORTED_FIELDS;
        }
    }

    /**
     * This class extends the default VCardSyntaxParser to reimplement the method
     * that parse the mails for manage the three mails (internet - work - home).
     * Native contact application doesn't discern mail type
     */
    private class BBVCardListener extends ContactParserListener {

        private AdditionalItemsAttributes oldAdditionals;

        private String[] fax = new String[]{null,null,null};

        public BBVCardListener(Contact contact, boolean addNewCategory) {
            super(contact, addNewCategory);
        }

        public void setMail(String content, ParamList plist,
                        Token group) throws ParseException{
            if(oldAdditionals == null){
                addNewEmailAddresses(content, plist);
            }else{
                setEmailAddresses(content, plist);
            }

        }

        private void addNewEmailAddresses(String content, ParamList plist)throws ParseException{

            if(plist.containsKey("INTERNET") && plist.getSize() == 1) {
                contactAddits.addEmail(ContactAdditionalAttributes.EMAIL_INTENET,
                                        content,plist.containsKey("pref"));
            } else if (plist.containsKey("HOME")) {
                if (((Hashtable)plist.getXParams()).containsKey("X-FUNAMBOL-INSTANTMESSENGER")) {
                    //Ignore IM Address because is not supported by devices
                }else{
                    contactAddits.addEmail(ContactAdditionalAttributes.EMAIL_HOME,
                                        content,plist.containsKey("pref"));
                }
            } else if (plist.containsKey("WORK")) {
                contactAddits.addEmail(ContactAdditionalAttributes.EMAIL_WORK,
                                        content,plist.containsKey("pref"));
            }

        }

        private void setEmailAddresses(String content, ParamList plist)throws ParseException{
            if (plist.containsKey("INTERNET") && plist.getSize() == 1) {
                contactAddits.setEmail(ContactAdditionalAttributes.EMAIL_INTENET,
                                        content,plist.containsKey("pref"));
            } else if (plist.containsKey("HOME")) {
                if (((Hashtable)plist.getXParams()).containsKey("X-FUNAMBOL-INSTANTMESSENGER")) {
                    //Ignore consider IM Address because is not supported by devices
                }else{
                    contactAddits.setEmail(ContactAdditionalAttributes.EMAIL_HOME,
                                        content,plist.containsKey("pref"));
                }
                
            } else if (plist.containsKey("WORK")) {
                contactAddits.setEmail(ContactAdditionalAttributes.EMAIL_WORK,
                                        content,plist.containsKey("pref"));
            }

        }

        protected void setFax(String content, PIMList list, ParamList plist){
     
            if(plist.containsKey("WORK")){
                Log.debug(TAG_LOG, "WORK FAX: -" + content+";");
                Log.debug(TAG_LOG, "WORK FAX: length:"+content.length());
                if(content.length()>0){
                    fax[0] = content;
                }
            }else if(plist.containsKey("HOME")){
                Log.debug(TAG_LOG, "HOME FAX: -" + content+";");
                if(content.length()>0){
                    fax[1] = content;
                }
            }else{
                Log.debug(TAG_LOG, "OTHER FAX: -" + content+";");
                if(content.length()>0){
                    fax[2] = content;
                }
            }

            Log.trace(TAG_LOG, "setting fax Fax[0]: "+fax[0]);
            Log.trace(TAG_LOG, "setting fax Fax[1]: "+fax[1]);
            Log.trace(TAG_LOG, "setting fax Fax[2]: "+fax[2]);
            
        }


        public void end() {
            Log.debug(TAG_LOG, "Vcard finalizing emails");

            String[] emails = contactAddits.getContactEmailsToStore();

            for(int i=0; i<emails.length;i++){
                contact.addString(Contact.EMAIL, Contact.ATTR_NONE, emails[i]);
            }

            Log.debug(TAG_LOG, "Vcard finalizing fax");
            
            if(fax[0] != null){
                Log.debug(TAG_LOG, "Setting WORK FAX to: " +  fax[0]);
                contact.addString(Contact.TEL, Contact.ATTR_FAX, fax[0]);
                contactAddits.setFaxType(ContactAdditionalAttributes.FAX_WORK);
            }else if(fax[1] != null){
                Log.debug(TAG_LOG, "Setting HOME FAX to: " +  fax[1]);
                contact.addString(Contact.TEL, Contact.ATTR_FAX, fax[1]);
                contactAddits.setFaxType(ContactAdditionalAttributes.FAX_HOME);
            }else if(fax[2] != null){
                Log.debug(TAG_LOG, "Setting OTHER FAX to: " +  fax[2]);
                contact.addString(Contact.TEL, Contact.ATTR_FAX, fax[2]);
                contactAddits.setFaxType(ContactAdditionalAttributes.FAX_OTHER);
            }else{
                Log.debug(TAG_LOG, "VCard doesn't have a fax number");
                contactAddits.setFaxType(ContactAdditionalAttributes.FAX_NONE);
            }

            Log.info(TAG_LOG, "Vcard parsing ended");
        }

        public void setOldAdditionals(AdditionalItemsAttributes addits){
            oldAdditionals = addits;
        }

        protected int[] getSupportedFields() {
            return SUPPORTED_FIELDS;
        }

        protected String preprocessNote(String text) {
            // We must replace all CRLF into single LF, otherwise the UI will
            // display an empty line
            return StringUtil.replaceAll(text, "\r\n", "\n");
        }
    }

    public AdditionalItemsAttributes handleAddedItem(PIMItem item)
    {
        if (item instanceof Contact)
        {
            String internet = "";
            String home  = "";
            String work  = "";

            Contact contact = (Contact)item;
            ContactAdditionalAttributes additionals = new ContactAdditionalAttributes();

            for(int i=0;i<contact.countValues(Contact.EMAIL);++i) {
                Log.debug(TAG_LOG, "Fetching email number: " + i);
                String value = contact.getString(Contact.EMAIL, i);
                if (value != null && value.length() > 0) {
                    switch (i) {
                        case 0:
                            internet = value;
                            break;
                        case 1:
                            home = value;
                            break;
                        case 2:
                            work = value;
                            break;
                    }
                }
            }
            additionals.addEmail(ContactAdditionalAttributes.EMAIL_INTENET, internet, false);
            additionals.addEmail(ContactAdditionalAttributes.EMAIL_HOME, home, false);
            additionals.addEmail(ContactAdditionalAttributes.EMAIL_WORK, work, false);
            
            return additionals;
        }
        return null;
    }

    public AdditionalItemsAttributes updateAdditionals(PIMItem newItem, AdditionalItemsAttributes oldAddits){
        if (newItem instanceof Contact)
        {
            Contact contact = (Contact)newItem;
            ContactAdditionalAttributes additionals = (ContactAdditionalAttributes)oldAddits;
           
            String internetAddits = additionals.getEmail(ContactAdditionalAttributes.EMAIL_INTENET);
            String homeAddits = additionals.getEmail(ContactAdditionalAttributes.EMAIL_HOME);
            String workAddits = additionals.getEmail(ContactAdditionalAttributes.EMAIL_WORK);


            String [] emailsContact = new String[]{"","",""};
            String newInternet = " ";
            String newHome = " ";
            String newWork = " ";

            for(int i=0;i<contact.countValues(Contact.EMAIL);++i) {
                Log.debug(TAG_LOG, "Fetching email number: " + i);
                String value = contact.getString(Contact.EMAIL, i);
                if (value != null && value.length() > 0) {
                    switch (i) {
                        case 0:
                            emailsContact[0] = value;
                            break;
                        case 1:
                            emailsContact[1] = value;
                            break;
                        case 2:
                            emailsContact[2] = value;
                            break;
                    }
                }
            }

            String[] emailsSub = new String[]{null,null,null};
            for(int i=0; i<emailsContact.length; i++){
                if(emailsContact[i].equals(internetAddits)){
                    newInternet =  emailsContact[i];
                }else if(emailsContact[i].equals(homeAddits)){
                    newHome = emailsContact[i];
                }else if(emailsContact[i].equals(workAddits)){
                    newWork = emailsContact[i];
                }else{
                    emailsSub[i] = emailsContact[i];
                }
            }

            for(int y=0; y<emailsSub.length; y++){
                if(newInternet.equals(" ") && emailsSub[y]!=null){
                    newInternet = emailsSub[y];
                }else if(newHome.equals(" ") && emailsSub[y]!=null){
                    newHome = emailsSub[y];
                }else if(newWork.equals(" ") && emailsSub[y]!=null){
                    newWork = emailsSub[y];
                }
            }


            additionals.setEmail(ContactAdditionalAttributes.EMAIL_INTENET, newInternet, false);
            additionals.setEmail(ContactAdditionalAttributes.EMAIL_HOME, newHome, false);
            additionals.setEmail(ContactAdditionalAttributes.EMAIL_WORK, newWork, false);

            
            return additionals;
        }
        return null;
    }

    public AdditionalItemsAttributes createAdditionals(PIMItem newItem){
        if (newItem instanceof Contact)
        {
            Contact contact = (Contact)newItem;
            Log.debug(TAG_LOG, "Creating new additionals...");
            contactAddits = new ContactAdditionalAttributes();


            String newInternet = " ";
            String newHome = " ";
            String newWork = " ";

            for(int i=0;i<contact.countValues(Contact.EMAIL);++i) {
                Log.debug(TAG_LOG, "Fetching email number: " + i);
                String value = contact.getString(Contact.EMAIL, i);
                if (value != null && value.length() > 0) {
                    switch (i) {
                        case 0:
                            newInternet = value;
                            break;
                        case 1:
                            newHome = value;
                            break;
                        case 2:
                            newWork = value;
                            break;
                    }
                }
            }

            contactAddits.addEmail(ContactAdditionalAttributes.EMAIL_INTENET, newInternet, false);
            contactAddits.addEmail(ContactAdditionalAttributes.EMAIL_HOME, newHome, false);
            contactAddits.addEmail(ContactAdditionalAttributes.EMAIL_WORK, newWork, false);

            String fax = null;

            for (int i = 0; i < contact.countValues(Contact.TEL); i++)
            {
                final String value = contact.getString(Contact.TEL, i);
                if (value != null) {
                    final int type = contact.getAttributes(Contact.TEL, i);
                    if(type == Contact.ATTR_FAX ){
                        fax = value;
                    }
                }
            }

            if (fax != null){
                contactAddits.setFaxType(ContactAdditionalAttributes.FAX_WORK);
            }else{
                Log.debug(TAG_LOG, "Contact does not have fax field filled");
            }


            return contactAddits;
        }
        return null;
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#toString(PIMList pimlist,PIMItem item)
     */
    public String itemToString(PIMItem item, AdditionalItemsAttributes additionals) throws PIMException
    {
        if (item instanceof Contact)
        {
            // should we remove this line after cleaning out the code
            ContactList list = (ContactList)getPIMList();

            Contact contact = (Contact)item;

            ByteArrayOutputStream ostream = new ByteArrayOutputStream();
            VCardFormatter formatter =  new BBVCardFormatter("UTF-8", (ContactAdditionalAttributes)additionals);


            formatter.format(contact, ostream, true);
            String vCard = ostream.toString();
            return vCard;
        }
        return null;
    }

    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#createPIMItem(PIMList pimlist)
     */
    public PIMItem createPIMItem(PIMList pimlist) throws PIMException
    {
        if (pimlist instanceof ContactList)
        {
            return ((ContactList)pimlist).createContact();
        }
        return null;
    }

    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public AdditionalItemsAttributes fillItem(PIMItem item, String pimString,
                                              AdditionalItemsAttributes additionals) throws PIMException
    {
        Log.debug(TAG_LOG, "pimString ["+pimString+"]");


        if(additionals == null){
            Log.debug(TAG_LOG, "additionals null - creating new additionals...");
            contactAddits = new ContactAdditionalAttributes();
        } else {
            contactAddits = (ContactAdditionalAttributes) additionals;
        }

        PIMList pimlist = getPIMList();
        if (pimlist instanceof ContactList)
        {
            ContactList list = (ContactList)pimlist;
            ByteArrayInputStream is = new ByteArrayInputStream(pimString.getBytes());
            PIM pim = PIM.getInstance();

            try {
                Contact newContact = (Contact)item;
                BBVCardListener lis = new BBVCardListener(newContact,ENABLED_ADD_NEW_CATECORY);
                lis.setOldAdditionals(additionals);
                VCardSyntaxParser parser = new VCardSyntaxParser(is);
                parser.setListener(lis);
                parser.parse();

            } catch (Exception pe) {
                Log.debug(TAG_LOG, "Exception while parsing the item " + pe.toString());
                throw new PIMException(pe.toString());
            }
        }
        return contactAddits;
    }

    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#setPIMProperties(PIMList pimlist, PIMItem item,String pimString)
     */
    public void updatePimItem(PIMList pimlist, PIMItem item,String pimString) throws PIMException
    {
        Log.debug(TAG_LOG, "pimString ["+pimString+"]");
        if (pimlist instanceof ContactList)
        {
            ContactList list = (ContactList)pimlist;
            ByteArrayInputStream is = new ByteArrayInputStream(pimString.getBytes());
            PIM pim = PIM.getInstance();

            try {
                Contact newContact = (Contact)item;
                //ContactParserListener lis = new ContactParserListener(newContact);
                BBVCardListener lis = new BBVCardListener(newContact,ENABLED_ADD_NEW_CATECORY);
                VCardSyntaxParser parser = new VCardSyntaxParser(is);
                parser.setListener(lis);
                parser.parse();
            } catch (ParseException pe) {
                Log.debug(TAG_LOG, "Exception while adding the item " + pe.toString());
                throw new PIMException(pe.toString());
            }
        }
    }

    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#find_and_deletePIMItem(PIMList pimlist, String itemKey, boolean deleteFlag)
     */
    public PIMItem findPIMItem(PIMList pimlist, String itemKey, boolean deleteFlag) throws PIMException
    {
        if (pimlist instanceof ContactList){
            ContactList _pimlist = (ContactList)pimlist;
            PIMItem item = null;
            Enumeration items = _pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                if (itemKey.equals(this.getUID(item)))
                {
                    if (deleteFlag)
                    {
                        _pimlist.removeContact((Contact)item);
                    }
                    items = null;
                    _pimlist = null;
                    return item;
                }
            }
            item = null;
            items = null;
            _pimlist = null;
        }
        return null;
    }
    //----------------------------------------------------------
    public Hashtable findPIMItems(PIMList pimlist, Vector itemKeys, boolean deleteFlag) throws PIMException
    {
        Hashtable ret = new Hashtable();
        if (pimlist instanceof ContactList)
        {
            ContactList _pimlist = (ContactList)pimlist;
            PIMItem item = null;
            Enumeration items = _pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                String uid = this.getUID(item);
                if (itemKeys.contains(uid))
                {
                    if (deleteFlag)
                    {
                        _pimlist.removeContact((Contact)item);
                    }
                    ret.put(uid,item);
                    itemKeys.removeElement(uid);
                }
            }
            item = null;
            items = null;
            _pimlist = null;
        }
        return ret;
    }


    //----------------------------------------------------------
    /*
     * (non-Javadoc)
     * @see com.funambol.blackberry.helper.PIMItemHelper#clearPIMItem(PIMItem _item)
     */
    public void clearItem(PIMItem item) throws PIMException
    {
        PIMItemHelper.clearPIMItem(item,Contact.UID, SUPPORTED_FIELDS);
    }

    public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws PIMException
    {
        copyUnsupportedFields(origItem, item, Contact.UID, SUPPORTED_FIELDS);
        // We shall copy also unsupported attributes of supported fields
        Contact source = (Contact)origItem;
        Contact target = (Contact)item;

        int[] attributes = source.getPIMList().getSupportedAttributes(Contact.TEL);

        for (int i = 0; i < source.countValues(Contact.TEL); i++)
        {
            final String value = source.getString(Contact.TEL, i);
            if (value != null) {
                final int attribute = source.getAttributes(Contact.TEL, i);

                if (attribute != Contact.ATTR_FAX &&
                    attribute != Contact.ATTR_PAGER &&
                    attribute != Contact.ATTR_MOBILE &&
                    attribute != Contact.ATTR_WORK &&
                    attribute != Contact.ATTR_HOME &&
                    attribute != Contact.ATTR_OTHER)
                {

                    Log.trace(TAG_LOG, "Preserving attribute: " + attribute);
                    target.addString(Contact.TEL, attribute, value);
                }
            }
        }
    }


    //----------------------------------------------------------
    public void deleteAllPIMItems(PIMList pimlist) throws PIMException
    {
        if (pimlist instanceof ContactList)
        {
            ContactList _pimlist = (ContactList)pimlist;
            PIMItem item = null;
            Enumeration items = _pimlist.items();
            while(items.hasMoreElements())
            {
                item = (PIMItem)items.nextElement();
                if (this.isInSyncRange(pimlist,item))
                {
                    _pimlist.removeContact((Contact)item);
                }
            }
            item = null;
            items = null;
            _pimlist = null;
        }
    }
    //----------------------------------------------------------
    public boolean isInSyncRange(PIMList pimlist, PIMItem _item) throws PIMException
    {
        return true;
    }

    public void setPIMProperties(PIMList pimlist, PIMItem item,String pimString) throws PIMException
    {
    }

    //----------------------------------------------------------
}

