/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;

import net.rim.blackberry.api.pdap.PIMListListener;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.system.ControlledAccessException;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.util.Persistable;

import com.funambol.blackberry.controller.SynchronizationController;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.blackberry.BBPIMListener;
import com.funambol.syncml.protocol.SyncFilter;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.protocol.SyncMLStatus;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SyncListener;

import com.funambol.util.StaticDataHelper;
import com.funambol.util.Log;

/**
 * Class for PIM sync management in BlackBerry devices
 */
public class BBPIMSyncSource implements SyncSource
{
    private static final String     TAG_LOG = "BBPIMSyncSource";

    private static final int        TRACKING_STORAGE_STATUS_NORMAL      = 0;
    private static final int        TRACKING_STORAGE_STATUS_ADD_ERR     = 1;
    private static final int        TRACKING_STORAGE_STATUS_DEL_ERR     = 2;
    private static final int        TRACKING_STORAGE_STATUS_UPD_ERR     = 4;
    
    public static final String      ERR_SAVE_ITEM_ON_SERVER_WARNING     = "message_not-save-on-server";
    
    protected int                     minCacheSize                        = 10;
    protected int                     maxCacheSize                        = 100;
    
    protected SynchronizationController controller;
    
    protected long persistenceStorageKey;
    protected long addAttributesPersistenceStorageKey;

    protected PIMList pimList;
    protected SyncSourceTrackingData trackingData;
    protected PIMItemHelper helper;
 
    /**
     * Represents the set of deleted items to be sent to the server. This set of
     * items if "frozen" at the beginSync and it is a copy of the del items
     * contained in the tacking data at that moment. The info is kept in a
     * hastable to perform fast searches. The key is the item UID.
     */
    protected Hashtable delItemsToSend;
    /**
     * Represents the set of new items to be sent to the server. This set of
     * items if "frozen" at the beginSync and it is a copy of the new items
     * contained in the tacking data at that moment
     */
    protected Vector      newItemsToSend;
    /**
     * Represents the set of updated items to be sent to the server. This set of
     * items if "frozen" at the beginSync and it is a copy of the updated items
     * contained in the tacking data at that moment
     */
    protected Vector      updItemsToSend;
    /**
     * This is the enumeration of the items to be deleted. The method
     * getNextDeleted reads from this sequence. This field is different
     * from the enumerations for add, upd, all, since it is not an iterator
     * on the complete list of items, but only on the subset of the deleted
     * ones. The filtering is performed during the init, rather than during the
     * getNext, and the reason is that we need to make sure deletes reported by
     * the PIM listener are real deletes.
     */
    protected Enumeration delItems;
 
    /**
     * This is the enumeration of all the items in the db used to send updates.
     * The method getNextUpdated reads from this sequence (and filter based on
     * updItemsToSend)
     */
    protected Enumeration updItems;
    /**
     * This is the enumeration of all the items in the db used to send adds.
     * The method getNextUpdated reads from this sequence (and filter based on
     * newItemsToSend)
     */
    protected Enumeration newItems;
     /**
     * This is the enumeration of all the items in the db used in slow syncs.
     * The method getNextItem reads from this sequence
     */
    protected Enumeration allItems;
    
    protected int totalAdded,totalUpdated,totalDeleted;
    protected int totalSent,totalOutOfRangeNotSent,totalNewSent,totalUpdatedSent,totalDeletedSent;
    
    /**
     * The number of items to be sent to the server in the session
     **/
    private int clientItemsNumber;
    
    /**
     * The number of items that the server announced to send in the session
     **/
    private int serverItemsNumber;

    /**
     * The number of new items to be sent to the server in the session
     **/
    private int clientAddItemsNumber;

    /**
     * The number of replaced items to be sent to the server in the session 
     **/
    private int clientReplaceItemsNumber;

    /**
     * The number of deleted items to be sent to the server in the session
     **/
    private int clientDeleteItemsNumber;

    /**
     * Status of the sync source summarized in an integer value. See constants
     * defined in SyncSource
     */
    protected int globalStatus = SyncSource.STATUS_SUCCESS;

    /**
     * PIM listener of the sync process
     **/
    private BBPIMListener pimListener = null;

    /**
     * Listener of the sync process
     **/
    private SyncListener listener = null;
    
    /**
     * Synchronization filter
     **/
    protected SyncFilter filter;
    
    /**
     * SyncSource configuration
     **/
    private SourceConfig config;

    /**
     * This is the set of additional information for the items manipuled by this
     * source. This structure allows extra information to be stored for each
     * individual PIM item.
     */
    protected Additionals additionalsMap;

    /**
     * This map contains items that will be updated/deleted by the current
     * SyncML message. This items are preloaded to speed the entire sync
     * process, avoiding linear scans over the list of available items.
     * The key is the item UID, and the value is the corresponding PIMItem.
     */
    protected Hashtable preloadedItems = null;

    /**
     * This map contains a mapping from luid to guid. This information is not
     * kept alive during a sync to speed up the synchronization. At the
     * beginning of a sync it is recomputed from the main map which is in the
     * additionals and goes from uid to luid
     */
    protected Hashtable luidUidMap = null;

    
    public BBPIMSyncSource(SourceConfig config,
                           PIMList list, PIMItemHelper helper,
                           long persistenceStorageKey,
                           long addAttributesPersistenceStorageKey) throws SyncException
    {
        this.config                 = config;
        this.pimList                = list;
        this.helper                 = helper;
        this.persistenceStorageKey  = persistenceStorageKey;
        this.addAttributesPersistenceStorageKey = addAttributesPersistenceStorageKey;
        if(addAttributesPersistenceStorageKey != -1) {
            initAdditionalsMap();
        }

        initTrackingData();
        filter = null;
    }

    public void setController(SynchronizationController controller) {
        this.controller = controller;
    }

    public void setPIMListener(BBPIMListener pimListener) {
        this.pimListener = pimListener;
    }

    public BBPIMListener getPIMListener() {
        return pimListener;
    }

    public void itemRemovedExternally(PIMItem item)
    {
        try {
            Log.debug(TAG_LOG, "BBPIMSyncSource - removing item from PIMListListener");
            final String uid = helper.getUID(item);
            Log.debug(TAG_LOG, "SyncSource ["+config.getName()+"] removed UID ["+uid+"]");
            trackingData.addDeletedKey(uid);
            helper.handleDeletedItem(item);
            this.removeAdditionals(uid);
        } catch(final PIMException e) {
            trackingData.setStatus(BBPIMSyncSource.TRACKING_STORAGE_STATUS_DEL_ERR | trackingData.getStatus());
        }
        if (!persistTrackingData()) {
            Log.error(TAG_LOG, "Unable to persist tracking data");
        }
        Log.trace(TAG_LOG, "itemRemoved completed");
    }

    public void itemUpdatedExternally(PIMItem oldItem, PIMItem newItem)
    {
        try {
            Log.debug(TAG_LOG, "updating item from PIMListListener");
            final String uid = helper.getUID(newItem);
            Log.debug(TAG_LOG, "SyncSource ["+config.getName()+"] updated UID ["+uid+"]");
            trackingData.addUpdatedKey(uid);
            if(additionalsMap != null){
                AdditionalItemsAttributes additionals = (AdditionalItemsAttributes) additionalsMap.get(uid);
                if(additionals != null){
                    AdditionalItemsAttributes newAdditionals = helper.updateAdditionals(newItem, additionals);
                    removeAdditionals(uid);
                    setAdditionals(uid, newAdditionals);
                }
            }
            helper.handleUpdatedItem(newItem);
        } catch(final PIMException e) {
            trackingData.setStatus(BBPIMSyncSource.TRACKING_STORAGE_STATUS_UPD_ERR | trackingData.getStatus());
        }
        if (!persistTrackingData()) {
            Log.error(TAG_LOG, "Unable to persist tracking data");
        }
        
        Log.trace(TAG_LOG, "itemUpdatedExternally completed");
    }

    public void itemAddedExternally(PIMItem item) {

        try {
            Log.debug(TAG_LOG, "adding item from PIMListListener");
            final String uid = helper.getUID(item);
            Log.debug(TAG_LOG, "SyncSource ["+config.getName()+"] added UID ["+uid+"]");
            trackingData.addNewKey(uid);
            
            AdditionalItemsAttributes existingAdditionals = additionalsMap.get(uid);
            AdditionalItemsAttributes additionals = helper.handleAddedItem(item);
            if(additionals != null){
                additionals.setLuid(uid);
                if(existingAdditionals != null) {
                    additionals = existingAdditionals;
                }
                this.setAdditionals(uid, additionals);
            }
        } catch(final PIMException e) {
            trackingData.setStatus(BBPIMSyncSource.TRACKING_STORAGE_STATUS_ADD_ERR | trackingData.getStatus());
        }
        if (!persistTrackingData()) {
            Log.error(TAG_LOG, "Unable to persist tracking data");
        }
        Log.trace(TAG_LOG, "itemAdded completed");
    }

    //----------------------------------------------------------- Public Methods
    
    /**
     * Returns the config of the source. The client can use this method
     * to obtain the config object and change some parameter. A setConfig()
     * must be called to actually change the source configuration.
     * @return the config of the source
     */
    public SourceConfig getConfig() {
        return config;
    }

    public void setConfig(SourceConfig config) {
        this.config = config;
    }
    
    //------------------------------------------------ SyncSource implementation
    /**
     * Returns the name of the source
     * @return the name of the source
     */
    public String getName() {
        return getConfig().getName();
    }

 
    /**
     * Returns the source URI
     * @return the absolute URI of the source
     */
    public String getSourceUri() {
        return getConfig().getRemoteUri();
    }

 
    /**
     * Returns the type of the source.
     * The types are defined as mime-types, for instance * text/x-vcard).
     * @return the type of the source
     */
    public String getType() {
        return getConfig().getType();
    }


    /**
     * Returns the encoding of the source.
     * The encoding can be 'b64' or 'none' only. The standard defines
     * also 'des' and '3des' but they are not implemented in this version
     * of the APIs.
     * @return the encoding of the source
     */
    public String getEncoding() {
        return getConfig().getEncoding();
    }


    /**
     * Returns the preferred sync mode of the source.
     * The preferred sync mode is the one that the SyncManager sends
     * to the server in the initialization phase. The server can respond
     * with a different alert code, to force, for instance, a slow.
     * @return the preferred sync mode for this source
     */
    public int getSyncMode() {
        return getConfig().getSyncMode();
    }

    /**
     * Returns the current filter for this SyncSource.
     */
    public SyncFilter getFilter() {
        return filter;
    }
     
    /**
     * Set a new filter for this SyncSource
     */
    public void setFilter(SyncFilter filter) {
        this.filter = filter;
    }
    
    /** 
     * Returns the next item of the store.
     **/
    public SyncItem getNextItem() throws SyncException {
        if (allItems != null && allItems.hasMoreElements()) {
            totalSent++;
            try {
                PIMItem pimItem = (PIMItem)allItems.nextElement();
                SyncItem item = createSyncItem(pimItem, SyncItem.STATE_UPDATED);
                Log.info(TAG_LOG, "Source "+getName()+": sending item "+item.getKey());
                return item;
            } catch (Exception e) {
                Log.error(TAG_LOG, "getNextItem cannot create SyncItem", e);
                throw new SyncException(SyncException.CLIENT_ERROR, "Cannot create SyncItem");
            }
        } else {
            Log.info(TAG_LOG, "Source "+getName()+": no more items to send for slow sync");
            return null;
        }
    }

    /**
     * Returns the next new item of the store
     * (not yet sent to the server)
     */
    public SyncItem getNextNewItem() throws SyncException {
        if (newItems == null || newItemsToSend.size() == 0) {
            Log.info(TAG_LOG, "Source " + getName() + ": no more new items to send");
            return null;
        }

        // We keep scanning the list until we find an item
        // that belongs to the new list
        while (newItems.hasMoreElements()) {
            try {
                PIMItem pimItem = (PIMItem)newItems.nextElement();

                // If this item belongs to the newItems list, then
                // we are done
                String uid = helper.getUID(pimItem);
                if (containsKey(newItemsToSend, uid, true)) {
                    SyncItem item = createSyncItem(pimItem, SyncItem.STATE_NEW);
                    Log.info(TAG_LOG, "Source "+getName()+": sending new item "+item.getKey());
                    // Increment the number of sent items
                    totalSent++;
                    totalNewSent++;
                    return item;
                }
            } catch (Exception e) {
                Log.error(TAG_LOG, "getNextItem: cannot create SyncItem", e);
                throw new SyncException(SyncException.CLIENT_ERROR, "Cannot create SyncItem");
            }
        }
        // If we get here, then it may be that the items got deleted by the user,
        // so we must ignore it, and finish
        Log.info(TAG_LOG, "the item was deleted");
        return null;
    }
     
    /**
     * Returns the first/next updated item of the store
     * (changed from the last sync)
     */
    public SyncItem getNextUpdatedItem() throws SyncException {
        if (updItems == null || updItemsToSend.size() == 0) {
            Log.info(TAG_LOG, "Source " + getName() + ": no more updated items to send");
            return null;
        }

        // We keep scanning the list until we find an item
        // that belongs to the new list
        while (updItems.hasMoreElements()) {
            try {
                PIMItem pimItem = (PIMItem)updItems.nextElement();

                // If this item belongs to the newItems list, then
                // we are done
                String uid = helper.getUID(pimItem);
                if (containsKey(updItemsToSend, uid, true)) {
                    SyncItem item = createSyncItem(pimItem, SyncItem.STATE_UPDATED);
                    Log.info(TAG_LOG, "Source "+getName()+": sending updated item "+item.getKey());
                    // Increment the number of sent items
                    totalSent++;
                    totalUpdatedSent++;
                    return item;
                }
            } catch (Exception e) {
                Log.error(TAG_LOG, "getNextUpdItem cannot create SyncItem", e);
                throw new SyncException(SyncException.CLIENT_ERROR, "Cannot create SyncItem");
            }
        }
        // If we get here, then it may be that the item got deleted by the user,
        // so we just ignore it and finish
        Log.info(TAG_LOG, "the item was deleted");
        return null;
    }
     
    /** 
     * Returns a SyncItem containing the key of the first/next
     * deleted item of the store (locally removed after the last sync,
     * but not yet deleted on server)
     */
    public SyncItem getNextDeletedItem() throws SyncException {
        if (delItemsToSend == null || delItemsToSend.size() == 0 ||
            delItems == null) {
            Log.info(TAG_LOG, "Source "+getName()+": no more deleted items to send");
            return null;
        }
        if (!delItems.hasMoreElements()) {
            Log.info(TAG_LOG, "Source "+getName()+": no more deleted items to send");
            delItems = null;
            return null;
        } else {
            SyncItem item = (SyncItem)delItems.nextElement();
            totalSent++;
            totalDeletedSent++;
            Log.info(TAG_LOG, "Source "+getName()+": sending item "+item.getKey());
            // No need to get the content here
            return item;
        }
    } 
     
    /**
     * Return the number of changes that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientItemsNumber() {
        return clientItemsNumber;
    }
     
    /**
     * Return the number of new items (add) that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of new items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientAddNumber() {
        return clientAddItemsNumber;
    }
     
    /**
     * Return the number of replaced items that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of replaced items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientReplaceNumber() {
        return clientReplaceItemsNumber;
    }
     
    /**
     * Return the number of deleted items that the client will send during the
     * session. This method, after the beginSync() call, should return
     * the number of delted items to be sent to the server.
     * The number of changes is computed by initXXXItems() during beginSync().
     * @return number of items to sent, or -1 if unknown
     */
    public int getClientDeleteNumber() {
        return clientDeleteItemsNumber;
    }
     
    /**
     * Return the number of changes that the server will send during the
     * session. This method, after the beginSync() call, should return
     * the number of items to be sent to the server.
     * @return number of changes from the server, or -1 if not announced.
     */
    public int getServerItemsNumber() {
        return serverItemsNumber;
    }
     
    /**
     * Set the number of changes that the server will send during the
     * session. This method is called by the engine to notify the Source
     * of the number of changes announced by the server. If the server
     * does not announce the number of changes, the engine will call
     * this method with parameter -1.
     * @param number of changes from the server, or -1 if not announced.
     */
    public void setServerItemsNumber(int number) {
        serverItemsNumber = number;
    }
     
    /** 
     * Return the Last Anchor for this source
     */
    public long getLastAnchor() {
        return getConfig().getLastAnchor();
    }
     
    /** 
     * Set the value of the Last Anchor for this source
     */
    public void setLastAnchor(long time) {
        getConfig().setLastAnchor(time);
    }
     
    /** 
     * Return the Next Anchor for this source
     */
    public long getNextAnchor() {
        return getConfig().getNextAnchor();
    }
     
    /** 
     * Set the value of the Next Anchor for this source
     */
    public void setNextAnchor(long time) {
        getConfig().setNextAnchor(time);
    }
     
    /**
     * Called after SyncManager preparation and initialization just before start
     * the synchronization of the SyncSource.
     * @param syncMode the synchronization type: one of the values in
     *         sync4j.framework.core.AlertCode
     * @throws SyncException in case of error. This will stop the sync process
     */
    public void beginSync(int syncMode) throws SyncException {
        Log.info(TAG_LOG, "Begin sync for source '" + getName() + "' with mode " + syncMode);

        helper.init();
        
        totalAdded = 0;
        totalDeleted = 0;
        totalUpdated = 0;

        preloadedItems = null;
        luidUidMap = null;

        checkTrackingData();

        // Init lists
        switch(syncMode) {
            case SyncML.ALERT_CODE_REFRESH_FROM_CLIENT:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_REFRESH_FROM_CLIENT");
                // A refresh from client is like a slow here, except that we
                // reset the anchors so that if the sync fails for any reason,
                // the next sync will be a slow one
                getConfig().setLastAnchor(0);
                resetTrackingData();
            case SyncML.ALERT_CODE_SLOW:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_SLOW");
                initAllItems();
                
                // Init number of changes counter
                clientAddItemsNumber     = 0 ;
                clientReplaceItemsNumber = 0 ;
                clientDeleteItemsNumber  = 0 ;
                clientItemsNumber        = 0 ;
                break;
            case SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW");
                // A one way from client no slow is like a fast here
            case SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT");
                // A one way from client is like a fast here
            case SyncML.ALERT_CODE_FAST:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_FAST");
                initNewItems();
                initUpdItems();
                initDelItems();
                // Init number of changes counter
                clientAddItemsNumber     = (newItemsToSend != null) ? newItemsToSend.size() : 0;
                clientReplaceItemsNumber = (updItemsToSend != null) ? updItemsToSend.size() : 0;
                clientDeleteItemsNumber  = (delItemsToSend != null) ? delItemsToSend.size() : 0;
                
                clientItemsNumber        = clientAddItemsNumber + clientReplaceItemsNumber + clientDeleteItemsNumber;
                break;
                
            case SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_ONE_WAY_FROM_SERVER");
                // No modifications to send (it's not
                // strictly necessary to reset the lists,
                // because the engine will not ask items to
                // the SyncSource, but it's good to do it)
                newItems = null;
                updItems = null;
                delItems = null;
                
                // Init number of changes counter
                clientAddItemsNumber     = 0;
                clientReplaceItemsNumber = 0;
                clientDeleteItemsNumber  = 0;
                
                clientItemsNumber        = 0;
                break;
                
            case SyncML.ALERT_CODE_REFRESH_FROM_SERVER:
                Log.trace(TAG_LOG, "SyncML.ALERT_CODE_REFRESH_FROM_SERVER");
                // In this case, the SyncSource should
                // delete all the items in the database
                // (possibly asking the user before that)
                // No modifications to send.
                newItems = null;
                updItems = null;
                delItems = null;

                // Init number of changes counter
                clientAddItemsNumber     = 0;
                clientReplaceItemsNumber = 0;
                clientDeleteItemsNumber  = 0;
                
                clientItemsNumber        = 0;

                // When the user asks for a reset from server, the client
                // usually removes all items before starting the sync, to avoid
                // any timeout issue, so the following call has usually no
                // effect. It is necessary though to handle cases where the
                // server forces a refresh from server.
                deleteAllItems();

                // We reset the anchors so that if the sync gets interrupted at
                // the next sync we'll have slow sync and no data loss in case
                // of fast
                getConfig().setLastAnchor(0);

                // The following code lines are important to ensure the client throws away 
                // all tracking data before recover from server
                trackingData.clear();
                persistTrackingData();
                break;
                
            default:
                throw new SyncException(SyncException.SERVER_ERROR, "SyncSource "+getName()+
                                        ": invalid sync mode "+getSyncMode());
        }

        totalSent = 0;
        Log.debug(TAG_LOG, "For source ["+getName()+"] There are ["+clientAddItemsNumber+"] new items should be sent");
        Log.debug(TAG_LOG, "For source ["+getName()+"] There are ["+clientDeleteItemsNumber+"] deleted items should be sent");
        Log.debug(TAG_LOG, "For source ["+getName()+"] There are ["+clientReplaceItemsNumber+"] updated items should be sent");
        Log.debug(TAG_LOG, "For source ["+getName()+"] There are ["+clientItemsNumber+"] ALL items should be sent");
    }
     
    /**
     * Called just before committing the synchronization process by the
     * SyncManager. The SyncSource can stop the commit phase raising an
     * exception here.
     * @throws SyncException in case of error, to stop the commit.
     */
    public void endSync() throws SyncException {

        try {
            helper.tearDown();
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot tear down helper", e);
        }

        Log.info(TAG_LOG, "End sync for source " + getName());
        Log.info(TAG_LOG,
                "For source ["+getName()+"]: \n"+
                "there were ["+totalSent+"] total items sent \n" +
                "there were ["+totalOutOfRangeNotSent+"] total OutOfRange items were not sent \n" +
                
                "there were ["+totalNewSent    +"] NEW items sent \n" +
                "there were ["+totalDeletedSent+"] DELETED items sent \n" +
                "there were ["+totalUpdatedSent+"] UPDATED items sent \n" +
                
                "there were ["+totalAdded  +"] total items tried to add \n" +
                "there were ["+totalDeleted+"] total items tried to delete \n" +
                "there were ["+totalUpdated+"] total items tried to update \n"
        );

        // Release resources
        resetResource();

        // If the synch was successfull, we clear the tracker
        trackingData.clear();
        persistTrackingData();
    }
     
    /**
     * Set a sync listener.
     * @param listener the listener or null to remove it
     */
    public void setListener(SyncListener listener) {
        this.listener = listener;
    }
     
    /**
     * Returns the current listener (or null if not set)
     */
    public SyncListener getListener() {
        return listener;
    }
     
    /**
     * Returns the status of the sync source. The status is encoded as a bit
     * mask of the STATUS_* values
     */
    public int getStatus() {
        return globalStatus;
    }
     
    public void dataReceived(String date, int size) {
        //Do nothing here
    }

    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#addItem(com.funambol.syncml.spds.SyncItem)
     */
    public int addItem(SyncItem item) throws SyncException {
        totalAdded++;
        try
        {
            PIMItem pitem = helper.createItem();
            if (pitem!=null)
            {
                AdditionalItemsAttributes addits = null;
                
                AdditionalItemsAttributes additionals;
                additionals = helper.fillItem(pitem,new String(item.getContent()),addits);

                Log.trace(TAG_LOG, "About to commit PIMItem.");
                try {
                    helper.saveNewPIMItem(pitem);
                } catch (PIMException e) {
                    String error = helper.handlePIMException(e);
                    Log.error(TAG_LOG, "Unable to save new item in source [" + getName() + "]: " + error, e);
                    return SyncMLStatus.GENERIC_ERROR;
                }
                String key = helper.getUID(pitem);
                item.setKey(key);
                if(additionals != null) {
                    additionals.setLuid(key);
                    setAdditionals(key, additionals);
                }

                Log.trace(TAG_LOG, "Purposfully deleting new item key: " + key);
                trackingData.deleteNewKey(key,true);

                key = null;
                return SyncMLStatus.SUCCESS;
            } else {
                Log.trace(TAG_LOG, "Can not create a blank PIMItem.");
            }
        } catch(final PIMException e) {
            String error = helper.handlePIMException(e);
            Log.error(TAG_LOG, "PIMException while adding item to SyncSource [" + getName() + "]: " + error, e);
            return SyncMLStatus.GENERIC_ERROR;
        } catch(final Exception e) {
            Log.error(TAG_LOG, "RuntimeException while updating item to SyncSource ["
                      + getName() + "]", e);
            return SyncMLStatus.GENERIC_ERROR;
        }
        Log.error(TAG_LOG, "Unable to save new item in source [" + getName() + "]: no error");
        return SyncMLStatus.GENERIC_ERROR;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#updateItem(com.funambol.syncml.spds.SyncItem)
     */
    public int updateItem(SyncItem item) throws SyncException {
        Log.trace(TAG_LOG, "updateItem" + item.getKey());
        totalUpdated++;
        try
        {
            String uid = getUIDFromLuid(item.getKey());
            PIMItem pitem = findItem(uid);
            if (pitem != null) {
                AdditionalItemsAttributes additionals = getAdditionals(uid);
                helper.clearItem(pitem);

                AdditionalItemsAttributes newAdditionals;
                newAdditionals = helper.fillItem(pitem, new String(item.getContent()),
                                                 additionals);
               
                Log.trace(TAG_LOG, "About to commit PIMItem.");
                try {
                    helper.saveUpdatedPIMItem(pitem);
                } catch (PIMException e) {
                    String error = helper.handlePIMException(e);
                    Log.error(TAG_LOG, "Unable to save updated item in source [" + getName() + "]: " + error);
                    return SyncMLStatus.GENERIC_ERROR;
                }
                Log.trace(TAG_LOG, "Before trying to get UID of PIMItem.");
                item.setKey(uid);

                if(newAdditionals != null) {
                    updateAdditionals(uid, newAdditionals);
                }

                Log.trace(TAG_LOG, "Purposfully deleting updated item key: " + uid);
                trackingData.deleteUpdatedKey(uid,true);
                
                return SyncMLStatus.SUCCESS;
            }
            Log.error(TAG_LOG, "Unable to save updated item in source [" + getName() + "]: item not found");
            return SyncMLStatus.NOT_FOUND;
        } catch(final PIMException e) {
            String error = helper.handlePIMException(e);
            Log.error(TAG_LOG, "PIMException while updating item to SyncSource [" + getName() + "]: " + error, e);
            return SyncMLStatus.GENERIC_ERROR;
        } catch (Exception e) {
            // We had some sort of generic runtime exception
            Log.error(TAG_LOG, "RuntimeException while updating item to SyncSource ["
                      + getName() + "]", e);
            return SyncMLStatus.GENERIC_ERROR;
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#deleteItem(java.lang.String)
     */
    public int deleteItem(String key) throws SyncException {
        Log.trace(TAG_LOG, "deleteItem " + key);
        totalDeleted++;
        try
        {
            String uid = getUIDFromLuid(key);
            PIMItem item = findItem(uid);
            if (item != null) {
                helper.deleteItem(item);
                Log.trace(TAG_LOG, "Purposfully deleting deleted item key: " + item);
                trackingData.deleteDeletedKey(uid,true);
                removeAdditionals(uid);
                return SyncMLStatus.SUCCESS;
            }
            Log.error(TAG_LOG, "Unable to delete item in source [" + getName() + "]: no error");
            return SyncMLStatus.NOT_FOUND;
        }
        catch(final Exception e)
        {
            Log.error(TAG_LOG, "Exception while deleting item from SyncSource ["+getName()+"]", e);
            return SyncMLStatus.GENERIC_ERROR;
        }
    }

    /**
     * This method is invoked by the BBSyncManager once the modifications from
     * the server have been preprocessed and we know that items are going to be
     * updated/deleted. These items are preloaded here with a linear scan over
     * the list of PIM items.
     */
    public void prepareModifications(Vector itemsKey) {
        Log.info(TAG_LOG, "[prepareModifications]");

        preloadedItems = new Hashtable();
        // Copy all the keys into an hashtable, so we can handle big
        // set of changes in an almost linear time
        Hashtable keys = new Hashtable();
        for(int i=0;i<itemsKey.size();++i) {
            try {
                String luid = (String)itemsKey.elementAt(i);
                String uid = getUIDFromLuid(luid);
                keys.put(uid, luid);
            } catch (Exception e) {
                Log.error(TAG_LOG, "prepareModifications Cannot load uid from luid "
                          + e.toString());
            }
        }
        // Now scan the list of items until we have preloaded all the keys
        try {
            Enumeration items = helper.getItemsList();
            // Perform a single scan of all the items and load the ones that
            // are going to be updated/deleted
            int loaded = 0;
            while(items.hasMoreElements() && (loaded < itemsKey.size())) {
                PIMItem item = (PIMItem)items.nextElement();
                String itemUid = helper.getUID(item);
                if (keys.get(itemUid) != null) {
                    Log.debug(TAG_LOG, "prepareModifications preloading: " + itemUid);
                    preloadedItems.put(itemUid, item);
                    loaded++;
                }
            }
        } catch (Exception e) {
            Log.error(TAG_LOG, "Error while preloading items", e);
        }
    }

    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#setItemStatus(java.lang.String key, int status)
     */
    public void setItemStatus(String key, int status) {
        if (!SyncMLStatus.isSuccess(status)) {
            Log.error(TAG_LOG, "Could not save to server item with UID [" +
                       key+ "] for SyncSource ["+ getName()+
                       "] returned status ["+status+"]");
        }
        trackingData.deleteNewKey(key);
        trackingData.deleteUpdatedKey(key);
        trackingData.deleteDeletedKey(key);

        // If this item was succesfully deleted from the server, then we can
        // throw away its additionals
        if (SyncMLStatus.isSuccess(status)) {
            try {
                String uid = getUIDFromLuid(key);
                if (delItemsToSend != null && delItemsToSend.get(uid) != null) {
                    removeAdditionals(uid);
                }
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot remove additionals info", e);
            }
        }
    }

    /**
     * Creates a new SyncItem for the engine to store incoming items
     */
    public SyncItem createSyncItem(String key, String type, char state,
                                   String parent, long size) {

        SyncItem item = new SyncItem(key, type, state, parent);
        item.setObjectSize(size);
        return item;
    }


    public boolean shouldWarn()
    {
        int deleted = trackingData.countDeletedKey();

        try{
            int remainders = 0;
            Enumeration items = pimList.items();
            while(items.hasMoreElements()) {
                remainders++;
                items.nextElement();
            }
                
            int total = deleted + remainders;
            Log.info(TAG_LOG, "Considering warning user about " + deleted + " deletes when " + remainders + " items remain");
               
            //if (deleted > (.5 * remainders))
            if (deleted > ((2*total)/3)) {
                // 2/3 deleted or mail account deleted bug
                return true;
            }
        }
        catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public PIMItemHelper getHelper() {
        return helper;
    }

    public void deleteAllItems() throws SyncException {
        try {
            UISyncSourceController ssController = null;
            SyncListener lis = getListener();
            if (lis instanceof UISyncSourceController) {
                ssController = (UISyncSourceController)lis;
            }

            //count the PIMList elements
            int size = 0;
            Enumeration items = helper.getItemsList();
            while (items.hasMoreElements()) {
                PIMItem item = (PIMItem) items.nextElement();
                if (size == 1 && ssController != null) {
                    //Notify the ui that items are being deleted from the client
                    ssController.removingAllData();
                }
                size++;
            }

            //remove the PIMList elements
            items = helper.getItemsList();
            int i = 0;
            while (items.hasMoreElements()) {
                if (controller != null && controller.isCancelled()) {
                    //Reset anchors if a cancel action was performed
                    //after at least 1 item was cancelled
                    if (i>0) {
                        getConfig().setNextAnchor(0);
                        getConfig().setLastAnchor(0);
                        resetTrackingData();
                    }

                    // The sync has not started yet, so we must synthetize a
                    // report here
                    throw new SyncException(SyncException.CANCELLED, "Sync cancelled");
                }
                i++;
                PIMItem item = (PIMItem) items.nextElement();
                helper.deleteItem(item);

                //Notify the UI
                if (ssController != null) {
                    ssController.itemRemoved(i, size);
                }
            }
        } catch (SyncException ex) {
            Log.error(TAG_LOG, "resetClientData cannot delete device items", ex);
            throw ex;
        } catch (Exception ex) {
            Log.error(TAG_LOG, "resetClientData Cannot delete device items", ex);
            throw new SyncException(SyncException.CLIENT_ERROR, ex.toString());
        }
    }

    protected PIMItem findItem(String uid) throws PIMException {
        PIMItem item = null;
        if (preloadedItems != null) {
            try {
                item = (PIMItem) preloadedItems.get(uid);
                if (item != null) {
                    Log.trace(TAG_LOG, "Item " + uid + " was preloaded, good!");
                }
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot preload item: ", e);
            }
        }

        if (item == null) {
            item = helper.findItem(uid);
        }

        return item;
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initAllItems()
     */
    protected void initAllItems() throws SyncException
    {
        try {
            allItems = helper.getItemsList();
        } catch(final PIMException e) {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "Exception while initializing all items of SyncSource ["
                                    +getName()+"]");
        } catch (Throwable t) {
            Log.error(TAG_LOG, "Error during initialization of all items", t);
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "Exception while initializing all items of SyncSource ["
                                    +getName()+"]");
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initDelItems()
     */
    protected void initDelItems()
    {
        Log.trace(TAG_LOG, "Start building DelItems list");
        delItemsToSend = new Hashtable();
        Hashtable deleted = new Hashtable();
        String uid = null;

        while((uid = trackingData.popDeletedKey()) != null) {
            deleted.put(uid, uid);
        }
        // This is a workaround for a BB bug. In some cases
        // the systems sends bogus delete notifications for all items.
        // For this reason before sending a delete command to the server
        // we double check if the item is still present
        try {
            Enumeration items = helper.getItemsList();
            while (items.hasMoreElements()) {
                PIMItem item = (PIMItem)items.nextElement();
                uid = helper.getUID(item);
                // Check if this item is in the deleted list
                if (deleted.get(uid) != null) {
                    // The item was not really deleted, so this is a spurius
                    // delete
                    Log.error(TAG_LOG, "Detected spurious delete for item: " + uid);
                    deleted.remove(uid);
                }
            }
        } catch (Exception e) {
            Log.error(TAG_LOG, "Error while checking for spurious delete", e);
        }
        // Now build the actual list of deleted items
        Enumeration deletedUid = deleted.keys();
        while(deletedUid.hasMoreElements()) {
            uid = (String)deletedUid.nextElement();
            String luid = getLuidFromUID(uid);
            Log.trace(TAG_LOG, "Creating a sync item to be deleted with Luid ["+luid+"]");
            SyncItem delItem = new SyncItem(luid);
            delItem.setState(SyncItem.STATE_DELETED);
            delItemsToSend.put(uid, delItem);
        }
        // Initialize the iterator over the deleted items
        delItems = delItemsToSend.elements();
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initNewItems()
     */
    protected void initNewItems()
    {
        Log.trace(TAG_LOG, "initNewItems");
        try {
            newItems = helper.getItemsList();
            // Copy the set of new items into a local and frozen list
            newItemsToSend = trackingData.getNewKeys();
        } catch (PIMException pe) {
            Log.error(TAG_LOG, "Cannot get items list ", pe);
            newItems = null;
        }
    }
    
    /*
     * (non-Javadoc)
     * @see com.funambol.syncml.spds.SyncSource#initUpdItems()
     */
    protected void initUpdItems()
    {
        Log.trace(TAG_LOG, "initUpdItems");
        try {
            updItems = helper.getItemsList();
            // Copy the set of new items into a local and frozen list
            updItemsToSend = trackingData.getUpdatedKeys();
        } catch (PIMException pe) {
            Log.error(TAG_LOG, "Cannot get items list ", pe);
            updItems = null;
        }
    }

    /**
     * Retrieve a luid from a uid. If an additional with the proper mapping
     * information is not available, then the luid is assumed to be identical to
     * the uid. This is perfectly acceptable, as there is no requirement that
     * UID and LUID are different.
     */
    protected String getLuidFromUID(String uid) {
        String luid = uid;
        AdditionalItemsAttributes addits = getAdditionals(uid);
        if(addits != null){
            luid = addits.getLuid();
        }
        Log.trace(TAG_LOG, "[getLuidFromUID] uid=" + uid + " luid="+luid);
        return luid;
    }


    protected void buildUidLuidMap() {
        if (additionalsMap != null) {
            luidUidMap = new Hashtable();
            synchronized(additionalsMap) {
                Enumeration keys = additionalsMap.keys();
                while(keys.hasMoreElements()) {
                    String uid = (String)keys.nextElement();
                    AdditionalItemsAttributes addits   = (AdditionalItemsAttributes)additionalsMap.get(uid);
                    if (addits != null) {
                        String luid = addits.getLuid();
                        luidUidMap.put(luid, uid);
                    }
                }
            }
        }
    }

    /**
     * This method returns the UID of an item given its luid. This method relies
     * on a reverse hash map (luidUidMap) which is built on demand and once per
     * sync session. The map is then kept in sync with the additionalsMap by the
     * methods that manipulate the additionalsMap.
     *
     * @param luid is the item luid
     * @param the uid (fallback to luid is no mapping info is found)
     */
    protected String getUIDFromLuid(String luid) throws PIMException, SyncException {

        // We cache the reverse map, to speed up the operations. The reverse map
        // is built the first time this info is needed
        if (luidUidMap == null) {
            buildUidLuidMap();
        }
        String uid = luid;
        if (luidUidMap != null) {
            uid = (String)luidUidMap.get(luid);
        }
        if (uid == null) {
            uid = luid;
        }
        Log.trace(TAG_LOG, "[getUIDFromLuid] uid=" + uid + " luid="+luid);
        return uid;
    }
    
    private boolean containsKey(Vector items, String key, boolean remove) {
        if (items != null) {
            for(int i=0;i<items.size();++i) {
                String itemKey = (String) items.elementAt(i);
                if (itemKey != null && itemKey.equals(key)) {
                    if (remove) {
                        items.removeElementAt(i);
                    }
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Create a SyncItem that can be returned to the sync engine. The purpose of
     * this method is mainly to perform uid-luid mapping and update the
     * additional information associated to the item.
     *
     * @param pimItem the item
     * @param state the SyncItem state (@see SyncItem)
     * @return the create sync item (cannot be null)
     */
    private SyncItem createSyncItem(PIMItem pimItem, char state) throws Exception {
        String uid    = helper.getUID(pimItem);
        String luid   = getLuidFromUID(uid);
        SyncItem item = new SyncItem(luid);
        item.setState(state);
        AdditionalItemsAttributes additionals = getAdditionals(uid);

        if (additionals == null){
            additionals = helper.createAdditionals(pimItem);
            // This was an item that was received with an old version of the
            // client or it was created on the device, therefore it does not
            // have additionals yet. After creating them, we populate and
            // save them
            if (additionals != null) {
                additionals.setLuid(luid);
                setAdditionals(uid, additionals);
            }
        }

        item.setContent(helper.itemToString(pimItem, additionals).getBytes());
        return item;
    }

    protected void setAdditionals(String uid, AdditionalItemsAttributes addits) throws SyncException {
        if(additionalsMap != null){
            synchronized(additionalsMap) {
                additionalsMap.put(uid, addits);
                persistAdditionalsMap();
            }
            // Update the reverse map
            if (luidUidMap != null) {
                synchronized(luidUidMap) {
                    luidUidMap.put(addits.getLuid(), uid);
                }
            }
        }else{
            Log.debug(TAG_LOG, "[setAdditionals] additionalMap is not initializated for "
                      + config.getName());
        }
    }

    protected void removeAdditionals(String uid) throws SyncException {
        String luid = getLuidFromUID(uid);
        if(additionalsMap != null){
            synchronized(additionalsMap) {
                additionalsMap.remove(uid);
                persistAdditionalsMap();
            }
            // Update the reverse map
            if (luidUidMap != null && luid != null) {
                synchronized(luidUidMap) {
                    luidUidMap.remove(luid);
                }
            }
        }else{
            Log.debug(TAG_LOG, "[removeAdditionals] additionalMap is not initializated for "
                      + config.getName());
        }
    }

    protected void updateAdditionals(String uid, AdditionalItemsAttributes addits) throws SyncException {
        if(additionalsMap != null){
            // Update and persist the additionalsMap
            synchronized(additionalsMap) {
                additionalsMap.put(uid, addits);
                persistAdditionalsMap();
            }
            // Update the reverse map
            if (luidUidMap != null) {
                synchronized(luidUidMap) {
                    luidUidMap.put(addits.getLuid(), uid);
                }
            }
        }else{
            Log.debug(TAG_LOG, "[updateAdditionals] additionalMap is not initializated for "
                      +config.getName());
        }
    }

    protected AdditionalItemsAttributes getAdditionals(String uid) throws SyncException {
        AdditionalItemsAttributes additionals = null;
        if(additionalsMap != null) {
            synchronized(additionalsMap) {
                additionals = (AdditionalItemsAttributes)additionalsMap.get(uid);
            }
        }
        return additionals;
    }
 

    private boolean initAdditionalsMap() throws SyncException {
        try
        {
            Log.info(TAG_LOG, "[initAdditionalsMap]");
            long key = addAttributesPersistenceStorageKey;
            final PersistentObject pobj = PersistentStore.getPersistentObject(key);
            if (pobj == null || pobj.getContents() == null) {
                additionalsMap = new Additionals();
            } else {
                additionalsMap = (Additionals)pobj.getContents();
            }
            return true;
        } catch(Exception e) {
            Log.error(TAG_LOG, "Cannot load Additional Attributes Map: ", e);
            throw new SyncException(SyncException.CLIENT_ERROR, "Cannot create Additional Attributes map");
        }
    }

    private boolean persistAdditionalsMap() throws SyncException {
        Log.debug(TAG_LOG, "[persistAdditionalsMap]");
        try
        {
            long key = addAttributesPersistenceStorageKey;
            final PersistentObject pobj = PersistentStore.getPersistentObject(key);
            pobj.setContents(additionalsMap);
            pobj.commit();
            return true;
        }
        catch(ControlledAccessException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "The application could not persist persistAdditionalsMap");
        }catch(Exception e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
                                    "The application could not persist persistAdditionalsMap");
        }
    }

    private void checkTrackingData()
    {
        if (trackingData.getStatus() != BBPIMSyncSource.TRACKING_STORAGE_STATUS_NORMAL)
        {
            if (controller != null) {
                controller.serverOperationFailed();
            }
        }
    }


    private boolean initTrackingData() throws SyncException
    {
        try
        {
            final PersistentObject pobj = PersistentStore.getPersistentObject(persistenceStorageKey);
            if (pobj.getContents() == null) {
                trackingData = new SyncSourceTrackingData();
            } else {
                trackingData = (SyncSourceTrackingData)pobj.getContents();
            }
            return true;
        }
        catch(ControlledAccessException e) {
            throw new SyncException(SyncException.CLIENT_ERROR,
              "The application could not access to the TrackingData for SyncSource [" +
              getName()+"]");
        }
    }


    private boolean persistTrackingData() throws SyncException
    {
        try
        {
            final PersistentObject pobj = PersistentStore.getPersistentObject(persistenceStorageKey);
            pobj.setContents(trackingData);
            pobj.commit();
            return true;
        }
        catch(ControlledAccessException e)
        {
            throw new SyncException(SyncException.CLIENT_ERROR,
              "The application could not access to the TrackingData for SyncSource ["+getName()+"]");
        }
    }

    private void resetResource() {

        allItems = null;
        newItems = null;
        updItems = null;
        delItems = null;        

        // Init number of chages counters
        clientItemsNumber = 
        serverItemsNumber =
        clientAddItemsNumber =
        clientReplaceItemsNumber =
        clientDeleteItemsNumber = -1;
        
        totalAdded =
        totalDeleted =
        totalUpdated = 0;

        totalSent =
        totalNewSent =
        totalUpdatedSent =
        totalDeletedSent =
        totalOutOfRangeNotSent = 0;

        // Release memory for temporary objects
        luidUidMap = null;
        preloadedItems = null;
    }

    /**
     * An utility method to delete the tracking data that are on the queue
     * Deletes the tracking data, the backup and save the empty one in the store
     * After this method has been called all of the client modifications, 
     * deletions and updates are lost.
     */
    public void resetTrackingData() {
        // The following code lines are important to ensure the client throws away 
        // all tracking data before recover from server
        trackingData.clear();
        persistTrackingData();
    }


    /**
     * This class holds the uid - additionals attributes for an item
     * and can be persisted to the BB storage.
     */
    public static final class Additionals implements Persistable {
        private Hashtable addAttributesMap;

        public Additionals() {
            Log.debug(TAG_LOG, "creating additionals map");
            addAttributesMap = new Hashtable();
        }

        public AdditionalItemsAttributes put(String uid, AdditionalItemsAttributes additionalAttributes) {
            return (AdditionalItemsAttributes)this.addAttributesMap.put(uid, additionalAttributes);
        }

        public AdditionalItemsAttributes get(String uid) {
            return (AdditionalItemsAttributes)this.addAttributesMap.get(uid);
        }

        public Enumeration keys() {
            return this.addAttributesMap.keys();
        }

        public void remove(String uid) {
            this.addAttributesMap.remove(uid);
        }

    }
}
