/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol". 
 */

package com.funambol.blackberry.source;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import com.funambol.blackberry.Configuration;

import com.funambol.platform.FileAdapter;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.client.TrackerException;

import com.funambol.storage.StringKeyValueFileStore;
import com.funambol.syncml.client.ChangesTracker;
import com.funambol.syncml.client.CacheTracker;
import com.funambol.storage.StringKeyValueStore;

import com.funambol.util.Log;

public class PhotoTracker extends CacheTracker {

    private static final String TAG_LOG = "PhotoTracker";

    private String directory;

    public PhotoTracker(StringKeyValueStore keyStore, String directory) {
        super(keyStore);
        this.directory = directory;
    }

    public void begin(int syncMode) throws TrackerException {
        super.begin(syncMode);
        // Filter outgoing items if the user enabled the filter
        if (! Configuration.getPicturesSyncOld()) {
            // Remove the old items from the list of changes
            Enumeration adds = newItems.keys();
            while(adds.hasMoreElements()) {
                String fileName = (String)adds.nextElement();
                // Check the date for this item
                FileAdapter fa = null;
                try {
                    Log.trace(TAG_LOG, "Checking if " + fileName + " shall be synced");
                    fa = new FileAdapter(fileName);
                    Log.trace(TAG_LOG, "File last modified = " + fa.lastModified());
                    Log.trace(TAG_LOG, "First run timestamp = " + Configuration.getFirstRunTimestamp());
                    if (fa.lastModified() < Configuration.getFirstRunTimestamp()) {
                        Log.info(TAG_LOG, "Filtering out old picture: " + fileName);
                        newItems.remove(fileName);
                    }
                } catch (IOException ioe) {
                    Log.error(TAG_LOG, "Cannot check picture date ", ioe);
                } finally {
                    if (fa != null) {
                        try {
                            fa.close();
                        } catch (IOException e) {
                        }
                    }
                }
            }
        }
    }


    protected String computeFingerprint(SyncItem item) {
        Log.trace(TAG_LOG, "computeFingerprint");
        // We use the file size as fingerprint
        // This is not very accurate, but we do not expect pictures to be
        // changed on the device
        long size = 0;
        String sep = "";
        if (!directory.endsWith("/")) {
            sep = "/";
        }
        String fileName = item.getKey();
        FileAdapter file = null;
        try {
            file = new FileAdapter(fileName);
            size = file.getSize();
        } catch(Exception e) {
            Log.error(TAG_LOG, "Cannot compute file size " + fileName);
        } finally {
            if (file != null) {
                try {
                    file.close();
                } catch (Exception e) {
                }
            }
        }
        return "" + size;
    }
}


