/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import net.rim.device.api.util.Persistable;
import java.util.Vector;
import java.util.Enumeration;

/**
 * 
 */
public final class SyncSourceTrackingData implements Persistable
{
    private Vector bulkDelete;
    private Vector newItems;
    private Vector updatedItems;
    private Vector deletedItems;
    private Vector ignoreNewItems;
    private Vector ignoreUpdatedItems;
    private Vector ignoreDeletedItems;
    
    private int status;
    
    public SyncSourceTrackingData()
    {
        this.bulkDelete = new Vector();
        this.newItems = new Vector();
        this.deletedItems = new Vector();
        this.updatedItems = new Vector();
        this.ignoreNewItems = new Vector();
        this.ignoreDeletedItems = new Vector();
        this.ignoreUpdatedItems = new Vector();
        this.status = 0;
    }
    
    public void deleteNewKey(String key)
    {
        deleteNewKey(key, false);
    }
    
    public void deleteNewKey(String key, boolean ignoreOnFail)
    {
        synchronized(this.newItems)
        {
            if (!this.newItems.removeElement(key) && ignoreOnFail)
            {
                this.ignoreNewItems.addElement(key);
            }
        }
    }
    
    /**
     * This function always add the new key directly to the DB.
     * @param key The UID of the new item
     */
    public void addNewKey(String key)
    {
        synchronized(this.newItems)
        {
            if (this.ignoreNewItems.contains(key))
            {
                this.ignoreNewItems.removeElement(key);
            }
            else
            {
                this.newItems.addElement(key);
            }
        }
    }
    
    public String popNewKey()
    {
        synchronized(this.newItems)
        {
            int size = this.newItems.size();
            if (size==0)
            {
                return null;
            }
            String temp = (String)this.newItems.elementAt(size-1);
            this.newItems.removeElementAt(size-1);
            return temp;
        }
    }
    
    public int countNewKey()
    {
        synchronized(this.newItems)
        {
            return this.newItems.size();
        }
    }

    /**
     * This method copies the new items into a new vector that the user is free
     * to manipulate.
     */
    public Vector getNewKeys() {
        Vector res = new Vector();
        synchronized(newItems) {
            for (int i=0;i<newItems.size();i++) {
                res.addElement(newItems.elementAt(i));
            }
        }
        return res;
    }

    public void deleteUpdatedKey(String key)
    {
        deleteUpdatedKey(key, false);
    }
        
    public void deleteUpdatedKey(String key, boolean ignoreOnFail)
    {
        synchronized(this.updatedItems)
        {
            if (!this.updatedItems.removeElement(key) && ignoreOnFail)
            {
                this.ignoreUpdatedItems.addElement(key);
            }
        }
    }
    
    /**
     * This function will add the updated key to the DB if there is no similar key in the NewKeyDB and UpdatedKeyDB
     * @param key The UID of the updated item
     */
    public void addUpdatedKey(String key)
    {
        synchronized(this.updatedItems)
        {
            if (this.ignoreUpdatedItems.contains(key))
            {
                this.ignoreUpdatedItems.removeElement(key);
            }
            else
            {
                if (!this.updatedItems.contains(key) && !this.newItems.contains(key))
                {
                    this.updatedItems.addElement(key);
                }
            }
        }
    }

    /**
     * This method copies the new items into a new vector that the user is free
     * to manipulate.
     */
    public Vector getUpdatedKeys() {
        Vector res = new Vector();
        synchronized(newItems) {
            for (int i=0;i<updatedItems.size();i++) {
                res.addElement(updatedItems.elementAt(i));
            }
        }
        return res;
    }

    public String popUpdatedKey()
    {
        synchronized(this.updatedItems)
        {
            int size = this.updatedItems.size();
            if (size==0)
            {
                return null;
            }
            String temp = (String)this.updatedItems.elementAt(size-1);
            this.updatedItems.removeElementAt(size-1);
            return temp;
        }
    }
    
    public int countUpdatedKey()
    {
        synchronized(this.updatedItems)
        {
            return this.updatedItems.size();
        }
    }
    
    public void deleteDeletedKey(String key)
    {
        deleteDeletedKey(key, false);
    }
    
    public void deleteDeletedKey(String key, boolean ignoreOnFail)
    {
        synchronized(this.deletedItems)
        {
            if (!deletedItems.removeElement(key) && ignoreOnFail)
            {
                ignoreDeletedItems.addElement(key);
            }
        }
    }
    
    /**
     * This function will add the deleted key directly into DeletedKeyDB, after that
     * it will removes all similar keys in the NewKeyDB and UpdatedKeyDB
     * @param key The UID of the deleted item
     */
    public void addDeletedKey(String key)
    {
        synchronized(this)
        {
            if (ignoreDeletedItems.contains(key))
            {
                ignoreDeletedItems.removeElement(key);
            }
            else
            {
                deletedItems.addElement(key);
                newItems.removeElement(key);
                updatedItems.removeElement(key);
            }
        }
    }
    
    public String popDeletedKey()
    {
        synchronized(deletedItems)
        {
            int size = deletedItems.size();
            if (size==0)
            {
                return null;
            }
            String temp = (String)deletedItems.elementAt(size-1);
            deletedItems.removeElementAt(size-1);
            return temp;
        }
    }
    
    public int countDeletedKey()
    {
        synchronized(deletedItems)
        {
            return deletedItems.size();
        }
    }
    
    public void setStatus(int status)
    {
        this.status = status;
    }
    
    public int getStatus()
    {
        return status;
    }
    
    public void addBulkDeleteItem(String key)
    {
        bulkDelete.addElement(key);
    }
    
    public int countBulkDeleteItem()
    {
        return bulkDelete.size();
    }
    
    public Vector getBulkDeleteItem()
    {
        return bulkDelete;
    }
    
    public void clearBulkDeleteItem()
    {
        bulkDelete.removeAllElements();
    }
    
    public void clear()
    {
        synchronized(this)
        {
            status = 0;
            newItems.removeAllElements();
            deletedItems.removeAllElements();
            updatedItems.removeAllElements();
            ignoreNewItems.removeAllElements();
            ignoreDeletedItems.removeAllElements();
            ignoreUpdatedItems.removeAllElements();
        }
    }
    
    public String toString()
    {
        StringBuffer temp = new StringBuffer();
        temp.append("There are ["+this.countNewKey()+"] new items since last sync.\n")
            .append("There are ["+this.countUpdatedKey()+"] updated items since last sync.\n")
            .append("There are ["+this.countDeletedKey()+"] deleted items since last sync.\n")
            .append("The current status ["+this.status+"].\n");
        return temp.toString();
    }
    
    public Object clone()
    {
        synchronized(this)
        {
            SyncSourceTrackingData ret = new SyncSourceTrackingData();
            ret.status = status;
            int size = deletedItems.size();
            ret.deletedItems = new Vector(size);
            for(int i=size-1;0<=i;i--)
            {
                ret.deletedItems.addElement(deletedItems.elementAt(i));
            }
        
            size = newItems.size();
            ret.newItems = new Vector(size);
            for(int i=size-1;0<=i;i--)
            {
                ret.newItems.addElement(newItems.elementAt(i));
            }

            size = updatedItems.size();
            ret.updatedItems = new Vector(size);
            for(int i=size-1;0<=i;i--)
            {
                ret.updatedItems.addElement(updatedItems.elementAt(i));
            }
            
            size = ignoreDeletedItems.size();
            ret.ignoreDeletedItems = new Vector(size);
            for(int i=size-1;0<=i;i--)
            {
                ret.ignoreDeletedItems.addElement(ignoreDeletedItems.elementAt(i));
            }
        
            size = ignoreNewItems.size();
            ret.ignoreNewItems = new Vector(size);
            for(int i=size-1;0<=i;i--)
            {
                ret.ignoreNewItems.addElement(ignoreNewItems.elementAt(i));
            }

            size = ignoreUpdatedItems.size();
            ret.ignoreUpdatedItems = new Vector(size);
            for(int i=size-1;0<=i;i--)
            {
                ret.ignoreUpdatedItems.addElement(ignoreUpdatedItems.elementAt(i));
            }
            
            return ret;
        }
    }
    
    public void append(SyncSourceTrackingData obj)
    {
        synchronized(this)
        {
            int size = obj.deletedItems.size();
            for(int i=size-1;0<=i;i--)
            {
                deletedItems.addElement(obj.deletedItems.elementAt(i));
            }
            size = obj.newItems.size();
            for(int i=size-1;0<=i;i--)
            {
                newItems.addElement(obj.newItems.elementAt(i));
            }
            size = obj.updatedItems.size();
            for(int i=size-1;0<=i;i--)
            {
                updatedItems.addElement(obj.updatedItems.elementAt(i));
            }
        }
    }
    
} 
