/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui;

public interface DisplayManager extends Screen {

    public static final long NO_LIMIT = -1;

    /**
     * Pull a screen to the back
     * 
     * @param scr
     *            The screen to hide
     */
    public void hideScreen(Screen scr) throws Exception;

    public void showScreen(Screen scr) throws Exception;

    public void showScreen(Screen scr, boolean donotwait) throws Exception;
    
    /**
     * Helper function to prompt the user for a yes/no answer
     * 
     * @param question
     * @param defaultyes
     * @param timeToWait
     *            TODO
     * @return True on yes
     */
    public boolean askYesNoQuestion(String question, boolean defaultyes, long timeToWait);

    /**
     * Helper function to prompt the user for an accept/deny answer
     * 
     * @param question
     * @param defaultaccept
     * @param timeToWait
     *            TODO
     * @return True on accept
     */
    public boolean askAcceptDenyQuestion(String question, boolean defaultyes, long timeToWait);



    /**
     * Helper function to prompt to continue or cancel
     * 
     * @param message
     * @return True for next
     */
    public boolean promptNext(String message);

    /**
     * Helper function to display a simple notice
     * 
     * @param message
     */
    public void showMessage(String message);

    /**
     * Helper function to display a simple notice with a timeout
     *
     * @param message
     * @param delay message will disappear after the given delay (this is
     * expressed in milliseconds)
     */
    public void showMessage(String message,long delay);

    /**
     * Prompt the user to select an option from a list
     * 
     * @param message
     *            The message to prompt the user with
     * @param options
     *            An array of option names
     * @param values
     *            An array of option values
     * @param defaultValue
     *            The default selected value
     * @param needResponse
     *            TODO
     * @param cancelAfterDelay
     *            TODO
     * @return The chosen value
     */
    public int promptSelection(String message, String[] options, int[] values, int defaultValue,
            boolean needResponse, long cancelAfterDelay);
    
    public void toForeground();

    public void toBackground();

    public void loadBrowser(String url);

}
