/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 *
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.AccountScreenController;
import com.funambol.blackberry.controller.SynchronizationController;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.ui.AccountScreen;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.blackberry.ui.SynchronizationScreen;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.StringUtil;
import com.funambol.syncml.spds.SyncSource;

/**
 *
 */
public class RimAccountScreen extends MainScreen implements AccountScreen, UISyncSourceContainer,
                                                            SynchronizationScreen
{
    private static final String TAG_LOG = "RimAccountScreen";

    private BasicEditField            usernameField    = null;
    private BasicEditField            serverUriField   = null;
    private PasswordEditField         passwordField    = null;
    private GaugeField                progressBar      = null;
    private AccountScreenController   controller       = null;
    private boolean                   saveEnabled      = true;
    private RimProgressSyncSource     uiSource         = null;
    private RimAccountScreenMainVM    mainVm           = null;
    private LabelField                title            = null;

    private final MenuItem    saveMenu       = new MenuItem(StaticDataHelper
                                                     .getLanguage("menu_save"), 20000, 10) {

                                                 public void run() {
                                                     saveConfiguration();
                                                 }
                                             };

    private final MenuItem    quitMenu       = new MenuItem(StaticDataHelper
                                                     .getLanguage("menu_cancel"), 20000, 10) {

                                                 public void run() {
                                                     controller.hide();
                                                 }
                                             };

    private final MenuItem    discardMenu    = new MenuItem(StaticDataHelper
                                                     .getLanguage("menu_discard"), 20000, 10) {

                                                 public void run() {
                                                     resetValues();
                                                 }
                                             };



    public RimAccountScreen() {
        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT | Field.FIELD_LEFT | Field.FIELD_TOP | Manager.NO_VERTICAL_SCROLL);

        this.setFont(RimHelper.getDefaultFont());
        mainVm = new RimAccountScreenMainVM();

        title = new LabelField(StaticDataHelper.getLanguage("dialog_account_title"));
        title.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(title);

        LabelField serverUriLabel = null;
        if (Customization.getInstance().syncUriEditable()) {
            serverUriLabel = new LabelField(StaticDataHelper.getLanguage("conf_server-url"));
            serverUriLabel.setFont(getFont().derive(Font.BOLD));
            serverUriField = new BasicEditField("", Configuration.getSyncUrl());
        }

        LabelField usernameLabel = new LabelField(StaticDataHelper.getLanguage("conf_user"));
        usernameLabel.setFont(getFont().derive(Font.BOLD));
        usernameField = new BasicEditField("", Configuration.getUsername());

        LabelField passwordLabel = new LabelField(StaticDataHelper.getLanguage("conf_password"));
        passwordLabel.setFont(getFont().derive(Font.BOLD));
        passwordField = new PasswordEditField("", Configuration.getPassword());

        VerticalFieldManager fieldsManager = new VerticalFieldManager(Manager.VERTICAL_SCROLL |
                                                                      Manager.VERTICAL_SCROLLBAR);

        fieldsManager.add(usernameLabel);
        fieldsManager.add(usernameField);
        fieldsManager.add(new LabelField(""));
        fieldsManager.add(passwordLabel);
        fieldsManager.add(passwordField);
        fieldsManager.add(new LabelField(""));

        if (Customization.getInstance().syncUriEditable()) {
            fieldsManager.add(serverUriLabel);
            fieldsManager.add(serverUriField);
        }
        //fieldsManager.add(new LabelField(""));
        mainVm.add(fieldsManager);

        add(mainVm);
    }

    public void initialize(AccountScreenController controller) {
        this.controller = controller;
        uiSource = new RimProgressSyncSource("");
        AppSyncSource appSource = AppSyncSourceManager.getInstance().getSource(
                                                 AppSyncSourceManager.SYNC_FLAG_CONFIG);
        UISyncSourceController itemController = new UISyncSourceController(
                                                 controller.getController(), appSource);
        itemController.setUISyncSource(uiSource);
        SyncSource ss = appSource.getSyncSource();
        ss.setListener(itemController);
        appSource.setUISyncSourceController(itemController);
        uiSource.setSource(appSource);
        uiSource.setContainer(this);
        add(uiSource);
    }

    public void redraw() {
        invalidate();
    }

    public void checkFailed() {
        if (uiSource != null) {
            uiSource.reset();
        }
    }

    public void checkSucceeded() {
        if (uiSource != null) {
            uiSource.reset();
        }
    }

    /**
     * Invoked on user hitting the escape button. It prompts the user with a
     * dialog, if he wants to save/discard/cancel the changes in configuration
     * 
     */
    public boolean onClose() {
        if (hasChanges()) {
            final int response = Dialog.ask(Dialog.D_SAVE, StaticDataHelper
                    .getLanguage("dialog_save"));

            if (Dialog.SAVE == response) {
                saveConfiguration();
                return false;
            } else if (Dialog.DISCARD == response) {
                controller.resetValues();
            } else if (Dialog.CANCEL == response) {
                return false;
            }
        }
  
        if (getScreen().getScreenBelow() != null) {
            controller.getController().getScreenDisplayController().hideAccountScreen();
        } else {
            controller.hide();
        }
        return true;
    }

    private boolean hasChanges() {

        String serverUri;
        if (Customization.getInstance().syncUriEditable()) {
            serverUri = serverUriField.getText().trim();
        } else {
            serverUri = Customization.getInstance().getServerUriDefault();
        }
        String username = usernameField.getText().trim();
        String password = passwordField.getText().trim();

        return controller.hasChanges(serverUri, username, password);
    }


    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen.
     */
    public void saveConfiguration() {
        Log.debug(TAG_LOG, "saveConfiguration");

        String serverUri;
        if (Customization.getInstance().syncUriEditable()) {
            serverUri = serverUriField.getText().trim();
        } else {
            serverUri = Customization.getInstance().getServerUriDefault();
        }
        String username = usernameField.getText().trim();
        String password = passwordField.getText().trim();

        controller.saveAndCheck(serverUri, username, password);
    }

    public void disableSave() {
        saveEnabled = false;
    }

    public void enableSave() {
        saveEnabled = true;
    }

    /**
     * Adds "Save" menu item to the menu on Configuration screen
     * 
     * @param Menu
     *            : menu to which save menuitem is to be added
     * @param int : instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance) {
        if (saveEnabled) {
            menu.add(saveMenu);
            menu.add(discardMenu);
        }
        menu.add(quitMenu);
    }

    public SynchronizationController getSynchronizationController() {
        return controller;
    }

    private int getStatusHeight() {
        Font font = RimHelper.getDefaultFont();
        int fontHeight = font.getHeight();
        return fontHeight;
    }

    public void setSyncUrl(String url) {
        if (Customization.getInstance().syncUriEditable()) {
            serverUriField.setText(url);
        }
    }

    public void setUsername(String username) {
        usernameField.setText(username);
    }

    public void setPassword(String password) {
        passwordField.setText(password);
    }

    private void resetValues() {
        controller.resetValues();
        invalidate();
    }

    private class RimAccountScreenMainVM extends VerticalFieldManager {

        public RimAccountScreenMainVM() {
            super();
        }

        protected void sublayout(int maxWidth, int maxHeight) {
            // We occupy all the height, but the space for the status
            int screenHeight = getScreen().getVisibleHeight();
            int height = screenHeight - getStatusHeight() - title.getHeight();
            // Reserve some extra space to avoid scroll
            height = (height * 98) / 100;
            setExtent(maxWidth, height);
            // Now recompute the layout
            super.sublayout(maxWidth, height);
            // Reset the extension because the previous call may have changed it
            setExtent(maxWidth, height);
        }
    }
}
