/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.util.Vector;
import java.util.Enumeration;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.KeyListener;
import net.rim.device.api.system.TrackwheelListener;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Keypad;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.component.ListField;
import net.rim.device.api.ui.component.ListFieldCallback;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.ExternalAppManager;
import com.funambol.blackberry.controller.HomeScreenController;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.blackberry.ui.SynchronizationScreen;
import com.funambol.blackberry.controller.UpdaterManager;
import com.funambol.blackberry.controller.SynchronizationController;
import net.rim.device.api.ui.Manager;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;
import com.funambol.syncml.spds.SyncSource;


/**
 * RimBasicHomeScreen is an inner class that creates the home screen for SyncClient on
 * the BlackBerry. The class provides a basic home screen that can be extended
 * to accomodate specific versions behavior. The class extends MainScreen.
 * 
 */
public class RimBasicHomeScreen extends MainScreen implements HomeScreen, UISyncSourceContainer,
                                                              SynchronizationScreen {

    private static final String TAG_LOG = "RimBasicHomeScreen";

    private static final String  IMAGE_ARROWS_IDLE       = "arrows1.png";
    protected HomeScreenController  controller;

    private HomeScreenMenu       menu;

    protected HomeScreenList     listField;

    private LabelField           titleField;

    protected RimButtonSyncSource syncAllItem = null;

    /** This is the FieldList default style. By default we center horizontally */
    protected long style = Field.FIELD_HCENTER;

    private boolean invalidateNavClick;

    private int previousWidth = -1;
    private int previousHeight = -1;
    private int minHeight = -1;
    private int maxHeight = -1;
    private String pressed = "";

    /**
     * Creates the home screen with all the UI components on it
     */
    public RimBasicHomeScreen() {
        super(MainScreen.FOCUSABLE | MainScreen.NO_VERTICAL_SCROLL);
    }

    public void initialize(HomeScreenController controller) {
        
        this.setFont(RimHelper.getDefaultFont());
        
        this.controller = controller;

        menu = new HomeScreenMenu(controller);

        if (Customization.getInstance().showTitleWithVersion()) {
            titleField = new LabelField(Customization.getInstance().getApplicationTitle() + " "
                    + BBVersion.VERSION);
        } else {
            titleField = new LabelField(Customization.getInstance().getApplicationTitle());
        }

        titleField.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(titleField);

        VerticalFieldManager vManager = new VerticalFieldManager(
                Manager.VERTICAL_SCROLL | Manager.VERTICAL_SCROLLBAR);
        add(vManager);
        listField = new HomeScreenList();
        vManager.add(listField);

        // The MySeparatorField allows to fill the empty space under the
        // ListField, in order to avoid navigationClick events in this area.
        add(new MySeparatorField(MySeparatorField.NON_FOCUSABLE));
    }

    public void setSelectedIndex(int index) {
        class SetSelectionRequest implements Runnable {

            private int index;

            public SetSelectionRequest(int index) {
                this.index = index;
            }

            public void run() {
                listField.setSelectedIndex(index);
            }
        }

        SetSelectionRequest req = new SetSelectionRequest(index);

        if (SyncClient.getUiApplication().isEventThread()) {
            req.run();
        } else {
            SyncClient.getUiApplication().invokeLater(req);
        }

        return;
    }

    public void updateVisibleItems(int focusRow) {
        class UpdateVisibleItemsRequest implements Runnable {

            private int index;
            private int size;

            public UpdateVisibleItemsRequest(int size, int index) {
                this.index = index;
                this.size  = size;
            }

            public void run() {
                listField.setSize(size, index);
                listField.updateListOfSources();
                listField.setSelectedIndex(index);
                // Now recompute sizes and repaint everything
                computeListFieldHeight();
                listField.invalidate();
            }
        }

        if (listField != null) {
            UpdateVisibleItemsRequest req;
            req = new UpdateVisibleItemsRequest(controller.getVisibleItems().size(), focusRow);
            if (SyncClient.getUiApplication().isEventThread()) {
                req.run();
            } else {
                SyncClient.getUiApplication().invokeLater(req);
            }
        }
    }

    /**
     * Check the Customization class if the Sync all main screen item is 
     * enabled. If disabled the first item in main screen is the first enabled 
     * source, so the initial index is set to 0.
     * @return the first index related to a source item  
     */
    private int getFirstSourceItemIndex() {
        if (Customization.getInstance().syncAllOnMainScreenRequired()) {
            return 1;
        } else {
            return 0;
        }
    }

    public UISyncSource createSyncAllItem(String name, String status, Bitmap icon,
                                          Bitmap selectedIcon, Bitmap bg, Bitmap bgSel) {

        syncAllItem = new SyncAllItem(name, null, icon, selectedIcon, bg, bgSel);
        syncAllItem.setKeepBgWhenDisabled(Customization.getInstance().keepSyncAllBackgroundWhenDisabled());
        return syncAllItem;
    }
    
    public void sourceStarted(int syncMask) {
        Enumeration enabledSources = AppSyncSourceManager.getInstance().getEnabledAndWorkingSources();
        
        while (enabledSources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource) enabledSources.nextElement();
            //Index of the item to be started
            int index = appSource.getUiSourceIndex();
            int id = appSource.getId();
                if ((id & syncMask) != 0) {
                    setSelectedIndex(index);
                }
            index++;
        }
    }

   
    public void disableListeners() {
        listField.removeListeners();
    }

    public void enableListeners() {
        listField.addListeners();
    }

    protected int getNumRows() {
        return listField.numRows();
    }

    protected int getRowHeight() {
        return listField.getRowHeight();
    }

    /**
     * This method is invoked on user hitting the escape button when in home
     * screen. It only minimizes the program
     */
    public boolean onClose() {
        controller.quitMenuSelected();
        return true;
    }

    public SynchronizationController getSynchronizationController() {
        return controller;
    }

    /**
     * Builds the menu
     * 
     * @param Menu
     *            Menu to which menuitems are to be added
     * @param int
     *            instance
     */
    public void makeMenu(Menu theMenu, int instance) {
        menu.makeMenu(theMenu, instance);
    }

    public void redraw() {
        if (listField != null) {
            listField.invalidate();
        }
    }

    protected void onExposed() {
        computeListFieldHeight();
        super.onExposed();
        controller.exposed();
    }

    protected void onDisplay() {
        computeListFieldHeight();
        super.onDisplay();
        controller.exposed();
    }

    protected int getDrawableHeight() {
        return getContentHeight() - titleField.getHeight();
    }

    private void computeListFieldHeight() {

        int width = getContentWidth();
        int height = getContentHeight();

        if (previousWidth == width && previousHeight == height) {
            return;
        }

        maxHeight = -1;
        minHeight = -1;

        previousWidth = width;
        previousHeight = height;

        // The max and min sizes depend on the screen resolution
        if (maxHeight == -1 || minHeight == -1) {

            if (width == 320 && height == 240) {
                // This is the Curve like
                minHeight = 36;
                maxHeight = 42;
            } else if (width == 240 && height == 320) {
                // This is the Pearl flip
                minHeight = 36;
                maxHeight = 42;
            } else if (width == 480 && height == 320) {
                // This is the Bold like
                minHeight = 40;
                maxHeight = 54;
            } else if (width == 360 && height == 480) {
                // This is the Storm like
                minHeight = 40;
                maxHeight = 60;
            } else if (width == 480 && height == 360) {
                // This is the Bold2 like
                minHeight = 40;
                maxHeight = 54;
            } else if (width == 240 && height == 260) {
                // This is the Pearl like
                minHeight = 36;
                maxHeight = 42;
            } else {
                // These are the default values
                minHeight = 36;
                maxHeight = 42;
            }
        }

        if (listField.numRows() != 0 && height > 0) {

            int rowHeight = height / listField.numRows();
            if (rowHeight < minHeight) {
                rowHeight = minHeight;
            }
            if (rowHeight > maxHeight) {
                rowHeight = maxHeight;
            }
            listField.setRowHeight(rowHeight);
        }
    }


    private class HomeScreenListener implements KeyListener, TrackwheelListener {

        private final static int KEYCODE_ENTER = Keypad.KEY_ENTER;

        // KeypadListener
        public boolean keyChar(char key, int status, int time) {
            return false;
        }

        public boolean keyDown(int keycode, int time) {
            int mapped = Keypad.map(keycode);
            if (mapped == KEYCODE_ENTER || keycode == KEYCODE_ENTER) {
                controller.buttonPressed(listField.getSelectedIndex());
                return true;
            }

            char mappedChar = Character.toUpperCase((char) mapped);

            if (mappedChar > 'A' && mappedChar < 'Z') {
                pressed += (char) mappedChar;
                if (pressed.length() > 3) {
                    pressed = pressed.substring(1);
                }
            }

            return false;
        }

        public boolean keyRepeat(int keycode, int time) {
            return false;
        }

        public boolean keyStatus(int keycode, int time) {
            return false;
        }

        public boolean keyUp(int keycode, int time) {
            return false;
        }

        public boolean trackwheelClick(int status, int time) {
            controller.buttonPressed(listField.getSelectedIndex());
            return true;
        }

        public boolean trackwheelRoll(int amount, int status, int time) {
            return false;
        }

        public boolean trackwheelUnclick(int status, int time) {
            return false;
        }
    }

    private class HomeScreenList extends ListField implements ListFieldCallback {

        private HomeScreenListener listener;

        private Vector                   listItems = new Vector();

        private boolean                  blockMoves;


        public HomeScreenList() {

            // The number of rows is dynamic and it depends on the number of
            // active sources
            super(controller.getVisibleItems().size(), style);
            updateListOfSources();
            setEditable(false);
            setEmptyString("", DrawStyle.LEFT);

            setCallback(this);
            listener = new HomeScreenListener();
            addListeners();
        }

        protected void paint(Graphics g) {
            // We need to recompute the height here, because if we do it only in
            // the onDisplay, onExpose we sometime get the wrong screen size
            // (for example on the Storm we have the size when the keyboard is
            // still visible)
            computeListFieldHeight();
            super.paint(g);
        }

        public void updateListOfSources() {

            // Remove all existing items
            listItems.removeAllElements();

            // TODO: can we do a setSize at this point? This would help to make
            // the list more dynamic providing the ability to hide/show sources
            if (syncAllItem != null) {
                listItems.addElement(syncAllItem);
            }
            Vector appSources = controller.getVisibleItems();
            Enumeration iter  = appSources.elements();
            while(iter.hasMoreElements()) {
                // The first item may be the sync all button. Check for this
                // case
                RimButtonSyncSource item;
                AppSyncSource appSource = (AppSyncSource) iter.nextElement();
                // In this view we use a RimButtonSyncSource to represent
                // sources
                item = createButtonUISyncSource(appSource);

                // Is this the right place where the controller and the UI
                // object shall be created?
                UISyncSourceController itemController = new UISyncSourceController(
                        controller.getController(), appSource);
                appSource.setUISyncSource(item);
                itemController.setUISyncSource(item);
                SyncSource ss = appSource.getSyncSource();
                ss.setListener(itemController);
                appSource.setUISyncSourceController(itemController);
                item.setStatusSelectedIcon(Bitmap.getBitmapResource(IMAGE_ARROWS_IDLE));

                // All these buttons are associated to a given application
                // source
                item.setSource(appSource);
                item.setContainer(RimBasicHomeScreen.this);
                listItems.addElement(item);
            }
        }

        protected boolean navigationClick(int status, int time) {
            Log.trace(TAG_LOG, "navigationClick");
            if (status == 0 && !invalidateNavClick) {
                controller.buttonPressed(listField.getSelectedIndex());
                return true;
            }
            invalidateNavClick = false;
            return (status == 0);
        }

        private int numRows() {
            return listItems.size();
        }

        private RimButtonSyncSource createButtonUISyncSource(AppSyncSource appSource) {
            String iconName = appSource.getIconName();
            String iconDisabledName = appSource.getDisabledIconName();
            String name = appSource.getName();
            String bgName = Customization.getInstance().getButtonBackground();
            String bgSelName = Customization.getInstance().getButtonHighlightedBackground();
            Bitmap bg = Bitmap.getBitmapResource(bgName);
            Bitmap bgSel = Bitmap.getBitmapResource(bgSelName);
            Bitmap img = Bitmap.getBitmapResource(iconName);
            Bitmap imgDisabled = Bitmap.getBitmapResource(iconDisabledName);
            RimButtonSyncSource item = new RimButtonSyncSource(name, "", img, imgDisabled, bg, bgSel);
            return item;
        }

        // ListFieldCallback functions
        public void drawListRow(ListField listField, Graphics graphics, int index, int y, int width) {
            RimButtonSyncSource item = ((RimButtonSyncSource) listItems.elementAt(index));
            item.draw(graphics, 0, y, width, listField.getRowHeight());
        }

        public Object get(ListField listField, int index) {
            return null;
        }

        public int indexOfList(ListField listField, String prefix, int start) {
            return listField.getSelectedIndex();
        }

        public int getPreferredWidth(ListField listField) {
            return getContentWidth();
        }
        
        protected int moveFocus(int amount, int status, int time) {

            if (blockMoves) {
                return 0;
            }

            UISyncSource item;

            int oldSelection = getSelectedIndex();
            item = ((UISyncSource) listItems.elementAt(oldSelection));
            item.setSelected(false);

            // Forward the call
            int ret = super.moveFocus(amount, status, time);

            int newSelection = getSelectedIndex();

            // Get the next enabled item;
            item = ((UISyncSource) listItems.elementAt(newSelection));
            boolean stop = false;
            while (item.isDisabled() && !stop) {
                newSelection += amount;
                stop = newSelection < 0 || newSelection >= listItems.size();
                if (!stop) {
                    item = ((UISyncSource) listItems.elementAt(newSelection));
                }
            }
            
            if (stop) {
                newSelection = oldSelection;
                invalidateNavClick = true;
            }
            
            item = ((UISyncSource) listItems.elementAt(newSelection));

            item.setSelected(true);
            setSelectedIndex(newSelection);

            invalidate();

            return ret;
        }

        protected void moveFocus(int x, int y, int status, int time) {
            int oldSelection = getSelectedIndex();
            super.moveFocus(x, y, status, time);
            int newSelection = getSelectedIndex();
            UISyncSource item = (UISyncSource)listItems.elementAt(newSelection);
            if(!item.isDisabled()) {
                ((UISyncSource) listItems.elementAt(oldSelection)).setSelected(false);
                setSelectedIndex(newSelection);
            }
            else {
                setSelectedIndex(oldSelection);
                invalidateNavClick = true; 
            }
        }

        public void addListeners() {
            addKeyListener(listener);
            // If the device being used does not have a dedicated menu key,
            // then we shall not intercept the trackwheel
            if (StaticDataHelper.hasDedicatedMenuKey()) {
                addTrackwheelListener(listener);
            }
            blockMoves = false;
        }

        public void removeListeners() {
            removeKeyListener(listener);
            blockMoves = true;
        }

        public void setSelectedIndex(int index) {
            int current = getSelectedIndex();
            super.setSelectedIndex(index);
            ((UISyncSource) listItems.elementAt(current)).setSelected(false);
            ((UISyncSource) listItems.elementAt(index)).setSelected(true);
            invalidate();
        }
    }

    private class SyncAllItem extends RimButtonSyncSource {

        public SyncAllItem(String title, String status, Bitmap image,
                           Bitmap imageDisabled, Bitmap background,
                           Bitmap backgroundSelected) {

            super(title, status, image, imageDisabled, background,
                  backgroundSelected);
            setFont(getFont());
            setContainer(RimBasicHomeScreen.this);
        }

        protected void drawStatusText(Graphics graphics, int x, int y, int width, int height) {
            // Last sync takes bottom third
            //int fontHeight = (height / 3) - (PADDING * 2);
            int fontHeight = ((2* height) / 5) - (PADDING * 2);
            graphics.setFont(getFont().derive(Font.PLAIN, fontHeight));
            graphics.setColor(Color.BLACK);
            graphics.drawText(status, x + PADDING + 5, y - 4 + (height - fontHeight),
                    DrawStyle.LEFT | DrawStyle.TOP, width - PADDING);
        }

        protected void drawBackground(Graphics graphics, int x, int y, int width, int height) {
            Bitmap toDraw = null;
            if (selected) {
                toDraw = bgSelected;
            } else {
                toDraw = bg;
            }
            int imgWidth = toDraw.getWidth();
            while (width > -2) {
                graphics.drawBitmap(x - 1, y - 1, width + 2, height + 1, toDraw, 0, 0);
                width -= imgWidth;
                // Overlap a little bit to avoid border issues
                x += imgWidth - 2;
            }
        }

        private void drawBorder(Graphics graphics, int x, int y, int width, int height) {

            graphics.setColor(Color.BLACK);
            graphics.drawRect(x, y, width, height);
        }
    }

    private class HomeScreenMenu {

        private HomeScreenController homeScreenController;

        public HomeScreenMenu(HomeScreenController ctrl) {
            homeScreenController = ctrl;
        }

        /**
         * The menu
         */
        private final MenuItem syncAllMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_sync-all"), 20000, 10) {
                public void run() {
                    homeScreenController.syncMenuSelected();
                }
            };

        private final MenuItem syncMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_sync"), 20000, 10) {
                public void run() {
                    int index = listField.getSelectedIndex();
                    UISyncSource item;
                    item = (UISyncSource) listField.listItems.elementAt(index);
                    AppSyncSource source = item.getSource();
                    // Sync this specific source
                    homeScreenController.syncMenuSelected(source);
                }
            };

        private final MenuItem gotoMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_goto"), 20000, 10) {
                public void run() {
                    int index = listField.getSelectedIndex();
                    UISyncSource item;
                    item = (UISyncSource) listField.listItems.elementAt(index);
                    AppSyncSource source = item.getSource();
                    ExternalAppManager appManager = source.getAppManager();
                    try {
                        appManager.launch();
                    } catch (Exception e) {
                        Log.error(TAG_LOG, "Cannot invoke the application: ", e);
                    }
                }
            };

        private final MenuItem refreshMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_refresh"), 20000, 10) {
                public void run() {
                    homeScreenController.refreshMenuSelected();
                }
            };

        private final MenuItem cancelMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_cancel-sync"), 20000, 10) {
                public void run() {
                    homeScreenController.cancelMenuSelected();
                }
            };

        private final MenuItem sendLogMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_send-logs"), 20000, 10) {
                public void run() {
                    homeScreenController.sendLogs();
                }
            };

        private final MenuItem accountMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_account"), 20000, 10) {
                public void run() {
                    homeScreenController.accountMenuSelected();
                }
            };

        private final MenuItem configMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_config"), 20000, 10) {
                public void run() {
                    homeScreenController.configMenuSelected();
                }
            };

        
        private final MenuItem aboutMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_about"), 20000, 10) {
                 public void run() {
                     homeScreenController.aboutMenuSelected();
                 }
             };

        private final MenuItem updateMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_update"), 20000, 10) {
                public void run() {
                    homeScreenController.updateMenuSelected();
                }
            };

        private final MenuItem quitMenu =
            new MenuItem(StaticDataHelper.getLanguage("menu_quit"), 20000, 10) {
                public void run() {
                    homeScreenController.quitMenuSelected();
                }
            };

        public void makeMenu(Menu menu, int instance) {

            boolean inSync   = false;
            boolean isErrors = false;
            boolean isUpdate = false;

            try {
                inSync   = controller.isSynchronizing();
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot detect if sync is running", e);
            }

            try {
                isErrors = controller.numErrors() != 0;
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot detect if there are pending errors", e);
            }

            try {
                isUpdate = Customization.getInstance().enableUpdaterManager() && 
                           UpdaterManager.getInstance().isUpdateAvailable();
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot detect if an update is available", e);
            }

            if (inSync) {
                menu.add(cancelMenu);
            } else {
                if(listField.listItems.size() > 1) {
                    menu.add(syncAllMenu);
                }
                // We add the sync menu only if the highlighted item
                // is not the sync all
                if (Customization.getInstance().syncAllOnMainScreenRequired()) {
                    if (listField.getSelectedIndex() != 0) {
                        addSourceItemListMenu(menu);
                    }    
                } else {
                    addSourceItemListMenu(menu);
                }
                
            }
            menu.add(MenuItem.separator(20000));
            menu.add(quitMenu);
            menu.add(MenuItem.separator(20000));

            if (!inSync) {
                menu.add(accountMenu);
                menu.add(configMenu);
            }

            if (!inSync) {
                menu.add(refreshMenu);
            }

            if (Customization.getInstance().sendLogEnabled()) {
                if (isErrors || "LOG".equals(pressed)) {
                    menu.add(sendLogMenu);
                }
            }

            if (isUpdate) {
                menu.add(updateMenu);
            }
           
            menu.add(aboutMenu);
        }
        
        private void addSourceItemListMenu(Menu menu) {
            // Fix the menu text, depending on the selected source
            int index = listField.getSelectedIndex();
            UISyncSource item;
            item = (UISyncSource) listField.listItems.elementAt(index);
            AppSyncSource source = item.getSource();
            String menuText = StaticDataHelper.getLanguage("menu_sync");
            StringBuffer newMenuText = new StringBuffer(menuText);
            newMenuText.append(" ").append(source.getName());
            syncMenu.setText(newMenuText.toString());
            
            if (source.isEnabled()) {
                // Add the menu
                menu.add(syncMenu);

                ExternalAppManager appManager = source.getAppManager();
                // Add the proper goto menu, depending on the selected
                // source
                if (Customization.getInstance().gotoMenuEnabled() &&
                    appManager != null) {

                    menuText = StaticDataHelper.getLanguage("menu_goto");
                    newMenuText = new StringBuffer(menuText);
                    newMenuText.append(" ").append(source.getName());
                    gotoMenu.setText(newMenuText.toString());
                    // Add the menu
                    menu.add(gotoMenu);
                }
            }
        }
    }

    private class MySeparatorField extends SeparatorField {
        public MySeparatorField(long s) {
            super(s);
        }
        protected void layout(int width, int height) {
            setExtent(width, 0);
        }
    }


}
