/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;


import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.util.Log;

public class RimButtonSyncSource implements UISyncSource {

    protected static final int MARGIN_SIDE          = 5;
    protected static final int PADDING              = 2;

    protected String          title                 = null;
    protected String          status                = null;
    protected Bitmap          image                 = null;
    protected Bitmap          statusImage           = null;
    protected Bitmap          statusImageSelected   = null;
    protected Bitmap          imageDisabled         = null;
    protected Bitmap          bg                    = null;
    protected Bitmap          bgSelected            = null;
    protected boolean         enabled               = true;
    protected boolean         selected              = false;
    protected int             titleEnabledColor     = Color.BLACK;
    protected int             titleDisabledColor    = Color.LIGHTGREY;
    protected int             disabledStatusColor   = Color.LIGHTGREY;
    protected int             enabledStatusColor    = Color.BLACK;
    protected AppSyncSource   source                = null;
    protected Font            font                  = null;
    protected UISyncSourceContainer container       = null;
    protected boolean         keepBgWhenDisabled    = false;


    public RimButtonSyncSource(String title, String status, Bitmap image,
                               Bitmap imageDisabled, Bitmap background,
                               Bitmap backgroundSelected) {

        this.title = title;
        this.status = status;
        this.image = image;
        this.imageDisabled = imageDisabled;
        this.bg = background;
        this.bgSelected = backgroundSelected;
        this.font = RimHelper.getDefaultFont();
    }

    public void setTitle(String value) {
        title = value;
    }

    public void setFont(Font font) {
        this.font = font;
    }

    public void setContainer(UISyncSourceContainer container) {
        this.container = container;
    }

    public void setTitleEnabledColor(int color) {
        this.titleEnabledColor = color;
    }

    public void setTitleDisabledColor(int color) {
        this.titleDisabledColor = color;
    }

    public void setEnabled(boolean flag) {
        enabled = flag;
    }

    public boolean isDisabled() {
        return !enabled;
    }

    public void setStatusIcon(Bitmap image) {
        statusImage = image;
    }

    public void setStatusSelectedIcon(Bitmap image) {
        statusImageSelected = image;
    }

    public void redraw() {
        if (container != null) {
            container.redraw();
        }
    }

    public void setStatusString(String value) {
        status = value;
    }

    public void setSource(AppSyncSource source) {
        this.source = source;
    }

    public void setSelected(boolean selected) {
        this.selected = selected;
    }

    public boolean isSelected() {
        return selected;
    }

    public AppSyncSource getSource() {
        return source;
    }

    public void draw(Graphics graphics, int x, int y, int width, int height) {
        // Border
        Font originalFont = graphics.getFont();
        int originalColor = graphics.getColor();
        drawBackground(graphics, x, y, width, height);

        drawBorder(graphics, x, y, width, height);

        int leftImageWidth = drawLeftImage(graphics, x, y, height);

        int rightImageWidth = drawRightImage(graphics, y, width, height);

        drawTitleText(graphics, leftImageWidth, y, width - rightImageWidth - leftImageWidth, height);

        drawStatusText(graphics, leftImageWidth, y, width - rightImageWidth - leftImageWidth, height);


        graphics.setFont(originalFont);
        graphics.setColor(originalColor);
    }

    /**
     * Set whether or not to keep drawing the background when the item is disabled
     */
    public void setKeepBgWhenDisabled(boolean value) {
        keepBgWhenDisabled = value;
    }

    public void setDisabledStatusColor(int color) {
        disabledStatusColor = color;
    }

    public void setEnabledStatusColor(int color) {
        enabledStatusColor = color;
    }

    private void drawStatusText(Graphics graphics, int x, int y, int width, int height) {
        // Last sync takes bottom third
        //int fontHeight = (height / 3) - (PADDING * 2);
        int fontHeight = ((2* height) / 5) - (PADDING * 2);
        graphics.setFont(font.derive(Font.PLAIN, fontHeight));

        if (enabled) {
            graphics.setColor(enabledStatusColor);
        } else {
            graphics.setColor(disabledStatusColor);
        }

        graphics.drawText(status, x + PADDING + 5, y - 4 + (height - fontHeight),
                DrawStyle.LEFT | DrawStyle.TOP, width - PADDING);
    }

    private void drawTitleText(Graphics graphics, int x, int y, int width, int height) {
        //int fontHeight = ((int) (1.7 * (height / 3))) - (PADDING * 2);
        int fontHeight = ((int) ((3* height) / 5)) - (PADDING * 2);
        graphics.setFont(font.derive(Font.BOLD, fontHeight));

        if (enabled) {
            graphics.setColor(titleEnabledColor);
        } else {
            graphics.setColor(titleDisabledColor);
        }

        if (status != null) {
            // Title takes top 2/3 of list item
            graphics.drawText(title, x + PADDING + 3, y + PADDING + 2,
                              DrawStyle.LEFT | DrawStyle.TOP, width - x - (PADDING * 2));
        } else {
            // Title is vertically centered
            graphics.drawText(title, x + PADDING + 3, y + PADDING + (fontHeight / 2),
                    DrawStyle.LEFT | DrawStyle.TOP, width - x - (PADDING * 2));
        }

    }

    private int drawRightImage(Graphics graphics, int y, int width, int height) {
        // Image on right side

        int imageWidth = 0;
        int imageHeight;
        int imageTop;
        int imageLeft;
        Bitmap toDraw = null;
        if (selected) {
            toDraw = statusImageSelected;
        } else {
            toDraw = statusImage;
        }

        if (toDraw != null && enabled) {
            imageWidth = toDraw.getWidth();
            imageHeight = toDraw.getHeight();
            imageTop = y + ((height - imageHeight) / 2);
            imageLeft = (width - height) + ((height - imageWidth) / 2);

            graphics.drawBitmap(imageLeft, imageTop, imageWidth, imageHeight, toDraw, 0, 0);
        }

        return height;
    }

    private int drawLeftImage(Graphics graphics, int x, int y, int height) {
        Bitmap leftImage;

        if (enabled) {
            graphics.setColor(Color.BLACK);
            leftImage = image;
        } else {
            graphics.setColor(Color.LIGHTGREY);
            leftImage = imageDisabled;
        }

        int imageWidth = leftImage.getWidth();
        int imageHeight = leftImage.getHeight();
        int imageTop = y + ((height - imageHeight) / 2);
        int imageLeft = x + ((height - imageWidth) / 2);

        // Image on left side
        graphics.drawBitmap(imageLeft, imageTop, imageWidth, imageHeight, leftImage, 0, 0);

        return height;
    }

    private void drawBackground(Graphics graphics, int x, int y, int width, int height) {
        if (enabled || keepBgWhenDisabled) {
            Bitmap toDraw = null;
            if (selected) {
                toDraw = bgSelected;
            } else {
                toDraw = bg;
            }
            int imgWidth = toDraw.getWidth();
            while (width > -2) {
                graphics.drawBitmap(x - 1, y - 1, width + 2, height + 1, toDraw, 0, 0);
                width -= imgWidth;
                // Overlap a little bit to avoid border issues
                x += imgWidth - 2;
            }
        } else {
            graphics.setColor(Color.WHITE);
            graphics.fillRect(x - 1, y - 1, width + 2, height + 1);
        }
    }

    private void drawBorder(Graphics graphics, int x, int y, int width, int height) {

        graphics.setColor(Color.GRAY);
        graphics.drawLine(x, y - 1, x + width, y - 1);
        graphics.drawLine(x, y + height - 1, x + width, y + height - 1);
    }
}

