/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;
import java.util.Enumeration;

import net.rim.blackberry.api.mail.*;
import net.rim.device.api.system.DeviceInfo;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.NumericChoiceField;
import net.rim.device.api.ui.component.ObjectChoiceField;
import net.rim.device.api.ui.component.PasswordEditField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.system.Display;

import com.funambol.blackberry.Configuration;
import com.funambol.blackberry.Constants;
import com.funambol.blackberry.AppSyncSourceManager;
import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.Customization;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.DialogController;
import com.funambol.blackberry.ui.ConfigurationScreen;
import com.funambol.blackberry.ui.rim.component.ColoredBasicEditField;
import com.funambol.blackberry.ui.rim.component.ColoredLabelField;

import com.funambol.platform.FileAdapter;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.Log;
import com.funambol.util.LogContent;
import com.funambol.util.StaticDataHelper;
import com.funambol.util.FileAppender;
import com.funambol.util.Appender;


/**
 * Is an class that creates the configuration screen for configuring parameters
 * required for the SyncClient
 */
public class RimConfigurationScreen extends MainScreen implements ConfigurationScreen {

    private static final String TAG_LOG = "RimConfigurationScreen";

    private static final String[] PICTURES_DIR_LABEL = {
        StaticDataHelper.getLanguage("conf_pictures_phone_storage"),
        StaticDataHelper.getLanguage("conf_pictures_sdcard_storage"),
        StaticDataHelper.getLanguage("conf_pictures_custom_storage")
    };

    private static final String[] PICTURES_DIR_LABEL_NO_SDCARD = {
        StaticDataHelper.getLanguage("conf_pictures_phone_storage"),
        StaticDataHelper.getLanguage("conf_pictures_custom_storage")
    };

    private boolean enableFileBrowser = false;
    private String[] picturesDirSelectionName = null;

    // List of ColoredLabelField
    private Vector                enabledSourceLabelList  = new Vector(); 
    // List of CheckboxField
    private Vector                enabledSourceFieldList  = new Vector();
    // List of ColoredLabelField
    private Vector                sourceUriFieldList      = new Vector();
    // List of ColoredLabelField
    private Vector                disabledSourceLabelList = new Vector();

    private Vector                sourceMap               = new Vector();  

    private ObjectChoiceField     syncModeField           = null;

    private ObjectChoiceField     pollingPIMIntervalField = null;

    private NumericChoiceField    rangePastField          = null;

    private NumericChoiceField    rangeFutureField        = null;

    private ObjectChoiceField     syncTypeField           = null;

    private ObjectChoiceField     logLevelField           = null;

    private CheckboxField         blockInvitesField       = null;

    private CheckboxField         c2sPushField       = null;

    private boolean               hasChanges              = false;

    private Customization         customization           = Customization.getInstance();

    /**
     * This creates a MenuItem containing "Save" option. on selecting the save
     * option the configuration details are saved.
     */
    private final MenuItem        saveMenu                = new MenuItem(StaticDataHelper
                                                                  .getLanguage("menu_save"), 20000,
                                                                  10) {

                                                              public void run() {
                                                                  if (saveConfiguration()) {
                                                                      controller
                                                                              .getScreenDisplayController()
                                                                              .hideConfigScreen();
                                                                      hasChanges = false;
                                                                  }
                                                              }
                                                          };

    private final MenuItem        backMenu                = new MenuItem(StaticDataHelper
                                                                  .getLanguage("menu_back"), 20000,
                                                                  10) {

                                                              public void run() {
                                                                  onClose();
                                                              }
                                                          };

    private final MenuItem        sendLogMenu             = new MenuItem(StaticDataHelper
                                                                  .getLanguage("menu_sendlog"), 20000,
                                                                  10) {

                                                              public void run() {
                                                                  sendLog();
                                                              }
                                                          };


    private final Controller      controller;

    private CheckboxField         cdEnableField;

    private String            picturesCurrentDir   = null;
    private ColoredLabelField photosDirLabelField  = null;
    private PicturesLocationChoiceField picturesLocationChoiceField = null;
    private VerticalFieldManager   intervalVfm     = null;
    private HorizontalFieldManager intervalHfm     = null;
    private LabelField picturesOneWayField         = null;
    private CheckboxField picturesSyncOld          = null;
    private VerticalFieldManager sourcesFieldManager = null;
    private VerticalFieldManager syncDirectionFieldManager = null;
    private int sourcesIndex = 0;
    private Configuration configuration;



    /**
     * The constructor here creates the Configuration screen with all the UI
     * components on it and sets default input minValues
     */
    public RimConfigurationScreen(Controller ctrl) {

        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT | Field.FIELD_LEFT | Field.FIELD_TOP);

        Log.trace(TAG_LOG, "RimConfigurationScreen");

        this.setFont(RimHelper.getDefaultFont());

        controller = ctrl;
        configuration = Configuration.getInstance();

        ChangeListener cl = new ChangeListener();

        LabelField title = new LabelField(StaticDataHelper.getLanguage("conf_title"));
        title.setFont(getFont().derive(StaticDataHelper.TITLE_FONT_STYLE,
                StaticDataHelper.TITLE_FONT_SIZE));
        setTitle(title);

        String[] syncModes = getSyncModes();
        LabelField syncModeLabel = new LabelField(StaticDataHelper.
                                                     getLanguage("conf_sync_mode-title"));
        syncModeLabel.setFont(getFont().derive(Font.BOLD));
        syncModeField = new ObjectChoiceField("", syncModes, Configuration.getSyncMode());
        syncModeField.setFont(getFont());

        LabelField pollingPIMIntervalLabel = new LabelField(StaticDataHelper.
                                                     getLanguage("conf_polling-pim-duration"));
        pollingPIMIntervalLabel.setFont(getFont().derive(Font.BOLD));
        String[] choices = getIntervalChoices(customization.getPollingPimIntervalChoices());
        int index = getScheduledIntervalIndexFromMins(Configuration.getPollingInterval());
        pollingPIMIntervalField = new ObjectChoiceField("", choices, index);
        pollingPIMIntervalField.setFont(getFont());

        /*
         * TWO_WAY = 0; ONE_WAY_TO_SERVER = 1; ONE_WAY_TO_CLIENT = 2;
         */
        choices = getSyncTypes();
        // The MyLabelField is a custom LabelField which allows to force the field width
        // This is used within a HorizontalFieldManager.
        class MyLabelField extends LabelField {
            private int maxWidth;
            private final int PADDING_WIDTH = 5;
            public MyLabelField(String label, int maxWidth) {
                super(label);
                this.maxWidth = maxWidth-PADDING_WIDTH;
            }
            public int getPreferredWidth() {
                int labelWidth = getFont().getAdvance(getText());
                if(labelWidth<maxWidth) {
                    maxWidth = labelWidth + PADDING_WIDTH;
                }
                return maxWidth;
            }
        }
        int syncTypeMaxLength = Display.getWidth()-getFont().getAdvance(
                        StaticDataHelper.getLanguage("conf_sync-type-two-way"));
        
        MyLabelField syncTypeLabel = new MyLabelField(
                StaticDataHelper.getLanguage("conf_sync-type"), syncTypeMaxLength);
        syncTypeLabel.setFont(getFont().derive(Font.BOLD));
        syncTypeField = new ObjectChoiceField("", choices, Configuration.getSyncType());
        syncTypeField.setFont(getFont());

        LabelField rangeLabel = new LabelField(StaticDataHelper.getLanguage("conf_range"));
        rangeLabel.setFont(getFont().derive(Font.BOLD));

        rangePastField = new NumericChoiceField(StaticDataHelper.getLanguage("conf_range-past")
                + "  ", 1,// begin
                3,// end
                1,// increment
                Configuration.getRangePast() - 1);
        rangePastField.setEditable(false);
        rangePastField.setFont(getFont());

        rangeFutureField = new NumericChoiceField(StaticDataHelper.getLanguage("conf_range-future")
                + "  ", 1,// begin
                6,// end
                1,// increment
                Configuration.getRangeFuture() - 1);
        rangeFutureField.setEditable(false);
        rangeFutureField.setFont(getFont());

        choices = new String[] { StaticDataHelper.getLanguage("conf_log-level-0"),
                StaticDataHelper.getLanguage("conf_log-level-1"),
                StaticDataHelper.getLanguage("conf_log-level-2"),
                StaticDataHelper.getLanguage("conf_log-level-3") };

        LabelField logLevelLabel = new LabelField(StaticDataHelper.getLanguage("conf_log-level"));
        logLevelLabel.setFont(getFont().derive(Font.BOLD));
        logLevelField = new ObjectChoiceField("", choices, Configuration.getLogLevel());

        LabelField blockInvitesLabel = null;
        if (customization.invitesFiltering()) {
            blockInvitesLabel = new LabelField(StaticDataHelper
                    .getLanguage("conf_block-invites"));
            blockInvitesLabel.setFont(getFont().derive(Font.BOLD));
            blockInvitesField = new CheckboxField("", Configuration.getBlockIncomingInvites());
        }

        if (customization.isC2SPushEnabled()) {
            c2sPushField = new CheckboxField(StaticDataHelper
                    .getLanguage("conf_c2s-push"), configuration.getC2SPush());
        }

        // /////////////////////////////////////////////////////////////////////////

        final SeparatorField separator2 = new SeparatorField();
        final SeparatorField separator3 = new SeparatorField();
        final SeparatorField separator4 = new SeparatorField();

        sourcesIndex = getFieldCount();
        boolean sourcesAdded = updateListOfSources();
        add(new LabelField(""));

        if(sourcesAdded) {
            add(separator2);
            add(new LabelField(""));
        }

        intervalVfm = new VerticalFieldManager();
        HorizontalFieldManager hfm = new HorizontalFieldManager();
        hfm.add(syncModeLabel);
        hfm.add(syncModeField);
        intervalVfm.add(hfm);
        intervalHfm = new HorizontalFieldManager();
        intervalHfm.add(pollingPIMIntervalLabel);
        intervalHfm.add(pollingPIMIntervalField);

        if (Configuration.getSyncMode() == Configuration.SYNC_MODE_SCHEDULED) {
            intervalVfm.add(intervalHfm);
        }
        add(intervalVfm);

        add(new LabelField(""));

        if(customization.isC2SPushEnabled()){
            hfm = new HorizontalFieldManager();
            hfm.add(c2sPushField);
            add(hfm);
        }
        
        if (customization.isSyncDirectionVisible()) {
            syncDirectionFieldManager = new VerticalFieldManager();
            syncDirectionFieldManager.add(new LabelField(""));
            hfm = new HorizontalFieldManager();
            hfm.add(syncTypeLabel);
            hfm.add(syncTypeField);
            syncDirectionFieldManager.add(hfm);
            if (picturesOneWayField != null) {
                syncDirectionFieldManager.add(picturesOneWayField);
            }
            add(syncDirectionFieldManager);
        }

        add(new LabelField(""));

        if (customization.invitesFiltering()) {
            add(separator3);
            add(new LabelField(""));

            hfm = new HorizontalFieldManager();
            hfm.add(blockInvitesField);
            hfm.add(blockInvitesLabel);
            add(hfm);
        }
    
        if (customization.isRangePastVisible()||
                customization.isRangeFutureVisible()) {
            add(new LabelField(""));
            add(rangeLabel);
        }
        
        if (customization.isRangePastVisible()) {
            add(rangePastField);
        }
        
        if (customization.isRangeFutureVisible()) {
            add(rangeFutureField);
        }

        if (customization.isLogEnabledInSettingsScreen()) {
            add(separator4);
            add(new LabelField(""));
            hfm = new HorizontalFieldManager();
            hfm.add(logLevelLabel);
            hfm.add(logLevelField);
            add(hfm);
        }
        // /////////////////////////////////////////////////////////////////////////

        syncModeField.setChangeListener(cl);
        pollingPIMIntervalField.setChangeListener(cl);

        if (customization.isRangePastVisible()) {
            rangePastField.setChangeListener(cl);
        }
        
        if (customization.isRangeFutureVisible()) {
            rangeFutureField.setChangeListener(cl);
        }

        syncTypeField.setChangeListener(cl);
        logLevelField.setChangeListener(cl);

        if (customization.invitesFiltering()) {
            blockInvitesField.setChangeListener(cl);
        }

        if(customization.isC2SPushEnabled()){
            c2sPushField.setChangeListener(cl);
        }
    }
    
    public void onDisplay() {
        super.onDisplay();
        loadValues();
        reinitializePicturesDirOptions();
        invalidate();
    }
    
    public void onExpose() {
        super.onExposed();
        loadValues();
        reinitializePicturesDirOptions();
        invalidate();
    }

    public boolean updateListOfSources() {

        Log.trace(TAG_LOG, "updateListOfSources");

        // /////////////////////////////////////////////////////////////////////////
        // Add a field manager for the sources, this can be changed later on,
        // depending on sources being added/removed
        if (sourcesFieldManager != null) {
            delete(sourcesFieldManager);
        }
        
        if(syncDirectionFieldManager != null && picturesOneWayField != null) {
            syncDirectionFieldManager.delete(picturesOneWayField);
        }

        sourcesFieldManager = new VerticalFieldManager();

        ChangeListener cl = new ChangeListener();

        // Cleanup ol values
        picturesOneWayField = null;
        picturesSyncOld = null;
        enabledSourceLabelList.removeAllElements();
        enabledSourceFieldList.removeAllElements();
        sourceUriFieldList.removeAllElements();
        disabledSourceLabelList.removeAllElements();
        sourceMap.removeAllElements();


        // Populate the field manager with the necessary sources
        VerticalFieldManager vfm;
        HorizontalFieldManager hfm;

        ColoredLabelField cdLabelField        = null;
        Enumeration sources = AppSyncSourceManager.getInstance().getRegisteredSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();

            Log.trace(TAG_LOG, "Adding settings for source: " + appSource.getName());

            // If the source has no settings, then skip it
            if (!appSource.hasSettings()) {
                continue;
            }
            // If the source is not active, then skip it
            if (!Configuration.isSourceActive(appSource.getId())) {
                continue;
            }

            String enableString = appSource.getEnabledLabel();

            ColoredLabelField enableSourceLabel = new ColoredLabelField(enableString);
            enableSourceLabel.setFont(getFont().derive(Font.BOLD));

            boolean sourceEnabled = appSource.isEnabled();
            CheckboxField enableSyncSourceField = new CheckboxField("", sourceEnabled);

            String uri = Configuration.getSourceUri(appSource.getId());
            ColoredBasicEditField sourceUriField = new ColoredBasicEditField(StaticDataHelper
                    .getLanguage("conf_remote-uri")
                    + " ", uri);
            ColoredLabelField disabledSyncSourceLabel = new ColoredLabelField();

            // Contacts and photos are special as they have one extra option
            if (appSource.getId() == AppSyncSourceManager.SYNC_FLAG_CONTACTS) {
                cdEnableField = null;
                if (customization.syncCompanyDirectory()) {
                    cdEnableField = new CheckboxField("", uri.endsWith("-shared"));
                    cdLabelField = new ColoredLabelField(StaticDataHelper.getLanguage("custom_config_cd"));
                    FieldChangeListener listener = new CDListener(cl, sourceUriField);
                    cdEnableField.setChangeListener(listener);

                    if (!appSource.isEnabled()) {
                        cdEnableField.setEditable(false);
                        cdLabelField.setColor(Color.GRAY);
                    }
                }
            } else if (appSource.getId() == AppSyncSourceManager.SYNC_FLAG_PHOTOS) {
                //String value = StaticDataHelper.getLanguage("photo_config_dir");
                String dir   = Configuration.getPicturesDir();
                // Strip out the protocol to better fit in the screen
                if (dir.startsWith("file://")) {
                    dir = dir.substring(7);
                }
                //value = value + ":" + dir;
                photosDirLabelField = new ColoredLabelField(dir);
                picturesLocationChoiceField = new PicturesLocationChoiceField();
                String locationLabel = StaticDataHelper.getLanguage("conf_pictures_location_label");
                picturesLocationChoiceField.setLabel(locationLabel);
                reinitializePicturesDirOptions();

                if (!appSource.isEnabled()) {
                    photosDirLabelField.setColor(Color.GRAY);
                    picturesLocationChoiceField.setEditable(false);
                }
            }

            String disableString = appSource.getDisabledLabel();
            disabledSyncSourceLabel.setText(disableString);
            disabledSyncSourceLabel.setFont(getFont().derive(Font.ITALIC));

            // Add the UI components to the proper vectors
            enabledSourceLabelList.addElement(enableSourceLabel); // ColoredLabelField
            enabledSourceFieldList.addElement(enableSyncSourceField); // CheckboxField
            sourceUriFieldList.addElement(sourceUriField);            // ColoredBasicEditField
            disabledSourceLabelList.addElement(disabledSyncSourceLabel); // Colored

            sourceUriField.setChangeListener(cl);
            enableSyncSourceField.setChangeListener(cl);

            sourceMap.addElement(appSource);
        }

        boolean sourcesAdded = true;
        
        for(int i=0;i<enabledSourceFieldList.size();++i) {

            AppSyncSource appSource = (AppSyncSource) sourceMap.elementAt(i);

            // Show nothing if there is only one source, the source uri is not
            // visible and it is not the Pictures source, for which there
            // are further settings.
            if(enabledSourceFieldList.size() == 1 &&
                    !customization.isSourceUriVisible() &&
                    appSource.getId() != AppSyncSourceManager.SYNC_FLAG_PHOTOS) {
                sourcesAdded = false;
                break;
            }
            vfm = new VerticalFieldManager();
            hfm = new HorizontalFieldManager();

            CheckboxField enableSyncSourceField = (CheckboxField)
                                             enabledSourceFieldList.elementAt(i);
            ColoredLabelField enableSyncSourceLabel = (ColoredLabelField)
                                             enabledSourceLabelList.elementAt(i);
            ColoredBasicEditField syncSourceUriField = (ColoredBasicEditField)
                                             sourceUriFieldList.elementAt(i);
            ColoredLabelField disabledSyncSourceLabel = (ColoredLabelField)
                                             disabledSourceLabelList.elementAt(i);

            hfm.add(enableSyncSourceField);
            vfm.add(enableSyncSourceLabel);

            if (customization.isSourceUriVisible()) {
                vfm.add(syncSourceUriField);
            }
            
            if (   appSource.getId() == AppSyncSourceManager.SYNC_FLAG_CONTACTS
                && customization.syncCompanyDirectory()) {

                HorizontalFieldManager hfm2 = new HorizontalFieldManager();
                hfm2.add(cdEnableField);
                hfm2.add(cdLabelField);
                vfm.add(hfm2);
            } else if (appSource.getId() == AppSyncSourceManager.SYNC_FLAG_PHOTOS) {

                // We need a listener so that on enable/disable changes we
                // enable/disable the browse button
                PicturesCheckBoxListener checkBoxListener =
                                    new PicturesCheckBoxListener(enableSyncSourceField);
                

                HorizontalFieldManager hfm2 = new HorizontalFieldManager();
                picturesLocationChoiceField = new PicturesLocationChoiceField();

                String locationLabel = StaticDataHelper.getLanguage("conf_pictures_location_label");
                picturesLocationChoiceField.setLabel(locationLabel);
                reinitializePicturesDirOptions();

                // Init the one way label
                picturesOneWayField = new LabelField(StaticDataHelper.getLanguage("pictures_one_way_info"));
                // Init the sync old field
                boolean syncOld = Configuration.getPicturesSyncOld();
                picturesSyncOld = new CheckboxField(StaticDataHelper.getLanguage("pictures_sync_old"), syncOld);
                picturesSyncOld.setChangeListener(cl);
                if (!appSource.isEnabled()) {
                    picturesSyncOld.setEditable(false);
                }

                hfm2.add(picturesLocationChoiceField);
                vfm.add(hfm2);
                vfm.add(photosDirLabelField);

                vfm.add(picturesSyncOld);
            }

            // If the source does not work, we show an error and disable the
            // proper controls
            if (!appSource.isWorking()) {
                vfm.add(disabledSyncSourceLabel);
                syncSourceUriField.setEditable(false);
                enableSyncSourceField.setEditable(false);
            }
            hfm.add(vfm);
            sourcesFieldManager.add(hfm);
        }

        Log.trace(TAG_LOG, "Almost done");

        if(syncDirectionFieldManager != null && picturesOneWayField != null) {
            syncDirectionFieldManager.add(picturesOneWayField);
        }

        insert(sourcesFieldManager, sourcesIndex);
        Log.trace(TAG_LOG, "Really done");

        return sourcesAdded;
    }

    private void reinitializePicturesDirOptions() {
        Log.trace(TAG_LOG, "reinitialize pictures dir options " + Configuration.getPicturesDir());

        // This is the case where the picture sync source is not active
        if (picturesLocationChoiceField == null) {
            return;
        }


        if (BlackberryUtils.isSDCardInserted()) {
            picturesDirSelectionName = PICTURES_DIR_LABEL;
        } else {
            picturesDirSelectionName = PICTURES_DIR_LABEL_NO_SDCARD;
        }

        picturesLocationChoiceField.setChoices(picturesDirSelectionName);

        boolean isSDcardInserted = BlackberryUtils.isSDCardInserted();
        boolean isSDCardSelected = Configuration.getPicturesDir().equals(
                                               BlackberryUtils.getSDCardPicturesdir());
        boolean isInternalMemorySelected = Configuration.getPicturesDir().equals(
                                               BlackberryUtils.getInternalDeviceStorePicturesdir());

        // This is the current directory without the protocol specifier
        // (file://)
        if (picturesCurrentDir == null) {
            picturesCurrentDir = Configuration.getPicturesDir().substring(7);
        }

        if (Configuration.getPicturesDir() != null) {
            if (isSDcardInserted) {
                //SDCARD is inserted
                if (isInternalMemorySelected) {
                    picturesLocationChoiceField.setSelectedIndex(0);
                    // Hide default values
                    photosDirLabelField.setText("");
                } else if (isSDCardSelected) {
                    picturesLocationChoiceField.setSelectedIndex(1);
                    // Hide default values
                    photosDirLabelField.setText("");
                } else {
                    picturesLocationChoiceField.setSelectedIndex(2);
                    photosDirLabelField.setText(picturesCurrentDir);
                }
            } else {
                //SDCARD is NOT inserted
                if (isInternalMemorySelected) {
                    picturesLocationChoiceField.setSelectedIndex(0);
                    // Hide default values
                    photosDirLabelField.setText("");
                } else {
                    picturesLocationChoiceField.setSelectedIndex(1);
                    picturesCurrentDir = Configuration.getPicturesDir().substring(7);
                    photosDirLabelField.setText(picturesCurrentDir);
                }
            }
        }
    }
    
    private void loadValues() {

        for(int i=0;i<enabledSourceFieldList.size();++i) {
            CheckboxField enableSyncSourceField = (CheckboxField)
                                             enabledSourceFieldList.elementAt(i);
            ColoredBasicEditField uriField = (ColoredBasicEditField)
                                             sourceUriFieldList.elementAt(i);

            AppSyncSource appSource = (AppSyncSource)sourceMap.elementAt(i);
            int id = appSource.getId();
            boolean enabled = Configuration.isSourceEnabled(id);
            enableSyncSourceField.setChecked(enabled);
            String sourceUri = Configuration.getSourceUri(id);
            uriField.setText(sourceUri);

            if (appSource.getId() == AppSyncSourceManager.SYNC_FLAG_CONTACTS &&
                customization.syncCompanyDirectory()) {

                cdEnableField.setChecked(sourceUri.endsWith("-shared"));
            }
        }

        syncModeField.setSelectedIndex(Configuration.getSyncMode());
        int idx = getScheduledIntervalIndexFromMins(Configuration.getPollingInterval());
        pollingPIMIntervalField.setSelectedIndex(idx);
        syncTypeField.setSelectedIndex(Configuration.getSyncType());
        rangePastField.setSelectedIndex(Configuration.getRangePast() - 1);
        rangeFutureField.setSelectedIndex(Configuration.getRangeFuture() - 1);
        logLevelField.setSelectedIndex(Configuration.getLogLevel());
        if (customization.invitesFiltering()) {
            blockInvitesField.setChecked(Configuration.getBlockIncomingInvites());
        }

        if(customization.isC2SPushEnabled()){
            c2sPushField.setChecked(configuration.getC2SPush());
        }
        invalidate();
    }

    /**
     * Adds "Save" menu item to the menu on Configuration screen
     * 
     * @param Menu :
     *            menu to which save menuitem is to be added
     * @param int :
     *            instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance) {
        if (hasChanges) {
            menu.add(saveMenu);
        }
        menu.add(backMenu);

        // We need a valid email configuration to send logs via email
        if (customization.getInstance().enableFileLogging()) {
            if (customization.sendLogViaEmailEnabled() && BlackberryUtils.isBISEmailAvailable()) {
                // Since we had to re-implement the getContent method, we need
                // to make sure the log is in a file
                if (Log.getAppender() instanceof FileAppender) {
                    Session session = Session.getDefaultInstance();
                    if (session != null) {
                        menu.add(sendLogMenu);
                    }
                }
            }
        }
    }

    /**
     * Invoked on user hitting the escape button. It prompts the user with a
     * dialog, if he wants to save/discard/cancel the changes in configuration
     * 
     * @param void.
     * @return void.
     */
    public boolean onClose() {
        if (hasChanges) {
            final int response = Dialog.ask(Dialog.D_SAVE, StaticDataHelper
                    .getLanguage("dialog_save"));
            if (Dialog.SAVE == response) {
                hasChanges = false;
                saveConfiguration();
                loadValues();
            } else if (Dialog.DISCARD == response) {
                hasChanges = false;
                loadValues();
            } else if (Dialog.CANCEL == response) {
                return false;
            }
        }
        controller.getScreenDisplayController().hideConfigScreen();
        return true;
    }


    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen. It saves changes on the configuration into the
     * persistant store
     * 
     */
    public boolean saveConfiguration() {
        Log.trace(TAG_LOG, "saveConfiguration");

        boolean enablePollingPIM = false;
        int pollingPIMInterval = customization.getPollingPimIntervalChoices()[0];
        int rangePast = 1;
        int rangeFuture = 6;
        int syncType = 0;
        int syncMode = 0;
        int logLevel = 0;
        boolean blockInvites = false;
        boolean c2sPushEnabled = false;

        Vector sourceUri = new Vector();
        Vector sourceEnabled = new Vector();
        for(int i=0;i<sourceUriFieldList.size();++i) {
            ColoredBasicEditField uriField = (ColoredBasicEditField)sourceUriFieldList.elementAt(i);
            String uri = uriField.getText().trim();
            CheckboxField enabledField = (CheckboxField)enabledSourceFieldList.elementAt(i);
            Boolean enabled = new Boolean(enabledField.getChecked());

            // Add this item to the vectors
            sourceUri.addElement(uri);
            sourceEnabled.addElement(enabled);
        }

        syncMode = syncModeField.getSelectedIndex();
        pollingPIMInterval = getScheduledMinsFromIndex(pollingPIMIntervalField.getSelectedIndex());

        if(customization.isC2SPushEnabled()){
            c2sPushEnabled = c2sPushField.getChecked();
        }

        rangePast = rangePastField.getSelectedValue();
        rangeFuture = rangeFutureField.getSelectedValue();

        syncType = syncTypeField.getSelectedIndex();

        logLevel = logLevelField.getSelectedIndex();

        if (customization.invitesFiltering()) {
            blockInvites = blockInvitesField.getChecked();
        }

        if (sourceEnabledWithoutUri(sourceEnabled, sourceUri)) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-incomplete"));
            return false;
        }

        // Okay, save the configuration
        // TODO: save only the changed fields to optimize the process
        for(int i=0;i<sourceUri.size();++i) {
            Boolean enabled = (Boolean)sourceEnabled.elementAt(i);
            String  uri     = (String)sourceUri.elementAt(i);

            AppSyncSource appSource = (AppSyncSource) sourceMap.elementAt(i);
            int id = appSource.getId();
            Configuration.setSourceUri(id, uri);
            Configuration.setSourceEnabled(id, enabled.booleanValue());
        }

        Configuration.setSyncType(syncType);
        Configuration.setLogLevel(logLevel);

        Configuration.setPollingInterval(pollingPIMInterval);
        Configuration.setSyncMode(syncMode);

        if(customization.isC2SPushEnabled()){
            configuration.setC2SPush(c2sPushEnabled);
        }

        Configuration.setRangePast(rangePast);
        Configuration.setRangeFuture(rangeFuture);

        if (picturesSyncOld != null) {
            Configuration.setPicturesSyncOld(picturesSyncOld.getChecked());
        }

        if (customization.invitesFiltering()) {
            Configuration.setBlockIncomingInvites(blockInvites);
        }

        if (picturesLocationChoiceField != null) {
            String picturesDir = "file://" + picturesCurrentDir;
            Configuration.setPicturesDir(picturesDir);
        }

        loadValues();

        if (Configuration.save(controller) != Configuration.CONF_OK) {
            controller.getDialogController().showMessage(
                    StaticDataHelper.getLanguage("message_config-error") + ": "
                            + StaticDataHelper.getLanguage("message_config-error-save"));
            return false;
        }

        return true;
    }

    protected void onFocus(int arg0) {
        enableFileBrowser = false;
        reinitializePicturesDirOptions();
        super.onFocus(arg0);
        enableFileBrowser = true;
    }

    private boolean sourceEnabledWithoutUri(Vector sourceEnabled, Vector sourceUri) {
        if (sourceEnabled.size() != sourceUri.size()) {
            Log.error(TAG_LOG, "Internal error: sources enabled/uri cannot have different size");
            return true;
        }
        for (int i=0;i<sourceEnabled.size();++i) {
            Boolean enabled = (Boolean)sourceEnabled.elementAt(i);
            String  uri     = (String) sourceUri.elementAt(i);
            if (enabled.booleanValue() && "".equals(uri)) {
                return true;
            }
        }
        return false;
    }

    private boolean isAtLeastOneUriNull(Vector sourceUri) {
        for (int i=0;i<sourceUri.size();++i) {
            String uri = (String)sourceUri.elementAt(i);
            if (uri == null) {
                return true;
            }
        }
        return false;
    }

    private int[] getScheduledIntervals() {
        return customization.getPollingPimIntervalChoices();
    }

    private String[] getIntervalChoices(int[] minValues) {
        int size = minValues.length;

        String minutes = StaticDataHelper.getLanguage("conf-mins");
        String hour = StaticDataHelper.getLanguage("conf-hour");
        String hours = StaticDataHelper.getLanguage("conf-hours");
        String day = StaticDataHelper.getLanguage("conf-day");
        String days = StaticDataHelper.getLanguage("conf-days");

        String[] intervalChoices = new String[size];

        for (int i = 0; i < size; i++) {
            int hoursValue = minValues[i]/60;

            if (hoursValue == 0) {
                intervalChoices[i] = minValues[i] + " " + minutes;
            } else if (hoursValue == 1) {
                intervalChoices[i] = hoursValue + " " + hour;
            } else if (hoursValue < 24) {
                intervalChoices[i] = hoursValue + " " + hours;
            } else if (hoursValue == 24){
                intervalChoices[i] = hoursValue/24 + " " + day;
            } else if (hoursValue > 24){
                intervalChoices[i] = hoursValue/24 + " " + day;
            }
        }

        return intervalChoices;
    }


    private int getScheduledIntervalIndexFromMins(int mins) {
        int[]    ints = getScheduledIntervals();
        // Backward search in the array
        for(int i=0;i<ints.length;++i) {
            int val = ints[i];
            if (val == mins) {
                return i;
            }
        }
        Log.error(TAG_LOG, "Invalid scheduling interval value " + mins);
        // We should never get here, but if we do, we just choose the first
        // value in the list to avoid exceptions
        return 0;
    }

    private int getScheduledMinsFromIndex(int idx) {
        int[]    ints = getScheduledIntervals();
        try {
            return ints[idx];
        } catch (ArrayIndexOutOfBoundsException e) {
            Log.error(TAG_LOG, "Invalid scheduling interval index " + idx);
            return 15;
        }
    }

    private String[] getSyncModes() {
        String[] syncModes = null;

        String manualOption = StaticDataHelper.getLanguage("conf_sync_mode-manual");
        String scheduledOption = StaticDataHelper.getLanguage("conf_sync_mode-scheduled");

        Vector options = new Vector(2);

        options.addElement(manualOption);
        options.addElement(scheduledOption);

        boolean isPollEnabled =
                !(customization.disablePollOn8830() &&
                DeviceInfo.getDeviceName().equals("8830"));

        if (!isPollEnabled) {
            options.removeElement(scheduledOption);
        }

        options.trimToSize();

        syncModes = new String[options.size()];

        for (int i = 0; i < options.size(); i++) {
            syncModes[i] = (String) options.elementAt(i);
        }

        return syncModes;
    }
    
    private String[] getSyncTypes() {
        String types[];
        types = new String[] { StaticDataHelper.getLanguage("conf_sync-type-two-way"),
                StaticDataHelper.getLanguage("conf_sync-type-one-way-to-server"),
                StaticDataHelper.getLanguage("conf_sync-type-one-way-to-client"), };
        return types;
    }

    private void sendLog() {
        Log.debug(TAG_LOG, "Starting thread to send log");
        LogSenderThread lst = new LogSenderThread();
        lst.setPriority(Thread.MIN_PRIORITY);
        lst.start();
    }

    private class LogSenderThread extends Thread {

        public void run() {
            Log.trace(TAG_LOG, "LogSenderThread.run");

            // Prepare an email to send
            FileAppender appender = null;
            try {
                // Give a feedback to the user
                DialogController dc = controller.getDialogController();
                StringBuffer alertMsg = new StringBuffer(StaticDataHelper.getLanguage("sendlog_sending_to"));
                alertMsg.append(" ").append(Customization.getInstance().getLogEmailAddress());
                dc.showMessage(alertMsg.toString(), 3000);

                Session session = Session.getDefaultInstance();
                if (session == null) {
                    Log.error(TAG_LOG, "Cannot send log because session is null");
                    return;
                }

                Log.trace(TAG_LOG, "Getting transport");
                Transport transport = session.getTransport();

                // Preparing the message
                Store store = Session.getDefaultInstance().getStore();
                Folder[] folders = store.list(Folder.SENT);
                Folder sentfolder = folders[0];
                Message msg = new Message(sentfolder);
                // do not ask for the return recipt
                msg.setFlag(Message.Flag.REQUEST_READ_ACK, false);

                // Prepare the address
                Address addr = new Address(customization.getLogEmailAddress(),
                        customization.getLogEmailName());
                msg.addRecipient(Message.RecipientType.TO, addr);
                msg.setSubject("BlackBerry Sync Client log");

                Multipart mp = new Multipart();
                String mimeType = TextBodyPart.getTextPlainContentType();
                Log.trace(TAG_LOG, "Getting log content");

                //LogContent logContent = Log.getCurrentLogContent();
                BodyPart contentPart;

                // The getContent method is too slow on BB, we need to send the
                // log in a different way
                appender = (FileAppender)Log.getAppender();
                // Disable the log temporary
                appender.closeLogFile();

                // Read the log content
                InputStream is            = null;
                byte        fileContent[] = null;
                String fileName = null;
                FileAdapter file = null;
                try {
                    // Read the log content
                    String logFilePath = Customization.getInstance().getLogFileDirectory();
                    String logFileName = Customization.getInstance().getLogFileName();
                    if (logFilePath.endsWith("/")) {
                        fileName = logFilePath + logFileName;
                    } else {
                        fileName = logFilePath + "/" + logFileName;
                    }
                    file = new FileAdapter(fileName);
                    is = file.openInputStream();
                    fileContent = new byte[(int)file.getSize()];
                    is.read(fileContent);
                } catch (IOException ioe) {
                    Log.error(TAG_LOG, "Cannot read log content file: " + ioe.toString());
                } finally {
                    if (is != null) {
                        try {
                            is.close();
                        } catch (IOException ioe) {
                        }
                        is = null;
                    }
                    if (file != null) {
                        try {
                            file.close();
                        } catch (IOException ioe) {
                        }
                        file = null;
                    }
                }

                SupportedAttachmentPart cp = new SupportedAttachmentPart(mp);
                cp.setContentType(mimeType);
                if (fileContent != null) {
                    cp.setContent(fileContent);
                }
                cp.setFilename(Customization.getInstance().getLogFileName());
                contentPart = cp;
                mp.addBodyPart(contentPart);

                // If we have the rotated log, we shall attach that as well
                if (fileName != null) {
                    try {
                        file = new FileAdapter(fileName + ".sav.txt", true);
                        is = file.openInputStream();
                        fileContent = new byte[(int)file.getSize()];
                        is.read(fileContent);
                        cp = new SupportedAttachmentPart(mp);
                        cp.setContentType(mimeType);
                        if (fileContent != null) {
                            cp.setContent(fileContent);
                        }
                        cp.setFilename(Customization.getInstance().getLogFileName() + ".sav.txt");
                        contentPart = cp;
                        mp.addBodyPart(contentPart);
                    } catch (IOException ioe) {
                        // If we cannot attach it, we just ignore it
                        Log.error(TAG_LOG, "Cannot read log content file: " + ioe.toString());
                    } finally {
                        if (is != null) {
                            try {
                                is.close();
                            } catch (IOException ioe) {
                            }
                            is = null;
                        }
                        if (file != null) {
                            try {
                                file.close();
                            } catch (IOException ioe) {
                            }
                            file = null;
                        }
                    }
                }

                // Reinit the log file
                appender.initLogFile();
                appender = null;

                msg.setContent(mp);

                // Finally send the message
                Log.trace(TAG_LOG, "Sending message");
                transport.send(msg);
            } catch (Exception e) {
                Log.error(TAG_LOG, "Cannot send the log as attachment " + e.toString());
            } finally {
                // Reenable log if it has not been enabled yet
                if (appender != null) {
                    appender.initLogFile();
                }
            }
        }
    }

    private class PicturesFileBrowserListener extends RimFileBrowserListener {

        private int previousIndex;
        private String previousDir;
        public PicturesFileBrowserListener(int previousIndex, String previousDir) {
            this.previousIndex = previousIndex;
            this.previousDir   = previousDir;
        }

        public boolean chosen(Vector entries) {
            // We expect a single selection here
            if (entries.size() != 1) {
                return false;
            }
            String picturesDir = (String)entries.elementAt(0);
            // Set the new directory in the Configuration
            Log.debug(TAG_LOG, "Setting pictures dir " + "file://" + picturesDir);

            // Update the label with the current value
            photosDirLabelField.setText(picturesDir);
            picturesCurrentDir = picturesDir;
            return true;
        }

        public boolean cancelled() {
            // Restore the previous values
            String internalDir = BlackberryUtils.getInternalDeviceStorePicturesdir().substring(7);
            boolean done = false;
            if (internalDir.equals(previousDir)) {
                photosDirLabelField.setText("");
                done = true;
            }
            if (BlackberryUtils.isSDCardInserted()) {
                String sdDir = BlackberryUtils.getSDCardPicturesdir().substring(7);
                if (sdDir.equals(previousDir)) {
                    photosDirLabelField.setText("");
                    done = true;
                }
            }
            if (!done) {
                photosDirLabelField.setText(previousDir);
            }
            picturesLocationChoiceField.setSelectedIndex(previousIndex);
            picturesCurrentDir = previousDir;
            return true;
        }
    }

    private class PicturesLocationChoiceField extends ObjectChoiceField {

        private int previousIndex;

        public PicturesLocationChoiceField() {
            super();
        }

        public int getPreviousIndex() {
            return previousIndex;
        }

        protected boolean trackwheelClick(int status, int time) {
            // Mark the configuration as having changes at this point. This is
            // not optimal, but it is conservative
            hasChanges = true;
            // Remember the current index
            previousIndex = getSelectedIndex();
            return super.trackwheelClick(status, time);
        }

        protected void setSelectedIndex(int index, int context) {
            super.setSelectedIndex(index, context);
            if (context == 2) {
                // This is the case where the user clicked on the item to select
                // it
                switch(index) {
                    case 0:
                        picturesCurrentDir = BlackberryUtils.getInternalDeviceStorePicturesdir().substring(7);
                        photosDirLabelField.setText("");
                        break;
                    case 1:
                        if (BlackberryUtils.isSDCardInserted()) {
                            picturesCurrentDir = BlackberryUtils.getSDCardPicturesdir().substring(7);
                            photosDirLabelField.setText("");
                        } else {
                            showFileBrowser();
                        }
                        break;
                    case 2:
                        showFileBrowser();
                        break;
                    default:
                        break;
                }
            }
        }

        private void showFileBrowser() {
            Vector picturesExtensions = new Vector();
            picturesExtensions.addElement("jpg");
            picturesExtensions.addElement("jpeg");
            picturesExtensions.addElement("gif");
            picturesExtensions.addElement("png");
            int previousIndex = picturesLocationChoiceField.getPreviousIndex();
            PicturesFileBrowserListener listener = new PicturesFileBrowserListener(previousIndex, picturesCurrentDir);
            RimFileBrowser fileBrowser = new RimFileBrowser(controller, RimFileBrowser.ALL_MODE,
                    RimFileBrowser.SELECT_DIR_MODE, picturesExtensions);
            // Set the initial path in the browser. Note that this
            // path does not contain the "file://" prefix
            fileBrowser.setPath(picturesCurrentDir);
            fileBrowser.setListener(listener);
            fileBrowser.setCaseSensitive(false);
            try {
                controller.getScreenDisplayController().showScreen(fileBrowser);
            } catch (Exception e) {
                Log.error("Error while showing file browser " + e.toString());
            }
        }
    }

    class PicturesCheckBoxListener implements FieldChangeListener {

        private CheckboxField  field;

        public PicturesCheckBoxListener(CheckboxField field) {
            this.field = field;
        }

        public void fieldChanged(Field valueField, int context) {

            if (field.getChecked()) {
                // Pictures sync is enabled, we shall enable the browse button
                picturesLocationChoiceField.setEditable(true);
                picturesSyncOld.setEditable(true);
            } else {
                // Pictures sync is disabled, we shall disable the browse button
                picturesLocationChoiceField.setEditable(false);
                picturesSyncOld.setEditable(false);
            }
        }
    }


    /**
     * This class is only used if CUSTOM_COMPANY_DIRECTORY is enabled.
     * 
     */
    class CDListener implements FieldChangeListener {

        private ChangeListener listener;
        private BasicEditField  field;

        public CDListener(ChangeListener cl, BasicEditField field) {
            listener   = cl;
            this.field = field;
        }

        public void fieldChanged(Field cdField, int context) {

            if (cdField instanceof CheckboxField) {
                CheckboxField cd = (CheckboxField) cdField;

                String curr = field.getText();
                if (cd.getChecked()) {
                    if (curr.endsWith("-shared")) {
                        // Do nothing
                    } else {
                        curr = curr + "-shared";
                    }
                } else {
                    if (curr.endsWith("-shared")) {
                        curr = curr.substring(0, curr.length() - "-shared".length());
                        controller.getDialogController().showMessage(
                                StaticDataHelper.getLanguage("custom_message-remove-cd"));
                    } else {
                        // Do nothing
                    }
                }
                field.setText(curr);
            }
            listener.fieldChanged(field, context);
        }
    }

    class FixedWidthButtonField extends ButtonField {
        private int width;
 
        FixedWidthButtonField( String label, long style, int width){
            super( label, style);
            this.width = width;
        }
 
        public int getPreferredWidth(){
            return width;
        }
    }

    class ChangeListener implements FieldChangeListener {

        public void fieldChanged(Field field, int context) {
            if (context != FieldChangeListener.PROGRAMMATIC) {
                hasChanges = true;
            }
            if (field == syncModeField) {
                // We may need to hide or show the scheduled interval
                if (syncModeField.getSelectedIndex() == Configuration.SYNC_MODE_SCHEDULED) {
                    if (intervalVfm.getFieldCount() == 1) {
                        intervalVfm.add(intervalHfm);
                    }
                } else {
                    if (intervalVfm.getFieldCount() == 2) {
                        intervalVfm.delete(intervalHfm);
                    }
                }
            }
        }
    }
}
