/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.DrawStyle;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.component.GaugeField;

import com.funambol.blackberry.AppSyncSource;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.util.Log;
import com.funambol.util.StaticDataHelper;

/**
 * This class represents a sync source in the form of a progress bar. The
 * representation is a Field and therefore it can be contained in any screen or
 * field. Icons are not displayed and each received message moves the progress
 * bar forward. The current implementation assumes 3 messages are generated from
 * the controller, but it is easy to change the policy and providing a more
 * general schema for moving the progress bar.
 */
public class RimProgressSyncSource extends GaugeField implements UISyncSource {

    protected String          status                = null;
    protected AppSyncSource   source                = null;
    protected Font            font                  = null;
    protected UISyncSourceContainer container       = null;
    protected int             perc                  = 0;
    protected String          statusConnecting      = null;
    protected boolean         inProgress            = false;


    public RimProgressSyncSource(String status) {
        
        super(status, 0, 100, 0, GaugeField.NO_TEXT);
        this.status = status;
        this.font = RimHelper.getDefaultFont();

        // We recognize the status as progress indicators
        statusConnecting  = StaticDataHelper.getLanguage("status_connecting");
    }

    public void setTitle(String value) {
    }

    public void setFont(Font font) {
        this.font = font;
    }

    public void setContainer(UISyncSourceContainer container) {
        this.container = container;
    }

    public void setTitleEnabledColor(int color) {
    }

    public void setTitleDisabledColor(int color) {
    }

    public void setEnabled(boolean flag) {
    }

    public boolean isDisabled() {
        return false;
    }

    public void setStatusIcon(Bitmap image) {
    }

    public void setStatusSelectedIcon(Bitmap image) {
    }

    public void redraw() {
        invalidate();
    }

    public void reset() {
        inProgress = false;
        super.reset("", 0, 100, 0);
        perc = 0;
        status = null;
    }

    public void setStatusString(String value) {
        // Depending on the label, we shall update the percentage
       
        if (statusConnecting != null && statusConnecting.equals(value)) {
            inProgress = true;
            status = value;
        } else if (inProgress) {
        }

        if (inProgress) {
            // The sync takes about three messages
            perc += 33;
            if (perc > 100) {
                perc = 100;
            }
            setValue(perc);
            redraw();
        }
    }

    public void setSource(AppSyncSource source) {
        this.source = source;
    }

    public void setSelected(boolean selected) {
    }

    public boolean isSelected() {
        return true;
    }

    public AppSyncSource getSource() {
        return source;
    }

    public int getPreferredWidth() {
        return getScreen().getVisibleWidth() - 5;
    }

    public int getPreferredHeight() {
        return font.getHeight() + 8;
    }

    public void setKeepBgWhenDisabled(boolean value) {
    }

    public void setDisabledStatusColor(int color) {
    }

    public void setEnabledStatusColor(int color) {
    }

    protected void paint(Graphics g) {
        // paint the progress bar
        super.paint(g);
        // Now paint our custom text
        if (status != null) {
            int currentColor = g.getColor();
            if (currentColor != Color.BLACK) {
                g.setColor(Color.BLACK);
            } else {
                g.setColor(Color.YELLOW);
            }
            // We use a slightly smaller font so that the look is better
            int fontHeight = (font.getHeight() * 75) / 100;
            g.setFont(font.derive(Font.PLAIN, fontHeight));
            g.drawText(status, 5, 3, Graphics.HCENTER | Graphics.TOP);
        }
    }
}

