/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.util;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import net.rim.device.api.system.ApplicationDescriptor;
import net.rim.device.api.system.ApplicationManager;
import net.rim.device.api.ui.Font;
import net.rim.device.api.system.DeviceInfo;

import com.funambol.blackberry.BlackBerryLocalization;
import com.funambol.blackberry.BBVersion;
import com.funambol.blackberry.Constants;
import com.funambol.util.StringUtil;

/**
 * Supports general using, language dictionary access, logging. Statics data and
 * static methods
 */
public class StaticDataHelper {

    private static final String LOG_TAG = "StaticDataHelper";

    public static final int     TITLE_FONT_SIZE                = 16;
    public static final int     TITLE_FONT_STYLE               = Font.BOLD;
    public static final int     TITLE_PADDING                  = 5;
    
    public static final int     SYNC_TYPE_TWO_WAY              = 0;
    public static final int     SYNC_TYPE_ONE_WAY_TO_SERVER    = 1;
    public static final int     SYNC_TYPE_ONE_WAY_TO_CLIENT    = 2;
    public static final int     SYNC_TYPE_REFRESH_TO_SERVER    = 3;
    public static final int     SYNC_TYPE_REFRESH_TO_CLIENT    = 4;

    private static String operatingSystem = null;

    private static BlackBerryLocalization localization;

    // ----------------------------------------------------------------- Public
    // methods

    public static String getOS() {
        return operatingSystem;
    }

    /**
     * check OS version
     * @param handled OS
     * @param high x in x.y.z
     * @param mid  y in x.y.z
     * @param low  z in x.y.z
     * @return true if OS >= high.mid.low
     */
    public static boolean checkOS(String os, int high, int mid, int low)
    {
        if (os == null || os.length() == 0) {
            os = getOS();
        }

        if (os == null || !(os.length() > 3)) {
            return false;
        }

        try {

            final int major = Integer.valueOf(os.substring(0, 1)).intValue();
            final int middle = Integer.valueOf(os.substring(2, 3)).intValue();

            int minor = 0;

            if (os.length() > 4) {
                minor = Integer.valueOf(os.substring(4, 5)).intValue();
            }

            boolean gte = (
                    (major > high)
                    ||
                    (major == high && middle > mid)
                    ||
                    (major == high && middle == mid && minor >= low)
                    );

            return gte;
        } catch (Exception e) {
            Log.error(LOG_TAG, "Cannot check OS version: " + os, e);
            return false;
        }
    }

    public static String[] getOSVersion() {

        Log.trace(LOG_TAG, "getOSVersion");

        String os = getOS();
        Log.trace(LOG_TAG, "OS version string is: " + os);
        if (os == null) {
            return new String[0];
        }

        String[] tokens = StringUtil.split(os, ".");
        for(int i=0;i<tokens.length;++i) {
            Log.trace(LOG_TAG, "Ver["+i+"]=" + tokens[i]);
        }
        return tokens;
    }

    /**
     * Returns true iff the device has a dedicated key to open the menu. The
     * implementation is based on the device model and it works for the models
     * currently supported by our APIs (>= 4.2.1)
     */
    public static boolean hasDedicatedMenuKey() {
        String deviceName = DeviceInfo.getDeviceName();
        if (deviceName.startsWith("7130")  ||
            deviceName.startsWith("8700")  ||
            deviceName.startsWith("8703")  ||
            deviceName.startsWith("8705")  ||
            deviceName.startsWith("8707")) {

            return false;
        } else {
            return true;
        }
    }
    
    /**
     * Make a String by value of <i>tag</i>.
     * 
     * @param key
     *            key to find
     * @return key value
     */
    public static String getLanguage(String key) {
        return localization.getLanguage(key);
    }

    public static void init(Class c) throws Exception {
        loadVersion();
        localization = new BlackBerryLocalization();
        setOS();
    }

    /**
     * Must be called on initialization
     */
    public static void loadVersion() {
        String version = BBVersion.VERSION;
        Constants.VER = version;
        Log.info(LOG_TAG, "Version: " + version);
    }


    private static void setOS() {
        final ApplicationDescriptor[] appDes = ApplicationManager.getApplicationManager()
                .getVisibleApplications();
        String version = "";
        for (int i = appDes.length - 1; i >= 0; --i) {
            if (appDes[i].getModuleName().equals("net_rim_bb_ribbon_app")) {
                version = appDes[i].getVersion();
                break;
            }
        }
        operatingSystem = version;
    }

}
