/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;

import javax.microedition.pim.PIMException;
import javax.microedition.pim.PIMItem;
import javax.microedition.pim.PIMList;

import net.rim.blackberry.api.pdap.PIMListListener;
import net.rim.blackberry.api.pdap.BlackBerryPIMList;
import net.rim.device.api.system.ControlledAccessException;
import net.rim.device.api.system.PersistentObject;
import net.rim.device.api.system.PersistentStore;
import net.rim.device.api.util.Persistable;
import net.rim.blackberry.api.pdap.BlackBerryContact;

import junit.framework.*;

import com.funambol.blackberry.controller.SynchronizationController;
import com.funambol.blackberry.helper.PIMItemHelper;
import com.funambol.syncml.protocol.SyncFilter;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.protocol.SyncMLStatus;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncException;
import com.funambol.syncml.spds.SyncItem;
import com.funambol.syncml.spds.SyncSource;

import com.funambol.util.Log;

public class BBPIMSyncSourceTest extends TestCase
{
    private static int id = 0;
    private int numItemsRequests = 0;

    public BBPIMSyncSourceTest(String name) {
        super(name);
    }

    public void setUp() {
        numItemsRequests = 0;
    }

    public void testSendingSpeedSlowSync() throws Exception {

        SourceConfig config = new SourceConfig();
        TestPIMList list = new TestPIMList();
        TestHelper helper = new TestHelper(list);

        BBPIMSyncSource source = new BBPIMSyncSource(config, list, helper, 1, 2);

        // Populate the db with 1000 items
        for(int i=0;i<1000;++i) {
            TestItem item = new TestItem(list);
            list.addItem(item);
        }
        // Now we can simulate a slow sync
        numItemsRequests = 0;
        source.beginSync(SyncML.ALERT_CODE_SLOW);
        int numItems = 0;
        SyncItem item = source.getNextItem();
        while (item != null) {
            item = source.getNextItem();
            numItems++;
        }
        // We expect only one iteration over the list of items
        // And we assume that anytime the enumeration is asked, then
        // its items are iterated
        System.out.println("numItemsRequests = " + numItemsRequests);
        System.out.println("numItems = " + numItems);
        assertTrue(numItems == 1000);
        assertTrue(numItemsRequests == 1);
    }

    public void testReceivingSpeedSlowSync() throws Exception {

        SourceConfig config = new SourceConfig();
        TestPIMList list = new TestPIMList();
        TestHelper helper = new TestHelper(list);

        BBPIMSyncSource source = new BBPIMSyncSource(config, list, helper, 1, 2);
        source.beginSync(SyncML.ALERT_CODE_SLOW);
        // Receive one thousands add
        for(int i=0;i<1000;++i) {
            byte content[] = ("Content " + i).getBytes();
            SyncItem sItem = new SyncItem("id"+i, "text/plain", SyncItem.STATE_NEW,
                                          null, content);
            source.addItem(sItem);
        }
        System.out.println("numItemsRequests = " + numItemsRequests);
        assertTrue(numItemsRequests == 1);

        Enumeration elements = list.items();
        int numItems = 0;
        while(elements.hasMoreElements()) {
            TestItem pimItem = (TestItem)elements.nextElement();
            numItems++;
        }
        System.out.println("numItems = " + numItems);
        assertTrue(numItems == 1000);
    }

    public void testSpeedSlowSync() throws Exception {

        SourceConfig config = new SourceConfig();
        TestPIMList list = new TestPIMList();
        TestHelper helper = new TestHelper(list);

        BBPIMSyncSource source = new BBPIMSyncSource(config, list, helper, 1, 2);

        // Populate the db with 1000 items
        Vector keys = new Vector();
        for(int i=0;i<1000;++i) {
            TestItem item = new TestItem(list);
            list.addItem(item);
            keys.addElement(helper.getUID(item));
        }
        // Now we can simulate the sending phase
        numItemsRequests = 0;
        source.beginSync(SyncML.ALERT_CODE_SLOW);
        int numItems = 0;
        SyncItem item = source.getNextItem();
        while (item != null) {
            item = source.getNextItem();
            numItems++;
        }
        // We expect only one iteration over the list of items
        // And we assume that anytime the enumeration is asked, then
        // its items are iterated
        System.out.println("numItemsRequests = " + numItemsRequests);
        System.out.println("numItems = " + numItems);
        assertTrue(numItems == 1000);
        assertTrue(numItemsRequests == 1);

        // Now receive the items. Simulate a single message with all the changes
        source.prepareModifications(keys);
        System.out.println("numItemsRequests = " + numItemsRequests);
        assertTrue(numItemsRequests == 2);

        // Receive one thousands update
        for(int i=0;i<1000;++i) {
            byte content[] = ("Content " + i).getBytes();
            SyncItem sItem = new SyncItem((String)keys.elementAt(i),
                                          "text/plain", SyncItem.STATE_UPDATED,
                                          null, content);
            source.updateItem(sItem);
        }
        System.out.println("numItemsRequests = " + numItemsRequests);
        assertTrue(numItemsRequests == 2);

        Enumeration elements = list.items();
        numItems = 0;
        while(elements.hasMoreElements()) {
            TestItem pimItem = (TestItem)elements.nextElement();
            numItems++;
        }
        System.out.println("numItems = " + numItems);
        assertTrue(numItems == 1000);
    }

    public void testSpeedSlowSync2() throws Exception {

        SourceConfig config = new SourceConfig();
        TestPIMList list = new TestPIMList();
        TestHelper helper = new TestHelper(list);

        BBPIMSyncSource source = new BBPIMSyncSource(config, list, helper, 1, 2);

        // Populate the db with 1000 items
        Vector keys = new Vector();
        for(int i=0;i<1000;++i) {
            TestItem item = new TestItem(list);
            list.addItem(item);
            keys.addElement(helper.getUID(item));
        }
        // Now we can simulate the sending phase
        numItemsRequests = 0;
        source.beginSync(SyncML.ALERT_CODE_SLOW);
        int numItems = 0;
        SyncItem item = source.getNextItem();
        while (item != null) {
            item = source.getNextItem();
            numItems++;
        }
        // We expect only one iteration over the list of items
        // And we assume that anytime the enumeration is asked, then
        // its items are iterated
        System.out.println("numItemsRequests = " + numItemsRequests);
        System.out.println("numItems = " + numItems);
        assertTrue(numItems == 1000);
        assertTrue(numItemsRequests == 1);

        // Now receive the items. Simulate a bunch of messages with 50 changes
        // each
        for(int i=0;i<keys.size();i+=50) {
            Vector keysSubset = new Vector();
            for(int j=i;j<i+50;++j) {
                String key = (String)keys.elementAt(j);
                keysSubset.addElement(key);
            }
            source.prepareModifications(keysSubset);
            // Receive 50 updates
            for(int j=0;j<50;++j) {
                byte content[] = ("Content " + i).getBytes();
                SyncItem sItem = new SyncItem((String)keysSubset.elementAt(j),
                        "text/plain", SyncItem.STATE_UPDATED,
                        null, content);
                source.updateItem(sItem);
            }
        }
        System.out.println("numItemsRequests = " + numItemsRequests);
        assertTrue(numItemsRequests == (1 + (1000 / 50)));

        // Check that the db contains 1000 items
        Enumeration elements = list.items();
        numItems = 0;
        while(elements.hasMoreElements()) {
            TestItem pimItem = (TestItem)elements.nextElement();
            numItems++;
        }
        System.out.println("numItems = " + numItems);
        assertTrue(numItems == 1000);
    }

    public void testBeginSyncRefreshFromServer() throws Exception {
        SourceConfig config = new SourceConfig();
        TestPIMList list = new TestPIMList();
        TestHelper helper = new TestHelper(list);

        // Populate the list with two items
        TestItem item = new TestItem(list);
        list.addItem(item);
        item = new TestItem(list);
        list.addItem(item);

        BBPIMSyncSource source = new BBPIMSyncSource(config, list, helper, 1, 2);
        source.beginSync(SyncML.ALERT_CODE_REFRESH_FROM_SERVER);

        // Verify that the list is empty now
        Enumeration items = list.items();
        assertTrue(!items.hasMoreElements());
    }

    private class TestItem implements PIMItem {
        private String key;
        private TestPIMList list;

        public TestItem(TestPIMList list) {
            key = "" + id++;
            this.list = list;
        }

        public void setStringArray(int a,int b,int c, String[] values) {
        }

        public void setString(int a,int b,int c, String value) {
        }

        public void setInt(int a,int b,int c, int v) {
        }

        public void setDate(int a,int b,int c, long v) {
        }

        public void setBoolean(int a,int b,int c, boolean v) {
        }

        public void setBinary(int a,int b,int c, byte[] v, int d, int e) {
        }

        public int maxCategories() {
            return 0;
        }

        public String[] getCategories() {
            return null;
        }

        public void removeFromCategory(String cat) {
        }

        public void addToCategory(String cat) {
        }

        public int getAttributes(int a, int b) {
            return 0;
        }

        public void removeValue(int a, int b) {
        }

        public int countValues(int a) {
            return 0;
        }

        public void addBoolean(int a, int b, boolean v) {
        }

        public boolean getBoolean(int a, int b) {
            return false;
        }

        public void addStringArray(int a, int b, String v[]) {
        }

        public String[] getStringArray(int a, int b) {
            return null;
        }

        public void addString(int a, int b, String v) {
        }

        public String getString(int a, int b) {
            if (a == BlackBerryContact.UID) {
                return key;
            }
            return null;
        }

        public void addDate(int a, int b, long v) {
        }

        public long getDate(int a, int b) {
            return 0;
        }

        public void addInt(int a, int b, int v) {
        }

        public int getInt(int a, int b) {
            return 0;
        }

        public void addBinary(int a, int b, byte v[], int c, int d) {
        }

        public byte[] getBinary(int a, int b) {
            return null;
        }

        public int[] getFields() {
            return null;
        }

        public boolean isModified() {
            return false;
        }

        public void commit() throws PIMException {
            list.addItem(this);
        }

        public PIMList getPIMList() {
            return list;
        }
    }

    private class TestHelper extends PIMItemHelper {

        private TestPIMList list;

        public TestHelper(PIMList list) {
            super(list);
            this.list = (TestPIMList)list;
        }

        public String itemToString(PIMItem item,
                                   AdditionalItemsAttributes additionals)
                                   throws PIMException {
            return "TestItem";
        }
    
        public String getUID(PIMItem item) throws PIMException {

            return item.getString(BlackBerryContact.UID, 0);
        }
    
        public PIMItem createItem() throws PIMException {
            TestItem item = new TestItem(list);
            return item;
        }
    
        public AdditionalItemsAttributes fillItem(PIMItem item, String pimString,
                                                  AdditionalItemsAttributes additionals)
        throws PIMException {

            return additionals;
        }

        public void clearItem(PIMItem item) throws PIMException {
        }

        public void copyUnsupportedFields(PIMItem origItem, PIMItem item) throws
        PIMException {
        }
 
    
        public boolean deleteItem(PIMItem item) throws PIMException {
            return list.remove(item);
        }

        public boolean isInSyncRange(PIMItem item) throws PIMException {
            return true;
        }
    }

    private class TestPIMList implements PIMList {

        Hashtable items = new Hashtable();

        public void addItem(PIMItem item) {
            String uid = item.getString(BlackBerryContact.UID, 0);
            items.put(uid, item);
        }

        public boolean remove(PIMItem item) {
            String uid = item.getString(BlackBerryContact.UID, 0);
            return items.remove(uid) != null;
            
        }

        public void addCategory(String category) {
        }

        public void close() {
        }

        public void deleteCategory(String category, boolean deleteUnassignedItems) {
        }

        public String getArrayElementLabel(int stringArrayField, int arrayElement) {
            return null;
        }

        public String getAttributeLabel(int attribute) {
            return null;
        }

        public String[] getCategories() {
            return null;
        }

        public int getFieldDataType(int field) {
            return 0;
        }

        public String getFieldLabel(int field) {
            return null;
        }

        public String getName() {
            return null;
        }

        public int[] getSupportedArrayElements(int stringArrayField) {
            return null;
        }

        public int[] getSupportedAttributes(int field) {
            return null;
        }
        public int[] getSupportedFields() {
            return null;
        }

        public boolean isCategory(String category) {
            return false;
        }
        public boolean isSupportedArrayElement(int stringArrayField, int arrayElement) {
            return true;
        }
        
        public boolean isSupportedAttribute(int field, int attribute) {
            return true;
        }
        public boolean isSupportedField(int field) {
            return true;
        }
        public Enumeration items() {
            numItemsRequests++;
            return items.elements();
        }
        public Enumeration items(PIMItem matching) {
            return items.elements();
        }
        public Enumeration items(String matching) {
            return items.elements();
        }
        public Enumeration itemsByCategory(String category) {
            return items.elements();
        }
        public int maxCategories() {
            return 0;
        }
        public int maxValues(int field) {
            return 0;
        }
        public void renameCategory(String currentCategory, String newCategory) {
        }
        public int stringArraySize(int stringArrayField) {
            return 0;
        }
    }
}
 
